@@ -1,7 +1,5 @@
 /*
- * $Id: alive.h 16106 2009-03-10 15:16:31Z rmanfredi $
- *
- * Copyright (c) 2002-2003, Raphael Manfredi
+ * Copyright (c) 2002-2003, 2014 Raphael Manfredi
  *
  *----------------------------------------------------------------------
  * This file is part of gtk-gnutella.
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -30,7 +28,7 @@
  * Alive status checking ping/pongs.
  *
  * @author Raphael Manfredi
- * @date 2002-2003
+ * @date 2002-2003, 2014
  */
 
 #ifndef _core_alive_h_
@@ -38,19 +36,26 @@
 
 #include "common.h"
 
+#include "lib/timestamp.h"		/* For time_delta_t */
+
 struct gnutella_node;
 struct guid;
 
+struct alive;
+typedef struct alive alive_t;
+
 /*
  * Public interface.
  */
 
-gpointer alive_make(struct gnutella_node *n, int max);
-void alive_free(gpointer obj);
-gboolean alive_send_ping(gpointer obj);
-gboolean alive_ack_ping(gpointer obj, const struct guid *);
-void alive_ack_first(gpointer obj, const struct guid *);
-void alive_get_roundtrip_ms(gconstpointer obj, guint32 *avg, guint32 *last);
+alive_t *alive_make(struct gnutella_node *n, int max);
+void alive_free(alive_t *a);
+bool alive_send_ping(alive_t *a);
+bool alive_ack_ping(alive_t *a, const struct guid *);
+void alive_ack_first(alive_t *a, const struct guid *);
+void alive_get_roundtrip_ms(const alive_t *a, uint32 *avg, uint32 *last);
+time_delta_t alive_elapsed(const alive_t *a);
 
 #endif /* _core_alive_h_ */
 
+/* vi: set ts=4 sw=4 cindent: */
