@@ -1,6 +1,4 @@
 /*
- * $Id: bogons.c 17194 2009-11-05 21:44:06Z rmanfredi $
- *
  * Copyright (c) 2004, Raphael Manfredi
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -35,18 +33,18 @@
 
 #include "common.h"
 
-RCSID("$Id: bogons.c 17194 2009-11-05 21:44:06Z rmanfredi $")
-
 #include "bogons.h"
 #include "settings.h"
 
 #include "lib/ascii.h"
 #include "lib/file.h"
-#include "lib/glib-missing.h"
 #include "lib/halloc.h"
+#include "lib/host_addr.h"
 #include "lib/iprange.h"
 #include "lib/parse.h"
 #include "lib/path.h"
+#include "lib/str.h"
+#include "lib/tm.h"
 #include "lib/walloc.h"
 #include "lib/watcher.h"
 
@@ -55,48 +53,48 @@ RCSID("$Id: bogons.c 17194 2009-11-05 21:44:06Z rmanfredi $")
 
 #include "lib/override.h"		/* Must be the last header included */
 
-static const char bogon[] = "bogon";
-
 static const char bogons_file[] = "bogons.txt";
 static const char bogons_what[] = "Bogus IP addresses";
 
 static struct iprange_db *bogons_db; /**< The database of bogus CIDR ranges */
+static time_t bogons_mtime;			 /**< Modification time of loaded file */
 
 /**
  * Load bogons data from the supplied FILE.
  *
  * @returns the amount of entries loaded.
  */
-static int
+static int G_COLD
 bogons_load(FILE *f)
 {
 	char line[1024];
-	char *p;
-	guint32 ip, netmask;
+	uint32 ip, netmask;
 	int linenum = 0;
 	int bits;
 	iprange_err_t error;
+	filestat_t buf;
 
 	bogons_db = iprange_new();
+	if (-1 == fstat(fileno(f), &buf)) {
+		g_warning("cannot stat %s: %m", bogons_file);
+	} else {
+		bogons_mtime = buf.st_mtime;
+	}
 
 	while (fgets(line, sizeof(line), f)) {
 		linenum++;
-		if (*line == '\0' || *line == '#')
-			continue;
 
 		/*
 		 * Remove all trailing spaces in string.
 		 * Otherwise, lines which contain only spaces would cause a warning.
 		 */
 
-		p = strchr(line, '\0');
-		while (--p >= line) {
-			guchar c = (guchar) *p;
-			if (!is_ascii_space(c))
-				break;
-			*p = '\0';
+		if (!file_line_chomp_tail(ARYLEN(line), NULL)) {
+			g_warning("%s, line %d: too long a line", bogons_file, linenum);
+			break;
 		}
-		if ('\0' == *line)
+
+		if (file_line_is_skipable(line))
 			continue;
 
 		if (!string_to_ip_and_mask(line, &ip, &netmask)) {
@@ -106,8 +104,7 @@ bogons_load(FILE *f)
 		}
 
 		bits = netmask_to_cidr(netmask);
-		error = iprange_add_cidr(bogons_db, ip, bits,
-					deconstify_gpointer(bogon));
+		error = iprange_add_cidr(bogons_db, ip, bits, 1);
 
 		switch (error) {
 		case IPR_ERR_OK:
@@ -124,9 +121,9 @@ bogons_load(FILE *f)
 	iprange_sync(bogons_db);
 
 	if (GNET_PROPERTY(reload_debug)) {
-		g_message("loaded %u bogus IP ranges (%u hosts)",
+		g_debug("loaded %u bogus IP ranges (%u hosts)",
 			iprange_get_item_count(bogons_db),
-			iprange_get_host_count(bogons_db));
+			iprange_get_host_count4(bogons_db));
 	}
 
 	return iprange_get_item_count(bogons_db);
@@ -137,7 +134,7 @@ bogons_load(FILE *f)
  * addresses changed.
  */
 static void
-bogons_changed(const char *filename, gpointer unused_udata)
+bogons_changed(const char *filename, void *unused_udata)
 {
 	FILE *f;
 	char buf[80];
@@ -151,8 +148,9 @@ bogons_changed(const char *filename, gpointer unused_udata)
 
 	bogons_close();
 	count = bogons_load(f);
+	fclose(f);
 
-	gm_snprintf(buf, sizeof(buf), "Reloaded %d bogus IP ranges.", count);
+	str_bprintf(ARYLEN(buf), "Reloaded %d bogus IP ranges.", count);
 	gcu_statusbar_message(buf);
 }
 
@@ -169,28 +167,22 @@ bogons_changed(const char *filename, gpointer unused_udata)
  * The selected file will then be monitored and a reloading will occur
  * shortly after a modification.
  */
-static void
+static void G_COLD
 bogons_retrieve(void)
 {
 	FILE *f;
 	int idx;
 	char *filename;
-#ifndef OFFICIAL_BUILD
-	static file_path_t fp[3];
-#else
-	static file_path_t fp[2];
-#endif
+	file_path_t fp[4];
+	unsigned length;
 
-	file_path_set(&fp[0], settings_config_dir(), bogons_file);
-	file_path_set(&fp[1], PRIVLIB_EXP, bogons_file);
-#ifndef OFFICIAL_BUILD
-	file_path_set(&fp[2], PACKAGE_EXTRA_SOURCE_DIR, bogons_file);
-#endif
+	length = settings_file_path_load(fp, bogons_file, SFP_DFLT);
 
-	f = file_config_open_read_norename_chosen(
-			bogons_what, fp, G_N_ELEMENTS(fp), &idx);
+	g_assert(length <= N_ITEMS(fp));
 
-	if (!f)
+	f = file_config_open_read_norename_chosen(bogons_what, fp, length, &idx);
+
+	if (NULL == f)
 	   return;
 
 	filename = make_pathname(fp[idx].dir, fp[idx].name);
@@ -224,20 +216,23 @@ bogons_close(void)
  *
  * @returns TRUE if found, and FALSE if not.
  */
-gboolean
+bool
 bogons_check(const host_addr_t ha)
 {
-	if (NET_TYPE_IPV4 == host_addr_net(ha)) {
-		guint32 ip = host_addr_ipv4(ha);
-		return bogons_db && iprange_get(bogons_db, ip);
-	} else if (NET_TYPE_IPV6 == host_addr_net(ha)) {
-		host_addr_t to;
+	if G_UNLIKELY(NULL == bogons_db)
+		return FALSE;
 
-		if (host_addr_convert(ha, &to, NET_TYPE_IPV4))
-			return bogons_check(to);
-	}
+	/*
+	 * If the bogons file is too ancient, there is a risk it may flag an
+	 * IP as bogus whereas it is no longer reserved.  IPv4 address shortage
+	 * makes that likely.
+	 *		--RAM, 2010-11-07
+	 */
 
-	return FALSE;
+	if (delta_time(tm_time(), bogons_mtime) > 15552000)	/* ~6 months */
+		return !host_addr_is_routable(ha);
+
+	return 0 != iprange_get_addr(bogons_db, ha);
 }
 
 /* vi: set ts=4 sw=4 cindent: */
