@@ -1,6 +1,4 @@
 /*
- * $Id: dime.h 16104 2009-03-10 14:58:09Z rmanfredi $
- *
  * Copyright (c) 2004, Jeroen Asselman
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -41,21 +39,22 @@
 #include "common.h"
 
 struct dime_record;
+struct pslist;
 
 struct dime_record *dime_record_alloc(void);
 void dime_record_free(struct dime_record **record_ptr);
 
 size_t dime_create_record(const struct dime_record *record,
-			char **data_ptr, gboolean first, gboolean last);
+			char **data_ptr, bool first, bool last);
 
-gboolean dime_record_set_data(struct dime_record *record,
+bool dime_record_set_data(struct dime_record *record,
 			const void *data, size_t size);
-gboolean dime_record_set_id(struct dime_record *record, const char *id);
-gboolean dime_record_set_type_uri(struct dime_record *, const char *type);
-gboolean dime_record_set_type_mime(struct dime_record *, const char *type);
+bool dime_record_set_id(struct dime_record *record, const char *id);
+bool dime_record_set_type_uri(struct dime_record *, const char *type);
+bool dime_record_set_type_mime(struct dime_record *, const char *type);
 
-void dime_list_free(GSList **list_ptr);
-GSList *dime_parse_records(const char *data, size_t size);
+void dime_list_free(struct pslist **list_ptr);
+struct pslist *dime_parse_records(const char *data, size_t size);
 
 const char *dime_record_type(const struct dime_record *record);
 size_t dime_record_type_length(const struct dime_record *record);
