@@ -1,8 +1,6 @@
 /*
- * $Id: dump.c 16990 2009-09-25 15:23:43Z rmanfredi $
- *
- * Copyright (c) 2009, Raphael Manfredi
- * Copyright (c) 2006, Christian Biere
+ * Copyright (c) 2009, 2012 Raphael Manfredi
+ * Copyright (c) 2006 Christian Biere
  *
  *----------------------------------------------------------------------
  * This file is part of gtk-gnutella.
@@ -20,7 +18,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -30,35 +28,91 @@
  *
  * Traffic dumping, for later analysis with barracuda.
  *
+ * To use traffic dumping, one needs to create named pipes in the
+ * configuration directory (~/.gtk-gnutella, usually).
+ *
+ * When told to dump data, gtk-gnutella opens the corresponding named
+ * pipe for writing, the user needing to open it for reading to get
+ * the traffic dumps.
+ *
+ * Dumping is non-blocking and automatically stops when the pipe becomes
+ * full, meaning the reader stopped or cannot extract data fast enough.
+ *
+ * Two properties (which can be set through a shell command) are governing
+ * whether traffic dumping should be done, with explicit names:
+ *
+ *    dump_received_gnutella_packets
+ *    dump_transmitted_gnutella_packets
+ *
+ * Because indiscriminate traffic dumping can create a lot of output, three
+ * additional properties are used (RX = receive, TX = transmit):
+ *
+ *    dump_rx_addrs
+ *    dump_tx_from_addrs
+ *    dump_tx_to_addrs
+ *
+ * These contain a comma-separated list of IP addresses that are used to
+ * filter traffic: only traffic sent to or originating from the listed
+ * addresses (depending on whether we're filtering for TX or RX) is dumped.
+ * An empty set means no filtering is done at all, i.e. everything is logged.
+ *
+ * When dumping TX traffic, we can filter on both from/to, but RX traffic
+ * can only be filtered on its from address.  This is because RX traffic is
+ * dumped just after being received and before we know the fate of the packet,
+ * whereas TX traffic is dumped when the packet is sent out, at which time we
+ * know both its origin and its destination.
+ *
+ * Dumps are made with a header, hence one must use "barracuda -D" to
+ * post-process the logs.
+ *
+ * Here is a sample traffic dumping session:
+ *
+ * # create the named pipes
+ * mknod ~/.gtk-gnutella/packets_rx.dump p
+ * mknod ~/.gtk-gnutella/packets_tx.dump p
+ *
+ * # prepare TX traffic reading
+ * gzip -9 <~/.gtk-gnutella/packets_tx.dump >traffic.gz
+ *
+ * # request gtk-gnutella dumpping
+ * echo set dump_transmitted_gnutella_packets TRUE | gtk-gnutella --shell
+ *
+ * # when done, hit ^C and post-process with barracuda
+ * gzcat traffic.gz | barracuda -D | less
+ *
+ * # to cleanly stop dumps, use this instead of hitting ^C
+ * echo set dump_transmitted_gnutella_packets FALSE | gtk-gnutella --shell
+ *
  * @author Raphael Manfredi
- * @date 2009
+ * @date 2009, 2012
  * @author Christian Biere
  * @date 2006
  */
 
 #include "common.h"
 
-RCSID("$Id: dump.c 16990 2009-09-25 15:23:43Z rmanfredi $")
-
 #include "nodes.h"
 #include "settings.h"
 
 #include "lib/fd.h"
 #include "lib/file.h"
+#include "lib/halloc.h"
+#include "lib/ipset.h"
 #include "lib/path.h"
 #include "lib/pmsg.h"
 #include "lib/slist.h"
-#include "lib/halloc.h"
 
 #include "if/gnet_property.h"
 #include "if/gnet_property_priv.h"
 
 #include "lib/override.h"		/* Must be the last header included */
 
+#define DUMP_BUFFER_MAX	(256 * 1024UL)	/* Max amount we keep in memory */
+
 /**
  * Barracuda header flags.
  */
-enum dump_header_flags {                                                        
+enum dump_header_flags {
 	DH_F_UDP  = (1 << 0),
 	DH_F_TCP  = (1 << 1),
 	DH_F_IPV4 = (1 << 2),
@@ -80,7 +134,7 @@ struct dump_header {
  *	uint8_t addr[16];
  *	uint8_t port[2];
  */
-	guchar data[19];
+	uchar data[19];
 };
 
 /**
@@ -89,7 +143,7 @@ struct dump_header {
 struct dump {
 	const char * const filename;
 	slist_t *slist;
-	const gboolean *dump_var;
+	const bool *dump_var;
 	size_t fill;
 	int fd;
 	int initialized;
@@ -116,20 +170,24 @@ static struct dump dump_tx = {
 	PROP_DUMP_TRANSMITTED_GNUTELLA_PACKETS,	/* dump_property */
 };
 
+static ipset_t dump_rx_addrs = IPSET_INIT;
+static ipset_t dump_tx_from_addrs = IPSET_INIT;
+static ipset_t dump_tx_to_addrs = IPSET_INIT;
+
 /**
  * Fill dump header with node address information.
  */
 static void
-dump_header_set(struct dump_header *dh, const struct gnutella_node *node)
+dump_header_set(struct dump_header *dh, const gnutella_node_t *node)
 {
-	memset(dh, 0, sizeof dh);
+	ZERO(dh);
 
 	dh->data[0] = NODE_IS_UDP(node) ? DH_F_UDP : DH_F_TCP;
 	switch (host_addr_net(node->addr)) {
 	case NET_TYPE_IPV4:
 		{
-			guint32 ip;
-			
+			uint32 ip;
+
 			dh->data[0] |= DH_F_IPV4;
 			ip = host_addr_ipv4(node->addr);
 			poke_be32(&dh->data[1], ip);
@@ -155,14 +213,12 @@ dump_disable(struct dump *dump)
 	pmsg_slist_free(&dump->slist);
 	dump->fill = 0;
 
-	if (dump->fd >= 0) {
-		close(dump->fd);
-		dump->fd = -1;
-	}
+	if (dump->fd >= 0)
+		fd_close(&dump->fd);
+
 	dump->initialized = FALSE;
-	if (*dump->dump_var) {
+	if (*dump->dump_var)
 		gnet_prop_set_boolean_val(dump->dump_property, FALSE);
-	}
 }
 
 /**
@@ -170,7 +226,7 @@ dump_disable(struct dump *dump)
  *
  * @return TRUE if initialized.
  */
-static gboolean
+static bool
 dump_initialize(struct dump *dump)
 {
 	char *pathname;
@@ -224,7 +280,7 @@ dump_flush(struct dump *dump)
 {
 	while (dump->fill > 0) {
 		ssize_t written;
-		struct iovec *iov;
+		iovec_t *iov;
 		int iov_cnt;
 
 		iov = pmsg_slist_to_iovec(dump->slist, &iov_cnt, NULL);
@@ -233,11 +289,11 @@ dump_flush(struct dump *dump)
 
 		if ((ssize_t)-1 == written) {
 			if (!is_temporary_error(errno)) {
-				g_warning("error writing to %s: %s -- disabling dumping",
-					dump->filename, g_strerror(errno));
+				g_warning("error writing to %s: %m -- disabling dumping",
+					dump->filename);
 				dump_disable(dump);
 			}
-			if (dump->fill >= 256 * 1024UL) {
+			if (dump->fill >= DUMP_BUFFER_MAX) {
 				g_warning(
 					"queue is full: %s -- disabling dumping", dump->filename);
 				dump_disable(dump);
@@ -260,16 +316,21 @@ dump_flush(struct dump *dump)
  * Dump packet received from node.
  */
 static void
-dump_packet_from(struct dump *dump, const struct gnutella_node *node)
+dump_packet_from(struct dump *dump, const gnutella_node_t *node)
 {
-	struct dump_header dh;	
+	struct dump_header dh;
+
+	g_assert(node != NULL);
 
 	if (!dump_initialize(dump))
 		return;
 
+	if (!ipset_contains_addr(&dump_rx_addrs, node->addr, TRUE))
+		return;
+
 	dump_header_set(&dh, node);
-	dump_append(dump, dh.data, sizeof dh.data);
-	dump_append(dump, node->header, sizeof node->header);
+	dump_append(dump, ARYLEN(dh.data));
+	dump_append(dump, ARYLEN(node->header));
 	dump_append(dump, node->data, node->size);
 	dump_flush(dump);
 }
@@ -280,15 +341,15 @@ dump_packet_from(struct dump *dump, const struct gnutella_node *node)
  */
 static void
 dump_packet_from_to(struct dump *dump,
-	const struct gnutella_node *from, const struct gnutella_node *to,
+	const gnutella_node_t *from, const gnutella_node_t *to,
 	const pmsg_t *mb)
 {
-	struct dump_header dh_to;	
-	struct dump_header dh_from;	
+	struct dump_header dh_to;
+	struct dump_header dh_from;
 
 	g_assert(to != NULL);
 	g_assert(mb != NULL);
-	g_assert(pmsg_read_base(mb) == pmsg_start(mb));
+	g_assert(pmsg_is_unread(mb));
 
 	if (!dump_initialize(dump))
 		return;
@@ -297,16 +358,23 @@ dump_packet_from_to(struct dump *dump,
 	 * This is only for Gnutella packets, leave DHT messages out.
 	 */
 
-	if (GTA_MSG_DHT == gnutella_header_get_function(pmsg_start(mb)))
+	if (GTA_MSG_DHT == gnutella_header_get_function(pmsg_phys_base(mb)))
+		return;
+
+	if (!ipset_contains_addr(&dump_tx_to_addrs, to->addr, TRUE))
 		return;
 
 	if (NULL == from) {
-		struct gnutella_node local;
+		gnutella_node_t local;
 		local.peermode = NODE_IS_UDP(to) ? NODE_P_UDP : NODE_P_NORMAL;
 		local.addr = listen_addr();
 		local.port = GNET_PROPERTY(listen_port);
+		if (!ipset_contains_addr(&dump_tx_from_addrs, local.addr, TRUE))
+			return;
 		dump_header_set(&dh_from, &local);
 	} else {
+		if (!ipset_contains_addr(&dump_tx_from_addrs, from->addr, TRUE))
+			return;
 		dump_header_set(&dh_from, from);
 	}
 
@@ -314,10 +382,10 @@ dump_packet_from_to(struct dump *dump,
 	dh_to.data[0] |= DH_F_TO;
 	if (pmsg_prio(mb) != PMSG_P_DATA)
 		dh_to.data[0] |= DH_F_CTRL;
-		
-	dump_append(dump, dh_to.data, sizeof dh_to.data);
-	dump_append(dump, dh_from.data, sizeof dh_from.data);
-	dump_append(dump, pmsg_read_base(mb), pmsg_size(mb));
+
+	dump_append(dump, ARYLEN(dh_to.data));
+	dump_append(dump, ARYLEN(dh_from.data));
+	dump_append(dump, pmsg_start(mb), pmsg_size(mb));
 	dump_flush(dump);
 }
 
@@ -325,7 +393,7 @@ dump_packet_from_to(struct dump *dump,
  * Dump packet received from node.
  */
 void
-dump_rx_packet(const struct gnutella_node *node)
+dump_rx_packet(const gnutella_node_t *node)
 {
 	if (GNET_PROPERTY(dump_received_gnutella_packets)) {
 		dump_packet_from(&dump_rx, node);
@@ -340,7 +408,7 @@ dump_rx_packet(const struct gnutella_node *node)
  */
 void
 dump_tx_tcp_packet(
-	const struct gnutella_node *from, const struct gnutella_node *to,
+	const gnutella_node_t *from, const gnutella_node_t *to,
 	const pmsg_t *mb)
 {
 	if (GNET_PROPERTY(dump_transmitted_gnutella_packets)) {
@@ -361,7 +429,7 @@ void
 dump_tx_udp_packet(const gnet_host_t *to, const pmsg_t *mb)
 {
 	if (GNET_PROPERTY(dump_transmitted_gnutella_packets)) {
-		struct gnutella_node udp;
+		gnutella_node_t udp;
 
 		g_assert(to != NULL);
 		g_assert(mb != NULL);
@@ -381,6 +449,31 @@ dump_tx_udp_packet(const gnet_host_t *to, const pmsg_t *mb)
 	}
 }
 
+/**
+ * Dump RX traffic coming from listed addresses, all addresses if empty.
+ */
+void
+dump_rx_set_addrs(const char *s)
+{
+	ipset_set_addrs(&dump_rx_addrs, s);
+}
+
+/**
+ * Dump TX traffic coming from listed addresses, all addresses if empty.
+ */
+void dump_tx_set_from_addrs(const char *s)
+{
+	ipset_set_addrs(&dump_tx_from_addrs, s);
+}
+
+/**
+ * Dump TX traffic sent to listed addresses, all addresses if empty.
+ */
+void dump_tx_set_to_addrs(const char *s)
+{
+	ipset_set_addrs(&dump_tx_to_addrs, s);
+}
+
 /**
  * Initialize traffic dumping.
  */
@@ -400,6 +493,10 @@ dump_close(void)
 		dump_disable(&dump_rx);
 	if (dump_tx.initialized)
 		dump_disable(&dump_tx);
+
+	ipset_clear(&dump_rx_addrs);
+	ipset_clear(&dump_tx_from_addrs);
+	ipset_clear(&dump_tx_to_addrs);
 }
 
 /* vi: set ts=4 sw=4 cindent: */
