@@ -294,7 +294,7 @@ hostiles_load(FILE *f, hostiles_t which)
 
 	hostile_db[which] = iprange_new();
 
-	while (fgets(ARYLEN(line), f)) {
+	while (fgets(line, sizeof(line), f)) {
 		linenum++;
 
 		/*
@@ -429,7 +429,7 @@ hostiles_retrieve(hostiles_t which)
 
 	case HOSTILE_GLOBAL:
 		{
-			file_path_t fp[3];
+			file_path_t fp[4];
 			FILE *f;
 			int idx;
 			uint length;
