@@ -1,6 +1,4 @@
 /*
- * $Id: huge.c 17300 2010-02-26 16:33:57Z rmanfredi $
- *
  * Copyright (c) 2002-2003, Ch. Tronche & Raphael Manfredi
  *
  * Started by Ch. Tronche (http://tronche.com/) 28/04/2002
@@ -21,7 +19,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -31,6 +29,9 @@
  *
  * HUGE support (Hash/URN Gnutella Extension).
  *
+ * Server side: computation of SHA1 hash digests and replies.
+ * SHA1 is defined in RFC 3174.
+ *
  * @author Raphael Manfredi
  * @date 2002-2003
  * @author Ch. Tronche (http://tronche.com/)
@@ -39,23 +40,27 @@
 
 #include "common.h"
 
-RCSID("$Id: huge.c 17300 2010-02-26 16:33:57Z rmanfredi $")
-
 #include "huge.h"
-#include "share.h"
-#include "gmsg.h"
+
 #include "dmesh.h"
+#include "gmsg.h"
+#include "nodes.h"
+#include "settings.h"
+#include "share.h"
+#include "spam.h"
 #include "verify_sha1.h"
 #include "verify_tth.h"
 #include "version.h"
-#include "settings.h"
-#include "spam.h"
 
 #include "lib/atoms.h"
 #include "lib/base32.h"
+#include "lib/cq.h"
 #include "lib/file.h"
+#include "lib/gnet_host.h"
 #include "lib/halloc.h"
+#include "lib/hashing.h"
 #include "lib/header.h"
+#include "lib/hikset.h"
 #include "lib/parse.h"
 #include "lib/pattern.h"
 #include "lib/sha1.h"
@@ -69,13 +74,10 @@ RCSID("$Id: huge.c 17300 2010-02-26 16:33:57Z rmanfredi $")
 
 #include "lib/override.h"		/* Must be the last header included */
 
-/***
- *** Server side: computation of SHA1 hash digests and replies.
- *** SHA1 is defined in RFC 3174.
- ***/
+#define HUGE_SHA1_CACHE_FREQ	60	/* seconds, for SHA1 cache dumps */
 
 /**
- * There's an in-core cache (the GHashTable sha1_cache), and a
+ * There's an in-core cache (the hash table ``sha1_cache''), and a
  * persistent copy (normally in ~/.gtk-gnutella/sha1_cache). The
  * in-core cache is filled with the persistent one at launch. When the
  * "shared_file" (the records describing the shared files, see
@@ -99,16 +101,16 @@ struct sha1_cache_entry {
 	const struct tth *tth;		/**< TTH (binary; atom)				*/
     filesize_t  size;			/**< File size                      */
     time_t mtime;				/**< Last modification time         */
-    gboolean shared;			/**< There's a known entry for this
+    bool shared;				/**< There's a known entry for this
                                      file in the share library      */
 };
 
-static GHashTable *sha1_cache;
+static hikset_t *sha1_cache;
 
 /**
  * cache_dirty = TRUE means that in-core cache is different from the disk one.
  */
-static gboolean cache_dirty;
+static bool cache_dirty;
 static time_t cache_dumped;
 
 static cpattern_t *has_http_urls;
@@ -143,18 +145,18 @@ static void update_volatile_cache(
  */
 static void
 add_volatile_cache_entry(const char *filename, filesize_t size, time_t mtime,
-	const struct sha1 *sha1, const struct tth *tth, gboolean known_to_be_shared)
+	const struct sha1 *sha1, const struct tth *tth, bool known_to_be_shared)
 {
 	struct sha1_cache_entry *item;
-   
-	item = walloc(sizeof *item);
+
+	WALLOC(item);
 	item->file_name = atom_str_get(filename);
 	item->size = size;
 	item->mtime = mtime;
 	item->sha1 = atom_sha1_get(sha1);
 	item->tth = tth ? atom_tth_get(tth) : NULL;
 	item->shared = known_to_be_shared;
-	g_hash_table_insert(sha1_cache, deconstify_gchar(item->file_name), item);
+	hikset_insert_key(sha1_cache, &item->file_name);
 }
 
 /* Disk cache */
@@ -180,8 +182,8 @@ cache_entry_print(FILE *f, const char *filename,
 	g_return_if_fail(sha1);
 	g_return_if_fail(size > 0);
 
-	uint64_to_string_buf(size, size_buf, sizeof size_buf);
-	uint64_to_string_buf(mtime, mtime_buf, sizeof mtime_buf);
+	uint64_to_string_buf(size, ARYLEN(size_buf));
+	uint64_to_string_buf(mtime, ARYLEN(mtime_buf));
 
 	fprintf(f, "%s\t%s\t%s\t%s\n", bitprint_to_urn_string(sha1, tth),
 		size_buf, mtime_buf, filename);
@@ -200,15 +202,14 @@ add_persistent_cache_entry(const char *filename, filesize_t size,
 	pathname = make_pathname(settings_config_dir(), "sha1_cache");
 	f = file_fopen(pathname, "a");
 	if (f) {
-		struct stat sb;
+		filestat_t sb;
 
 		/*
 		 * If we're adding the very first entry (file empty), then emit header.
 		 */
 
 		if (fstat(fileno(f), &sb)) {
-			g_warning("add_persistent_cache_entry: could not stat \"%s\"",
-				pathname);
+			g_warning("%s(): could not stat \"%s\": %m", G_STRFUNC, pathname);
 		} else {
 			if (0 == sb.st_size) {
 				fputs(sha1_persistent_cache_file_header, f);
@@ -217,15 +218,14 @@ add_persistent_cache_entry(const char *filename, filesize_t size,
 		}
 		fclose(f);
 	} else {
-		g_warning("add_persistent_cache_entry: could not open \"%s\"",
-			pathname);
+		g_warning("%s(): could not open \"%s\": %m", G_STRFUNC, pathname);
 	}
 	HFREE_NULL(pathname);
 }
 
 struct dump_cache_context {
 	FILE *f;
-	gboolean forced;
+	bool forced;
 };
 
 /**
@@ -233,13 +233,11 @@ struct dump_cache_context {
  * called by dump_cache to dump the whole in-memory cache onto disk.
  */
 static void
-dump_cache_one_entry(gpointer unused_key, gpointer value, gpointer udata)
+dump_cache_one_entry(void *value, void *udata)
 {
 	struct sha1_cache_entry *e = value;
 	struct dump_cache_context *ctx = udata;
 
-	(void) unused_key;
-
 	if (ctx->forced || e->shared) {
 		cache_entry_print(ctx->f,
 			e->file_name, e->sha1, e->tth, e->size, e->mtime);
@@ -250,23 +248,23 @@ dump_cache_one_entry(gpointer unused_key, gpointer value, gpointer udata)
  * Dump the whole in-memory cache onto disk.
  */
 static void
-dump_cache(gboolean force)
+dump_cache(bool force)
 {
 	FILE *f;
 	file_path_t fp;
 
 	if (!force && !cache_dirty)
 		return;
-	
+
 	file_path_set(&fp, settings_config_dir(), "sha1_cache");
 	f = file_config_open_write("SHA-1 cache", &fp);
 	if (f) {
 		struct dump_cache_context ctx;
-		
+
 		fputs(sha1_persistent_cache_file_header, f);
 		ctx.f = f;
 		ctx.forced = force;
-		g_hash_table_foreach(sha1_cache, dump_cache_one_entry, &ctx);
+		hikset_foreach(sha1_cache, dump_cache_one_entry, &ctx);
 		if (file_config_close(f, &fp)) {
 			cache_dirty = FALSE;
 		}
@@ -287,28 +285,25 @@ dump_cache(gboolean force)
  * the line and calls add_volatile_cache_entry() to append the record
  * to the in-memory cache.
  */
-static void
+static void G_COLD
 parse_and_append_cache_entry(char *line)
 {
-	const char *file_name;
-	char *file_name_end;
 	const char *p, *end; /* pointers to scan the line */
 	int c, error;
 	filesize_t size;
 	time_t mtime;
 	struct sha1 sha1;
 	struct tth tth;
-	gboolean has_tth;
+	bool has_tth;
 
 	/* Skip comments and blank lines */
-	c = line[0];
-	if (c == '\0' || c == '#' || c == '\n')
+	if (file_line_is_skipable(line))
 		return;
 
 	/* Scan until file size */
 
 	p = line;
-	while ((c = *p) != '\0' && c != '\t' && c != '\n') {
+	while ((c = *p) != '\0' && c != '\t') {
 		p++;
 	}
 
@@ -325,7 +320,7 @@ parse_and_append_cache_entry(char *line)
 		if (
 			*p != '\t' ||
 			(p - sha1_digest_ascii) != SHA1_BASE32_SIZE ||
-			SHA1_RAW_SIZE != base32_decode(sha1.data, sizeof sha1.data,
+			SHA1_RAW_SIZE != base32_decode(VARLEN(sha1),
 								sha1_digest_ascii, SHA1_BASE32_SIZE)
 		) {
 			goto failure;
@@ -356,46 +351,61 @@ parse_and_append_cache_entry(char *line)
 
 	/* p is now supposed to point to the file name */
 
-	file_name = p;
-	file_name_end = strchr(file_name, '\n');
-
-	if (!file_name_end) {
+	if (vstrchr(p, '\t') != NULL)
 		goto failure;
+
+	/*
+	 * Validate that the file still exists and was not modified since its
+	 * insertion in the cache before recording it.
+	 * 		--RAM, 2017-10-19
+	 */
+
+	{
+		filestat_t st;
+
+		if (-1 == stat(p, &st))
+			return;		/* No file, or cannot access it */
+
+		if (!S_ISREG(st.st_mode))
+			return;		/* Not a regular file */
+
+		if (UNSIGNED(st.st_size) != size)
+			return;		/* File was modified */
+
+		if (delta_time(st.st_mtime, mtime) > 0)
+			return;		/* File was modified */
 	}
 
-	/* Set string end markers */
-	*file_name_end = '\0';
-
-	add_volatile_cache_entry(file_name, size, mtime,
+	add_volatile_cache_entry(p, size, mtime,
 		&sha1, has_tth ? &tth : NULL, FALSE);
 	return;
 
 failure:
-	g_warning("Malformed line in SHA1 cache file: %s", line);
+	g_warning("malformed line in SHA1 cache file: %s", line);
 }
 
 /**
  * Read the whole persistent cache into memory.
  */
-static void
+static void G_COLD
 sha1_read_cache(void)
 {
 	FILE *f;
 	file_path_t fp[1];
-	gboolean truncated = FALSE;
+	bool truncated = FALSE;
 
 	g_return_if_fail(settings_config_dir());
 
 	file_path_set(fp, settings_config_dir(), "sha1_cache");
-	f = file_config_open_read("SHA-1 cache", fp, G_N_ELEMENTS(fp));
+	f = file_config_open_read("SHA-1 cache", fp, N_ITEMS(fp));
 	if (f) {
 		for (;;) {
 			char buffer[4096];
 
-			if (NULL == fgets(buffer, sizeof buffer, f))
+			if (NULL == fgets(buffer, sizeof(buffer), f))
 				break;
 
-			if (NULL == strchr(buffer, '\n')) {
+			if (!file_line_chomp_tail(ARYLEN(buffer), NULL)) {
 				truncated = TRUE;
 			} else if (truncated) {
 				truncated = FALSE;
@@ -408,16 +418,75 @@ sha1_read_cache(void)
 	}
 }
 
+static bool
+huge_spam_check(shared_file_t *sf, const struct sha1 *sha1)
+{
+	if (NULL != sha1 && spam_sha1_check(sha1)) {
+		g_warning("file \"%s\" is listed as spam (SHA1)", shared_file_path(sf));
+		return TRUE;
+	}
+
+	if (
+		spam_check_filename_size(shared_file_name_nfc(sf),
+			shared_file_size(sf))
+	) {
+		g_warning("file \"%s\" is listed as spam (Name)", shared_file_path(sf));
+		return TRUE;
+	}
+	return FALSE;
+}
+
+static cevent_t *cache_dump_ev;
+
+/**
+ * Callout queue callback invoked when we should flush the SHA1 cache.
+ */
+static void
+cache_dump_due(cqueue_t *cq, void *unused_obj)
+{
+	(void) unused_obj;
+
+	cq_zero(cq, &cache_dump_ev);	/* Indicates callback fired */
+	dump_cache(FALSE);
+}
+
+/**
+ * Dump the cache at most about once per HUGE_SHA1_CACHE_FREQ secs..
+ */
+static void
+cache_dump_schedule(void)
+{
+	time_delta_t t;
+
+	cache_dirty = TRUE;
+
+	if G_UNLIKELY(0 == cache_dumped) {
+		t = 0;
+	} else {
+		t = delta_time(tm_time(), cache_dumped);
+		if (t >= HUGE_SHA1_CACHE_FREQ)
+			t = 0;
+		else
+			t = HUGE_SHA1_CACHE_FREQ - t;
+	}
+	if (0 == t) {
+		dump_cache(FALSE);
+	} else if (NULL == cache_dump_ev) {
+		cache_dump_ev = cq_main_insert(t * 1000, cache_dump_due, NULL);
+	}
+}
+
 /**
  ** Asynchronous computation of hash value
  **/
 
-gboolean
-huge_update_hashes(struct shared_file *sf,
+bool
+huge_update_hashes(shared_file_t *sf,
 	const struct sha1 *sha1, const struct tth *tth)
 {
 	struct sha1_cache_entry *cached;
-	struct stat sb;
+	filestat_t sb;
+	const sha1_t *osha1;
 
 	shared_file_check(sf);
 	g_return_val_if_fail(sha1, FALSE);
@@ -427,8 +496,8 @@ huge_update_hashes(struct shared_file *sf,
 	 */
 
 	if (-1 == stat(shared_file_path(sf), &sb)) {
-		g_warning("discarding SHA1 for file \"%s\": can't stat(): %s",
-			shared_file_path(sf), g_strerror(errno));
+		g_warning("discarding SHA1 for file \"%s\": can't stat(): %m",
+			shared_file_path(sf));
 		shared_file_remove(sf);
 		return TRUE;
 	}
@@ -437,32 +506,39 @@ huge_update_hashes(struct shared_file *sf,
 		g_warning("file \"%s\" was modified whilst SHA1 was computed",
 			shared_file_path(sf));
 		shared_file_set_modification_time(sf, sb.st_mtime);
-		return request_sha1(sf);					/* Retry! */
+		request_sha1(sf);					/* Retry! */
+		return TRUE;
 	}
 
-	if (spam_sha1_check(sha1)) {
-		g_warning("file \"%s\" is listed as spam", shared_file_path(sf));
+	if (huge_spam_check(sf, sha1)) {
 		shared_file_remove(sf);
 		return FALSE;
 	}
 
-	shared_file_set_sha1(sf, sha1);
+	/*
+	 * Testing for the SHA1 already being present avoids problems when
+	 * we are coming here simply to update the TTH of a completed file.
+	 * In that case, we already have the SHA1 normally, hence we will
+	 * never call shared_file_set_sha1().
+	 * 		--RAM, 2017-10-19
+	 */
+
+	osha1 = shared_file_sha1(sf);
+
+	if (NULL == osha1 || !sha1_eq(sha1, osha1))
+		shared_file_set_sha1(sf, sha1);
+
 	shared_file_set_tth(sf, tth);
 
 	/* Update cache */
 
-	cached = g_hash_table_lookup(sha1_cache,
-					cast_to_gconstpointer(shared_file_path(sf)));
+	cached = hikset_lookup(sha1_cache, shared_file_path(sf));
 
 	if (cached) {
 		update_volatile_cache(cached, shared_file_size(sf),
 			shared_file_modification_time(sf), sha1, tth);
-		cache_dirty = TRUE;
 
-		/* Dump the cache at most about once per minute. */
-		if (!cache_dumped || delta_time(tm_time(), cache_dumped) > 60) {
-			dump_cache(FALSE);
-		}
+		cache_dump_schedule(); 	/* Dump cache once per minute */
 	} else {
 		add_volatile_cache_entry(shared_file_path(sf),
 			shared_file_size(sf), shared_file_modification_time(sf),
@@ -471,20 +547,20 @@ huge_update_hashes(struct shared_file *sf,
 			shared_file_size(sf), shared_file_modification_time(sf),
 			sha1, tth);
 	}
-	if (NULL == tth) {
-		request_tigertree(sf, FALSE);
-	}
 	return TRUE;
 }
 
 /**
- * Get the next file waiting for its hash to be computed from the queue
- * (actually a stack).
+ * Look whether we still need to compute the SHA1 of the given shared file
+ * by looking into our in-core cache to see whether the entry we have is
+ * up-to-date.
  *
- * @return this file.
+ * @param sf	the shared file for which we want to compute the SHA1
+ *
+ * @return TRUE if the file need SHA1 recomputation.
  */
-static gboolean
-huge_need_sha1(struct shared_file *sf)
+static bool
+huge_need_sha1(shared_file_t *sf)
 {
 	struct sha1_cache_entry *cached;
 
@@ -494,39 +570,28 @@ huge_need_sha1(struct shared_file *sf)
 	 * After a rescan, there might be files in the queue which are
 	 * no longer shared.
 	 */
-	if (!(SHARE_F_INDEXED & shared_file_flags(sf)))
+
+	if (!shared_file_indexed(sf))
 		return FALSE;
 
-	/*
-	 * XXX HACK ALERT
-	 *
-	 * We need to be careful here, because each time the library is rescanned,
-	 * we add file to the list of SHA1 to recompute if we don't have them
-	 * yet.  This means that when we rescan the library during a computation,
-	 * we'll add duplicates to our working queue.
-	 *
-	 * Fortunately, we can probe our in-core cache to see if what we have
-	 * is already up-to-date.
-	 *
-	 * XXX It would be best to maintain a hash table of all the filenames
-	 * XXX in our workqueue and not enqueue the work in the first place.
-	 * XXX		--RAM, 21/05/2002
-	 */
+	if G_UNLIKELY(NULL == sha1_cache)
+		return FALSE;		/* Shutdown occurred (processing TEQ event?) */
 
-	cached = g_hash_table_lookup(sha1_cache, shared_file_path(sf));
-	if (cached) {
-		struct stat sb;
+	cached = hikset_lookup(sha1_cache, shared_file_path(sf));
+
+	if (cached != NULL) {
+		filestat_t sb;
 
 		if (-1 == stat(shared_file_path(sf), &sb)) {
-			g_warning("ignoring SHA1 recomputation request for \"%s\": %s",
-				shared_file_path(sf), g_strerror(errno));
+			g_warning("ignoring SHA1 recomputation request for \"%s\": %m",
+				shared_file_path(sf));
 			return FALSE;
 		}
 		if (
-			cached->size + (off_t) 0 == sb.st_size + (filesize_t) 0 &&
+			cached->size + (fileoffset_t) 0 == sb.st_size + (filesize_t) 0 &&
 			cached->mtime == sb.st_mtime
 		) {
-			if (GNET_PROPERTY(dbg) > 1) {
+			if (GNET_PROPERTY(share_debug) > 1) {
 				g_warning("ignoring duplicate SHA1 work for \"%s\"",
 					shared_file_path(sf));
 			}
@@ -547,21 +612,25 @@ huge_need_sha1(struct shared_file *sf)
  * is put in a queue for it's SHA1 digest to be computed.
  */
 
-static gboolean
+static bool
 huge_verify_callback(const struct verify *ctx, enum verify_status status,
 	void *user_data)
 {
-	struct shared_file *sf = user_data;
+	shared_file_t *sf = user_data;
 
 	shared_file_check(sf);
+
 	switch (status) {
 	case VERIFY_START:
+		if (!huge_need_sha1(sf))
+			return FALSE;
 		gnet_prop_set_boolean_val(PROP_SHA1_REBUILDING, TRUE);
-		return huge_need_sha1(sf);
+		return TRUE;
 	case VERIFY_PROGRESS:
-		return 0 != (SHARE_F_INDEXED & shared_file_flags(sf));
+		return shared_file_indexed(sf);
 	case VERIFY_DONE:
 		huge_update_hashes(sf, verify_sha1_digest(ctx), NULL);
+		request_tigertree(sf, TRUE);
 		/* FALL THROUGH */
 	case VERIFY_ERROR:
 	case VERIFY_SHUTDOWN:
@@ -576,22 +645,24 @@ huge_verify_callback(const struct verify *ctx, enum verify_status status,
 }
 
 /**
- * Put the shared file on the stack of the things to do. Activate the timer if
- * this wasn't done already.
+ * Put the shared file on the stack of the things to do.
+ *
+ * We first begin with the computation of the SHA1, and when completed we
+ * will continue with the TTH computation.
  */
 static void
-queue_shared_file_for_sha1_computation(struct shared_file *sf)
+queue_shared_file_for_sha1_computation(shared_file_t *sf)
 {
 	int inserted;
-	
+
  	shared_file_check(sf);
 
 	inserted = verify_sha1_enqueue(FALSE, shared_file_path(sf),
 					shared_file_size(sf), huge_verify_callback,
 					shared_file_ref(sf));
-	if (!inserted) {
+
+	if (!inserted)
 		shared_file_unref(&sf);
-	}
 }
 
 /**
@@ -599,9 +670,9 @@ queue_shared_file_for_sha1_computation(struct shared_file *sf)
  *
  * @return true (in the C sense) if it is, or false otherwise.
  */
-static gboolean
+static bool
 cached_entry_up_to_date(const struct sha1_cache_entry *cache_entry,
-	const struct shared_file *sf)
+	const shared_file_t *sf)
 {
 	return cache_entry->size == shared_file_size(sf)
 		&& cache_entry->mtime == shared_file_modification_time(sf);
@@ -610,59 +681,84 @@ cached_entry_up_to_date(const struct sha1_cache_entry *cache_entry,
 /**
  * External interface to check whether the sha1 for shared_file is known.
  */
-gboolean
-sha1_is_cached(const struct shared_file *sf)
+bool
+sha1_is_cached(const shared_file_t *sf)
 {
 	const struct sha1_cache_entry *cached;
 
-	cached = g_hash_table_lookup(sha1_cache, shared_file_path(sf));
+	cached = hikset_lookup(sha1_cache, shared_file_path(sf));
 	return cached && cached_entry_up_to_date(cached, sf);
 }
 
+/**
+ * Quickly check whether file changed since we cached its SHA1 / TTH.
+ *
+ * This is used when attempting to resume a seeded file, to quickly check
+ * whether it has been changing since the last time we checked it.
+ *
+ * @param path		full path to file we wish to check
+ * @param size		file size on disk
+ * @param mtime		mtime in the file's i-node on disk
+ *
+ * @return TRUE if cache is up-to-date.
+ */
+bool
+huge_cached_is_uptodate(const char *path, filesize_t size, time_t mtime)
+{
+	const struct sha1_cache_entry *cached = hikset_lookup(sha1_cache, path);
+
+	if (NULL == cached)
+		return FALSE;
+
+	return cached->size == size && cached->mtime == mtime;
+}
 
 /**
  * External interface to call for getting the hash for a shared_file.
- *
- * @return if shared_file_remove() was called, FALSE is returned and
- *         "sf" is no longer valid. Otherwise TRUE is returned.
  */
-gboolean
-request_sha1(struct shared_file *sf)
+void
+request_sha1(shared_file_t *sf)
 {
 	struct sha1_cache_entry *cached;
 
 	shared_file_check(sf);
 
-	cached = g_hash_table_lookup(sha1_cache, shared_file_path(sf));
+	if (!shared_file_indexed(sf))
+		return;		/* "stale" shared file, has been superseded or removed */
+
+	cached = hikset_lookup(sha1_cache, shared_file_path(sf));
+
 	if (cached && cached_entry_up_to_date(cached, sf)) {
-		if (spam_sha1_check(cached->sha1)) {
-			g_warning("file \"%s\" is listed as spam", shared_file_path(sf));
-			shared_file_remove(sf);
-			return FALSE;
-		} else {
-			cache_dirty = TRUE;
-			cached->shared = TRUE;
-			shared_file_set_sha1(sf, cached->sha1);
-			shared_file_set_tth(sf, cached->tth);
-			request_tigertree(sf, FALSE);
-			return TRUE;
+		cache_dirty = TRUE;
+		cached->shared = TRUE;
+		shared_file_set_sha1(sf, cached->sha1);
+		shared_file_set_tth(sf, cached->tth);
+
+		if (NULL == cached->tth || !shared_file_tth_is_available(sf)) {
+			if (GNET_PROPERTY(share_debug) > 1) {
+				if (NULL == cached->tth)
+					g_debug("no known TTH entry for \"%s\"", shared_file_path(sf));
+				else
+					g_debug("no TTH %s entry cached for \"%s\"",
+						tth_base32(cached->tth), shared_file_path(sf));
+			}
+
+			request_tigertree(sf, NULL == cached->tth);
 		}
 	} else {
-
-		if (GNET_PROPERTY(dbg) > 4) {
+		if (GNET_PROPERTY(share_debug) > 1) {
 			if (cached)
-				g_message("Cached SHA1 entry for \"%s\" outdated: "
+				g_debug("cached SHA1 entry for \"%s\" outdated: "
 					"had mtime %lu, now %lu",
 					shared_file_path(sf),
-					(gulong) cached->mtime,
-					(gulong) shared_file_modification_time(sf));
+					(ulong) cached->mtime,
+					(ulong) shared_file_modification_time(sf));
 			else
-				g_message("Queuing \"%s\" for SHA1 computation",
+				g_debug("queuing \"%s\" for SHA1 computation",
 						shared_file_path(sf));
 		}
 
 		queue_shared_file_for_sha1_computation(sf);
-		return TRUE;
 	}
 }
 
@@ -673,15 +769,15 @@ request_sha1(struct shared_file *sf)
  * things using the same pattern with other letters, as being rather
  * improbable hashes.
  */
-gboolean
+bool
 huge_improbable_sha1(const char *buf, size_t len)
 {
 	size_t ilen = 0;			/* Length of the improbable sequence */
 	size_t i, longest = 0;
 
 	for (i = 1; i < len; i++) {
-		guchar previous, c;
-		
+		uchar previous, c;
+
 		previous = buf[i - 1];
 		c = buf[i];
 
@@ -709,28 +805,28 @@ huge_improbable_sha1(const char *buf, size_t len)
  *
  * @return TRUE if the SHA1 was valid and properly decoded, FALSE on error.
  */
-gboolean
+bool
 huge_sha1_extract32(const char *buf, size_t len, struct sha1 *sha1,
-	const struct gnutella_node *n)
+	const gnutella_node_t *n)
 {
 	if (len != SHA1_BASE32_SIZE || huge_improbable_sha1(buf, len))
 		goto bad;
 
-	if (SHA1_RAW_SIZE != base32_decode(sha1->data, sizeof sha1->data, buf, len))
+	if (SHA1_RAW_SIZE != base32_decode(PTRLEN(sha1), buf, len))
 		goto bad;
 
 	/*
 	 * Make sure the decoded value in `sha1' is "valid".
 	 */
 
-	if (huge_improbable_sha1(sha1->data, sizeof sha1->data)) {
-		if (GNET_PROPERTY(dbg)) {
+	if (huge_improbable_sha1(ARYLEN(sha1->data))) {
+		if (GNET_PROPERTY(share_debug)) {
 			if (is_printable(buf, len)) {
 				g_warning("%s has improbable SHA1 (len=%lu): %.*s, hex: %s",
 					gmsg_node_infostr(n),
 					(unsigned long) len,
 					(int) MIN(len, (size_t) INT_MAX),
-					buf, data_hex_str(sha1->data, sizeof sha1->data));
+					buf, data_hex_str(ARYLEN(sha1->data)));
 			} else
 				goto bad;		/* SHA1 should be printable originally */
 		}
@@ -740,7 +836,7 @@ huge_sha1_extract32(const char *buf, size_t len, struct sha1 *sha1,
 	return TRUE;
 
 bad:
-	if (GNET_PROPERTY(dbg)) {
+	if (GNET_PROPERTY(share_debug)) {
 		if (is_printable(buf, len)) {
 			g_warning("%s has bad SHA1 (len=%u): %.*s",
 				gmsg_node_infostr(n),
@@ -758,20 +854,20 @@ bad:
 	return FALSE;
 }
 
-gboolean
+bool
 huge_tth_extract32(const char *buf, size_t len, struct tth *tth,
-	const struct gnutella_node *n)
+	const gnutella_node_t *n)
 {
 	if (len != TTH_BASE32_SIZE)
 		goto bad;
 
-	if (TTH_RAW_SIZE != base32_decode(tth->data, sizeof tth->data, buf, len))
+	if (TTH_RAW_SIZE != base32_decode(PTRLEN(tth), buf, len))
 		goto bad;
 
 	return TRUE;
 
 bad:
-	if (GNET_PROPERTY(dbg)) {
+	if (GNET_PROPERTY(share_debug)) {
 		if (is_printable(buf, len)) {
 			g_warning("%s has bad TTH (len=%u): %.*s",
 				gmsg_node_infostr(n),
@@ -794,7 +890,7 @@ bad:
  * X-Alt but is really an old X-Gnutella-Alternate-Location containing a
  * list of HTTP URLs.
  */
-static gboolean
+static bool
 huge_is_pure_xalt(const char *value, size_t len)
 {
 	host_addr_t addr;
@@ -815,7 +911,7 @@ huge_is_pure_xalt(const char *value, size_t len)
 	if (string_to_host_addr(value, NULL, &addr))
 		return TRUE;
 
-	if (pattern_qsearch(has_http_urls, value, len, 0, qs_any))
+	if (pattern_search(has_http_urls, value, len, 0, qs_any))
 		return FALSE;
 
 	return TRUE;
@@ -826,16 +922,36 @@ huge_is_pure_xalt(const char *value, size_t len)
  * about other sources for this file.
  *
  * Also knows about "Alternate-Location", "Alt-Location", "X-Alt" and "X-Falt".
+ *
+ * @param sha1		the SHA1 for which we're parsing alt-locs
+ * @param header	the headers supplied by the remote host
+ * @param origin	if non-NULL, this is the host supplying the alt-locs
  */
 void
-huge_collect_locations(const struct sha1 *sha1, const header_t *header)
+huge_collect_locations(const sha1_t *sha1, const header_t *header,
+	const gnet_host_t * origin)
 {
 	char *alt;
 	size_t len;
+	const char *user_agent;
 
 	g_return_if_fail(sha1);
 	g_return_if_fail(header);
 
+	/*
+	 * This code can be invoked on the download path (when we analyse
+	 * locations sent by the server) or on the upload path (when we
+	 * analyze those sent by the user to whom we are uploading).
+	 *
+	 * Therefore the user agent name can be held in the Server or
+	 * User-Agent header, depending.
+	 */
+
+	user_agent = header_get(header, "User-Agent");	/* Uploading */
+
+	if (NULL == user_agent)
+		user_agent = header_get(header, "Server");	/* Downloading */
+
 	alt = header_get(header, "X-Gnutella-Alternate-Location");
 
 	/*
@@ -850,7 +966,7 @@ huge_collect_locations(const struct sha1 *sha1, const header_t *header)
 		alt = header_get(header, "Alt-Location");
 
 	if (alt != NULL) {
-		dmesh_collect_locations(sha1, alt);
+		dmesh_collect_locations(sha1, alt, origin, user_agent);
 		return;
 	}
 
@@ -865,9 +981,9 @@ huge_collect_locations(const struct sha1 *sha1, const header_t *header)
 		 */
 
 		if (huge_is_pure_xalt(alt, len))
-			dmesh_collect_compact_locations(sha1, alt);
+			dmesh_collect_compact_locations(sha1, alt, origin, user_agent);
 		else
-			dmesh_collect_locations(sha1, alt);
+			dmesh_collect_locations(sha1, alt, origin, user_agent);
     }
 
 	/*
@@ -877,38 +993,103 @@ huge_collect_locations(const struct sha1 *sha1, const header_t *header)
 	alt = header_get(header, "X-Falt");
 
 	if (alt != NULL) {
-		dmesh_collect_fw_hosts(sha1, alt);
+		dmesh_collect_fw_hosts(sha1, alt, origin, user_agent);
 	}
 }
 
+/**
+ * Iterator callback to check whether SHA1 cache entry is still being
+ * shared.  Otherwise, it is removed from the set.
+ *
+ * @return TRUE if the item was freed and needs to be dropped from the cache.
+ */
+static bool
+cache_entry_is_shared(void *v, void *unused_udata)
+{
+	struct sha1_cache_entry *e = v;
+	shared_file_t *sf;
+
+	(void) unused_udata;
+
+	sf = shared_file_by_sha1(e->sha1);
+
+	if G_UNLIKELY(SHARE_REBUILDING == sf)
+		return FALSE;		/* Cannot decide */
+
+	if (NULL == sf) {
+		/* Entry no longer shared */
+
+		atom_str_free_null(&e->file_name);
+		atom_sha1_free_null(&e->sha1);
+		atom_tth_free_null(&e->tth);
+		WFREE(e);
+
+		return TRUE;
+	}
+
+	shared_file_unref(&sf);
+	return FALSE;
+}
+
+/**
+ * Purge the SHA1 cache.
+ *
+ * This is meant to be called once, after the first library rescan.
+ *
+ * It will remove from the cache any entry for which we hold a SHA1 that
+ * is not currently being shared.
+ *
+ * This is important because we recompute the TTH of seeded files, and they
+ * are inserted into the persistent cache: we do not want to hold them
+ * forever.
+ *
+ * Users may also remove files from their library by removing entire directories
+ * from the sharing filesystem tree.  The files may still be on the filesystem
+ * but end-up being unshared, and we do not want to keep them in the cache
+ * (and in memory) if they are actually not going to be useful at all.
+ */
+void
+huge_sha1_cache_prune(void)
+{
+	size_t pruned;
+
+	pruned = hikset_foreach_remove(sha1_cache, cache_entry_is_shared, NULL);
+
+	if (GNET_PROPERTY(share_debug)) {
+		g_info("%s(): pruned %zu entr%s from SHA1 cache",
+			G_STRFUNC, pruned, plural_y(pruned));
+	}
+
+	if (pruned != 0)
+		cache_dump_schedule();
+}
+
 /**
  * Initialize the HUGE layer.
  */
 void
 huge_init(void)
 {
-	sha1_cache = g_hash_table_new(pointer_hash_func, NULL);
+	sha1_cache = hikset_create(		/* Keys are atoms */
+		offsetof(struct sha1_cache_entry, file_name), HASH_KEY_SELF, 0);
 	sha1_read_cache();
-	has_http_urls = pattern_compile("http://");
+	has_http_urls = pattern_compile("http://", FALSE);
 }
 
 /**
  * Free SHA1 cache entry.
  */
-static gboolean
-cache_free_entry(gpointer unused_key, gpointer v, gpointer unused_udata)
+static void
+cache_free_entry(void *v, void *unused_udata)
 {
 	struct sha1_cache_entry *e = v;
 
-	(void) unused_key;
 	(void) unused_udata;
 
 	atom_str_free_null(&e->file_name);
 	atom_sha1_free_null(&e->sha1);
 	atom_tth_free_null(&e->tth);
-	wfree(e, sizeof *e);
-
-	return TRUE;
+	WFREE(e);
 }
 
 /**
@@ -919,9 +1100,8 @@ huge_close(void)
 {
 	dump_cache(FALSE);
 
-	g_hash_table_foreach_remove(sha1_cache, cache_free_entry, NULL);
-	g_hash_table_destroy(sha1_cache);
-	sha1_cache = NULL;
+	hikset_foreach(sha1_cache, cache_free_entry, NULL);
+	hikset_free_null(&sha1_cache);
 
 	pattern_free(has_http_urls);
 	has_http_urls = NULL;
