@@ -1,6 +1,4 @@
 /*
- * $Id: ioheader.c 16377 2009-04-02 19:03:59Z rmanfredi $
- *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -35,8 +33,6 @@
 
 #include "common.h"
 
-RCSID("$Id: ioheader.c 16377 2009-04-02 19:03:59Z rmanfredi $")
-
 #include "ioheader.h"
 #include "sockets.h"
 #include "bsched.h"
@@ -44,7 +40,9 @@ RCSID("$Id: ioheader.c 16377 2009-04-02 19:03:59Z rmanfredi $")
 #include "lib/getline.h"
 #include "lib/header.h"
 #include "lib/inputevt.h"
+#include "lib/log.h"		/* For log_printable() */
 #include "lib/misc.h"
+#include "lib/str.h"
 #include "lib/walloc.h"
 
 #include "if/gnet_property_priv.h"
@@ -61,8 +59,8 @@ enum io_opaque_magic { IO_OPAQUE_MAGIC = 0x578da161U };
  */
 struct io_header {
 	enum io_opaque_magic magic;
-	gpointer resource;				/**< Resource for which we're parsing */
-	gpointer *io_opaque;			/**< Where we're referenced in resource */
+	void *resource;					/**< Resource for which we're parsing */
+	void **io_opaque;				/**< Where we're referenced in resource */
 	struct gnutella_socket *socket;	/**< Socket on which we're reading */
 	bsched_bws_t bws;				/**< Bandwidth scheduler to use */
 	header_t *header;
@@ -70,8 +68,8 @@ struct io_header {
 	const struct io_error *error;	/**< Error callbacks */
 	io_done_cb_t process_header;	/**< Called when all headers are read */
 	io_start_cb_t header_read_start;/**< Called when reading first byte */
-	GString *text;					/**< Full header text */
-	guint read_bytes;				/**< Amount of bytes read */
+	str_t *text;					/**< Full header text */
+	uint read_bytes;				/**< Amount of bytes read */
 	int flags;
 };
 
@@ -79,9 +77,9 @@ struct io_header {
  * Internal consistency checks.
  */
 static void
-io_check(const gpointer opaque)
+io_check(const void *opaque)
 {
-	struct io_header *ih = opaque;
+	const struct io_header *ih = opaque;
 
 	g_assert(ih);
 	g_assert(IO_OPAQUE_MAGIC == ih->magic);
@@ -95,31 +93,28 @@ io_check(const gpointer opaque)
  * Free the opaque I/O data.
  */
 void
-io_free(gpointer opaque)
+io_free(void *opaque)
 {
 	struct io_header *ih = opaque;
 
 	io_check(opaque);
 	*ih->io_opaque = NULL;
 
-	if (ih->header)
-		header_free(ih->header);
-	if (ih->getline)
-		getline_free(ih->getline);
-	if (ih->text)
-		g_string_free(ih->text, TRUE);
+	header_free_null(&ih->header);
+	getline_free_null(&ih->getline);
+	str_destroy_null(&ih->text);
 
 	ih->magic = 0;
-	wfree(ih, sizeof(*ih));
+	WFREE(ih);
 }
 
 /**
  * Fetch header structure from opaque I/O data.
  */
 struct header *
-io_header(const gpointer opaque)
+io_header(const void *opaque)
 {
-	struct io_header *ih = opaque;
+	const struct io_header *ih = opaque;
 
 	io_check(opaque);
 
@@ -130,9 +125,9 @@ io_header(const gpointer opaque)
  * Fetch getline structure from opaque I/O data.
  */
 struct getline *
-io_getline(const gpointer opaque)
+io_getline(const void *opaque)
 {
-	struct io_header *ih = opaque;
+	const struct io_header *ih = opaque;
 
 	io_check(opaque);
 
@@ -146,24 +141,24 @@ io_getline(const gpointer opaque)
  * The returned data will be freed when io_free() is called.
  */
 char *
-io_gettext(const gpointer opaque)
+io_gettext(const void *opaque)
 {
-	struct io_header *ih = opaque;
+	const struct io_header *ih = opaque;
 
 	io_check(opaque);
 	g_assert(ih->flags & IO_SAVE_HEADER);	/* They must have requested this */
 	g_assert(ih->text);
 
-	return ih->text->str;
+	return str_2c(ih->text);
 }
 
 /**
  * How many bytes we received so far during header parsing.
  */
-guint
-io_get_read_bytes(const gpointer opaque)
+uint
+io_get_read_bytes(const void *opaque)
 {
-	struct io_header *ih = opaque;
+	const struct io_header *ih = opaque;
 
 	io_check(opaque);
 
@@ -190,11 +185,13 @@ io_header_parse(struct io_header *ih)
 nextline:
 	switch (getline_read(ih->getline, s->buf, s->pos, &parsed)) {
 	case READ_OVERFLOW:
-		g_warning("io_header_parse: line too long, disconnecting from %s",
-			host_addr_to_string(s->addr));
-		dump_hex(stderr, "Leading Data", s->buf, MIN(s->pos, 256));
-		fprintf(stderr, "------ Header Dump:\n");
-		header_dump(stderr, header, "-----");
+		g_warning("%s(): line too long, disconnecting from %s",
+			G_STRFUNC, host_addr_to_string(s->addr));
+		if (log_printable(LOG_STDERR)) {
+			dump_hex(stderr, "Leading Data", s->buf, MIN(s->pos, 256));
+			fprintf(stderr, "------ Header Dump:\n");
+			header_dump(stderr, header, "-----");
+		}
 		(*ih->error->line_too_long)(ih->resource, header);
 		return;
 		/* NOTREACHED */
@@ -223,9 +220,9 @@ nextline:
 		 */
 
 		g_assert(ih->text != NULL);
-		/* No g_string_append_len() in glib 1.2, use g_string_append() --RAM */
-		g_string_append(ih->text, getline_str(ih->getline));
-		g_string_append_c(ih->text, '\n');
+		str_cat_len(ih->text,
+			getline_str(ih->getline), getline_length(ih->getline));
+		str_putc(ih->text, '\n');
 	}
 
 	if (ih->flags & IO_SAVE_FIRST) {
@@ -259,7 +256,7 @@ nextline:
 
 	error = header_append(header,
 		getline_str(ih->getline), getline_length(ih->getline));
-	
+
 	switch (error) {
 	case HEAD_OK:
 		getline_reset(ih->getline);
@@ -273,23 +270,28 @@ nextline:
 			(*ih->error->header_error_tell)(ih->resource, error);
 		/* FALL THROUGH */
 	case HEAD_EOH_REACHED:
-		g_warning("io_header_parse: %s, disconnecting from %s",
-			header_strerror(error),	host_addr_to_string(s->addr));
-		fprintf(stderr, "------ Header Dump:\n");
-		header_dump(stderr, header, "-----");
-		dump_hex(stderr, "Header Line", getline_str(ih->getline),
-			MIN(getline_length(ih->getline), 128));
+		g_warning("%s(): %s, disconnecting from %s",
+			G_STRFUNC, header_strerror(error), host_addr_to_string(s->addr));
+		if (log_printable(LOG_STDERR)) {
+			fprintf(stderr, "------ Header Dump:\n");
+			header_dump(stderr, header, "-----");
+			dump_hex(stderr, "Header Line", getline_str(ih->getline),
+				MIN(getline_length(ih->getline), 128));
+		}
 		(*ih->error->header_error)(ih->resource, error);
 		return;
 		/* NOTREACHED */
 	default:					/* Error, but try to continue */
 		if (GNET_PROPERTY(dbg)) {
-			g_warning("io_header_parse: %s, from %s",
-				header_strerror(error), host_addr_to_string(s->addr));
-			dump_hex(stderr, "Header Line",
-				getline_str(ih->getline), getline_length(ih->getline));
-			fprintf(stderr, "------ Header Dump (so far):\n");
-			header_dump(stderr, header, "-----");
+			g_warning("%s(): %s, from %s",
+				G_STRFUNC, header_strerror(error),
+				host_addr_to_string(s->addr));
+			if (log_printable(LOG_STDERR)) {
+				dump_hex(stderr, "Header Line",
+					getline_str(ih->getline), getline_length(ih->getline));
+				fprintf(stderr, "------ Header Dump (so far):\n");
+				header_dump(stderr, header, "-----");
+			}
 		}
 		getline_reset(ih->getline);
 		goto nextline;			/* Go process other lines we may have read */
@@ -301,7 +303,7 @@ nextline:
 
 	if ((ih->flags & IO_HEAD_ONLY) && s->pos) {
         if (GNET_PROPERTY(dbg)) {
-            g_message("remote %s sent extra bytes after headers",
+            g_debug("remote %s sent extra bytes after headers",
                 host_addr_to_string(s->addr));
             dump_hex(stderr, "Extra Data", s->buf, MIN(s->pos, 512));
         }
@@ -351,7 +353,7 @@ nextline:
  * Read data is then handed out to io_header_parse() for analysis.
  */
 static void
-io_read_data(gpointer data, int unused_source, inputevt_cond_t cond)
+io_read_data(void *data, int unused_source, inputevt_cond_t cond)
 {
 	struct io_header *ih = data;
 	struct gnutella_socket *s = ih->socket;
@@ -386,8 +388,8 @@ io_read_data(gpointer data, int unused_source, inputevt_cond_t cond)
 	g_assert(s->buf_size >= s->pos);
 	count = s->buf_size - s->pos;
 	if (count < 1) {
-		g_warning("ih_header_read: incoming buffer full, "
-			"disconnecting from %s", host_addr_to_string(s->addr));
+		g_warning("%s(): incoming buffer full, disconnecting from %s",
+			G_STRFUNC, host_addr_to_string(s->addr));
 		dump_hex(stderr, "Leading Data", s->buf, MIN(s->pos, 256));
 		(*ih->error->input_buffer_full)(ih->resource);
 		return;
@@ -424,6 +426,37 @@ io_read_data(gpointer data, int unused_source, inputevt_cond_t cond)
 	io_header_parse(ih);
 }
 
+/**
+ * This routine is only used when doing HTTP pipelining, to be able to
+ * process already received network traffic from a connection, which is
+ * expected to be found in the socket's buffer.
+ *
+ * Read data is then handed out to io_header_parse() for analysis.
+ */
+void
+io_add_header(void *opaque)
+{
+	struct io_header *ih = opaque;
+	struct gnutella_socket *s;
+
+	io_check(opaque);
+
+	s = ih->socket;
+	g_assert(s != NULL);
+
+	/*
+	 * First time we read data, notify them once if needed.
+	 */
+
+	if (ih->header_read_start) {
+		(*ih->header_read_start)(ih->resource);
+		ih->header_read_start = NULL;
+	}
+
+	ih->read_bytes += s->pos;
+	io_header_parse(ih);
+}
+
 /**
  * Setup input callback and context for reading/parsing the header.
  * The I/O parsing context is directly written into the structure.
@@ -435,8 +468,8 @@ io_read_data(gpointer data, int unused_source, inputevt_cond_t cond)
  */
 void
 io_get_header(
-	gpointer resource,			/**< Resource for which we're reading headers */
-	gpointer *io_opaque,		/**< Field address in resource's structure */
+	void *resource,				/**< Resource for which we're reading headers */
+	void **io_opaque,			/**< Field address in resource's structure */
 	bsched_bws_t bws,			/**< B/w scheduler from which we read */
 	struct gnutella_socket *s,	/**< Socket from which we're reading */
 	int flags,					/**< I/O parsing flags */
@@ -461,7 +494,7 @@ io_get_header(
 	 * Create and initialize the callback argument used during header reading.
 	 */
 
-	ih = walloc(sizeof *ih);
+	WALLOC(ih);
 	ih->magic = IO_OPAQUE_MAGIC;
 	ih->resource = resource;
 	ih->io_opaque = io_opaque;
@@ -473,7 +506,7 @@ io_get_header(
 	ih->process_header = done;
 	ih->header_read_start = start;
 	ih->header = (flags & IO_SINGLE_LINE) ? NULL : header_make();
-	ih->text = (flags & IO_SAVE_HEADER) ? g_string_sized_new(DFLT_SIZE) : NULL;
+	ih->text = (flags & IO_SAVE_HEADER) ? str_new(DFLT_SIZE) : NULL;
 	ih->read_bytes = 0;
 
 	/*
@@ -510,7 +543,7 @@ io_get_header(
  */
 void
 io_continue_header(
-	gpointer opaque,			/**< Existing header parsing context */
+	void *opaque,				/**< Existing header parsing context */
 	int flags,					/**< New I/O parsing flags */
 	io_done_cb_t done,			/**< Mandatory: final callback when all done */
 	io_start_cb_t start)		/**< Optional: called when reading 1st byte */
@@ -535,13 +568,12 @@ io_continue_header(
 		g_assert(ih->flags & IO_SAVE_HEADER);
 
 		if (flags & IO_SAVE_HEADER)
-			g_string_truncate(ih->text, 0);
+			str_setlen(ih->text, 0);
 		else {
-			g_string_free(ih->text, TRUE);
-			ih->text = NULL;
+			str_destroy_null(&ih->text);
 		}
 	} else if (flags & IO_SAVE_HEADER)
-		ih->text = g_string_sized_new(DFLT_SIZE);
+		ih->text = str_new(DFLT_SIZE);
 
 	header_reset(ih->header);
 	ih->flags = flags;
