@@ -452,7 +452,7 @@ ipp_cache_parse(ipp_cache_t *ic, FILE *f)
 	bit_array_init(tag_used, NUM_IPP_CACHE_TAGS);
 	bit_array_clear_range(tag_used, 0, NUM_IPP_CACHE_TAGS - 1U);
 
-	while (fgets(ARYLEN(line), f)) {
+	while (fgets(line, sizeof(line), f)) {
 		const char *tag_name, *value;
 		char *sp;
 		bool damaged;
