@@ -1,8 +1,6 @@
 /*
- * $Id: ipp_cache.h 16324 2009-03-31 07:00:23Z rmanfredi $
- *
  * Copyright (c) 2009, Raphael Manfredi
- * Copyright (c) 2006, Christian Biere 
+ * Copyright (c) 2006, Christian Biere
  *
  *----------------------------------------------------------------------
  * This file is part of gtk-gnutella.
@@ -20,7 +18,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -41,7 +39,7 @@
 
 #include "common.h"
 
-struct ipp_cache;
+#include "lib/host_addr.h"
 
 typedef struct ipp_cache ipp_cache_t;
 
@@ -54,13 +52,13 @@ enum ipp_cache_id {
 };
 
 void ipp_cache_insert(enum ipp_cache_id cid,
-	const host_addr_t addr, guint16 port);
-gboolean ipp_cache_lookup(enum ipp_cache_id cid,
-	const host_addr_t addr, guint16 port);
+	const host_addr_t addr, uint16 port);
+bool ipp_cache_lookup(enum ipp_cache_id cid,
+	const host_addr_t addr, uint16 port);
 time_t ipp_cache_get_timestamp(enum ipp_cache_id cid,
-	const host_addr_t addr, guint16 port);
-gboolean ipp_cache_remove(enum ipp_cache_id cid,
-	const host_addr_t addr, guint16 port);
+	const host_addr_t addr, uint16 port);
+bool ipp_cache_remove(enum ipp_cache_id cid,
+	const host_addr_t addr, uint16 port);
 
 void ipp_cache_init(void);
 void ipp_cache_load_all(void);
@@ -72,25 +70,25 @@ void ipp_cache_close(void);
  */
 
 static inline void
-tls_cache_insert(const host_addr_t addr, guint16 port)
+tls_cache_insert(const host_addr_t addr, uint16 port)
 {
 	ipp_cache_insert(IPP_CACHE_TLS, addr, port);
 }
 
 static inline void
-tls_cache_remove(const host_addr_t addr, guint16 port)
+tls_cache_remove(const host_addr_t addr, uint16 port)
 {
 	ipp_cache_remove(IPP_CACHE_TLS, addr, port);
 }
 
-static inline gboolean
-tls_cache_lookup(const host_addr_t addr, guint16 port)
+static inline bool
+tls_cache_lookup(const host_addr_t addr, uint16 port)
 {
 	return ipp_cache_lookup(IPP_CACHE_TLS, addr, port);
 }
 
 static inline time_t
-tls_cache_get_timestamp(const host_addr_t addr, guint16 port)
+tls_cache_get_timestamp(const host_addr_t addr, uint16 port)
 {
 	return ipp_cache_get_timestamp(IPP_CACHE_TLS, addr, port);
 }
@@ -100,13 +98,13 @@ tls_cache_get_timestamp(const host_addr_t addr, guint16 port)
  */
 
 static inline void
-g2_cache_insert(const host_addr_t addr, guint16 port)
+g2_cache_insert(const host_addr_t addr, uint16 port)
 {
 	ipp_cache_insert(IPP_CACHE_G2, addr, port);
 }
 
-static inline gboolean
-g2_cache_lookup(const host_addr_t addr, guint16 port)
+static inline bool
+g2_cache_lookup(const host_addr_t addr, uint16 port)
 {
 	return ipp_cache_lookup(IPP_CACHE_G2, addr, port);
 }
@@ -116,13 +114,13 @@ g2_cache_lookup(const host_addr_t addr, guint16 port)
  */
 
 static inline void
-local_addr_cache_insert(const host_addr_t addr, guint16 port)
+local_addr_cache_insert(const host_addr_t addr, uint16 port)
 {
 	ipp_cache_insert(IPP_CACHE_LOCAL_ADDR, addr, port);
 }
 
-static inline gboolean
-local_addr_cache_lookup(const host_addr_t addr, guint16 port)
+static inline bool
+local_addr_cache_lookup(const host_addr_t addr, uint16 port)
 {
 	return ipp_cache_lookup(IPP_CACHE_LOCAL_ADDR, addr, port);
 }
