@@ -1,7 +1,5 @@
 /*
- * $Id: mq.c 17088 2009-10-18 15:01:49Z rmanfredi $
- *
- * Copyright (c) 2002-2003, Raphael Manfredi
+ * Copyright (c) 2002-2003, 2014 Raphael Manfredi
  *
  *----------------------------------------------------------------------
  * This file is part of gtk-gnutella.
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -30,13 +28,11 @@
  * Message queues, common code between TCP and UDP sending stacks.
  *
  * @author Raphael Manfredi
- * @date 2002-2003
+ * @date 2002-2003, 2014
  */
 
 #include "common.h"
 
-RCSID("$Id: mq.c 17088 2009-10-18 15:01:49Z rmanfredi $")
-
 #define MQ_INTERNAL
 #include "mq.h"
 
@@ -44,12 +40,17 @@ RCSID("$Id: mq.c 17088 2009-10-18 15:01:49Z rmanfredi $")
 #include "gmsg.h"
 #include "gnet_stats.h"
 
+#include "lib/array_util.h"
 #include "lib/cq.h"
-#include "lib/glib-missing.h"	/* For gm_snprintf() */
 #include "lib/halloc.h"
+#include "lib/htable.h"
+#include "lib/plist.h"
 #include "lib/pmsg.h"
+#include "lib/str.h"
+#include "lib/stringify.h"		/* For plural() */
 #include "lib/unsigned.h"		/* For size_saturate_add() */
 #include "lib/walloc.h"
+#include "lib/xsort_data.h"
 
 #include "if/gnet_property_priv.h"
 
@@ -59,90 +60,130 @@ RCSID("$Id: mq.c 17088 2009-10-18 15:01:49Z rmanfredi $")
 
 static void qlink_free(mqueue_t *q);
 static void mq_update_flowc(mqueue_t *q);
-static gboolean make_room_header(
-	mqueue_t *q, char *header, guint prio, int needed, int *offset);
-static void mq_swift_timer(cqueue_t *cq, gpointer obj);
+static bool make_room_header(
+	mqueue_t *q, const char *header, uint prio, int needed, int *offset);
+static void mq_swift_timer(cqueue_t *cq, void *obj);
 
-guint32 mq_debug(const mqueue_t *q)
+/**
+ * @return queue's fullness status.
+ */
+mq_status_t
+mq_status(const mqueue_t *q)
 {
+	mq_check_consistency(q);
+
+	if (0 == q->count)
+		return MQ_S_EMPTY;
+
+	if (q->flags & MQ_SWIFT)
+		return MQ_S_SWIFT;
+
+	if (q->flags & MQ_FLOWC)
+		return MQ_S_FLOWC;
+
+	return (q->size > q->lowat) ? MQ_S_WARNZONE : MQ_S_DELAY;
+}
+
+uint32 mq_debug(const mqueue_t *q)
+{
+	mq_check_consistency(q);
 	return *q->debug;
 }
 
-gboolean
+bool
 mq_is_flow_controlled(const mqueue_t *q)
 {
+	mq_check_consistency(q);
 	return 0 != (q->flags & MQ_FLOWC);
 }
 
-gboolean
+bool
 mq_is_swift_controlled(const mqueue_t *q)
 {
+	mq_check_consistency(q);
 	return 0 != (q->flags & MQ_SWIFT);
 }
 
 int
 mq_maxsize(const mqueue_t *q)
 {
+	mq_check_consistency(q);
 	return q->maxsize;
 }
 
 int
 mq_size(const mqueue_t *q)
 {
+	mq_check_consistency(q);
 	return q->size;
 }
 
 int
 mq_lowat(const mqueue_t *q)
 {
+	mq_check_consistency(q);
 	return q->lowat;
 }
 
 int
 mq_hiwat(const mqueue_t *q)
 {
+	mq_check_consistency(q);
 	return q->hiwat;
 }
 
 int
 mq_count(const mqueue_t *q)
 {
+	mq_check_consistency(q);
 	return q->count;
 }
 
 int
 mq_pending(const mqueue_t *q)
 {
+	mq_check_consistency(q);
 	return q->size + tx_pending(q->tx_drv);
 }
 
+int
+mq_tx_pending(const mqueue_t *q)
+{
+	mq_check_consistency(q);
+	return tx_pending(q->tx_drv);
+}
+
 struct bio_source *
 mq_bio(const mqueue_t *q)
 {
+	mq_check_consistency(q);
 	return tx_bio_source(q->tx_drv);
 }
 
-struct gnutella_node *
+gnutella_node_t *
 mq_node(const mqueue_t *q)
 {
+	mq_check_consistency(q);
 	return q->node;
 }
 
 /**
  * Would `additional' bytes of traffic cause the queue to enter flow-control?
  */
-gboolean
+bool
 mq_would_flow_control(const mqueue_t *q, size_t additional)
 {
+	mq_check_consistency(q);
 	return size_saturate_add(q->size, additional) >= UNSIGNED(q->hiwat);
 }
 
 /**
  * Are we already at or above the low watermark?
  */
-gboolean
+bool
 mq_above_low_watermark(const mqueue_t *q)
 {
+	mq_check_consistency(q);
 	return q->size >= q->lowat;
 }
 
@@ -155,22 +196,20 @@ mq_info(const mqueue_t *q)
 	static char buf[160];
 
 	if (q->magic != MQ_MAGIC) {
-		gm_snprintf(buf, sizeof(buf),
-			"queue 0x%lx INVALID (bad magic)", (gulong) q);
+		str_bprintf(ARYLEN(buf), "queue %p INVALID (bad magic)", q);
 	} else {
-		gboolean udp = NODE_IS_UDP(q->node);
+		bool udp = NODE_USES_UDP(q->node);
 
-		gm_snprintf(buf, sizeof(buf),
-			"queue 0x%lx [%s %s node %s%s%s%s%s] (%d item%s, %d byte%s)",
-			(gulong) q, udp ? "UDP" : "TCP",
+		str_bprintf(ARYLEN(buf),
+			"queue %p [%s %s node %s%s%s%s%s] (%d item%s, %d byte%s)",
+			(void *) q, udp ? "UDP" : "TCP",
 			NODE_IS_ULTRA(q->node) ? "ultra" :
 			udp ? "remote" : "leaf", node_addr(q->node),
 			(q->flags & MQ_FLOWC) ? " FLOWC" : "",
 			(q->flags & MQ_DISCARD) ? " DISCARD" : "",
 			(q->flags & MQ_SWIFT) ? " SWIFT" : "",
 			(q->flags & MQ_WARNZONE) ? " WARNZONE" : "",
-			q->count, q->count == 1 ? "" : "s",
-			q->size, q->size == 1 ? "" : "s"
+			PLURAL(q->count), PLURAL(q->size)
 		);
 	}
 
@@ -181,41 +220,41 @@ mq_info(const mqueue_t *q)
 /*
  * This hashtable tracks the queue owning a given glist linkable.
  */
-static GHashTable *qown = NULL;
+static htable_t *qown = NULL;
 
 /**
  * Add linkable into queue
  */
 static void
-mq_add_linkable(mqueue_t *q, GList *l)
+mq_add_linkable(mqueue_t *q, plist_t *l)
 {
 	mqueue_t *owner;
 
 	g_assert(q->magic == MQ_MAGIC);
 	g_assert(l != NULL);
 	g_assert(l->data != NULL);
-	
-	if (qown == NULL)
-		qown = NOT_LEAKING(g_hash_table_new(NULL, NULL));
 
-	owner = g_hash_table_lookup(qown, l);
+	if (qown == NULL)
+		qown = NOT_LEAKING(htable_create(HASH_KEY_SELF, 0));
+
+	owner = htable_lookup(qown, l);
 	if (owner) {
-		g_warning("BUG: added linkable 0x%lx already owned by %s%s",
-			(gulong) l, owner == q ? "ourselves" : "other", mq_info(owner));
+		g_critical("BUG: added linkable %p already owned by %s%s",
+			(void *) l, owner == q ? "ourselves" : "other", mq_info(owner));
 		if (owner != q)
-			g_warning("BUG: will make linkable 0x%lx belong to %s",
-				(gulong) l, mq_info(q));
+			g_warning("BUG: will make linkable %p belong to %s",
+				(void *) l, mq_info(q));
 		g_assert_not_reached();
 	}
 
-	g_hash_table_insert(qown, l, q);
+	htable_insert(qown, l, q);
 }
 
 /**
  * Remove linkable from queue
  */
 static void
-mq_remove_linkable(mqueue_t *q, GList *l)
+mq_remove_linkable(mqueue_t *q, plist_t *l)
 {
 	mqueue_t *owner;
 
@@ -223,16 +262,16 @@ mq_remove_linkable(mqueue_t *q, GList *l)
 	g_assert(l != NULL);
 	g_assert(qown != NULL);		/* Must have added something before */
 
-	owner = g_hash_table_lookup(qown, l);
+	owner = htable_lookup(qown, l);
 
 	if (owner == NULL)
-		g_error("BUG: removed linkable 0x%lx from %s belongs to no queue!",
-			(gulong) l, mq_info(q));
+		g_error("BUG: removed linkable %p from %s belongs to no queue!",
+			(void *) l, mq_info(q));
 	else if (owner != q)
-		g_error("BUG: removed linkable 0x%lx from %s is from another queue!",
-			(gulong) l, mq_info(q));
+		g_error("BUG: removed linkable %p from %s is from another queue!",
+			(void *) l, mq_info(q));
 	else
-		g_hash_table_remove(qown, l);
+		htable_remove(qown, l);
 }
 
 /**
@@ -248,12 +287,12 @@ mq_check_track(mqueue_t *q, int offset, const char *where, int line)
 	g_assert(q);
 
 	if (qown == NULL)
-		qown = NOT_LEAKING(g_hash_table_new(NULL, NULL));
+		qown = NOT_LEAKING(htable_create(HASH_KEY_SELF, 0));
 
 	if (q->magic != MQ_MAGIC)
 		g_error("BUG: %s at %s:%d", mq_info(q), where, line);
 
-	qcount = g_list_length(q->qhead);
+	qcount = plist_length(q->qhead);
 	if (qcount != q->count)
 		g_error("BUG: "
 			"%s has wrong q->count of %d (counted %d in list) at %s:%d",
@@ -263,7 +302,7 @@ mq_check_track(mqueue_t *q, int offset, const char *where, int line)
 		return;
 
 	for (n = 0; n < q->qlink_count; n++) {
-		GList *item = q->qlink[n];
+		plist_t *item = q->qlink[n];
 		mqueue_t *owner;
 
 		if (item == NULL)
@@ -276,7 +315,7 @@ mq_check_track(mqueue_t *q, int offset, const char *where, int line)
 
 		g_assert(qown);		/* If we have a qlink, we have added items */
 
-		owner = g_hash_table_lookup(qown, item);
+		owner = htable_lookup(qown, item);
 		if (owner != q)
 			g_error("BUG: linkable #%d/%d from %s "
 				"%s at %s:%d",
@@ -303,7 +342,8 @@ mq_check_track(mqueue_t *q, int offset, const char *where, int line)
  * Polymorphic operations.
  */
 
-#define MQ_PUTQ(o,m)		((o)->ops->putq((o), (m)))
+#define MQ_PUTQ(o,m)	((o)->ops->putq((o), (m)))
+#define MQ_FLUSHED(o)	((o)->ops->flushed(o))
 
 /**
  * Free queue and all enqueued messages.
@@ -314,12 +354,14 @@ mq_check_track(mqueue_t *q, int offset, const char *where, int line)
 void
 mq_free(mqueue_t *q)
 {
-	GList *l;
+	plist_t *l;
 	int n;
 
+	mq_check_consistency(q);
+
 	tx_free(q->tx_drv);		/* Get rid of lower layers */
 
-	for (n = 0, l = q->qhead; l; l = g_list_next(l)) {
+	for (n = 0, l = q->qhead; l; l = plist_next(l)) {
 		n++;
 		pmsg_free(l->data);
 		l->data = NULL;
@@ -331,13 +373,12 @@ mq_free(mqueue_t *q)
 	if (q->qlink)
 		qlink_free(q);
 
-	cq_cancel(callout_queue, &q->swift_ev);
-	g_list_free(q->qhead);
+	cq_cancel(&q->swift_ev);
+	plist_free_null(&q->qhead);
 	pmsg_slist_free(&q->qwait);
 
-	q->qhead = NULL;
 	q->magic = 0;
-	wfree(q, sizeof(*q));
+	WFREE(q);
 }
 
 /**
@@ -347,13 +388,13 @@ mq_free(mqueue_t *q)
  * The underlying message is freed and the size information on the
  * queue is updated, but not the flow-control information.
  */
-static GList *
-mq_rmlink_prev(mqueue_t *q, GList *l, int size)
+static plist_t *
+mq_rmlink_prev(mqueue_t *q, plist_t *l, int size)
 {
-	GList *prev = g_list_previous(l);
+	plist_t *prev = plist_prev(l);
 
 	mq_remove_linkable(q, l);
-	q->qhead = g_list_remove_link(q->qhead, l);
+	q->qhead = plist_remove_link(q->qhead, l);
 	if (q->qtail == l)
 		q->qtail = prev;
 
@@ -364,7 +405,7 @@ mq_rmlink_prev(mqueue_t *q, GList *l, int size)
 
 	pmsg_free(l->data);
 	l->data = NULL;
-	g_list_free_1(l);
+	plist_free_1(l);
 
 	return prev;
 }
@@ -373,7 +414,7 @@ mq_rmlink_prev(mqueue_t *q, GList *l, int size)
  * A "swift" checkpoint was reached.
  */
 static void
-mq_swift_checkpoint(mqueue_t *q, gboolean initial)
+mq_swift_checkpoint(mqueue_t *q, bool initial)
 {
 	int elapsed = q->swift_elapsed;	/* Elapsed since we were scheduled */
 	int target_to_lowmark;
@@ -385,10 +426,20 @@ mq_swift_checkpoint(mqueue_t *q, gboolean initial)
 	int extra;
 
 	g_assert(q->flags & MQ_FLOWC);
-	g_assert(q->size > q->lowat);	/* Or we would have left FC */
 
 	q->swift_ev = NULL;				/* Event fired, we may not reinstall it */
 
+	/*
+	 * We do not necessarily leave the flow-control state when q->size
+	 * drops below q->lowat for TCP queues, where we want to reach a
+	 * state where we're not in the middle of sending one large message.
+	 *
+	 * When we're below the low watermark, there's nothing for us to do.
+	 */
+
+	if (q->size <= q->lowat)
+		goto done;
+
 	/*
 	 * For next period, the elapsed time will be...
 	 */
@@ -438,72 +489,35 @@ mq_swift_checkpoint(mqueue_t *q, gboolean initial)
 		needed = extra + flushed_till_next_timer / 4;
 	}
 
-	if (initial) {
-		/*
-		 * First time we're in "swift" mode.
-		 *
-		 * Purge pending queries, since they are getting quite old.
-		 * Leave our queries in for now (they have hops=0).
-		 */
+	/*
+	 * To cut the dependency between MQ and the messages being held within,
+	 * things are decoupled thusly:
+	 *
+	 * The MQ requests, via a user-supplied callback, a vector of message
+	 * templates.  For instance for Gnutella messages. these are just Gnutella
+	 * headers.
+	 *
+	 * These headers are used in turn to request message pruning from the
+	 * queue by comparing messages held in the queue with these headers,
+	 * using the user-supplied ``msg_headcmp'' comparison callback.
+	 */
 
-		gnutella_header_set_function(&q->header, GTA_MSG_SEARCH);
-		gnutella_header_set_hops(&q->header, 1);
-		gnutella_header_set_ttl(&q->header, GNET_PROPERTY(max_ttl));
+	if (q->uops->msg_templates != NULL) {
+		size_t i, tcnt = 0;
+		iovec_t *templates = q->uops->msg_templates(initial, &tcnt);
 
-		if (needed > 0)
-			make_room_header(q, (char*) &q->header, PMSG_P_DATA, needed, NULL);
+		for (i = 0; i < tcnt && needed > 0; i++) {
+			int old_size = q->size;
+			const void *base = iovec_base(&templates[i]);
 
-		/*
-		 * Whether or not we were able to make enough room at this point
-		 * is not important, for the initial checkpoint.  Indeed, since
-		 * we are now in "swift" mode, more query messages will be dropped
-		 * at the next iteration, since we'll start dropping query hits,
-		 * and hits are more prioritary than queries.
-		 */
-
-	} else {
-		int ttl;
-
-		/*
-		 * We're going to drop query hits...
-		 *
-		 * We start with the lowest prioritary query hit: low hops count
-		 * and high TTL, and we progressively increase until we can drop
-		 * the amount we need to drop.
-		 *
-		 * Note that we will never be able to drop the partially written
-		 * message at the tail of the queue, even if it is less prioritary
-		 * than our comparison point.
-		 */
-
-		gnutella_header_set_function(&q->header, GTA_MSG_SEARCH_RESULTS);
-
-		/*
-		 * Loop until we reach hops=hard_ttl_limit or we have finished
-		 * removing enough data from the queue.
-		 */
-
-		for (ttl = GNET_PROPERTY(hard_ttl_limit); ttl >= 0; ttl--) {
-			int old_size;
-
-			if (needed <= 0)
+			if (make_room_header(q, base, PMSG_P_DATA, needed, NULL))
 				break;
 
-			old_size = q->size;
-			gnutella_header_set_hops(&q->header,
-				GNET_PROPERTY(hard_ttl_limit) - ttl);
-			gnutella_header_set_ttl(&q->header, ttl);
-
-			if (
-				make_room_header(q, (char*) &q->header, PMSG_P_DATA,
-					needed, NULL)
-			)
-				break;
-
-			needed -= (old_size - q->size);		/* Amount we removed */
+			needed -= old_size - q->size;		/* Amount we removed */
 		}
 	}
 
+done:
 	mq_update_flowc(q);		/* May cause us to leave "swift" mode */
 
 	/*
@@ -516,8 +530,7 @@ mq_swift_checkpoint(mqueue_t *q, gboolean initial)
 	if (q->flags & MQ_SWIFT) {
 		q->flowc_written = 0;
 		q->last_size = q->size;
-		q->swift_ev = cq_insert(callout_queue,
-			q->swift_elapsed, mq_swift_timer, q);
+		q->swift_ev = cq_main_insert(q->swift_elapsed, mq_swift_timer, q);
 	}
 }
 
@@ -525,13 +538,13 @@ mq_swift_checkpoint(mqueue_t *q, gboolean initial)
  * Callout queue callback: periodic "swift" mode timer.
  */
 static void
-mq_swift_timer(cqueue_t *unused_cq, gpointer obj)
+mq_swift_timer(cqueue_t *cq, void *obj)
 {
 	mqueue_t *q = obj;
 
-	(void) unused_cq;
 	g_assert((q->flags & (MQ_FLOWC|MQ_SWIFT)) == (MQ_FLOWC|MQ_SWIFT));
 
+	cq_zero(cq, &q->swift_ev);
 	mq_swift_checkpoint(q, FALSE);
 }
 
@@ -539,15 +552,15 @@ mq_swift_timer(cqueue_t *unused_cq, gpointer obj)
  * Callout queue callback invoked when the queue must enter "swift" mode.
  */
 static void
-mq_enter_swift(cqueue_t *unused_cq, gpointer obj)
+mq_enter_swift(cqueue_t *cq, void *obj)
 {
 	mqueue_t *q = obj;
 
-	(void) unused_cq;
 	g_assert((q->flags & (MQ_FLOWC|MQ_SWIFT)) == MQ_FLOWC);
 
 	q->flags |= MQ_SWIFT;
 
+	cq_zero(cq, &q->swift_ev);
 	node_tx_swift_changed(q->node);
 	mq_swift_checkpoint(q, TRUE);
 }
@@ -574,14 +587,12 @@ mq_enter_flowc(mqueue_t *q)
 	q->last_size = q->size;
 	q->swift_elapsed = node_flowc_swift_grace(q->node) * 1000;
 	q->swift_elapsed = MAX(q->swift_elapsed, 1);
-
-	q->swift_ev =
-		cq_insert(callout_queue, q->swift_elapsed, mq_enter_swift, q);
+	q->swift_ev = cq_main_insert(q->swift_elapsed, mq_enter_swift, q);
 
 	node_tx_enter_flowc(q->node);	/* Signal flow control */
 
 	if (MQ_DEBUG_LVL(q) > 4)
-		g_message("entering FLOWC for node %s (%d bytes queued)",
+		g_debug("MQ entering FLOWC for node %s (%d bytes queued)",
 			node_addr(q->node), q->size);
 }
 
@@ -594,7 +605,7 @@ mq_leave_flowc(mqueue_t *q)
 	g_assert(q->flags & MQ_FLOWC);
 
 	if (MQ_DEBUG_LVL(q) > 4)
-		g_message("leaving %s for node %s (%d bytes queued)",
+		g_debug("MQ leaving %s for node %s (%d bytes queued)",
 			(q->flags & MQ_SWIFT) ? "SWIFT" : "FLOWC",
 			node_addr(q->node), q->size);
 
@@ -602,7 +613,7 @@ mq_leave_flowc(mqueue_t *q)
 	if (q->qlink)
 		qlink_free(q);
 
-	cq_cancel(callout_queue, &q->swift_ev);
+	cq_cancel(&q->swift_ev);
 	node_tx_leave_flowc(q->node);	/* Signal end flow control */
 }
 
@@ -616,7 +627,8 @@ static void
 mq_update_flowc(mqueue_t *q)
 {
 	if (q->flags & MQ_FLOWC) {
-		if (q->size <= q->lowat) {
+	 	/* Never leave flow-control when the last message is partially sent */
+		if (q->size <= q->lowat && MQ_FLUSHED(q)) {
 			mq_leave_flowc(q);
 			q->flags &= ~MQ_WARNZONE;		/* no flow-control */
 		}
@@ -640,7 +652,7 @@ mq_update_flowc(mqueue_t *q)
 void
 mq_clear(mqueue_t *q)
 {
-	g_assert(q);
+	mq_check_consistency(q);
 
 	if (q->count == 0)
 		return;					/* Queue is empty */
@@ -655,7 +667,7 @@ mq_clear(mqueue_t *q)
 	q->flags |= MQ_CLEAR;
 
 	while (q->qhead) {
-		GList *l = q->qhead;
+		plist_t *l = q->qhead;
 		pmsg_t *mb = l->data;
 
 		/*
@@ -695,7 +707,7 @@ mq_clear(mqueue_t *q)
 void
 mq_discard(mqueue_t *q)
 {
-	g_assert(q);
+	mq_check_consistency(q);
 
 	q->flags |= MQ_DISCARD;
 }
@@ -706,7 +718,7 @@ mq_discard(mqueue_t *q)
 void
 mq_shutdown(mqueue_t *q)
 {
-	g_assert(q);
+	mq_check_consistency(q);
 
 	tx_shutdown(q->tx_drv);		/* No further output will be made */
 }
@@ -717,7 +729,7 @@ mq_shutdown(mqueue_t *q)
 void
 mq_flush(mqueue_t *q)
 {
-	g_assert(q);
+	mq_check_consistency(q);
 
 	tx_flush(q->tx_drv);
 }
@@ -725,17 +737,18 @@ mq_flush(mqueue_t *q)
 /**
  * Compare two pointers to links based on their relative priorities, then
  * based on their held Gnutella messages.
- * -- qsort() callback
+ * -- qsort() callback but with an extra data parameter
  */
 static int
-qlink_cmp(const void *a, const void *b)
+qlink_cmp(const void *a, const void *b, void *data)
 {
-	const GList * const *l1 = a, * const *l2 = b;
+	const plist_t * const *l1 = a, * const *l2 = b;
 	const pmsg_t *m1 = (*l1)->data, *m2 = (*l2)->data;
 
-	if (pmsg_prio(m1) == pmsg_prio(m2))
-		return gmsg_cmp(pmsg_start(m1), pmsg_start(m2), TRUE);
-	else
+	if (pmsg_prio(m1) == pmsg_prio(m2)) {
+		const mqueue_t *q = data;
+		return q->uops->msg_cmp(pmsg_phys_base(m1), pmsg_phys_base(m2));
+	} else
 		return pmsg_prio(m1) < pmsg_prio(m2) ? -1 : +1;
 }
 
@@ -745,28 +758,29 @@ qlink_cmp(const void *a, const void *b)
 static void
 qlink_create(mqueue_t *q)
 {
-	GList *l;
+	plist_t *l;
 	int n;
 
 	g_assert(q->qlink == NULL);
 
-	q->qlink = halloc(q->count * sizeof q->qlink[0]);
+	HALLOC_ARRAY(q->qlink, q->count);
 
 	/*
 	 * Prepare sorting of queued messages.
 	 *
-	 * What's sorted is queue links, but the comparison factor is the
-	 * gmsg_cmp() routine to compare the Gnutella messages.
+	 * What's sorted is queue links, but the sorting criteria is the
+	 * user-supplied msg_cmp routine to compare the messages, when they
+	 * have the same priority.
 	 */
 
-	for (l = q->qhead, n = 0; l && n < q->count; l = g_list_next(l), n++) {
+	for (l = q->qhead, n = 0; l && n < q->count; l = plist_next(l), n++) {
 		g_assert(l->data != NULL);
 		q->qlink[n] = l;
 	}
 
 	if (l || n != q->count)
-		g_error("BUG: queue count of %d for 0x%lx is wrong (has %d)",
-			q->count, (gulong) q, g_list_length(q->qhead));
+		g_error("BUG: queue count of %d for %p is wrong (has %zd)",
+			q->count, (void *) q, plist_length(q->qhead));
 
 	/*
 	 * We use `n' and not `q->count' in case the warning above is emitted,
@@ -774,7 +788,7 @@ qlink_create(mqueue_t *q)
 	 */
 
 	q->qlink_count = n;
-	qsort(q->qlink, n, sizeof q->qlink[0], qlink_cmp);
+	xsort_with_data(q->qlink, n, sizeof q->qlink[0], qlink_cmp, q);
 
 	mq_check(q, 0);
 }
@@ -796,10 +810,10 @@ qlink_free(mqueue_t *q)
  * before the position indicated by `hint'.
  */
 static void
-qlink_insert_before(mqueue_t *q, int hint, GList *l)
+qlink_insert_before(mqueue_t *q, int hint, plist_t *l)
 {
 	g_assert(hint >= 0 && hint < q->qlink_count);
-	g_assert(qlink_cmp(&q->qlink[hint], &l) >= 0);	/* `hint' >= `l' */
+	g_assert(qlink_cmp(&q->qlink[hint], &l, q) >= 0);	/* `hint' >= `l' */
 	g_assert(l->data != NULL);
 
 	mq_check(q, -1);
@@ -818,28 +832,20 @@ qlink_insert_before(mqueue_t *q, int hint, GList *l)
 	 */
 
 	q->qlink_count++;
-	q->qlink = hrealloc(q->qlink, q->qlink_count * sizeof q->qlink[0]);
+	HREALLOC_ARRAY(q->qlink, q->qlink_count);
 
-	{
-		int i;
-
-		/* Shift right */
-		for (i = q->qlink_count - 1; i > hint; i--) {
-			q->qlink[i] = q->qlink[i - 1];
-		} 
-	}
-	q->qlink[hint] = l;
+	ARRAY_INSERT(q->qlink, hint, q->qlink_count, l);
 }
 
 /**
  * Insert linkable `l' within the sorted qlink array of linkables.
  */
 static void
-qlink_insert(mqueue_t *q, GList *l)
+qlink_insert(mqueue_t *q, plist_t *l)
 {
 	int low = 0;
 	int high = q->qlink_count - 1;
-	GList **qlink = q->qlink;
+	plist_t **qlink = q->qlink;
 
 	g_assert(l->data != NULL);
 
@@ -852,7 +858,7 @@ qlink_insert(mqueue_t *q, GList *l)
 	if (high < 0) {
 		g_assert(q->count == 1);		/* `l' is already part of the queue */
 		q->qlink_count++;
-		q->qlink = hrealloc(q->qlink, q->qlink_count * sizeof q->qlink[0]);
+		HREALLOC_ARRAY(q->qlink, q->qlink_count);
 		q->qlink[0] = l;
 		return;
 	}
@@ -861,7 +867,7 @@ qlink_insert(mqueue_t *q, GList *l)
 	 * If lower than the beginning, insert at the head.
 	 */
 
-	if (qlink[low] != NULL && qlink_cmp(&l, &qlink[low]) <= 0) {
+	if (qlink[low] != NULL && qlink_cmp(&l, &qlink[low], q) <= 0) {
 		qlink_insert_before(q, low, l);
 		return;
 	}
@@ -870,9 +876,9 @@ qlink_insert(mqueue_t *q, GList *l)
 	 * If higher than the tail, insert at the tail.
 	 */
 
-	if (qlink[high] != NULL && qlink_cmp(&l, &qlink[high]) >= 0) {
+	if (qlink[high] != NULL && qlink_cmp(&l, &qlink[high], q) >= 0) {
 		q->qlink_count++;
-		q->qlink = hrealloc(q->qlink, q->qlink_count * sizeof q->qlink[0]);
+		HREALLOC_ARRAY(q->qlink, q->qlink_count);
 		q->qlink[q->qlink_count - 1] = l;
 		return;
 	}
@@ -946,12 +952,12 @@ qlink_insert(mqueue_t *q, GList *l)
 				return;
 			}
 
-			if (qlink_cmp(&l, &qlink[lowest_non_null]) < 0) {
+			if (qlink_cmp(&l, &qlink[lowest_non_null], q) < 0) {
 				high = lowest_non_null - 1;
 				continue;
 			}
 
-			if (qlink_cmp(&l, &qlink[highest_non_null]) > 0) {
+			if (qlink_cmp(&l, &qlink[highest_non_null], q) > 0) {
 				low = highest_non_null + 1;
 				continue;
 			}
@@ -970,7 +976,7 @@ qlink_insert(mqueue_t *q, GList *l)
 		 * Regular dichotomic case.
 		 */
 
-		c = qlink_cmp(&qlink[mid], &l);
+		c = qlink_cmp(&qlink[mid], &l, q);
 
 		if (c == 0) {
 			qlink_insert_before(q, mid, l);
@@ -997,9 +1003,9 @@ qlink_insert(mqueue_t *q, GList *l)
  * @param l			the linkable to remove from the qlink indexer
  */
 static void
-qlink_remove(mqueue_t *q, GList *l)
+qlink_remove(mqueue_t *q, plist_t *l)
 {
-	GList **qlink = q->qlink;
+	plist_t **qlink = q->qlink;
 	int n = q->qlink_count;
 
 	g_assert(qlink);
@@ -1014,12 +1020,12 @@ qlink_remove(mqueue_t *q, GList *l)
 	 */
 
 	if (n > q->count * 3) {
-		GList **dest = qlink;
+		plist_t **dest = qlink;
 		int copied = 0;
-		gboolean found = FALSE;
+		bool found = FALSE;
 
 		while (n-- > 0) {
-			GList *entry = *qlink++;
+			plist_t *entry = *qlink++;
 			if (entry == NULL)
 				continue;
 			else if (l == entry) {
@@ -1046,14 +1052,10 @@ qlink_remove(mqueue_t *q, GList *l)
 		/* Should have been found -- FALL THROUGH */
 	}
 
-	/*
-	 * Used to be an assertion, but it is non-fatal.  Warn copiously though.
-	 */
-
-	g_error("BUG: linkable 0x%lx for %s not found "
-		"(qlink has %d slots, queue has %d counted items, really %d) at %s:%d",
-		(gulong) l, mq_info(q),
-		q->qlink_count, q->count, g_list_length(q->qhead),
+	g_error("BUG: linkable %p for %s not found "
+		"(qlink has %d slots, queue has %d counted items, really %zd) at %s:%d",
+		(void *) l, mq_info(q),
+		q->qlink_count, q->count, plist_length(q->qhead),
 		_WHERE_, __LINE__);
 }
 
@@ -1075,21 +1077,20 @@ qlink_remove(mqueue_t *q, GList *l)
  *
  * @returns TRUE if we were able to make enough room.
  */
-static gboolean
+static bool
 make_room_internal(mqueue_t *q,
-	char *header, size_t msglen, guint prio, int needed, int *offset)
+	const char *header, size_t msglen, uint prio, int needed, int *offset)
 {
 	int n;
 	int dropped = 0;				/* Amount of messages dropped */
-	gboolean qlink_corrupted = FALSE;	/* BUG catcher */
 
 	g_assert(needed > 0);
 	mq_check(q, 0);
 
 	if (MQ_DEBUG_LVL(q) > 5)
-		g_message("%s try to make room for %d bytes in queue 0x%lx (node %s)",
+		g_debug("MQ %s try to make room for %d bytes in queue %p (node %s)",
 			(q->flags & MQ_SWIFT) ? "SWIFT" : "FLOWC",
-			needed, (gulong) q, node_addr(q->node));
+			needed, (void *) q, node_addr(q->node));
 
 	if (q->qhead == NULL)			/* Queue is empty */
 		return FALSE;
@@ -1109,7 +1110,7 @@ make_room_internal(mqueue_t *q,
 	 * in the loop below).  When we break out because we found a more
 	 * prioritary message, we remember the index in the array and return it
 	 * to the caller.  If the message is finally inserted in the queue,
-	 * we can insert its GList link right before that index.
+	 * we can insert its plist_t link right before that index.
 	 *
 	 * This adds some complexity but it will avoid millions of calls to
 	 * qlink_cmp(), which is costly.
@@ -1119,9 +1120,8 @@ make_room_internal(mqueue_t *q,
 	 * network, so we can NULLify the corresponding slot in the qlink array.
 	 */
 
-restart:
 	for (n = 0; needed >= 0 && n < q->qlink_count; n++) {
-		GList *item = q->qlink[n];
+		plist_t *item = q->qlink[n];
 		pmsg_t *cmb;
 		char *cmb_start;
 		int cmb_size;
@@ -1133,41 +1133,15 @@ restart:
 		if (item == NULL)
 			continue;
 
-		/*
-		 * BUG catcher -- I've seen situations where item->data is NULL
-		 * at this point, which means something is deeply corrupted...
-		 *		--RAM, 2006-07-14
-		 */
-
-		if (item->data == NULL) {
-			g_error("BUG: NULL data for qlink item #%d/%d in %s at %s:%d",
-				n, q->qlink_count, mq_info(q), _WHERE_, __LINE__);
-
-			if (qlink_corrupted) {
-				g_error(
-					"BUG: trying to ignore still invalid qlink entry at %s:%d",
-						_WHERE_, __LINE__);
-				continue;
-			}
-
-			qlink_corrupted = TRUE;		/* Try to mend it */
-			qlink_free(q);
-			qlink_create(q);
-
-			g_error("BUG: recreated qlink and restarting at %s:%d",
-				_WHERE_, __LINE__);
-			goto restart;
-		}
-
 		cmb = item->data;
-		cmb_start = pmsg_start(cmb);
+		cmb_start = pmsg_phys_base(cmb);
 
 		/*
 		 * Any partially written message, however unimportant, cannot be
 		 * removed or we'd break the flow of messages.
 		 */
 
-		if (pmsg_read_base(cmb) != cmb_start)	/* Started to write it  */
+		if (pmsg_start(cmb) != cmb_start)	/* Started to write it  */
 			continue;
 
 		/*
@@ -1179,10 +1153,18 @@ restart:
 		 * (it's necessarily >= 0 if we're in the loop)
 		 */
 
-		if (gmsg_cmp(cmb_start, header, msglen != 0) >= 0) {
-			if (offset != NULL)
-				*offset = n;
-			break;
+		if (0 == msglen) {
+			if (q->uops->msg_headcmp(cmb_start, header) >= 0) {
+				if (offset != NULL)
+					*offset = n;
+				break;
+			}
+		} else {
+			if (q->uops->msg_cmp(cmb_start, header) >= 0) {
+				if (offset != NULL)
+					*offset = n;
+				break;
+			}
 		}
 
 		/*
@@ -1201,15 +1183,17 @@ restart:
 		 * Drop message.
 		 */
 
-		if (MQ_DEBUG_LVL(q) > 4) {
-			gmsg_log_dropped_pmsg(cmb, "to %s %s node %s, in favor of %s",
+		if (MQ_DEBUG_LVL(q) > 4 && q->uops->msg_log != NULL) {
+			q->uops->msg_log(cmb, "to %s %s node %s, in favor of %s",
 				(q->flags & MQ_SWIFT) ? "SWIFT" : "FLOWC",
-				NODE_IS_UDP(q->node) ? "UDP" : "TCP",
+				NODE_USES_UDP(q->node) ? "UDP" : "TCP",
 				node_addr(q->node), msglen ?
 					gmsg_infostr_full(header, msglen) : gmsg_infostr(header));
 		}
 
-		gnet_stats_count_flowc(pmsg_start(cmb));
+		if (q->uops->msg_flowc != NULL)
+			q->uops->msg_flowc(q->node, cmb);
+
 		cmb_size = pmsg_size(cmb);
 
 		g_assert(q->qlink[n] == item);
@@ -1227,7 +1211,7 @@ restart:
 		node_add_txdrop(q->node, dropped);	/* Dropped during TX */
 
 	if (MQ_DEBUG_LVL(q) > 5)
-		g_message("%s end purge: %d bytes (count=%d) for node %s, need=%d",
+		g_debug("MQ %s end purge: %d bytes (count=%d) for node %s, need=%d",
 			(q->flags & MQ_SWIFT) ? "SWIFT" : "FLOWC",
 			q->size, q->count, node_addr(q->node), needed);
 
@@ -1269,11 +1253,11 @@ restart:
  *
  * @returns TRUE if we were able to make enough room.
  */
-static gboolean
-make_room(mqueue_t *q, pmsg_t *mb, int needed, int *offset)
+static bool
+make_room(mqueue_t *q, const pmsg_t *mb, int needed, int *offset)
 {
-	char *header = pmsg_start(mb);
-	guint prio = pmsg_prio(mb);
+	const char *header = pmsg_phys_base(mb);
+	uint prio = pmsg_prio(mb);
 	size_t msglen = pmsg_written_size(mb);
 
 	return make_room_internal(q, header, msglen, prio, needed, offset);
@@ -1283,9 +1267,9 @@ make_room(mqueue_t *q, pmsg_t *mb, int needed, int *offset)
  * Same as make_room(), but we are not given a "pmsg_t" as a comparison
  * point but a Gnutella header and a message priority explicitly.
  */
-static gboolean
+static bool
 make_room_header(
-	mqueue_t *q, char *header, guint prio, int needed, int *offset)
+	mqueue_t *q, const char *header, uint prio, int needed, int *offset)
 {
 	return make_room_internal(q, header, 0, prio, needed, offset);
 }
@@ -1298,9 +1282,9 @@ mq_puthere(mqueue_t *q, pmsg_t *mb, int msize)
 {
 	int needed;
 	int qlink_offset = -1;
-	GList *new = NULL;
-	gboolean make_room_called = FALSE;
-	gboolean has_normal_prio = (pmsg_prio(mb) == PMSG_P_DATA);
+	plist_t *new = NULL;
+	bool make_room_called = FALSE;
+	bool has_normal_prio = (pmsg_prio(mb) == PMSG_P_DATA);
 
 	mq_check(q, 0);
 
@@ -1313,18 +1297,20 @@ mq_puthere(mqueue_t *q, pmsg_t *mb, int msize)
 	if (
 		(q->flags & MQ_FLOWC) &&
 		has_normal_prio &&
-		gmsg_can_drop(pmsg_start(mb), msize) &&
+		gmsg_can_drop(pmsg_phys_base(mb), msize) &&
 		((make_room_called = TRUE)) &&			/* Call make_room() once only */
 		!make_room(q, mb, msize, &qlink_offset)
 	) {
 		g_assert(pmsg_is_unread(mb));			/* Not partially written */
-		if (MQ_DEBUG_LVL(q) > 4)
-			gmsg_log_dropped_pmsg(mb, "to %s %s node %s, %d bytes queued",
+		if (MQ_DEBUG_LVL(q) > 4 && q->uops->msg_log != NULL)
+			q->uops->msg_log(mb, "to %s %s node %s, %d bytes queued",
 				(q->flags & MQ_SWIFT) ? "SWIFT" : "FLOWC",
-				NODE_IS_UDP(q->node) ? "UDP" : "TCP",
+				NODE_USES_UDP(q->node) ? "UDP" : "TCP",
 				node_addr(q->node), q->size);
 
-		gnet_stats_count_flowc(pmsg_start(mb));
+		if (q->uops->msg_flowc != NULL)
+			q->uops->msg_flowc(q->node, mb);
+
 		pmsg_free(mb);
 		node_inc_txdrop(q->node);		/* Dropped during TX */
 		return;
@@ -1354,24 +1340,26 @@ mq_puthere(mqueue_t *q, pmsg_t *mb, int msize)
 
 		g_assert(pmsg_is_unread(mb));			/* Not partially written */
 
-		gnet_stats_count_flowc(pmsg_start(mb));
+		if (q->uops->msg_flowc != NULL)
+			q->uops->msg_flowc(q->node, mb);
 
 		if (has_normal_prio) {
-			if (MQ_DEBUG_LVL(q) > 4)
-				gmsg_log_dropped_pmsg(mb,
+			if (MQ_DEBUG_LVL(q) > 4 && q->uops->msg_log != NULL) {
+				q->uops->msg_log(mb,
 					"to %s %s node %s, %d bytes queued [FULL]",
 					(q->flags & MQ_SWIFT) ? "SWIFT" : "FLOWC",
-					NODE_IS_UDP(q->node) ? "UDP" : "TCP",
+					NODE_USES_UDP(q->node) ? "UDP" : "TCP",
 					node_addr(q->node), q->size);
-
+			}
 			node_inc_txdrop(q->node);		/* Dropped during TX */
 		} else {
-			if (MQ_DEBUG_LVL(q) > 4)
-				gmsg_log_dropped_pmsg(mb,
+			if (MQ_DEBUG_LVL(q) > 0 && q->uops->msg_log != NULL) {
+				q->uops->msg_log(mb,
 					"to %s %s node %s, %d bytes queued [KILLING]",
 					(q->flags & MQ_SWIFT) ? "SWIFT" : "FLOWC",
-					NODE_IS_UDP(q->node) ? "UDP" : "TCP",
+					NODE_USES_UDP(q->node) ? "UDP" : "TCP",
 					node_addr(q->node), q->size);
+			}
 
 			/*
 			 * Is the check for UDP the correct fix or just a
@@ -1385,9 +1373,11 @@ mq_puthere(mqueue_t *q, pmsg_t *mb, int msize)
 			 * the parameter values). --RAM, 2006-12-29
 			 */
 
-			if (!NODE_IS_UDP(q->node)) {
+			if (!NODE_USES_UDP(q->node)) {
 				node_bye(q->node, 502, "Send queue reached %d bytes",
 					q->maxsize);
+			} else {
+				node_inc_txdrop(q->node);	/* Dropped during TX */
 			}
 		}
 
@@ -1410,20 +1400,15 @@ mq_puthere(mqueue_t *q, pmsg_t *mb, int msize)
 	 */
 
 	if (has_normal_prio) {
-		new = q->qhead = g_list_prepend(q->qhead, mb);
+		new = q->qhead = plist_prepend(q->qhead, mb);
 		if (q->qtail == NULL)
 			q->qtail = q->qhead;
 	} else {
-		GList *l;
-		guint prio = pmsg_prio(mb);
-		gboolean inserted = FALSE;
+		plist_t *l;
+		uint prio = pmsg_prio(mb);
+		bool inserted = FALSE;
 
-		/*
-		 * Unfortunately, there's no g_list_insert_after() or equivalent,
-		 * so we break the GList encapsulation.
-		 */
-
-		for (l = q->qtail; l; l = l->prev) {
+		for (l = q->qtail; l; l = plist_prev(l)) {
 			pmsg_t *m = l->data;
 
 			if (
@@ -1435,19 +1420,11 @@ mq_puthere(mqueue_t *q, pmsg_t *mb, int msize)
 				 * we are, then leave the loop.
 				 */
 
-				new = g_list_alloc();
+				q->qhead = plist_insert_after(q->qhead, l, mb);
+				new = plist_next(l);
 
-				new->data = mb;
-				new->prev = l;
-				new->next = l->next;
-
-				if (l->next)
-					l->next->prev = new;
-				else {
-					g_assert(l == q->qtail);	/* Inserted at tail */
+				if (l == q->qtail)				/* Inserted at tail */
 					q->qtail = new;				/* New tail */
-				}
-				l->next = new;
 
 				inserted = TRUE;
 				break;
@@ -1462,7 +1439,7 @@ mq_puthere(mqueue_t *q, pmsg_t *mb, int msize)
 		if (!inserted) {
 			g_assert(l == NULL);
 
-			new = q->qhead = g_list_prepend(q->qhead, mb);
+			new = q->qhead = plist_prepend(q->qhead, mb);
 			if (q->qtail == NULL)
 				q->qtail = q->qhead;
 		}
@@ -1510,6 +1487,7 @@ mq_puthere(mqueue_t *q, pmsg_t *mb, int msize)
 void
 mq_putq(mqueue_t *q, pmsg_t *mb)
 {
+	mq_check_consistency(q);
 	MQ_PUTQ(q, mb);
 }
 
@@ -1529,4 +1507,4 @@ mq_get_cops(void)
 	return &mq_cops;
 }
 
-/* vi: set ts=4: */
+/* vi: set ts=4 sw=4 cindent: */
