@@ -1,7 +1,5 @@
 /*
- * $Id: mq_tcp.c 16738 2009-05-19 20:19:26Z rmanfredi $
- *
- * Copyright (c) 2002-2003, Raphael Manfredi
+ * Copyright (c) 2002-2003, 2014 Raphael Manfredi
  *
  *----------------------------------------------------------------------
  * This file is part of gtk-gnutella.
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -30,13 +28,11 @@
  * Message queues, writing to a TCP stack.
  *
  * @author Raphael Manfredi
- * @date 2002-2003
+ * @date 2002-2003, 2014
  */
 
 #include "common.h"
 
-RCSID("$Id: mq_tcp.c 16738 2009-05-19 20:19:26Z rmanfredi $")
-
 #define MQ_INTERNAL
 #include "mq.h"
 
@@ -47,6 +43,7 @@ RCSID("$Id: mq_tcp.c 16738 2009-05-19 20:19:26Z rmanfredi $")
 #include "gnet_stats.h"
 #include "dump.h"
 
+#include "lib/plist.h"
 #include "lib/pmsg.h"
 #include "lib/walloc.h"
 
@@ -59,20 +56,30 @@ RCSID("$Id: mq_tcp.c 16738 2009-05-19 20:19:26Z rmanfredi $")
 #define MQ_MINIOV		2		/**< Minimum amount of I/O vectors in service */
 #define MQ_MINSEND		256		/**< Minimum size we try to send */
 
-static void mq_tcp_service(gpointer data);
+static void mq_tcp_service(void *data);
 static const struct mq_ops mq_tcp_ops;
 
 /**
  * Create new message queue capable of holding `maxsize' bytes, and
  * owned by the supplied node.
+ *
+ * @param maxsize		the overall sum of message size that can be held
+ * @param n				the network node to which the message queue is attached
+ * @oaram nd			the top of the TX stack to use to send out messages
+ * @param uops			user-defined operations
  */
 mqueue_t *
-mq_tcp_make(int maxsize, struct gnutella_node *n, struct txdriver *nd)
+mq_tcp_make(int maxsize,
+	gnutella_node_t *n, struct txdriver *nd, const struct mq_uops *uops)
 {
 	mqueue_t *q;
 
-	q = walloc0(sizeof *q);
+	node_check(n);
+	tx_check(nd);
+	g_assert(uops != NULL);
+	g_assert(maxsize > 0);
 
+	WALLOC0(q);
 	q->magic = MQ_MAGIC;
 	q->node = n;
 	q->tx_drv = nd;
@@ -82,6 +89,7 @@ mq_tcp_make(int maxsize, struct gnutella_node *n, struct txdriver *nd)
 	q->qwait = slist_new();
 	q->ops = &mq_tcp_ops;
 	q->cops = mq_get_cops();
+	q->uops = uops;
 	q->debug = GNET_PROPERTY_PTR(mq_tcp_debug);
 
 	tx_srv_register(nd, mq_tcp_service, q);
@@ -93,19 +101,19 @@ mq_tcp_make(int maxsize, struct gnutella_node *n, struct txdriver *nd)
  * Service routine for TCP message queue.
  */
 static void
-mq_tcp_service(gpointer data)
+mq_tcp_service(void *data)
 {
 	mqueue_t *q = (mqueue_t *) data;
-	static struct iovec iov[MQ_MAXIOV];
+	static iovec_t iov[MQ_MAXIOV];
 	int iovsize;
 	int iovcnt;
 	int sent;
 	ssize_t r;
-	GList *l;
+	plist_t *l;
 	int dropped;
 	int maxsize;
-	gboolean saturated;
-	gboolean has_prioritary = FALSE;
+	bool saturated;
+	bool has_prioritary = FALSE;
 
 again:
 	mq_check(q, 0);
@@ -128,9 +136,8 @@ again:
 	maxsize = MAX(MQ_MINSEND, maxsize);
 
 	for (l = q->qtail; l && iovsize > 0; /* empty */) {
-		struct iovec *ie;
+		iovec_t *ie;
 		pmsg_t *mb = (pmsg_t *) l->data;
-		char *mbs = pmsg_start(mb);
 
 		/*
 		 * Don't build too much.
@@ -143,18 +150,18 @@ again:
 		 * Honour hops-flow, and ensure there is a route for possible replies.
 		 */
 
-		if (pmsg_check(mb, q)) {
+		if (pmsg_can_send(mb, q)) {
 			/* send the message */
-			l = g_list_previous(l);
+			l = plist_prev(l);
 			iovsize--;
 			ie = &iov[iovcnt++];
-			ie->iov_base = deconstify_gpointer(mb->m_rptr);
-			ie->iov_len = pmsg_size(mb);
-			maxsize -= ie->iov_len;
+			iovec_set(ie, deconstify_pointer(mb->m_rptr), pmsg_size(mb));
+			maxsize -= iovec_len(ie);
 			if (pmsg_prio(mb))
 				has_prioritary = TRUE;
 		} else {
-			gnet_stats_count_flowc(mbs);	/* Done before message freed */
+			if (q->uops->msg_flowc != NULL)
+				q->uops->msg_flowc(q->node, mb);	/* Done before msg freed */
 			if (q->qlink)
 				q->cops->qlink_remove(q, l);
 
@@ -183,7 +190,9 @@ again:
 
 	r = tx_writev(q->tx_drv, iov, iovcnt);
 
-	g_assert((ssize_t) -1 == r || !tx_has_error(q->tx_drv));
+	g_assert_log((ssize_t) -1 == r || !tx_has_error(q->tx_drv),
+		"%s(): written=%zd, yet error is reported by TX layer \"%s\"",
+		G_STRFUNC, r, tx_error_layer_name(q->tx_drv));
 
 	if ((ssize_t) -1 == r || r == 0) {
 		q->last_written = 0;
@@ -214,30 +223,18 @@ again:
 	saturated = FALSE;
 
 	for (l = q->qtail; l && r > 0 && iovsize > 0; iovsize--) {
-		struct iovec *ie = &iov[iovcnt++];
+		iovec_t *ie = &iov[iovcnt++];
 		pmsg_t *mb = (pmsg_t *) l->data;
 
-		if ((guint) r >= ie->iov_len) {			/* Completely written */
-			char *mb_start = pmsg_start(mb);
-			guint8 function = gmsg_function(mb_start);
+		if ((uint) r >= iovec_len(ie)) {		/* Completely written */
 			sent++;
 			pmsg_mark_sent(mb);
-            gnet_stats_count_sent(q->node,
-				function, gmsg_hops(mb_start), pmsg_size(mb));
-			switch (function) {
-			case GTA_MSG_SEARCH:
-				node_inc_tx_query(q->node);
-				break;
-			case GTA_MSG_SEARCH_RESULTS:
-				node_inc_tx_qhit(q->node);
-				break;
-			default:
-				break;
-			}
-			r -= ie->iov_len;
+			if (q->uops->msg_sent != NULL)
+				q->uops->msg_sent(q->node, mb);
+			r -= iovec_len(ie);
 			if (q->qlink)
 				q->cops->qlink_remove(q, l);
-			l = q->cops->rmlink_prev(q, l, ie->iov_len);
+			l = q->cops->rmlink_prev(q, l, iovec_len(ie));
 		} else {
 			g_assert(r > 0 && r < pmsg_size(mb));
 			g_assert(r < q->size);
@@ -302,19 +299,19 @@ update_servicing:
  * @param from		for TX traffic dump: the origin of message, NULL if local
  */
 void
-mq_tcp_putq(mqueue_t *q, pmsg_t *mb, const struct gnutella_node *from)
+mq_tcp_putq(mqueue_t *q, pmsg_t *mb, const gnutella_node_t *from)
 {
 	int size;				/* Message size */
 	char *mbs;				/* Start of message */
-	guint8 function;		/* Gnutella message function */
-	guint8 hops;			/* Gnutella message hop count */
-	gboolean prioritary;	/* Is message prioritary? */
-	gboolean error = FALSE;
+	bool prioritary;		/* Is message prioritary? */
+	bool error = FALSE;
+
+	mq_check_consistency(q);
 
 	dump_tx_tcp_packet(from, q->node, mb);
 
 again:
-	g_assert(q);
+	mq_check_consistency(q);
 	g_assert(!pmsg_was_sent(mb));
 	g_assert(pmsg_is_unread(mb));
 	g_assert(q->ops == &mq_tcp_ops);	/* Is a TCP queue */
@@ -335,7 +332,7 @@ again:
 
 	size = pmsg_size(mb);
 	if (size == 0) {
-		g_warning("mq_tcp_putq: called with empty message");
+		g_carp("%s: called with empty message", G_STRFUNC);
 		goto cleanup;
 	}
 
@@ -350,20 +347,19 @@ again:
 	if (q->putq_entered > 0) {
 		if (debugging(20))
 			g_warning(
-				"mq_tcp_putq: %s recursion detected (%d already pending)",
-				mq_info(q), slist_length(q->qwait));
+				"%s: %s recursion detected (%u already pending)",
+				G_STRFUNC, mq_info(q), slist_length(q->qwait));
 		slist_append(q->qwait, mb);
 		return;
 	}
 
 	q->putq_entered++;
-	
-	mbs = pmsg_start(mb);
-	function = gmsg_function(mbs);
-	hops = gmsg_hops(mbs);
+
+	mbs = pmsg_phys_base(mb);
 	prioritary = pmsg_prio(mb) != PMSG_P_DATA;
 
-	gnet_stats_count_queued(q->node, function, hops, size);
+	if (q->uops->msg_queued != NULL)
+		q->uops->msg_queued(q->node, mb);
 
 	/*
 	 * If queue is empty, attempt a write immediatly.
@@ -372,7 +368,7 @@ again:
 	if (q->qhead == NULL) {
 		ssize_t written;
 
-		if (pmsg_check(mb, q)) {
+		if (pmsg_can_send(mb, q)) {
 			if (prioritary)
 				node_flushq(q->node);
 
@@ -384,7 +380,9 @@ again:
 			 * reports data written and not -1.
 			 *		--RAM, 2006-12-29
 			 */
-			g_assert((ssize_t) -1 == written || !tx_has_error(q->tx_drv));
+			g_assert_log((ssize_t) -1 == written || !tx_has_error(q->tx_drv),
+				"%s(): written=%zd, yet error reported by TX layer \"%s\"",
+				G_STRFUNC, written, tx_error_layer_name(q->tx_drv));
 
 			if ((ssize_t) -1 == written) {
 				error = TRUE;
@@ -399,7 +397,8 @@ again:
 				}
 			}
 		} else {
-			gnet_stats_count_flowc(mbs);
+			if (q->uops->msg_flowc != NULL)
+				q->uops->msg_flowc(q->node, mb);
 			node_inc_txdrop(q->node);		/* Dropped during TX */
 			written = -1;
 		}
@@ -411,18 +410,8 @@ again:
 
 		if (written == size) {
 			pmsg_mark_sent(mb);
-			node_inc_sent(q->node);
-            gnet_stats_count_sent(q->node, function, hops, size);
-			switch (function) {
-			case GTA_MSG_SEARCH:
-				node_inc_tx_query(q->node);
-				break;
-			case GTA_MSG_SEARCH_RESULTS:
-				node_inc_tx_qhit(q->node);
-				break;
-			default:
-				break;
-			}
+			if (q->uops->msg_sent != NULL)
+				q->uops->msg_sent(q->node, mb);
 			goto cleanup;
 		}
 
@@ -469,8 +458,8 @@ cleanup:
 		if (mb) {
 			if (debugging(20))
 				g_warning(
-					"mq_tcp_putq: %s flushing waiting (%d still pending)",
-					mq_info(q), slist_length(q->qwait));
+					"%s: %s flushing waiting (%u still pending)",
+					G_STRFUNC, mq_info(q), slist_length(q->qwait));
 			goto again;
 		}
 	}
@@ -489,8 +478,25 @@ mq_no_putq(mqueue_t *unused_q, pmsg_t *unused_mb)
 	g_error("plain mq_putq() forbidden on TCP queue -- use mq_tcp_putq()");
 }
 
+/**
+ * Is the last enqueued message still unwritten, as opposed to having been
+ * partially sent already?
+ */
+static bool
+mq_tcp_flushed(const mqueue_t *q)
+{
+	pmsg_t *mb;
+
+	if (NULL == q->qtail)
+		return TRUE;			/* Empty queue */
+
+	mb = q->qtail->data;
+	return pmsg_is_unread(mb);
+}
+
 static const struct mq_ops mq_tcp_ops = {
 	mq_no_putq,			/* putq */
+	mq_tcp_flushed,		/* flushed */
 };
 
 /* vi: set ts=4 sw=4 cindent: */
