@@ -1,6 +1,4 @@
 /*
- * $Id: routing.h 17289 2010-02-25 17:06:34Z rmanfredi $
- *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -38,6 +36,7 @@ struct guid;
 
 typedef enum {
 	ROUTE_NONE = 0,			/**< No route, message stops here */
+	ROUTE_LEAVES,			/**< Route to leaves only */
 	ROUTE_ONE,				/**< Route to single node */
 	ROUTE_ALL_BUT_ONE,		/**< Route to all nodes but one */
 	ROUTE_MULTI				/**< Route to list of nodes */
@@ -51,8 +50,9 @@ struct route_dest {
 	route_type_t type;
 	union {
 		struct gnutella_node *u_node;
-		GSList *u_nodes;	/**< For ROUTE_MULTI */
+		struct pslist *u_nodes;	/**< For ROUTE_MULTI */
 	} ur;
+	unsigned duplicate:1;	/**< Set if message was a duplicate */
 };
 
 /**
@@ -68,14 +68,19 @@ void gnet_reset_guid(void);
 
 void routing_init(void);
 void routing_close(void);
-void message_set_muid(gnutella_header_t *header, guint8 function);
-gboolean route_message(struct gnutella_node **, struct route_dest *);
-void routing_node_remove(struct gnutella_node *);
-void message_add(const struct guid *muid, guint8, struct gnutella_node *);
-GSList *route_towards_guid(const struct guid *guid);
-gboolean route_exists_for_reply(const struct guid *muid, guint8 function);
+void routing_clear_all(void);
+void message_set_muid(gnutella_header_t *header, uint8 function);
+bool route_message(struct gnutella_node **, struct route_dest *);
+void routing_node_remove(void *node);
+void message_add(const struct guid *muid, uint8, struct gnutella_node *);
+void message_forget(const struct guid *muid, uint8, struct gnutella_node *);
+struct pslist *route_towards_guid(const struct guid *guid);
+bool route_exists_for_reply(const struct guid *muid, uint8 function);
+bool route_guid_pushable(const struct guid *guid);
+void route_udp_mark_deflatable(const struct gnutella_node *n);
+void route_udp_mark_semi_reliable(const struct gnutella_node *n);
 
-gboolean route_proxy_add(const struct guid *guid, struct gnutella_node *n);
+bool route_proxy_add(const struct guid *guid, struct gnutella_node *n);
 void route_proxy_remove(const struct guid *guid);
 struct gnutella_node *route_proxy_find(const struct guid *guid);
 
@@ -83,4 +88,5 @@ void route_starving_add(const struct guid *guid, route_starving_cb_t cb);
 void route_starving_remove(const struct guid *guid);
 
 #endif /* _core_routing_h_ */
+
 /* vi: set ts=4 sw=4 cindent: */
