@@ -1,6 +1,4 @@
 /*
- * $Id: rx_link.h 16104 2009-03-10 14:58:09Z rmanfredi $
- *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -47,14 +45,14 @@ const struct rxdrv_ops *rx_link_get_ops(void);
  * Callbacks used by the link layer.
  */
 struct rx_link_cb {
-	void (*add_rx_given)(gpointer owner, ssize_t amount);
-	void (*read_error)(gpointer owner,
-			const char *reason, ...) PRINTF_FUNC_PTR(2, 3);
-	void (*got_eof)(gpointer owner);
+	void (*add_rx_given)(void *owner, ssize_t amount);
+	void (*read_error)(void *owner,
+			const char *reason, ...) G_PRINTF_PTR(2, 3);
+	void (*got_eof)(void *owner);
 };
 
 /**
- * Arguments to be passed when the layer is intantiated.
+ * Arguments to be passed when the layer is instantiated.
  */
 struct rx_link_args {
 	const struct rx_link_cb *cb;	/**< Callbacks */
