@@ -1,6 +1,4 @@
 /*
- * $Id: rxbuf.h 15557 2008-08-10 08:00:42Z rmanfredi $
- *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -44,12 +42,11 @@
  */
 
 pdata_t *rxbuf_new(void);
-void rxbuf_free(gpointer p);
+void rxbuf_free(void *p);
 
 void rxbuf_init(void);
 void rxbuf_close(void);
 
 #endif	/* _core_rxbuf_h_ */
 
-/* vi: set ts=4: */
-
+/* vi: set ts=4 sw=4 cindent: */
