@@ -1,6 +1,4 @@
 /*
- * $Id: spam_sha1.h 13599 2007-05-11 18:15:46Z cbiere $
- *
  * Copyright (c) 2007, Christian Biere
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -40,7 +38,7 @@
 
 struct sha1;
 
-gboolean spam_sha1_check(const struct sha1 *sha1);
+bool spam_sha1_check(const struct sha1 *sha1);
 void spam_sha1_add(const struct sha1 *sha1);
 void spam_sha1_sync(void);
 void spam_sha1_init(void);
