@@ -1,6 +1,4 @@
 /*
- * $Id: topless.c 14365 2007-08-08 05:05:08Z cbiere $
- *
  * Copyright (c) 2007, Christian Biere
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -27,16 +25,15 @@
  * @ingroup core
  * @file
  *
+ * Main loop entry point when running topless.
  */
 
 #include "common.h"
 
-RCSID("$Id: topless.c 14365 2007-08-08 05:05:08Z cbiere $")
-
 #include "topless.h"
 
 #ifndef USE_TOPLESS
-gboolean running_topless;
+bool running_topless;
 #endif	/* USE_TOPLESS */
 
 void
