@@ -1,6 +1,4 @@
 /*
- * $Id: tx_deflate.c 17209 2009-11-08 21:37:28Z rmanfredi $
- *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -37,20 +35,22 @@
 
 #include "common.h"
 
-RCSID("$Id: tx_deflate.c 17209 2009-11-08 21:37:28Z rmanfredi $")
-
 #include <zlib.h>
 
-#include "sockets.h"
-#include "hosts.h"
 #include "tx.h"
 #include "tx_deflate.h"
+#include "hosts.h"
+#include "sockets.h"
 
 #include "if/gnet_property_priv.h"
 
+#include "lib/cq.h"
 #include "lib/endian.h"
+#include "lib/mempcpy.h"
+#include "lib/tm.h"
 #include "lib/walloc.h"
 #include "lib/zlib_util.h"
+
 #include "lib/override.h"		/* Must be the last header included */
 
 /*
@@ -66,7 +66,8 @@ RCSID("$Id: tx_deflate.c 17209 2009-11-08 21:37:28Z rmanfredi $")
  */
 
 #define BUFFER_COUNT	2
-#define BUFFER_NAGLE	200		/**< 200 ms */
+#define BUFFER_NAGLE	500		/**< 500 ms */
+#define BUFFER_DELAY	2		/**< 2 secs -- max Nagle delay */
 
 struct buffer {
 	char *arena;				/**< Buffer arena */
@@ -82,23 +83,29 @@ struct attr {
 	struct buffer buf[BUFFER_COUNT];
 	size_t buffer_size;			/**< Buffer size used */
 	size_t buffer_flush;		/**< Flush after that many bytes */
+	double ratio;				/**< Overall compression ratio */
+	double ratio_ema;			/**< EMA of compression ratio */
 	int fill_idx;				/**< Filled buffer index */
 	int send_idx;				/**< Buffer to be sent */
 	z_streamp outz;				/**< Compressing stream */
 	txdrv_t *nd;				/**< Network driver, underneath us */
-	size_t unflushed;			/**< Amount of bytes written since last flush */
+	size_t unflushed;			/**< Amount of input bytes since last flush */
+	size_t flushed;				/**< Amount of output bytes since last flush */
+	size_t total_input;			/**< Total amount of input bytes flushed */
+	size_t total_output;		/**< Total amount of output bytes flushed */
 	int flags;					/**< Operating flags */
 	cqueue_t *cq;				/**< The callout queue to use for Nagle */
 	cevent_t *tm_ev;			/**< The timer event */
 	const struct tx_deflate_cb *cb;	/**< Layer-specific callbacks */
 	tx_closed_t closed;			/**< Callback to invoke when layer closed */
-	gpointer closed_arg;		/**< Argument for closing routine */
-	gboolean nagle;				/**< Whether to use Nagle or not */
+	void *closed_arg;			/**< Argument for closing routine */
+	time_t nagle_start;			/**< When we started the Nagle timer */
 	struct {
-		gboolean	enabled;	/**< Whether to use gzip encapsulation */
-		guint32		size;		/**< Payload size counter for gzip */
+		bool		enabled;	/**< Whether to use gzip encapsulation */
+		uint32		size;		/**< Payload size counter for gzip */
 		uLong		crc;		/**< CRC-32 accumlator for gzip */
 	} gzip;
+	unsigned nagle:1;			/**< Whether to use Nagle or not */
 };
 
 /*
@@ -110,9 +117,13 @@ struct attr {
 #define DF_FLUSH		0x00000004	/**< Flushing started */
 #define DF_SHUTDOWN		0x00000008	/**< Stack has shut down */
 
-static void deflate_nagle_timeout(cqueue_t *cq, gpointer arg);
+static void deflate_nagle_timeout(cqueue_t *cq, void *arg);
 static size_t tx_deflate_pending(txdrv_t *tx);
 
+#define tx_deflate_debugging(lvl) \
+	G_UNLIKELY(GNET_PROPERTY(tx_deflate_debug) > (lvl) && \
+		tx_debug_host(&tx->host))
+
 /**
  * Write ready-to-be-sent buffer to the lower layer.
  */
@@ -142,14 +153,14 @@ deflate_send(txdrv_t *tx)
 
 	r = tx_write(tx->lower, b->rptr, len);
 
-	if (GNET_PROPERTY(dbg) > 9)
-		printf("deflate_send: (%s) wrote %lu bytes (buffer #%d) [%c%c]\n",
-			gnet_host_to_string(&tx->host), (unsigned long) r, attr->send_idx,
+	if (tx_deflate_debugging(9)) {
+		g_debug("TX %s: (%s) wrote %zu/%zu bytes (buffer #%d) [%c%c]",
+			G_STRFUNC, gnet_host_to_string(&tx->host), r, len, attr->send_idx,
 			(attr->flags & DF_FLOWC) ? 'C' : '-',
 			(attr->flags & DF_FLUSH) ? 'f' : '-');
-
+	}
 	if ((ssize_t) -1 == r) {
-		tx->flags |= TX_ERROR;
+		tx_error(tx);
 		return;
 	}
 
@@ -158,10 +169,10 @@ deflate_send(txdrv_t *tx)
 	 */
 
 	if ((size_t) r == len) {
-		if (GNET_PROPERTY(dbg) > 9)
-			printf("deflate_send: (%s) buffer #%d is empty\n",
-				gnet_host_to_string(&tx->host), attr->send_idx);
-
+		if (tx_deflate_debugging(9)) {
+			g_debug("TX %s: (%s) buffer #%d is empty",
+				G_STRFUNC, gnet_host_to_string(&tx->host), attr->send_idx);
+		}
 		attr->send_idx = -1;			/* Signals: is now free */
 		b->wptr = b->rptr = b->arena;	/* Buffer is now empty */
 		return;
@@ -195,6 +206,35 @@ deflate_nagle_start(txdrv_t *tx)
 
 	attr->tm_ev = cq_insert(attr->cq, BUFFER_NAGLE, deflate_nagle_timeout, tx);
 	attr->flags |= DF_NAGLE;
+	attr->nagle_start = tm_time();
+}
+
+/**
+ * Delay the nagle timer when more data is coming.
+ */
+static void
+deflate_nagle_delay(txdrv_t *tx)
+{
+	struct attr *attr = tx->opaque;
+
+	g_assert(attr->flags & DF_NAGLE);
+	g_assert(NULL != attr->tm_ev);
+	g_assert(attr->nagle);				/* Nagle is allowed */
+
+	/*
+	 * We push back the initial delay a little while when more data comes,
+	 * hoping that enough will be output so that we end up sending the TX
+	 * buffer without having to trigger a flush too soon, since that would
+	 * degrade compression performance.
+	 *
+	 * If too much time elapsed since the Nagle timer started, do not
+	 * postpone the flush otherwise we might delay time-sensitive messages.
+	 */
+
+	if (delta_time(tm_time(), attr->nagle_start) < BUFFER_DELAY) {
+		int delay = cq_remaining(attr->tm_ev);
+		cq_resched(attr->tm_ev, MAX(delay, BUFFER_NAGLE / 2));
+	}
 }
 
 /**
@@ -208,7 +248,7 @@ deflate_nagle_stop(txdrv_t *tx)
 	g_assert(attr->flags & DF_NAGLE);
 	g_assert(NULL != attr->tm_ev);
 
-	cq_cancel(attr->cq, &attr->tm_ev);
+	cq_cancel(&attr->tm_ev);
 	attr->flags &= ~DF_NAGLE;
 }
 
@@ -239,21 +279,110 @@ deflate_rotate_and_send(txdrv_t *tx)
 	if (attr->fill_idx >= BUFFER_COUNT)
 		attr->fill_idx = 0;
 
-	if (GNET_PROPERTY(dbg) > 9)
-		printf("deflate_rotate_and_send: (%s) fill buffer now #%d [%c%c]\n",
-			gnet_host_to_string(&tx->host), attr->fill_idx,
+	if (tx_deflate_debugging(9)) {
+		g_debug("TX %s: (%s) fill buffer now #%d [%c%c]",
+			G_STRFUNC, gnet_host_to_string(&tx->host), attr->fill_idx,
 			(attr->flags & DF_FLOWC) ? 'C' : '-',
 			(attr->flags & DF_FLUSH) ? 'f' : '-');
-
+	}
 	deflate_send(tx);
 }
 
+/**
+ * Compute amount of buffered output data awaiting to be sent.
+ */
+static size_t
+deflate_buffered(const txdrv_t *tx)
+{
+	const struct attr *attr = tx->opaque;
+	const struct buffer *b;
+	size_t buffered;
+
+	b = &attr->buf[attr->fill_idx];	/* Buffer we fill */
+	buffered = b->wptr - b->rptr;
+
+	if (-1 != attr->send_idx) {
+		b = &attr->buf[attr->send_idx];	/* Buffer we send */
+		buffered += b->wptr - b->rptr;
+	}
+
+	return buffered;
+}
+
+/**
+ * Enter or leave flow-control.
+ */
+static void
+deflate_set_flowc(txdrv_t *tx, bool on)
+{
+	struct attr *attr = tx->opaque;
+
+	if (on) {
+		attr->flags |= DF_FLOWC;		/* Enter flow control */
+	} else {
+		attr->flags &= ~DF_FLOWC;		/* Leave flow control state */
+	}
+
+	if (tx_deflate_debugging(4)) {
+		g_debug("TX %s: (%s) %s flow-control [%c%c]",
+			G_STRFUNC, gnet_host_to_string(&tx->host),
+			on ? "entering" : "leaving",
+			(attr->flags & DF_FLOWC) ? 'C' : '-',
+			(attr->flags & DF_FLUSH) ? 'f' : '-');
+	}
+
+	if (NULL != attr->cb->flow_control)
+		attr->cb->flow_control(tx->owner, on ? deflate_buffered(tx) : 0);
+}
+
+/**
+ * Pending data were all flushed.
+ */
+static void
+deflate_flushed(txdrv_t *tx)
+{
+	struct attr *attr = tx->opaque;
+	double flush = 0.0;
+
+	g_assert(size_is_non_negative(attr->unflushed));
+
+	attr->total_input += attr->unflushed;
+	attr->total_output += attr->flushed;
+
+	if G_UNLIKELY(0 == attr->total_input)
+		goto done;
+
+	attr->ratio = 1.0 - ((double) attr->total_output / attr->total_input);
+
+	if (0 != attr->unflushed) {
+		/*
+		 * Fast EMA for compression ratio is computed for the last n=3 flushes,
+		 * so the smoothing factor sm=2/(n+1) is 1/2.
+		 */
+
+		flush = 1.0 - ((double) attr->flushed / attr->unflushed);
+		attr->ratio_ema += (flush / 2.0) - (attr->ratio_ema / 2.0);
+	}
+
+	if (tx_deflate_debugging(4)) {
+		g_debug("TX %s: (%s) deflated %zu bytes into %zu "
+			"(%.2f%%, EMA=%.2f%%, overall %.2f%%)",
+			G_STRFUNC, gnet_host_to_string(&tx->host),
+			attr->unflushed, attr->flushed,
+			100 * flush, 100 * attr->ratio_ema, 100 * attr->ratio);
+	}
+
+done:
+	attr->unflushed = attr->flushed = 0;
+	attr->flags &= ~DF_FLUSH;
+}
+
 /**
  * Flush compression within filling buffer.
  *
  * @return success status, failure meaning we shutdown.
  */
-static gboolean
+static bool
 deflate_flush(txdrv_t *tx)
 {
 	struct attr *attr = tx->opaque;
@@ -262,24 +391,18 @@ deflate_flush(txdrv_t *tx)
 	int ret;
 	int old_avail;
 
-	/*
-	 * Normally we come here with something already written, so outz->next_in
-	 * should be initialized.  If it is not the case, then it means we
-	 * never wrote anything, so there is nothing for us to do: we're simply
-	 * called through tx_deflate_close().
-	 */
-
-	if (NULL == outz->next_in)
-		goto done;
-
 retry:
 	b = &attr->buf[attr->fill_idx];	/* Buffer we fill */
 
-	if (GNET_PROPERTY(dbg) > 9)
-		printf("deflate_flush: (%s) flushing (buffer #%d) [%c%c]\n",
-			gnet_host_to_string(&tx->host), attr->fill_idx,
+	if (tx_deflate_debugging(9)) {
+		g_debug("TX %s: (%s) flushing %zu bytes "
+			"(buffer #%d, flushed %zu, unflushed %zu) [%c%c]",
+			G_STRFUNC, gnet_host_to_string(&tx->host),
+			b->wptr - b->rptr, attr->fill_idx,
+			attr->flushed, attr->unflushed,
 			(attr->flags & DF_FLOWC) ? 'C' : '-',
 			(attr->flags & DF_FLUSH) ? 'f' : '-');
+	}
 
 	/*
 	 * Prepare call to deflate().
@@ -288,7 +411,7 @@ retry:
 	 * be consumed.
 	 */
 
-	outz->next_out = cast_to_gpointer(b->wptr);
+	outz->next_out = cast_to_pointer(b->wptr);
 	outz->avail_out = old_avail = b->end - b->wptr;
 
 	outz->avail_in = 0;
@@ -299,16 +422,16 @@ retry:
 
 	switch (ret) {
 	case Z_BUF_ERROR:				/* Nothing to flush */
-		return TRUE;
+		goto done;
 	case Z_OK:
 	case Z_STREAM_END:
 		break;
 	default:
 		attr->flags |= DF_SHUTDOWN;
-		tx->flags |= TX_ERROR;
+		tx_error(tx);
 
 		/* XXX: The callback must not destroy the tx! */
-		attr->cb->shutdown(tx->owner, "Compression flush failed: %s",
+		(*attr->cb->shutdown)(tx->owner, "Compression flush failed: %s",
 				zlib_strerror(ret));
 		return FALSE;
 	}
@@ -318,6 +441,7 @@ retry:
 
 		written = old_avail - outz->avail_out;
 		b->wptr += written;
+		attr->flushed += written;
 
 		if (NULL != attr->cb->add_tx_deflated)
 			attr->cb->add_tx_deflated(tx->owner, written);
@@ -331,17 +455,13 @@ retry:
 	 */
 
 	if (0 == outz->avail_out) {
-		if (attr->send_idx >= 0) {				/* Send buffer not sent yet */
-			attr->flags |= DF_FLOWC|DF_FLUSH;	/* Enter flow control */
-
-			if (GNET_PROPERTY(dbg) > 4)
-				printf("Compressing TX stack for peer %s enters FLOWC/FLUSH\n",
-					gnet_host_to_string(&tx->host));
-
+		if (attr->send_idx >= 0) {			/* Send buffer not sent yet */
+			attr->flags |= DF_FLUSH;		/* In flush mode */
+			deflate_set_flowc(tx, TRUE);	/* Starting flow-control */
 			return TRUE;
 		}
 
-		deflate_rotate_and_send(tx);			/* Can set TX_ERROR */
+		deflate_rotate_and_send(tx);		/* Can set TX_ERROR */
 
 		if (tx->flags & TX_ERROR)
 			return FALSE;
@@ -350,9 +470,7 @@ retry:
 	}
 
 done:
-
-	attr->unflushed = 0;
-	attr->flags &= ~DF_FLUSH;
+	deflate_flushed(tx);
 
 	return TRUE;		/* Fully flushed */
 }
@@ -388,21 +506,21 @@ deflate_flush_send(txdrv_t *tx)
  * If we can send the buffer, flush it and send it.  Otherwise, reschedule.
  */
 static void
-deflate_nagle_timeout(cqueue_t *unused_cq, gpointer arg)
+deflate_nagle_timeout(cqueue_t *cq, void *arg)
 {
 	txdrv_t *tx = arg;
 	struct attr *attr = tx->opaque;
 
-	(void) unused_cq;
+	cq_zero(cq, &attr->tm_ev);
+
 	if (-1 != attr->send_idx) {		/* Send buffer still incompletely sent */
 
-		if (GNET_PROPERTY(dbg) > 9)
-			printf("deflate_nagle_timeout: (%s) buffer #%d unsent,"
-				" exiting [%c%c]\n",
-				gnet_host_to_string(&tx->host), attr->send_idx,
+		if (tx_deflate_debugging(9)) {
+			g_debug("TX %s: (%s) buffer #%d unsent, exiting [%c%c]",
+				G_STRFUNC, gnet_host_to_string(&tx->host), attr->send_idx,
 				(attr->flags & DF_FLOWC) ? 'C' : '-',
 				(attr->flags & DF_FLUSH) ? 'f' : '-');
-
+		}
 
 		attr->tm_ev =
 			cq_insert(attr->cq, BUFFER_NAGLE, deflate_nagle_timeout, tx);
@@ -410,14 +528,14 @@ deflate_nagle_timeout(cqueue_t *unused_cq, gpointer arg)
 	}
 
 	attr->flags &= ~DF_NAGLE;
-	attr->tm_ev = NULL;
 
-	if (GNET_PROPERTY(dbg) > 9) {
-		printf("deflate_nagle_timeout: (%s) flushing (buffer #%d) [%c%c]\n",
-			gnet_host_to_string(&tx->host), attr->fill_idx,
+	if (tx_deflate_debugging(9)) {
+		struct buffer *b = &attr->buf[attr->fill_idx];	/* Buffer to send */
+		g_debug("TX %s: (%s) flushing %zu bytes (buffer #%d) [%c%c]",
+			G_STRFUNC, gnet_host_to_string(&tx->host),
+			b->wptr - b->rptr, attr->fill_idx,
 			(attr->flags & DF_FLOWC) ? 'C' : '-',
 			(attr->flags & DF_FLUSH) ? 'f' : '-');
-		fflush(stdout);
 	}
 
 	deflate_flush_send(tx);
@@ -430,27 +548,35 @@ deflate_nagle_timeout(cqueue_t *unused_cq, gpointer arg)
  * @return the amount of input bytes that were consumed ("added"), -1 on error.
  */
 static int
-deflate_add(txdrv_t *tx, gconstpointer data, int len)
+deflate_add(txdrv_t *tx, const void *data, int len)
 {
 	struct attr *attr = tx->opaque;
 	z_streamp outz = attr->outz;
 	int added = 0;
 
-	if (GNET_PROPERTY(dbg) > 9) {
-		printf("deflate_add: (%s) given %lu bytes (buffer #%d, nagle %s, "
-			"unflushed %lu) [%c%c]\n",
-			gnet_host_to_string(&tx->host), (gulong) len, attr->fill_idx,
-			(attr->flags & DF_NAGLE) ? "on" : "off", (gulong) attr->unflushed,
+	if (tx_deflate_debugging(9)) {
+		g_debug("TX %s: (%s) given %u bytes (buffer #%d, nagle %s, "
+			"unflushed %zu) [%c%c]%s", G_STRFUNC,
+			gnet_host_to_string(&tx->host), len, attr->fill_idx,
+			(attr->flags & DF_NAGLE) ? "on" : "off", attr->unflushed,
 			(attr->flags & DF_FLOWC) ? 'C' : '-',
-			(attr->flags & DF_FLUSH) ? 'f' : '-');
-		fflush(stdout);
+			(attr->flags & DF_FLUSH) ? 'f' : '-',
+			(tx->flags & TX_ERROR) ? " ERROR" : "");
 	}
 
+	/*
+	 * If an error was already reported, the whole deflate stream is dead
+	 * and we cannot accept any more data.
+	 */
+
+	if G_UNLIKELY(tx->flags & TX_ERROR)
+		return -1;
+
 	while (added < len) {
 		struct buffer *b = &attr->buf[attr->fill_idx];	/* Buffer we fill */
 		int ret;
 		int old_added = added;
-		gboolean flush_started = (attr->flags & DF_FLUSH) ? TRUE : FALSE;
+		bool flush_started = (attr->flags & DF_FLUSH) ? TRUE : FALSE;
 		int old_avail;
 		const char *in, *old_in;
 
@@ -458,12 +584,12 @@ deflate_add(txdrv_t *tx, gconstpointer data, int len)
 		 * Prepare call to deflate().
 		 */
 
-		outz->next_out = cast_to_gpointer(b->wptr);
+		outz->next_out = cast_to_pointer(b->wptr);
 		outz->avail_out = old_avail = b->end - b->wptr;
 
 		in = data;
 		old_in = &in[added];
-		outz->next_in = deconstify_gpointer(old_in);
+		outz->next_in = deconstify_pointer(old_in);
 		outz->avail_in = len - added;
 
 		g_assert(outz->avail_out > 0);
@@ -480,7 +606,7 @@ deflate_add(txdrv_t *tx, gconstpointer data, int len)
 
 		if (Z_OK != ret) {
 			attr->flags |= DF_SHUTDOWN;
-			attr->cb->shutdown(tx->owner, "Compression failed: %s",
+			(*attr->cb->shutdown)(tx->owner, "Compression failed: %s",
 				zlib_strerror(ret));
 			return -1;
 		}
@@ -489,12 +615,13 @@ deflate_add(txdrv_t *tx, gconstpointer data, int len)
 		 * Update the parameters.
 		 */
 
-		b->wptr = cast_to_gpointer(outz->next_out);
-		added = cast_to_gchar_ptr(outz->next_in) - in;
+		b->wptr = cast_to_pointer(outz->next_out);
+		added = ptr_diff(outz->next_in, in);
 
 		g_assert(added >= old_added);
 
 		attr->unflushed += added - old_added;
+		attr->flushed += old_avail - outz->avail_out;
 
 		if (NULL != attr->cb->add_tx_deflated)
 			attr->cb->add_tx_deflated(tx->owner, old_avail - outz->avail_out);
@@ -502,10 +629,21 @@ deflate_add(txdrv_t *tx, gconstpointer data, int len)
 		if (attr->gzip.enabled) {
 			size_t r;
 
-			r = cast_to_gchar_ptr(outz->next_in) - old_in;
+			r = ptr_diff(outz->next_in, old_in);
 			attr->gzip.size += r;
 			attr->gzip.crc = crc32(attr->gzip.crc,
-								cast_to_gconstpointer(old_in), r);
+								cast_to_constpointer(old_in), r);
+		}
+
+		if (tx_deflate_debugging(9)) {
+			g_debug("TX %s: (%s) deflated %d bytes into %d "
+				"(buffer #%d, nagle %s, flushed %zu, unflushed %zu) [%c%c]",
+				G_STRFUNC, gnet_host_to_string(&tx->host),
+				added, old_avail - outz->avail_out, attr->fill_idx,
+				(attr->flags & DF_NAGLE) ? "on" : "off",
+				attr->flushed, attr->unflushed,
+				(attr->flags & DF_FLOWC) ? 'C' : '-',
+				(attr->flags & DF_FLUSH) ? 'f' : '-');
 		}
 
 		/*
@@ -516,16 +654,11 @@ deflate_add(txdrv_t *tx, gconstpointer data, int len)
 
 		if (0 == outz->avail_out) {
 			if (attr->send_idx >= 0) {
-				attr->flags |= DF_FLOWC;	/* Enter flow control */
-
-				if (GNET_PROPERTY(dbg) > 4)
-					printf("Compressing TX stack for peer %s enters FLOWC\n",
-						gnet_host_to_string(&tx->host));
-
+				deflate_set_flowc(tx, TRUE);	/* Enter flow control */
 				return added;
 			}
 
-			deflate_rotate_and_send(tx);	/* Can set TX_ERROR */
+			deflate_rotate_and_send(tx);		/* Can set TX_ERROR */
 
 			if (tx->flags & TX_ERROR)
 				return -1;
@@ -539,10 +672,8 @@ deflate_add(txdrv_t *tx, gconstpointer data, int len)
 		 * space avaialable.
 		 */
 
-		if (flush_started && 0 == outz->avail_in) {
-			attr->unflushed = 0;
-			attr->flags &= ~DF_FLUSH;
-		}
+		if (flush_started && 0 == outz->avail_in)
+			deflate_flushed(tx);
 	}
 
 	g_assert(0 == outz->avail_in);
@@ -551,7 +682,9 @@ deflate_add(txdrv_t *tx, gconstpointer data, int len)
 	 * Start Nagle if not already on.
 	 */
 
-	if (!(attr->flags & DF_NAGLE))
+	if (attr->flags & DF_NAGLE)
+		deflate_nagle_delay(tx);
+	else
 		deflate_nagle_start(tx);
 
 	/*
@@ -574,7 +707,7 @@ deflate_add(txdrv_t *tx, gconstpointer data, int len)
  * Called by lower layer when it is ready to process more data.
  */
 static void
-deflate_service(gpointer data)
+deflate_service(void *data)
 {
 	txdrv_t *tx = data;
 	struct attr *attr = tx->opaque;
@@ -582,17 +715,17 @@ deflate_service(gpointer data)
 
 	g_assert(attr->send_idx < BUFFER_COUNT);
 
-	if (GNET_PROPERTY(dbg) > 9)
-		printf("deflate_service: (%s) %s(buffer #%d, %lu bytes held) [%c%c]\n",
-			gnet_host_to_string(&tx->host),
+	if (tx_deflate_debugging(9)) {
+		g_debug("TX %s: (%s) %s(buffer #%d, %zu bytes held) [%c%c]",
+			G_STRFUNC, gnet_host_to_string(&tx->host),
 			(tx->flags & TX_ERROR) ? "ERROR " : "",
 			attr->send_idx,
 			attr->send_idx >= 0 ?
-				(unsigned long) (attr->buf[attr->send_idx].wptr -
-						attr->buf[attr->send_idx].rptr) : 0,
+				(attr->buf[attr->send_idx].wptr -
+					attr->buf[attr->send_idx].rptr) : 0,
 			(attr->flags & DF_FLOWC) ? 'C' : '-',
 			(attr->flags & DF_FLUSH) ? 'f' : '-');
-
+	}
 	/*
 	 * First, attempt to transmit the whole send buffer, if any pending.
 	 */
@@ -615,11 +748,11 @@ deflate_service(gpointer data)
 	b = &attr->buf[attr->fill_idx];	/* Buffer we fill */
 
 	if (b->wptr >= b->end) {
-		if (GNET_PROPERTY(dbg) > 9)
-			printf("deflate_service: (%s) sending fill buffer #%d, %lu bytes\n",
-				gnet_host_to_string(&tx->host), attr->fill_idx,
-				(unsigned long) (b->wptr - b->rptr));
-
+		if (tx_deflate_debugging(9)) {
+			g_debug("TX %s: (%s) sending fill buffer #%d, %zu bytes",
+				G_STRFUNC, gnet_host_to_string(&tx->host), attr->fill_idx,
+				b->wptr - b->rptr);
+		}
 		deflate_rotate_and_send(tx);	/* Can set TX_ERROR */
 
 		if (tx->flags & TX_ERROR)
@@ -638,13 +771,8 @@ deflate_service(gpointer data)
 	 * have at least a free `fill' buffer.
 	 */
 
-	if (attr->flags & DF_FLOWC) {
-		attr->flags &= ~DF_FLOWC;	/* Leave flow control state */
-
-		if (GNET_PROPERTY(dbg) > 4)
-			printf("Compressing TX stack for peer %s leaves FLOWC\n",
-				gnet_host_to_string(&tx->host));
-	}
+	if (attr->flags & DF_FLOWC)
+		deflate_set_flowc(tx, FALSE);	/* Leave flow control state */
 
 	/*
 	 * If closing, we're done once we have flushed everything we could.
@@ -664,13 +792,13 @@ deflate_service(gpointer data)
 		}
 	}
 
-
-	if (GNET_PROPERTY(dbg) > 9)
-		printf("deflate_service: (%s) %sdone locally [%c%c]\n",
-			gnet_host_to_string(&tx->host),
+	if (tx_deflate_debugging(9)) {
+		g_debug("TX %s: (%s) %sdone locally [%c%c]",
+			G_STRFUNC, gnet_host_to_string(&tx->host),
 			(tx->flags & TX_ERROR) ? "ERROR " : "",
 			(attr->flags & DF_FLOWC) ? 'C' : '-',
 			(attr->flags & DF_FLUSH) ? 'f' : '-');
+	}
 
 	/*
 	 * If upper layer wants servicing, do it now.
@@ -692,8 +820,8 @@ deflate_service(gpointer data)
  *
  * @return NULL if there is an initialization problem.
  */
-static gpointer
-tx_deflate_init(txdrv_t *tx, gpointer args)
+static void *
+tx_deflate_init(txdrv_t *tx, void *args)
 {
 	struct attr *attr;
 	struct tx_deflate_args *targs = args;
@@ -704,8 +832,7 @@ tx_deflate_init(txdrv_t *tx, gpointer args)
 	g_assert(tx);
 	g_assert(NULL != targs->cb);
 
-	outz = walloc(sizeof *outz);
-
+	WALLOC(outz);
 	outz->zalloc = zlib_alloc_func;
 	outz->zfree = zlib_free_func;
 	outz->opaque = NULL;
@@ -728,6 +855,19 @@ tx_deflate_init(txdrv_t *tx, gpointer args)
 	 * depending on the nature of the traffic, of course).
 	 *
 	 *		--RAM, 2009-04-09
+	 *
+	 * For Ultra <-> Ultra connections we use window_bits = 15 and mem_level = 9
+	 * and request a best compression because the amount of ultra connections
+	 * is far less than the number of leaf connections and modern machines
+	 * can cope with a "best" compression overhead.
+	 *
+	 * This is now controlled with the "reduced" argument, so this layer does
+	 * not need to know whether we're an ultra node or even what an ultra
+	 * node is... It just knows whether we have to setup a fully compressed
+	 * connection or a reduced one (both in terms of memory usage and level
+	 * of compression).
+	 *
+	 *		--RAM, 2011-11-29
 	 */
 
 	{
@@ -735,7 +875,8 @@ tx_deflate_init(txdrv_t *tx, gpointer args)
 		int mem_level = MAX_MEM_LEVEL;		/* Must be 1 .. MAX_MEM_LEVEL */
 		int level = Z_BEST_COMPRESSION;
 
-		if (GNET_PROPERTY(current_peermode) == NODE_P_ULTRA) {
+		if (targs->reduced) {
+			/* Ultra -> Leaf connection */
 			window_bits = 14;
 			mem_level = 6;
 			level = Z_DEFAULT_COMPRESSION;
@@ -754,23 +895,20 @@ tx_deflate_init(txdrv_t *tx, gpointer args)
 	if (Z_OK != ret) {
 		g_warning("unable to initialize compressor for peer %s: %s",
 			gnet_host_to_string(&tx->host), zlib_strerror(ret));
-		wfree(outz, sizeof *outz);
+		WFREE(outz);
 		return NULL;
 	}
 
-	attr = walloc(sizeof *attr);
-
+	WALLOC0(attr);
 	attr->cq = targs->cq;
 	attr->cb = targs->cb;
 	attr->buffer_size = targs->buffer_size;
 	attr->buffer_flush = targs->buffer_flush;
-	attr->nagle = targs->nagle;
+	attr->nagle = booleanize(targs->nagle);
 	attr->gzip.enabled = targs->gzip;
 
 	attr->outz = outz;
-	attr->flags = 0;
 	attr->tm_ev = NULL;
-	attr->unflushed = 0;
 
 	for (i = 0; i < BUFFER_COUNT; i++) {
 		struct buffer *b = &attr->buf[i];
@@ -796,8 +934,7 @@ tx_deflate_init(txdrv_t *tx, gpointer args)
 
 		b = &attr->buf[attr->fill_idx];	/* Buffer we fill */
 		g_assert(sizeof header <= (size_t) (b->end - b->wptr));
-		memcpy(b->wptr, header, sizeof header);
-		b->wptr += sizeof header;
+		b->wptr = mempcpy(b->wptr, header, sizeof header);
 
 		attr->gzip.crc = crc32(0, NULL, 0);
 		attr->gzip.size = 0;
@@ -841,9 +978,9 @@ tx_deflate_destroy(txdrv_t *tx)
 		g_warning("while freeing compressor for peer %s: %s",
 			gnet_host_to_string(&tx->host), zlib_strerror(ret));
 
-	wfree(attr->outz, sizeof *attr->outz);
-	cq_cancel(attr->cq, &attr->tm_ev);
-	wfree(attr, sizeof *attr);
+	WFREE(attr->outz);
+	cq_cancel(&attr->tm_ev);
+	WFREE(attr);
 }
 
 /**
@@ -852,18 +989,17 @@ tx_deflate_destroy(txdrv_t *tx)
  * @return amount of bytes written, or -1 on error.
  */
 static ssize_t
-tx_deflate_write(txdrv_t *tx, gconstpointer data, size_t len)
+tx_deflate_write(txdrv_t *tx, const void *data, size_t len)
 {
 	struct attr *attr = tx->opaque;
 
-	if (GNET_PROPERTY(dbg) > 9)
-		printf("tx_deflate_write: (%s) (buffer #%d, nagle %s, "
-			"unflushed %lu) [%c%c]\n",
-			gnet_host_to_string(&tx->host), attr->fill_idx,
-			(attr->flags & DF_NAGLE) ? "on" : "off", (gulong) attr->unflushed,
+	if (tx_deflate_debugging(9)) {
+		g_debug("TX %s: (%s) (buffer #%d, nagle %s, unflushed %zu) [%c%c]",
+			G_STRFUNC, gnet_host_to_string(&tx->host), attr->fill_idx,
+			(attr->flags & DF_NAGLE) ? "on" : "off", attr->unflushed,
 			(attr->flags & DF_FLOWC) ? 'C' : '-',
 			(attr->flags & DF_FLUSH) ? 'f' : '-');
-
+	}
 	/*
 	 * If we're flow controlled or shut down, don't accept anything.
 	 */
@@ -880,20 +1016,20 @@ tx_deflate_write(txdrv_t *tx, gconstpointer data, size_t len)
  * @return amount of bytes written, or -1 on error.
  */
 static ssize_t
-tx_deflate_writev(txdrv_t *tx, struct iovec *iov, int iovcnt)
+tx_deflate_writev(txdrv_t *tx, iovec_t *iov, int iovcnt)
 {
 	struct attr *attr = tx->opaque;
 	int sent = 0;
 
-	if (GNET_PROPERTY(dbg) > 9)
-		printf("tx_deflate_writev: (%s) (buffer #%d, nagle %s, "
-			"unflushed %lu) [%c%c]\n",
-			gnet_host_to_string(&tx->host), attr->fill_idx,
-			(attr->flags & DF_NAGLE) ? "on" : "off", (gulong) attr->unflushed,
+	if (tx_deflate_debugging(9)) {
+		g_debug("TX %s: (%s) (buffer #%d, nagle %s, unflushed %zu) [%c%c]",
+			G_STRFUNC, gnet_host_to_string(&tx->host), attr->fill_idx,
+			(attr->flags & DF_NAGLE) ? "on" : "off", attr->unflushed,
 			(attr->flags & DF_FLOWC) ? 'C' : '-',
 			(attr->flags & DF_FLUSH) ? 'f' : '-');
+	}
 
-	while (iovcnt--) {
+	while (iovcnt-- > 0) {
 		int ret;
 
 		/*
@@ -901,29 +1037,29 @@ tx_deflate_writev(txdrv_t *tx, struct iovec *iov, int iovcnt)
 		 */
 
 		if (attr->flags & (DF_FLOWC|DF_SHUTDOWN))
-			return sent;
+			break;
 
-		ret = deflate_add(tx, iov->iov_base, iov->iov_len);
+		ret = deflate_add(tx, iovec_base(iov), iovec_len(iov));
 
 		if (-1 == ret)
 			return -1;
 
 		sent += ret;
-		if ((guint) ret < iov->iov_len) {
+		if (UNSIGNED(ret) < iovec_len(iov)) {
 			/* Could not write all, flow-controlled */
 			break;
 		}
 		iov++;
 	}
 
-	if (GNET_PROPERTY(dbg) > 9)
-		printf("tx_deflate_writev: (%s) sent %lu bytes (buffer #%d, nagle %s, "
-			"unflushed %lu) [%c%c]\n",
-			gnet_host_to_string(&tx->host), (gulong) sent, attr->fill_idx,
-			(attr->flags & DF_NAGLE) ? "on" : "off", (gulong) attr->unflushed,
+	if (tx_deflate_debugging(9)) {
+		g_debug("TX %s: (%s) sent %lu bytes (buffer #%d, nagle %s, "
+			"unflushed %zu) [%c%c]", G_STRFUNC,
+			gnet_host_to_string(&tx->host), (ulong) sent, attr->fill_idx,
+			(attr->flags & DF_NAGLE) ? "on" : "off", attr->unflushed,
 			(attr->flags & DF_FLOWC) ? 'C' : '-',
 			(attr->flags & DF_FLUSH) ? 'f' : '-');
-
+	}
 	return sent;
 }
 
@@ -954,16 +1090,18 @@ static size_t
 tx_deflate_pending(txdrv_t *tx)
 {
 	const struct attr *attr = tx->opaque;
-	const struct buffer *b;
 	size_t pending;
 
-	b = &attr->buf[attr->fill_idx];	/* Buffer we fill */
-	pending = b->wptr - b->rptr;
-	pending += attr->unflushed;		/* Some of those made it to buffer */
+	pending = deflate_buffered(tx);
 
-	if (-1 != attr->send_idx) {
-		b = &attr->buf[attr->send_idx];	/* Buffer we send */
-		pending += b->wptr - b->rptr;
+	/*
+	 * Account for deflation of pending bytes, using the current compression
+	 * ratio (EMA) to estimate how much we're going to emit.
+	 */
+
+	if (attr->unflushed != 0) {
+		size_t projected = attr->unflushed * (1.0 - attr->ratio_ema);
+		pending += attr->flushed >= projected ? 1 : projected - attr->flushed;
 	}
 
 	return pending;
@@ -979,7 +1117,7 @@ tx_deflate_flush(txdrv_t *tx)
 
 	if (attr->flags & DF_NAGLE) {
 		g_assert(NULL != attr->tm_ev);
-		cq_expire(attr->cq, attr->tm_ev);
+		cq_expire(attr->tm_ev);
 	} else if (!(attr->flags & DF_FLOWC))
 		deflate_flush_send(tx);
 }
@@ -1005,19 +1143,20 @@ tx_deflate_shutdown(txdrv_t *tx)
  * Once this is done, invoke the supplied callback.
  */
 static void
-tx_deflate_close(txdrv_t *tx, tx_closed_t cb, gpointer arg)
+tx_deflate_close(txdrv_t *tx, tx_closed_t cb, void *arg)
 {
 	struct attr *attr = tx->opaque;
 
 	g_assert(tx->flags & TX_CLOSING);
 
-	if (GNET_PROPERTY(dbg) > 9)
-		printf("tx_deflate_close: (%s) send=%d buffer #%d, nagle %s, "
-			"unflushed %lu) [%c%c]\n",
+	if (tx_deflate_debugging(9)) {
+		g_debug("TX %s: (%s) send=%d buffer #%d, nagle %s, "
+			"unflushed %zu) [%c%c]", G_STRFUNC,
 			gnet_host_to_string(&tx->host), attr->send_idx, attr->fill_idx,
-			(attr->flags & DF_NAGLE) ? "on" : "off", (gulong) attr->unflushed,
+			(attr->flags & DF_NAGLE) ? "on" : "off", attr->unflushed,
 			(attr->flags & DF_FLOWC) ? 'C' : '-',
 			(attr->flags & DF_FLUSH) ? 'f' : '-');
+	}
 
 	/*
 	 * Flush whatever we can.
@@ -1028,27 +1167,26 @@ tx_deflate_close(txdrv_t *tx, tx_closed_t cb, gpointer arg)
 	if (attr->gzip.enabled && 0 == tx_deflate_pending(tx)) {
 		/* See RFC 1952 - GZIP file format specification version 4.3 */
 		struct buffer *b;
-		guint32 trailer[2]; /* 0: CRC32, 1: SIZE % (1 << 32) */
+		uint32 trailer[2]; /* 0: CRC32, 1: SIZE % (1 << 32) */
 
 		/* We don't want to send the trailer more than once */
 		attr->gzip.enabled = FALSE;
 
 		attr->send_idx = 0;
 		b = &attr->buf[attr->send_idx];
-		poke_le32(&trailer[0], (guint32) attr->gzip.crc);
+		poke_le32(&trailer[0], (uint32) attr->gzip.crc);
 		poke_le32(&trailer[1], attr->gzip.size);
 
 		g_assert(sizeof trailer <= (size_t) (b->end - b->wptr));
-		memcpy(b->wptr, trailer, sizeof trailer);
-		b->wptr += sizeof trailer;
+		b->wptr = mempcpy(b->wptr, trailer, sizeof trailer);
 
 		deflate_send(tx);
 	}
 
 	if (0 == tx_deflate_pending(tx)) {
-		if (GNET_PROPERTY(dbg) > 9)
-			printf("tx_deflate_close: flushed everything immediately\n");
-
+		if (tx_deflate_debugging(9)) {
+			g_debug("TX %s: flushed everything immediately", G_STRFUNC);
+		}
 		(*cb)(tx, arg);
 		return;
 	}
@@ -1060,17 +1198,20 @@ tx_deflate_close(txdrv_t *tx, tx_closed_t cb, gpointer arg)
 	attr->closed = cb;
 	attr->closed_arg = arg;
 
-	if (GNET_PROPERTY(dbg) > 9)
-		printf("tx_deflate_close: (%s) delayed! send=%d buffer #%d, nagle %s, "
-			"unflushed %lu) [%c%c]\n",
-			gnet_host_to_string(&tx->host),
+	if (tx_deflate_debugging(9)) {
+		g_debug("TX %s: (%s) delayed! send=%d buffer #%d, nagle %s, "
+			"flushed %zu, unflushed %zu) [%c%c]",
+			G_STRFUNC, gnet_host_to_string(&tx->host),
 			attr->send_idx, attr->fill_idx,
-			(attr->flags & DF_NAGLE) ? "on" : "off", (gulong) attr->unflushed,
+			(attr->flags & DF_NAGLE) ? "on" : "off",
+			attr->flushed, attr->unflushed,
 			(attr->flags & DF_FLOWC) ? 'C' : '-',
 			(attr->flags & DF_FLUSH) ? 'f' : '-');
+	}
 }
 
 static const struct txdrv_ops tx_deflate_ops = {
+	"deflate",				/**< name */
 	tx_deflate_init,		/**< init */
 	tx_deflate_destroy,		/**< destroy */
 	tx_deflate_write,		/**< write */
