@@ -1,6 +1,4 @@
 /*
- * $Id: tx_link.c 16369 2009-04-02 16:54:32Z cbiere $
- *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -39,8 +37,6 @@
 
 #include "common.h"
 
-RCSID("$Id: tx_link.c 16369 2009-04-02 16:54:32Z cbiere $")
-
 #include "sockets.h"
 #include "tx.h"
 #include "tx_link.h"
@@ -63,7 +59,7 @@ struct attr {
  * Invoked when the output file descriptor can accept more data.
  */
 static void
-is_writable(gpointer data, int unused_source, inputevt_cond_t cond)
+is_writable(void *data, int unused_source, inputevt_cond_t cond)
 {
 	txdrv_t *tx = data;
 	struct attr *attr = tx->opaque;
@@ -72,7 +68,7 @@ is_writable(gpointer data, int unused_source, inputevt_cond_t cond)
 	g_assert(tx->flags & TX_SERVICE);		/* Servicing enabled */
 
 	if (cond & INPUT_EVENT_EXCEPTION) {
-		tx->flags |= TX_ERROR;
+		tx_error(tx);
 		attr->cb->eof_remove(tx->owner, _("Write failed (Input Exception)"));
 		return;
 	}
@@ -94,8 +90,8 @@ is_writable(gpointer data, int unused_source, inputevt_cond_t cond)
  *
  * Always succeeds, so never returns NULL.
  */
-static gpointer
-tx_link_init(txdrv_t *tx, gpointer args)
+static void *
+tx_link_init(txdrv_t *tx, void *args)
 {
 	struct tx_link_args *targs = args;
 	struct attr *attr;
@@ -103,11 +99,11 @@ tx_link_init(txdrv_t *tx, gpointer args)
 	g_assert(tx);
 	g_assert(targs->cb != NULL);
 
-	attr = walloc(sizeof *attr);
+	WALLOC(attr);
 
 	/*
 	 * Because we handle servicing of the upper layers explicitely within
-	 * the TX stack (i.e. upper layers detect that we were enable to comply
+	 * the TX stack (i.e. upper layers detect that we were unable to comply
 	 * with the whole write and enable us), there is no I/O callback attached
 	 * to the I/O source: we only create it to benefit from bandwidth limiting
 	 * through calls to bio_write() and bio_writev().
@@ -136,7 +132,7 @@ tx_link_destroy(txdrv_t *tx)
 
 	bsched_source_remove(attr->bio);
 
-	wfree(attr, sizeof *attr);
+	WFREE(attr);
 }
 
 static inline int
@@ -154,27 +150,24 @@ tx_link_write_error(txdrv_t *tx, const char *func)
 	 * robust against bugs in the components we rely on. --RAM, 09/10/2003
 	 */
 	case EINPROGRESS:		/* Weird, but seen it -- RAM, 07/10/2003 */
-		g_warning("%s(fd=%d) failed with weird errno = %d (%s), "
-			"assuming EAGAIN", func, attr->wio->fd(attr->wio), errno,
-			g_strerror(errno));
+		g_warning("%s(fd=%d) failed with weird errno = %d: %m -- "
+			"assuming EAGAIN", func, attr->wio->fd(attr->wio), errno);
 		return 0;
 
 	case EPIPE:
 	case ECONNRESET:
-		tx->flags |= TX_ERROR;
+	case ECONNABORTED:
+		tx_error(tx);
 		attr->cb->eof_remove(tx->owner,
 			_("Write failed: %s"), g_strerror(errno));
 		return -1;
 
 	default:
 		{
-			int saved_errno = errno;
 			wrap_io_t *wio = ((struct attr *) tx->opaque)->wio;
 			int fd = wio->fd(wio);
-			g_warning(
-				"%s: write failed on fd #%d with unexpected errno: %d (%s)",
-				func, fd, saved_errno, g_strerror(saved_errno));
-			errno = saved_errno;
+			g_warning("%s: write failed on fd #%d with unexpected error: %m",
+				func, fd);
 		}
 		/* FALL THROUGH */
 
@@ -182,6 +175,9 @@ tx_link_write_error(txdrv_t *tx, const char *func)
 #ifdef EDQUOT
 	case EDQUOT:
 #endif /* EDQUOT */
+#ifdef ESHUTDOWN
+	case ESHUTDOWN:
+#endif	/* ESHUTDOWN */
 	case EACCES:
 	case EFBIG:
 	case EHOSTDOWN:
@@ -190,7 +186,7 @@ tx_link_write_error(txdrv_t *tx, const char *func)
 	case ENETDOWN:
 	case ENETUNREACH:
 	case ETIMEDOUT:
-		tx->flags |= TX_ERROR;
+		tx_error(tx);
 		attr->cb->eof_shutdown(tx->owner,
 			_("Write failed: %s"), g_strerror(errno));
 		return -1;
@@ -205,7 +201,7 @@ tx_link_write_error(txdrv_t *tx, const char *func)
  * @return amount of bytes written, or -1 on error.
  */
 static ssize_t
-tx_link_write(txdrv_t *tx, gconstpointer data, size_t len)
+tx_link_write(txdrv_t *tx, const void *data, size_t len)
 {
 	struct attr *attr = tx->opaque;
 	ssize_t r;
@@ -226,7 +222,7 @@ tx_link_write(txdrv_t *tx, gconstpointer data, size_t len)
  * @return amount of bytes written, or -1 on error.
  */
 static ssize_t
-tx_link_writev(txdrv_t *tx, struct iovec *iov, int iovcnt)
+tx_link_writev(txdrv_t *tx, iovec_t *iov, int iovcnt)
 {
 	struct attr *attr = tx->opaque;
 	ssize_t r;
@@ -249,7 +245,7 @@ tx_link_enable(txdrv_t *tx)
 {
 	struct attr *attr = tx->opaque;
 
-	bio_add_callback(attr->bio, is_writable, (gpointer) tx);
+	bio_add_callback(attr->bio, is_writable, tx);
 }
 
 /**
@@ -315,6 +311,7 @@ tx_link_bio_source(txdrv_t *tx)
 }
 
 static const struct txdrv_ops tx_link_ops = {
+	"link",				/**< name */
 	tx_link_init,		/**< init */
 	tx_link_destroy,	/**< destroy */
 	tx_link_write,		/**< write */
