@@ -1,6 +1,4 @@
 /*
- * $Id: tx_link.h 8913 2005-08-01 17:37:35Z rmanfredi $
- *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -36,7 +34,10 @@
 #ifndef _core_tx_link_h_
 #define _core_tx_link_h_
 
+#include "common.h"
+
 #include "tx.h"
+#include "if/core/bsched.h"
 
 const struct txdrv_ops *tx_link_get_ops(void);
 
@@ -44,19 +45,19 @@ const struct txdrv_ops *tx_link_get_ops(void);
  * Callbacks used by the link layer.
  */
 struct tx_link_cb {
-	void (*add_tx_written)(gpointer owner, gint amount);
-	void (*eof_remove)(gpointer owner, const gchar *reason, ...);
-	void (*eof_shutdown)(gpointer owner, const gchar *reason, ...);
-	void (*unflushq)(gpointer owner);
+	void (*add_tx_written)(void *owner, int amount);
+	void (*eof_remove)(void *owner, const char *reason, ...);
+	void (*eof_shutdown)(void *owner, const char *reason, ...);
+	void (*unflushq)(void *owner);
 };
 
 /**
- * Arguments to be passed when the layer is intantiated.
+ * Arguments to be passed when the layer is instantiated.
  */
 struct tx_link_args {
-	struct tx_link_cb *cb;			/**< Callbacks */
-	struct bsched *bs;				/**< Bandwidth scheduler to use */
+	const struct tx_link_cb *cb;	/**< Callbacks */
 	struct wrap_io *wio;			/**< I/O wrapping routines */
+	bsched_bws_t bws;				/**< Bandwidth scheduler to use */
 };
 
 #endif	/* _core_tx_link_h_ */
