@@ -1,7 +1,5 @@
 /*
- * $Id: uhc.c 16990 2009-09-25 15:23:43Z rmanfredi $
- *
- * Copyright (c) 2004, Raphael Manfredi
+ * Copyright (c) 2004, 2015 Raphael Manfredi
  *
  *----------------------------------------------------------------------
  * This file is part of gtk-gnutella.
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -30,13 +28,11 @@
  * UDP Host Cache.
  *
  * @author Raphael Manfredi
- * @date 2004
+ * @date 2004, 2015
  */
 
 #include "common.h"
 
-RCSID("$Id: uhc.c 16990 2009-09-25 15:23:43Z rmanfredi $")
-
 #include "gnutella.h"
 #include "guid.h"
 #include "hcache.h"
@@ -46,14 +42,22 @@ RCSID("$Id: uhc.c 16990 2009-09-25 15:23:43Z rmanfredi $")
 #include "sockets.h"
 #include "udp.h"
 #include "uhc.h"
+#include "ghc.h"
 
 #include "lib/adns.h"
 #include "lib/atoms.h"
 #include "lib/cq.h"
 #include "lib/endian.h"
-#include "lib/glib-missing.h"
+#include "lib/halloc.h"
+#include "lib/hashing.h"
 #include "lib/hashlist.h"
+#include "lib/hstrfn.h"
+#include "lib/mempcpy.h"
 #include "lib/parse.h"
+#include "lib/shuffle.h"
+#include "lib/str.h"
+#include "lib/stringify.h"
+#include "lib/walloc.h"
 
 #include "if/gnet_property_priv.h"
 #include "if/bridge/c2ui.h"
@@ -63,7 +67,7 @@ RCSID("$Id: uhc.c 16990 2009-09-25 15:23:43Z rmanfredi $")
 
 #define UHC_MAX_ATTEMPTS 3		/**< Maximum connection / resolution attempts */
 #define UHC_TIMEOUT		 20000	/**< Host cache timeout, milliseconds */
-#define UHC_RETRY_AFTER	 3600	/**< Frequency of contacts for an UHC (secs) */
+#define UHC_RETRY_AFTER	 180	/**< Frequency of contacts for an UHC (secs) */
 
 /**
  * Request context, used when we decide to get hosts via the UDP host caches.
@@ -76,7 +80,7 @@ static struct uhc_context {
 	cevent_t *timeout_ev;		/**< Ping timeout */
 	int attempts;				/**< Connection / resolution attempts */
 	host_addr_t addr;			/**< Resolved IP address for host */
-	guint16 port;				/**< Port of selected host cache */
+	uint16 port;				/**< Port of selected host cache */
 	struct guid muid;			/**< MUID of the ping */
 } uhc_ctx;
 
@@ -85,7 +89,7 @@ static hash_list_t *uhc_list;	/**< List of ``struct uhc'' */
 struct uhc {
 	const char	*host;	/**< An UHC host as "<host>:<port>" (string atom) */
 	time_t		stamp;	/**< Timestamp of the last request */
-	guint		used;	/**< How often have we tried to contact it */
+	uint		used;	/**< How often have we tried to contact it */
 };
 
 /**
@@ -97,23 +101,24 @@ static const struct {
 #if defined(USE_LOCAL_UHC)
 	{ "localhost:6346" },
 #else	/* !USE_LOCAL_UHC */
-	{ "drei.gtkg.net:62666" },
-	{ "g6.dns6.org:1337" },
-	{ "gnutelladev1.udp-host-cache.com:1234" },
-	{ "gnutelladev2.udp-host-cache.com:5678" },
-	{ "gwc.ak-electron.eu:12060" },
-	{ "gwc.chickenkiller.com:8080" },
-	{ "leet.gtkg.org:1337" },
-	{ "tertiary.udp-host-cache.com:9999" },
-	{ "yang.cloud.bishopston.net:33558" },
-	{ "yin.cloud.bishopston.net:33558" },
+	/* Crabs */
+	{ "useast.gnutella.dyslexicfish.net:3558" },
+	{ "uswest.gnutella.dyslexicfish.net:3558" },
+	{ "uk.gnutella.dyslexicfish.net:3558" },
+	{ "au.gnutella.dyslexicfish.net:3558" },
+	/* Peers */
+	{ "1.uhc.gtk-gnutella.nl:19104" },
+	{ "2.uhc.gtk-gnutella.nl:4876" },
+	{ "useast.gnutella.dyslexicfish.net:15321" },
+	{ "uswest.gnutella.dyslexicfish.net:18037" },
+	{ "uk.gnutella.dyslexicfish.net:41481" },
 #endif	/* USE_LOCAL_UHC */
 };
 
-static gboolean uhc_connecting = FALSE;
+static bool uhc_connecting = FALSE;
 
-static void uhc_host_resolved(const host_addr_t *addr, size_t n,
-				gpointer uu_udata);
+static void uhc_host_resolved(const host_addr_t *addr, size_t n, void *udata);
+static void uhc_send_ping(void);
 
 /**
  * Parse hostname:port and return the hostname and port parts.
@@ -124,15 +129,15 @@ static void uhc_host_resolved(const host_addr_t *addr, size_t n,
  *
  * @return TRUE if we successfully parsed the string.
  */
-static gboolean
-uhc_get_host_port(const char *hp, const char **host, guint16 *port)
+static bool
+uhc_get_host_port(const char *hp, const char **host, uint16 *port)
 {
 	static char hostname[MAX_HOSTLEN + 1];
 	const char *ep;
-	guint32 u;
+	uint32 u;
 	int error;
 	size_t len;
-
+	char *p;
 
 	g_assert(hp);
 	g_assert(host);
@@ -148,8 +153,8 @@ uhc_get_host_port(const char *hp, const char **host, guint16 *port)
 	len = ep - hp;
 	if (len >= sizeof hostname)
 		return FALSE;
-	memcpy(hostname, hp, len);
-	hostname[len] = '\0';
+	p = mempcpy(hostname, hp, len);
+	*p = '\0';
 
 	g_assert(':' == *ep);
 	ep++;
@@ -167,39 +172,36 @@ uhc_get_host_port(const char *hp, const char **host, guint16 *port)
 static struct uhc *
 uhc_new(const char *host)
 {
-	static const struct uhc zero_uhc;
 	struct uhc *uhc;
 
-	g_assert(host);
-	uhc = g_malloc(sizeof *uhc);
-	*uhc = zero_uhc;
+	g_assert(host != NULL);
+
+	WALLOC0(uhc);
 	uhc->host = atom_str_get(host);
-	uhc->stamp = 0;
-	uhc->used = 0;
 	return uhc;
 }
 
 static void
 uhc_free(struct uhc **ptr)
 {
-	if (*ptr) {	
+	if (*ptr) {
 		struct uhc *uu = *ptr;
 		atom_str_free_null(&uu->host);
-		G_FREE_NULL(uu);
+		WFREE(uu);
 		*ptr = NULL;
 	}
 }
 
-static guint
-uhc_hash(gconstpointer key)
+static uint
+uhc_hash(const void *key)
 {
 	const struct uhc *uhc = key;
 
-	return g_str_hash(uhc->host);
+	return string_mix_hash(uhc->host);
 }
 
 static int
-uhc_equal(gconstpointer p, gconstpointer q)
+uhc_equal(const void *p, const void *q)
 {
 	const struct uhc *a = p, *b = q;
 
@@ -208,7 +210,7 @@ uhc_equal(gconstpointer p, gconstpointer q)
 
 
 static void
-uhc_list_add(const char *host)
+uhc_list_append(const char *host)
 {
 	struct uhc *uhc;
 
@@ -221,15 +223,14 @@ uhc_list_add(const char *host)
 		return;
 	}
 
-	if (random_value(100) < 50) {
-		hash_list_append(uhc_list, uhc);
-	} else {
-		hash_list_prepend(uhc_list, uhc);
-	}
+	if (GNET_PROPERTY(bootstrap_debug) > 1)
+		g_debug("adding UHC %s", host);
+
+	hash_list_append(uhc_list, uhc);
 }
 
 /**
- * @return NULL on error, a newly allocated string otherwise.
+ * @return NULL on error, a newly allocated string via halloc() otherwise.
  */
 static char *
 uhc_get_next(void)
@@ -237,12 +238,14 @@ uhc_get_next(void)
 	struct uhc *uhc;
 	char *host;
 	time_t now;
+	size_t n;
 
 	g_return_val_if_fail(uhc_list, NULL);
-	
+
 	now = tm_time();
-	uhc = hash_list_head(uhc_list);
-	if (NULL == uhc)
+
+	n = hash_list_count(uhc_list);
+	if (0 == n)
 		return NULL;
 
 	/*
@@ -252,11 +255,23 @@ uhc_get_next(void)
 	 * If we come here, it's because we're lacking hosts for establishing
 	 * a Gnutella connection, after we exhausted our caches.
 	 */
-	if (uhc->stamp && delta_time(now, uhc->stamp) < UHC_RETRY_AFTER)
-		return NULL;
 
+	while (n-- != 0) {
+		uhc = hash_list_head(uhc_list);
+
+		g_assert(uhc != NULL);	/* We computed count on entry */
+
+		if (delta_time(now, uhc->stamp) >= UHC_RETRY_AFTER)
+			goto found;
+
+		hash_list_moveto_tail(uhc_list, uhc);
+	}
+
+	return NULL;
+
+found:
 	uhc->stamp = now;
-	host = g_strdup(uhc->host);
+	host = h_strdup(uhc->host);
 
 	if (uhc->used < UHC_MAX_ATTEMPTS) {
 		uhc->used++;
@@ -269,21 +284,58 @@ uhc_get_next(void)
 	return host;
 }
 
+/**
+ * Re-initialize UHC list from hardwired set.
+ */
+static void G_COLD
+uhc_hardwired_reload(void)
+{
+	uint i;
+
+	g_return_if_fail(NULL != uhc_list);
+
+	for (i = 0; i < N_ITEMS(boot_hosts); i++) {
+		const char *host, *ep, *uhc;
+		uint16 port;
+
+		uhc = boot_hosts[i].uhc;
+
+		/* Some consistency checks */
+		uhc_get_host_port(uhc, &host, &port);
+		g_assert(NULL != host);
+		g_assert(0 != port);
+
+		ep = is_strprefix(uhc, host);
+		g_assert(NULL != ep);
+		g_assert(':' == ep[0]);
+
+		uhc_list_append(uhc);
+	}
+
+	hash_list_shuffle(uhc_list);
+}
+
 /**
  * Pick host at random among the host array.
  *
  * @return TRUE if OK.
  */
-static gboolean
+static bool
 uhc_pick(void)
 {
-	gboolean success = FALSE;
+	bool success = FALSE;
 	char *uhc;
 
 	uhc = uhc_get_next();
 	if (NULL == uhc) {
 		if (GNET_PROPERTY(bootstrap_debug))
-			g_warning("BOOT ran out of UHCs");
+			g_warning("BOOT ran out of UHCs, switching to GHCs");
+
+		ghc_get_hosts();
+
+		if (0 == hash_list_count(uhc_list))
+			uhc_hardwired_reload();		/* For next time */
+
 		goto finish;
 	}
 
@@ -298,38 +350,52 @@ uhc_pick(void)
 	{
 		char msg[256];
 
-		gm_snprintf(msg, sizeof msg, _("Looking for UDP host cache %s"), uhc);
+		str_bprintf(ARYLEN(msg), _("Looking for UDP host cache %s"), uhc);
 		gcu_statusbar_message(msg);
 	}
+
 	success = TRUE;
 
 finish:
-	G_FREE_NULL(uhc);
+	HFREE_NULL(uhc);
 	return success;
 }
 
 /**
- * Try a random host cache.
+ * Try with next host in the (already shuffled) list.
  */
 static void
-uhc_try_random(void)
+uhc_try_next(void)
 {
+	host_addr_t addr;
+
 	g_assert(uhc_connecting);
 	g_assert(uhc_ctx.timeout_ev == NULL);
 
-	if (uhc_ctx.attempts >= UHC_MAX_ATTEMPTS || !uhc_pick()) {
+	if (!uhc_pick()) {
 		uhc_connecting = FALSE;
 		return;
 	}
-	uhc_ctx.attempts++;
 
 	/*
 	 * The following may recurse if resolution is synchronous, but
 	 * we're protected by the `attempts' counter.
 	 */
 
-	(void) adns_resolve(uhc_ctx.host, settings_dns_net(),
-				uhc_host_resolved, NULL);
+	if (string_to_host_addr(uhc_ctx.host, NULL, &addr)) {
+		uhc_ctx.addr = addr;
+
+		if (GNET_PROPERTY(bootstrap_debug) > 1)
+			g_debug("BOOT UDP pinging host cache \"%s\"", uhc_ctx.host);
+
+		uhc_send_ping();
+	} else {
+		if (GNET_PROPERTY(bootstrap_debug) > 1)
+			g_debug("BOOT UDP reolving host cache \"%s\"", uhc_ctx.host);
+
+		(void) adns_resolve(uhc_ctx.host, settings_dns_net(),
+					uhc_host_resolved, NULL);
+	}
 }
 
 /**
@@ -337,17 +403,16 @@ uhc_try_random(void)
  * get a reply within the specified timeout.
  */
 static void
-uhc_ping_timeout(cqueue_t *unused_cq, gpointer unused_obj)
+uhc_ping_timeout(cqueue_t *cq, void *unused_obj)
 {
-	(void) unused_cq;
 	(void) unused_obj;
 
 	if (GNET_PROPERTY(bootstrap_debug))
 		g_warning("no reply from UDP host cache %s:%u",
 			uhc_ctx.host, uhc_ctx.port);
 
-	uhc_ctx.timeout_ev = NULL;
-	uhc_try_random();
+	cq_zero(cq, &uhc_ctx.timeout_ev);
+	uhc_try_next();
 }
 
 /**
@@ -358,20 +423,22 @@ uhc_send_ping(void)
 {
 	g_assert(uhc_connecting);
 
-	guid_random_muid(&uhc_ctx.muid);	
+	guid_random_muid(&uhc_ctx.muid);
 
 	if (udp_send_ping(&uhc_ctx.muid, uhc_ctx.addr, uhc_ctx.port, TRUE)) {
 
-		if (GNET_PROPERTY(bootstrap_debug))
-			g_message("BOOT sent UDP SCP ping %s to %s:%u",
+		if (GNET_PROPERTY(bootstrap_debug) || GNET_PROPERTY(log_uhc_pings_tx)) {
+			g_debug("BOOT sent UDP SCP ping #%s to %s:%u",
 				guid_hex_str(&uhc_ctx.muid), uhc_ctx.host, uhc_ctx.port);
+		}
+
 		/*
 		 * Give GUI feedback.
 		 */
 		{
 			char msg[256];
 
-			gm_snprintf(msg, sizeof msg,
+			str_bprintf(ARYLEN(msg),
 				_("Sent ping to UDP host cache %s:%u"),
 				uhc_ctx.host, uhc_ctx.port);
 			gcu_statusbar_message(msg);
@@ -384,11 +451,12 @@ uhc_send_ping(void)
 
 		g_assert(uhc_ctx.timeout_ev == NULL);
 
-		uhc_ctx.timeout_ev = cq_insert(callout_queue,
-				UHC_TIMEOUT, uhc_ping_timeout, NULL);
+		uhc_ctx.timeout_ev = cq_main_insert(UHC_TIMEOUT,
+			uhc_ping_timeout, NULL);
 	} else {
-		g_message("BOOT failed to send UDP SCP to %s",
+		g_warning("BOOT failed to send UDP SCP to %s",
 			host_addr_port_to_string(uhc_ctx.addr, uhc_ctx.port));
+		uhc_try_next();
 	}
 }
 
@@ -396,7 +464,7 @@ uhc_send_ping(void)
  * Callback for adns_resolve(), invoked when the resolution is complete.
  */
 static void
-uhc_host_resolved(const host_addr_t *addrs, size_t n, gpointer uu_udata)
+uhc_host_resolved(const host_addr_t *addrs, size_t n, void *uu_udata)
 {
 	(void) uu_udata;
 	g_assert(addrs);
@@ -410,14 +478,66 @@ uhc_host_resolved(const host_addr_t *addrs, size_t n, gpointer uu_udata)
 			g_warning("could not resolve UDP host cache \"%s\"",
 				uhc_ctx.host);
 
-		uhc_try_random();
+		uhc_try_next();
 		return;
 	}
 
-	uhc_ctx.addr = addrs[random_u32() % n];
-	
-	if (GNET_PROPERTY(bootstrap_debug))
-		g_message("BOOT UDP host cache \"%s\" resolved to %s",
+	if (n > 1) {
+		size_t i;
+		host_addr_t *hav;
+		/* Current UHC was moved to tail by uhc_get_next() */
+		struct uhc *uhc = hash_list_tail(uhc_list);
+
+		/*
+		 * UHC resolved to multiple endpoints. Could be roundrobbin or
+		 * IPv4 and IPv6 addresss. Adding them as seperate entries: if the
+		 * IPv6 is unreachable we have an opportunity to skip it.
+		 * 		-- JA 24/7/2011
+		 *
+		 * Shuffle the address array before appending them to the UHC list.
+		 *		--RAM, 2015-10-01
+		 */
+
+		hav = HCOPY_ARRAY(addrs, n);
+		SHUFFLE_ARRAY_N(hav, n);
+
+		for (i = 0; i < n; i++) {
+			const char *host = host_addr_port_to_string(hav[i], uhc_ctx.port);
+			g_debug("BOOT UDP host cache \"%s\" resolved to %s (#%zu)",
+				uhc_ctx.host, host, i + 1);
+
+			uhc_list_append(host);
+		}
+
+		hash_list_remove(uhc_list, uhc);	/* Replaced by IP address list */
+		uhc_free(&uhc);
+
+		/*
+		 * We're going to continue and process the first address (in our
+		 * shuffled array).  Make sure it is put at the end of the list
+		 * and marked as being used, mimicing what uhc_get_next() would do.
+		 *		--RAM, 2015-10-01
+		 */
+
+		{
+			struct uhc key;
+
+			key.host = host_addr_port_to_string(hav[0], uhc_ctx.port);
+			uhc = hash_list_lookup(uhc_list, &key);
+			g_assert(uhc != NULL);	/* We added the entry above! */
+			uhc->stamp = tm_time();
+			uhc->used++;
+			hash_list_moveto_tail(uhc_list, uhc);
+		}
+
+		uhc_ctx.addr = hav[0];		/* Struct copy */
+		HFREE_NULL(hav);
+	} else {
+		uhc_ctx.addr = addrs[0];
+	}
+
+	if (GNET_PROPERTY(bootstrap_debug) > 1)
+		g_debug("BOOT UDP host cache \"%s\" resolved to %s",
 			uhc_ctx.host, host_addr_to_string(uhc_ctx.addr));
 
 
@@ -431,7 +551,7 @@ uhc_host_resolved(const host_addr_t *addrs, size_t n, gpointer uu_udata)
 /**
  * Check whether we're waiting for some UDP host cache pongs.
  */
-gboolean
+bool
 uhc_is_waiting(void)
 {
 	return uhc_connecting;
@@ -452,58 +572,59 @@ uhc_get_hosts(void)
 	if (uhc_connecting || GNET_PROPERTY(ancient_version))
 		return;
 
-	if (!udp_active())
+	if (!udp_active()) {
+		g_message("BOOT cannot contact UHCs (UDP inactive), using GHCs");
+		ghc_get_hosts();
 		return;
+	}
 
-	if (GNET_PROPERTY(bootstrap_debug))
-		g_message("BOOT will be contacting an UHC");
+	g_message("BOOT will be contacting an UHC");
 
 	/*
 	 * Reset context.
 	 */
 
 	uhc_connecting = TRUE;
-	uhc_ctx.attempts = 0;
 
 	g_assert(uhc_ctx.timeout_ev == NULL);
 
 	/*
-	 * Pick a random host.
+	 * Pick next host.
 	 */
 
-	uhc_try_random();
+	uhc_try_next();
 }
 
 /**
  * Called when a pong with an "IPP" extension was received.
  */
 void
-uhc_ipp_extract(gnutella_node_t *n, const char *payload, int paylen)
+uhc_ipp_extract(gnutella_node_t *n, const char *payload, int paylen,
+	enum net_type type)
 {
 	int i, cnt;
+	int len = NET_TYPE_IPV6 == type ? 18 : 6;
+	const void *p;
 
-	g_assert(0 == paylen % 6);
+	g_assert(0 == paylen % len);
 
-	cnt = paylen / 6;
+	cnt = paylen / len;
 
 	if (GNET_PROPERTY(bootstrap_debug))
-		g_message("extracting %d host%s in UDP IPP pong %s from %s (%s)",
-			cnt, cnt == 1 ? "" : "s",
-			guid_hex_str(gnutella_header_get_muid(&n->header)), node_addr(n),
-			uhc_connecting ? "expected" : "unsollicited");
+		g_debug("extracting %d host%s in UDP IPP pong #%s from %s",
+			PLURAL(cnt),
+			guid_hex_str(gnutella_header_get_muid(&n->header)), node_addr(n));
 
-	for (i = 0; i < cnt; i++) {
+	for (i = 0, p = payload; i < cnt; i++, p = const_ptr_add_offset(p, len)) {
 		host_addr_t ha;
-		guint16 port;
-
-		ha = host_addr_peek_ipv4(&payload[i * 6]);
-		port = peek_le16(&payload[i * 6 + 4]);
+		uint16 port;
 
+		host_ip_port_peek(p, type, &ha, &port);
 		hcache_add_caught(HOST_ULTRA, ha, port, "UDP-HC");
 
-		if (GNET_PROPERTY(bootstrap_debug) > 1)
-			g_message("BOOT collected %s from UDP IPP pong from %s",
-				host_addr_to_string(ha), node_addr(n));
+		if (GNET_PROPERTY(bootstrap_debug) > 2)
+			g_debug("BOOT collected %s from UDP IPP pong from %s",
+				host_addr_port_to_string(ha, port), node_addr(n));
 	}
 
 	if (!uhc_connecting)
@@ -521,8 +642,8 @@ uhc_ipp_extract(gnutella_node_t *n, const char *payload, int paylen)
 		return;
 
 	if (GNET_PROPERTY(bootstrap_debug)) {
-		g_message("BOOT UDP cache \"%s\" replied: got %d host%s from %s",
-			uhc_ctx.host, cnt, cnt == 1 ? "" : "s", node_addr(n));
+		g_debug("BOOT UDP cache \"%s\" replied: got %d host%s from %s",
+			uhc_ctx.host, PLURAL(cnt), node_addr(n));
 	}
 
 	/*
@@ -532,10 +653,10 @@ uhc_ipp_extract(gnutella_node_t *n, const char *payload, int paylen)
 	if (cnt > 0) {
 		char msg[256];
 
-		cq_cancel(callout_queue, &uhc_ctx.timeout_ev);
+		cq_cancel(&uhc_ctx.timeout_ev);
 		uhc_connecting = FALSE;
 
-		gm_snprintf(msg, sizeof(msg),
+		str_bprintf(ARYLEN(msg),
 			NG_("Got %d host from UDP host cache %s",
 				"Got %d hosts from UDP host cache %s",
 				cnt),
@@ -543,47 +664,29 @@ uhc_ipp_extract(gnutella_node_t *n, const char *payload, int paylen)
 
 		gcu_statusbar_message(msg);
 	} else {
-		uhc_try_random();
+		uhc_try_next();
 	}
 }
 
 /**
  * Initializations.
  */
-void
+void G_COLD
 uhc_init(void)
 {
-	guint i;
-
 	g_return_if_fail(NULL == uhc_list);
+
 	uhc_list = hash_list_new(uhc_hash, uhc_equal);
-
-	for (i = 0; i < G_N_ELEMENTS(boot_hosts); i++) {
-		const char *host, *ep, *uhc;
-		guint16 port;
-
-		uhc = boot_hosts[i].uhc;
-
-		/* Some consistency checks */
-		uhc_get_host_port(uhc, &host, &port);
-		g_assert(NULL != host);
-		g_assert(0 != port);
-
-		ep = is_strprefix(uhc, host);
-		g_assert(NULL != ep);
-		g_assert(':' == ep[0]);
-
-		uhc_list_add(uhc);
-	}
+	uhc_hardwired_reload();
 }
 
 /**
  * Cleanup during process termination.
  */
-void
+void G_COLD
 uhc_close(void)
 {
-	cq_cancel(callout_queue, &uhc_ctx.timeout_ev);
+	cq_cancel(&uhc_ctx.timeout_ev);
 	uhc_connecting = FALSE;
 
 	if (uhc_list) {
