@@ -1,6 +1,4 @@
 /*
- * $Id: upload_stats.c 17240 2010-02-13 12:26:50Z rmanfredi $
- *
  * Copyright (c) 2001-2003, Raphael Manfredi
  * Copyright (c) 2002, Michael Tesch
  *
@@ -20,7 +18,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -54,18 +52,19 @@
 
 #include "common.h"
 
-RCSID("$Id: upload_stats.c 17240 2010-02-13 12:26:50Z rmanfredi $")
-
 #include "upload_stats.h"
 #include "share.h"
+#include "settings.h"
 
 #include "if/bridge/c2ui.h"
 
-#include "lib/atoms.h"
 #include "lib/ascii.h"
+#include "lib/atoms.h"
 #include "lib/file.h"
 #include "lib/halloc.h"
+#include "lib/hashing.h"
 #include "lib/hashlist.h"
+#include "lib/hikset.h"
 #include "lib/parse.h"
 #include "lib/stringify.h"
 #include "lib/timestamp.h"
@@ -77,13 +76,15 @@ RCSID("$Id: upload_stats.c 17240 2010-02-13 12:26:50Z rmanfredi $")
 
 #include "lib/override.h"		/* Must be the last header included */
 
-static gboolean dirty = FALSE;
-static char *stats_file;
-static hash_list_t *upload_stats_list;
-static GHashTable *upload_stats_by_sha1;
+static const char ul_stats_file[] = "upload_stats";
+static const char ul_stats_what[] = "upload statistics";
 
-static gboolean 
-ul_stats_eq(gconstpointer p, gconstpointer q)
+static bool dirty = FALSE;
+static hash_list_t *upload_stats_list;
+static hikset_t *upload_stats_by_sha1;
+
+static bool
+ul_stats_eq(const void *p, const void *q)
 {
 	const struct ul_stats *a = p, *b = q;
 
@@ -91,12 +92,13 @@ ul_stats_eq(gconstpointer p, gconstpointer q)
 	return a->pathname == b->pathname && a->size == b->size;
 }
 
-static guint
-ul_stats_hash(gconstpointer p)
+static uint
+ul_stats_hash(const void *p)
 {
 	const struct ul_stats *s = p;
-  
-	return g_str_hash(s->pathname) ^ s->size;
+
+	return hashing_mix32(
+		string_mix_hash(s->pathname) ^ integer_hash_fast(s->size));
 }
 
 /**
@@ -108,21 +110,21 @@ ul_stats_hash(gconstpointer p)
  * and recorded as such.
  */
 static struct ul_stats *
-upload_stats_find(const struct sha1 *sha1, const char *pathname, guint64 size)
+upload_stats_find(const struct sha1 *sha1, const char *pathname, uint64 size)
 {
 	struct ul_stats *s = NULL;
 
 	if (upload_stats_list) {
 		static const struct ul_stats zero_stats;
 		struct ul_stats key;
-		gconstpointer orig_key;
+		const void *orig_key;
 
 		g_assert(upload_stats_by_sha1);
 
 		if (sha1) {
-			s = g_hash_table_lookup(upload_stats_by_sha1, sha1);
+			s = hikset_lookup(upload_stats_by_sha1, sha1);
 			if (s)
-				return s;		/* Found it by SHA1 */
+				goto done;		/* Found it by SHA1 */
 		}
 
 		key = zero_stats;
@@ -130,7 +132,7 @@ upload_stats_find(const struct sha1 *sha1, const char *pathname, guint64 size)
 		key.size = size;
 
 		if (hash_list_find(upload_stats_list, &key, &orig_key))
-			s = deconstify_gpointer(orig_key);
+			s = deconstify_pointer(orig_key);
 		atom_str_free_null(&key.pathname);
 
 		if (s && sha1) {
@@ -142,34 +144,44 @@ upload_stats_find(const struct sha1 *sha1, const char *pathname, guint64 size)
 			} else {
 				/* SHA1 changed, file was modified */
 				struct ul_stats *old =
-					g_hash_table_lookup(upload_stats_by_sha1, s->sha1);
+					hikset_lookup(upload_stats_by_sha1, s->sha1);
 
 				g_assert(old == s);		/* Must be the same filename entry */
 
-				g_hash_table_remove(upload_stats_by_sha1, s->sha1);
+				hikset_remove(upload_stats_by_sha1, s->sha1);
 				atom_sha1_free(s->sha1);
 				s->sha1 = atom_sha1_get(sha1);
 			}
-			gm_hash_table_insert_const(upload_stats_by_sha1, s->sha1, s);
+			hikset_insert_key(upload_stats_by_sha1, &s->sha1);
 		}
 	}
 
+done:
+
 	/* We guarantee the SHA1 is present in the record if known */
 	g_assert(!(s && sha1) || s->sha1);
 
+	/* Postcondition: if we return something, it must be "correct" */
+	if (s != NULL) {
+		g_assert(atom_is_str(s->pathname));
+		g_assert(atom_is_str(s->filename));
+		g_assert(s->norm >= 0.0);
+	}
+
 	return s;
 }
 
 static void
 upload_stats_add(const char *pathname, filesize_t size, const char *name,
-	guint32 attempts, guint32 complete, guint64 ul_bytes,
+	uint32 attempts, uint32 complete, uint64 ul_bytes,
 	time_t rtime, time_t dtime, const struct sha1 *sha1)
 {
-	static const struct ul_stats zero_stats;
 	struct ul_stats *s;
 
-	s = walloc(sizeof *s);
-	*s = zero_stats;
+	g_assert(pathname != NULL);
+	g_assert(name != NULL);
+
+	WALLOC0(s);
 	s->pathname = atom_str_get(pathname);
 	s->filename = atom_str_get(name);
 	s->size = size;
@@ -184,27 +196,29 @@ upload_stats_add(const char *pathname, filesize_t size, const char *name,
 	if (!upload_stats_list) {
 		g_assert(!upload_stats_by_sha1);
 		upload_stats_list = hash_list_new(ul_stats_hash, ul_stats_eq);
-		upload_stats_by_sha1 = g_hash_table_new(sha1_hash, sha1_eq);
+		upload_stats_by_sha1 = hikset_create(
+			offsetof(struct ul_stats, sha1), HASH_KEY_FIXED, SHA1_RAW_SIZE);
 	}
 	hash_list_append(upload_stats_list, s);
 	if (s->sha1)
-		gm_hash_table_insert_const(upload_stats_by_sha1, s->sha1, s);
+		hikset_insert_key(upload_stats_by_sha1, &s->sha1);
 	gcu_upload_stats_gui_add(s);
 }
 
-void
-upload_stats_load_history(const char *ul_history_file_name)
+void G_COLD
+upload_stats_load_history(void)
 {
 	FILE *upload_stats_file;
+	file_path_t fp;
 	char line[FILENAME_MAX + 64];
-	guint lineno = 0;
+	uint lineno = 0;
 
 	gcu_upload_stats_gui_freeze();
-	
-	stats_file = g_strdup(ul_history_file_name);
+
+	file_path_set(&fp, settings_config_dir(), ul_stats_file);
 
 	/* open file for reading */
-	upload_stats_file = file_fopen_missing(ul_history_file_name, "r");
+	upload_stats_file = file_config_open_read(ul_stats_what, &fp, 1);
 	if (upload_stats_file == NULL)
 		goto done;
 
@@ -217,10 +231,16 @@ upload_stats_load_history(const char *ul_history_file_name)
 		size_t i;
 
 		lineno++;
-		if (line[0] == '#' || line[0] == '\n')
+
+		if (!file_line_chomp_tail(ARYLEN(line), NULL)) {
+			g_warning("%s: line %u too long, aborting", G_STRFUNC, lineno);
+			break;
+		}
+
+		if (file_line_is_skipable(line))
 			continue;
 
-		p = strchr(line, '\t');
+		p = vstrchr(line, '\t');
 		if (NULL == p)
 			goto corrupted;
 
@@ -235,7 +255,7 @@ upload_stats_load_history(const char *ul_history_file_name)
 		item.pathname = line;
 
 		for (i = 0; i < 8; i++) {
-			guint64 v;
+			uint64 v;
 			int error;
 			const char *endptr;
 
@@ -250,7 +270,7 @@ upload_stats_load_history(const char *ul_history_file_name)
 				/* We have a SHA1 or '*' if none known */
 				if ('*' != *p) {
 					size_t len = clamp_strlen(p, SHA1_BASE32_SIZE);
-					
+
 					error = !parse_base32_sha1(p, len, &sha1_buf);
 					item.sha1 = error ? NULL : &sha1_buf;
 				} else {
@@ -261,20 +281,22 @@ upload_stats_load_history(const char *ul_history_file_name)
 				break;
 			default:
 				v = parse_uint64(p, &endptr, 10, &error);
-				p = deconstify_gchar(endptr);
+				p = deconstify_char(endptr);
 			}
 
 			if (error || !is_ascii_space(*endptr))
 				goto corrupted;
 
+#define TIME(x)		MIN((x) + (time_t) 0, TIME_T_MAX + (uint64) 0)
+
 			switch (i) {
 			case 0: item.size = v; break;
 			case 1: item.attempts = v; break;
 			case 2: item.complete = v; break;
-			case 3: item.bytes_sent |= ((guint64) (guint32) v) << 32; break;
-			case 4: item.bytes_sent |= (guint32) v; break;
-			case 5: item.rtime = MIN(v + (time_t) 0, TIME_T_MAX + (guint64) 0);
-			case 6: item.dtime = MIN(v + (time_t) 0, TIME_T_MAX + (guint64) 0); 
+			case 3: item.bytes_sent |= ((uint64) (uint32) v) << 32; break;
+			case 4: item.bytes_sent |= (uint32) v; break;
+			case 5: item.rtime = TIME(v); break;
+			case 6: item.dtime = TIME(v); break;
 			case 7: break;	/* Already stored above */
 			default:
 				g_assert_not_reached();
@@ -282,7 +304,9 @@ upload_stats_load_history(const char *ul_history_file_name)
 			}
 		}
 
-		/* 
+#undef TIME
+
+		/*
 		 * We store the filenames UTF-8 encoded but the file might have been
 		 * edited or corrupted.
 		 */
@@ -295,11 +319,11 @@ upload_stats_load_history(const char *ul_history_file_name)
 		}
 
 		if (upload_stats_find(NULL, item.pathname, item.size)) {
-			g_warning("upload_stats_load_history():"
-				" Ignoring line %u due to duplicate file.", lineno);
+			g_warning("%s(): ignoring line %u due to duplicate file.",
+				G_STRFUNC, lineno);
 		} else if (upload_stats_find(item.sha1, item.pathname, item.size)) {
-			g_warning("upload_stats_load_history():"
-				" Ignoring line %u due to duplicate file.", lineno);
+			g_warning("%s(): ignoring line %u due to duplicate file.",
+				G_STRFUNC, lineno);
 		} else {
 			upload_stats_add(item.pathname, item.size, item.filename,
 				item.attempts, item.complete, item.bytes_sent,
@@ -320,9 +344,9 @@ done:
 }
 
 static void
-upload_stats_dump_item(gpointer p, gpointer user_data)
+upload_stats_dump_item(void *p, void *user_data)
 {
-	const struct shared_file *sf;
+	shared_file_t *sf;
 	FILE *out = user_data;
 	struct ul_stats *s = p;
 	char rtime_buf[TIME_T_DEC_BUFLEN];
@@ -341,8 +365,8 @@ upload_stats_dump_item(gpointer p, gpointer user_data)
 	}
 	escaped = url_escape_cntrl(pathname);
 
-	time_t_to_string_buf(s->rtime, rtime_buf, sizeof rtime_buf);
-	time_t_to_string_buf(s->dtime, dtime_buf, sizeof dtime_buf);
+	time_t_to_string_buf(s->rtime, ARYLEN(rtime_buf));
+	time_t_to_string_buf(s->dtime, ARYLEN(dtime_buf));
 
 	fprintf(out, "%s\t%s\t%u\t%u\t%lu\t%lu\t%s\t%s\t%s\n",
 		escaped,
@@ -358,31 +382,28 @@ upload_stats_dump_item(gpointer p, gpointer user_data)
 	if (escaped != pathname) {		/* File had escaped chars */
 		HFREE_NULL(escaped);
 	}
+	shared_file_unref(&sf);
 }
 
 /**
  * Save upload statistics to file.
  */
 static void
-upload_stats_dump_history(const char *ul_history_file_name)
+upload_stats_dump_history(void)
 {
 	FILE *out;
-	time_t now = tm_time();
-
-	g_return_if_fail(ul_history_file_name);
+	file_path_t fp;
 
 	/* open file for writing */
-	out = file_fopen(ul_history_file_name, "w");
-	if (NULL == out) {
-		return;
-	}
+	file_path_set(&fp, settings_config_dir(), ul_stats_file);
+	out = file_config_open_write(ul_stats_what, &fp);
 
-	fprintf(out,
-		"# THIS FILE IS AUTOMATICALLY GENERATED -- DO NOT EDIT\n"
-		"#\n"
-		"# Upload statistics saved on %s\n"
-		"#\n"
-		"\n"
+	if (NULL == out)
+		return;
+
+	file_config_preamble(out, "Upload statistics");
+
+	fputs(
 		"#\n"
 		"# Format is:\n"
 		"#    File basename <TAB> size <TAB> attempts <TAB> completed\n"
@@ -391,7 +412,8 @@ upload_stats_dump_history(const char *ul_history_file_name)
 		"#        <TAB> SHA1 (\"*\" if unknown)\n"
 		"#\n"
 		"\n",
-		timestamp_to_string(now));
+		out
+	);
 
 	/*
 	 * Don't check this sooner so that the file is cleared, if the user
@@ -402,8 +424,8 @@ upload_stats_dump_history(const char *ul_history_file_name)
 		hash_list_foreach(upload_stats_list, upload_stats_dump_item, out);
 	}
 
-	/* close file */
-	fclose(out);
+	file_config_close(out, &fp);
+	dirty = FALSE;
 }
 
 /**
@@ -416,12 +438,7 @@ upload_stats_flush_if_dirty(void)
 	if (!dirty)
 		return;
 
-	dirty = FALSE;
-
-	if (NULL != stats_file)
-		upload_stats_dump_history(stats_file);
-	else
-		g_warning("can't save upload statistics: no file name recorded");
+	upload_stats_dump_history();
 }
 
 /**
@@ -431,7 +448,7 @@ upload_stats_flush_if_dirty(void)
  * renamed and shared.
  */
 void
-upload_stats_enforce_local_filename(const struct shared_file *sf)
+upload_stats_enforce_local_filename(const shared_file_t *sf)
 {
 	struct ul_stats *s;
 	const struct sha1 *sha1;
@@ -445,7 +462,7 @@ upload_stats_enforce_local_filename(const struct shared_file *sf)
 	if (!sha1)
 		return;		/* File's SHA1 not known yet, nothing to do here */
 
-	s = g_hash_table_lookup(upload_stats_by_sha1, sha1);
+	s = hikset_lookup(upload_stats_by_sha1, sha1);
 
 	if (NULL == s)
 		return;							/* SHA1 not in stats, nothing to do */
@@ -470,7 +487,7 @@ upload_stats_enforce_local_filename(const struct shared_file *sf)
  * Called when an upload starts.
  */
 void
-upload_stats_file_begin(const struct shared_file *sf)
+upload_stats_file_begin(const shared_file_t *sf)
 {
 	struct ul_stats *s;
 	const char *pathname;
@@ -509,8 +526,8 @@ upload_stats_file_begin(const struct shared_file *sf)
  */
 static void
 upload_stats_file_add(
-	const struct shared_file *sf,
-	int comp, guint64 sent, gboolean update_dtime)
+	const shared_file_t *sf,
+	int comp, uint64 sent, bool update_dtime)
 {
 	const char *pathname = shared_file_path(sf);
 	filesize_t size = shared_file_size(sf);
@@ -547,7 +564,7 @@ upload_stats_file_add(
  * Called when an upload is aborted, to update the amount of bytes transferred.
  */
 void
-upload_stats_file_aborted(const struct shared_file *sf, filesize_t done)
+upload_stats_file_aborted(const shared_file_t *sf, filesize_t done)
 {
 	g_return_if_fail(sf);
 
@@ -559,7 +576,7 @@ upload_stats_file_aborted(const struct shared_file *sf, filesize_t done)
  * Called when an upload completes.
  */
 void
-upload_stats_file_complete(const struct shared_file *sf, filesize_t done)
+upload_stats_file_complete(const shared_file_t *sf, filesize_t done)
 {
 	g_return_if_fail(sf);
 
@@ -570,7 +587,7 @@ upload_stats_file_complete(const struct shared_file *sf, filesize_t done)
  * Called when an upload request is made.
  */
 void
-upload_stats_file_requested(const struct shared_file *sf)
+upload_stats_file_requested(const shared_file_t *sf)
 {
 	g_return_if_fail(sf);
 
@@ -582,13 +599,13 @@ upload_stats_prune_nonexistent(void)
 {
 	/* XXX */
 	/* for each row, get the filename, check if filename is ? */
-	g_warning("upload_stats_prune_nonexistent: not implemented!");
+	g_carp("%s(): not implemented!", G_STRFUNC);
 }
 
 /**
  * Clear all the upload stats data structure.
  */
-static void
+static void G_COLD
 upload_stats_free_all(void)
 {
 	if (upload_stats_list) {
@@ -599,13 +616,12 @@ upload_stats_free_all(void)
 			atom_str_free_null(&s->pathname);
 			atom_str_free_null(&s->filename);
 			if (s->sha1)
-				g_hash_table_remove(upload_stats_by_sha1, s->sha1);
+				hikset_remove(upload_stats_by_sha1, s->sha1);
 			atom_sha1_free_null(&s->sha1);
-			wfree(s, sizeof *s);
+			WFREE(s);
 		}
 		hash_list_free(&upload_stats_list);
-		g_hash_table_destroy(upload_stats_by_sha1);
-		upload_stats_by_sha1 = NULL;
+		hikset_free_null(&upload_stats_by_sha1);
 	}
 	dirty = TRUE;
 }
@@ -618,9 +634,7 @@ upload_stats_clear_all(void)
 {
 	gcu_upload_stats_gui_clear_all();
 	upload_stats_free_all();
-	if (stats_file) {
-		upload_stats_dump_history(stats_file);
-	}
+	upload_stats_dump_history();
 }
 
 /**
@@ -629,9 +643,8 @@ upload_stats_clear_all(void)
 void
 upload_stats_close(void)
 {
-	upload_stats_dump_history(stats_file);
+	upload_stats_dump_history();
 	upload_stats_free_all();
-	G_FREE_NULL(stats_file);
 }
 
 /* vi: set ts=4 sw=4 cindent: */
