@@ -1,6 +1,4 @@
 /*
- * $Id: kmsg.h 17044 2009-10-09 20:55:36Z rmanfredi $
- *
  * Copyright (c) 2008, Raphael Manfredi
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -57,19 +55,22 @@
 
 void kmsg_send_ping(knode_t *kn, const guid_t *muid);
 void kmsg_send_find_node(knode_t *kn, const kuid_t *id, const guid_t *muid,
-	pmsg_free_t mfree, gpointer marg);
+	pmsg_free_t mfree, void *marg);
 void kmsg_send_find_value(knode_t *kn, const kuid_t *id, dht_value_type_t type,
 	kuid_t **skeys, int scnt,
-	const guid_t *muid, pmsg_free_t mfree, gpointer marg);
-GSList *kmsg_build_store(
+	const guid_t *muid, pmsg_free_t mfree, void *marg);
+struct pslist *kmsg_build_store(
 	const void *token, size_t toklen, dht_value_t **vvec, int vcnt);
-	
+
 void kmsg_send_mb(knode_t *kn, pmsg_t *mb);
 
 void kmsg_serialize_contact(pmsg_t *mb, const knode_t *kn);
 knode_t *kmsg_deserialize_contact(bstr_t *bs);
 dht_value_t *kmsg_deserialize_dht_value(bstr_t *bs);
 
+void kmsg_init(void);
+void kmsg_close(void);
+
 #endif	/* _dht_kmsg_h_ */
 
 /* vi: set ts=4 sw=4 cindent: */
