@@ -1,6 +1,4 @@
 /*
- * $Id: knode.c 17363 2010-03-06 11:13:03Z rmanfredi $
- *
  * Copyright (c) 2006-2008, Raphael Manfredi
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -35,10 +33,11 @@
 
 #include "common.h"
 
-RCSID("$Id: knode.c 17363 2010-03-06 11:13:03Z rmanfredi $")
+#include <math.h>		/* For pow() */
 
 #include "knode.h"
 #include "kuid.h"
+#include "stable.h"		/* For stable_still_alive_probability() */
 
 #include "if/gnet_property_priv.h"
 #include "if/dht/kademlia.h"
@@ -47,30 +46,31 @@ RCSID("$Id: knode.c 17363 2010-03-06 11:13:03Z rmanfredi $")
 #include "core/hostiles.h"
 
 #include "lib/atoms.h"
-#include "lib/glib-missing.h"
+#include "lib/str.h"
 #include "lib/stringify.h"
 #include "lib/vendors.h"
 #include "lib/walloc.h"
+
 #include "lib/override.h"		/* Must be the last header included */
 
 /**
  * Hashing of knodes,
  */
 unsigned int
-knode_hash(gconstpointer key)
+knode_hash(const void *key)
 {
 	const knode_t *kn = key;
 
 	knode_check(kn);
 
-	return sha1_hash(kn->id);
+	return kuid_hash(kn->id);
 }
 
 /**
  * Equality of knodes.
  */
 int
-knode_eq(gconstpointer a, gconstpointer b)
+knode_eq(const void *a, const void *b)
 {
 	const knode_t *k1 = a;
 	const knode_t *k2 = b;
@@ -85,7 +85,7 @@ knode_eq(gconstpointer a, gconstpointer b)
  * Comparison of two knodes based on the last_seen time.
  */
 int
-knode_seen_cmp(gconstpointer a, gconstpointer b)
+knode_seen_cmp(const void *a, const void *b)
 {
 	const knode_t *k1 = a;
 	const knode_t *k2 = b;
@@ -96,6 +96,44 @@ knode_seen_cmp(gconstpointer a, gconstpointer b)
 	return CMP(k1->last_seen, k2->last_seen);
 }
 
+/**
+ * Comparison of two knodes based on their probability of being dead.
+ */
+int
+knode_dead_probability_cmp(const void *a, const void *b)
+{
+	const knode_t *k1 = a;
+	const knode_t *k2 = b;
+	double p1, p2;
+	double e;
+
+	p1 = knode_still_alive_probability(k1);
+	p2 = knode_still_alive_probability(k2);
+
+	/* Higher alive chances => lower dead probability */
+
+	e = p2 - p1;
+	if (e < 0.0)
+		e = -e;
+
+	if (e < 1e-15) {
+		time_delta_t d;
+
+		/*
+		 * Probabilities of presence are comparable.
+		 * The more ancient node is more likely to be alive.
+		 * Otherwise, the one we heard from last is more likely to be alive.
+		 */
+
+		d = delta_time(k1->first_seen, k2->first_seen);
+		if G_UNLIKELY(0 == d)
+			d = delta_time(k2->last_seen, k1->last_seen);	/* Inversed! */
+		return CMP(d, 0);
+	} else {
+		return CMP(p2, p1);
+	}
+}
+
 /**
  * Allocate new Kademlia node.
  *
@@ -112,13 +150,13 @@ knode_seen_cmp(gconstpointer a, gconstpointer b)
  */
 knode_t *
 knode_new(
-	const kuid_t *id, guint8 flags,
-	host_addr_t addr, guint16 port, vendor_code_t vcode,
-	guint8 major, guint8 minor)
+	const kuid_t *id, uint8 flags,
+	host_addr_t addr, uint16 port, vendor_code_t vcode,
+	uint8 major, uint8 minor)
 {
 	knode_t *kn;
 
-	kn = walloc0(sizeof *kn);
+	WALLOC0(kn);
 	kn->magic = KNODE_MAGIC;
 	kn->id = kuid_get_atom(id);
 	kn->vcode = vcode;
@@ -128,6 +166,7 @@ knode_new(
 	kn->major = major;
 	kn->minor = minor;
 	kn->status = KNODE_UNKNOWN;
+	kn->first_seen = tm_time();
 
 	if (flags & KDA_MSG_F_FIREWALLED)
 		kn->flags |= KNODE_F_FIREWALLED;
@@ -138,11 +177,31 @@ knode_new(
 	return kn;
 }
 
+/**
+ * Clone a Kademlia node.
+ *
+ * @return new node with a reference count of 1, in the "unknown" status.
+ */
+knode_t *
+knode_clone(const knode_t *kn)
+{
+	knode_t *cn;
+
+	WALLOC(cn);
+	*cn = *kn;						/* Struct copy */
+	cn->status = KNODE_UNKNOWN;		/* This instance is not in routing table */
+	cn->refcnt = 1;					/* New instance */
+	cn->id = kuid_get_atom(kn->id);	/* Increase reference count */
+	cn->rpc_pending = 0;
+
+	return cn;
+}
+
 /**
  * Can the node which timed-out in the past be considered again as the
  * target of an RPC, and therefore returned in k-closest lookups?
  */
-gboolean
+bool
 knode_can_recontact(const knode_t *kn)
 {
 	time_t grace;
@@ -153,12 +212,7 @@ knode_can_recontact(const knode_t *kn)
 	if (!kn->rpc_timeouts)
 		return TRUE;				/* Timeout condition was cleared */
 
-	/*
-	 * The grace period we want is 4 seconds times 2^timeouts, so it
-	 * ends up being 2^(timeouts + 2).
-	 */
-
-	grace = 1 << (kn->rpc_timeouts + 2);
+	grace = 1 << kn->rpc_timeouts;
 	elapsed = delta_time(tm_time(), kn->last_sent);
 
 	return elapsed > grace;
@@ -167,7 +221,7 @@ knode_can_recontact(const knode_t *kn)
 /**
  * Give a string representation of the node status.
  */
-const gchar *
+const char *
 knode_status_to_string(knode_status_t status)
 {
 	switch (status) {
@@ -196,8 +250,8 @@ knode_change_vendor(knode_t *kn, vendor_code_t vcode)
 		char vc_old[VENDOR_CODE_BUFLEN];
 		char vc_new[VENDOR_CODE_BUFLEN];
 
-		vendor_code_to_string_buf(kn->vcode.u32, vc_old, sizeof vc_old);
-		vendor_code_to_string_buf(vcode.u32, vc_new, sizeof vc_new);
+		vendor_code_to_string_buf(kn->vcode.u32, ARYLEN(vc_old));
+		vendor_code_to_string_buf(vcode.u32, ARYLEN(vc_new));
 
 		g_warning("DHT node %s at %s changed vendor from %s to %s",
 			kuid_to_hex_string(kn->id),
@@ -212,7 +266,7 @@ knode_change_vendor(knode_t *kn, vendor_code_t vcode)
  * Change node's version
  */
 void
-knode_change_version(knode_t *kn, guint8 major, guint8 minor)
+knode_change_version(knode_t *kn, uint8 major, uint8 minor)
 {
 	knode_check(kn);
 
@@ -229,7 +283,7 @@ knode_change_version(knode_t *kn, guint8 major, guint8 minor)
 /**
  * @return whether host can be kept as a valid contact
  */
-gboolean
+bool
 knode_is_usable(const knode_t *kn)
 {
 	knode_check(kn);
@@ -237,7 +291,7 @@ knode_is_usable(const knode_t *kn)
 	if (!host_is_valid(kn->addr, kn->port))
 		return FALSE;
 
-	if (hostiles_check(kn->addr))
+	if (hostiles_is_bad(kn->addr))
 		return FALSE;
 
 	return TRUE;
@@ -246,7 +300,7 @@ knode_is_usable(const knode_t *kn)
 /**
  * @return whether host's address is a valid DHT value creator.
  */
-gboolean
+bool
 knode_addr_is_usable(const knode_t *kn)
 {
 	knode_check(kn);
@@ -254,7 +308,7 @@ knode_addr_is_usable(const knode_t *kn)
 	if (!host_address_is_usable(kn->addr))
 		return FALSE;
 
-	if (hostiles_check(kn->addr))
+	if (hostiles_is_bad(kn->addr))
 		return FALSE;
 
 	return TRUE;
@@ -276,20 +330,20 @@ knode_addr_is_usable(const knode_t *kn)
  *
  * @return the buffer where printing was done.
  */
-const gchar *
+const char *
 knode_to_string_buf(const knode_t *kn, char buf[], size_t len)
 {
 	char host_buf[HOST_ADDR_PORT_BUFLEN];
 	char vc_buf[VENDOR_CODE_BUFLEN];
-	gchar kuid_buf[KUID_HEX_BUFLEN];
+	char kuid_buf[KUID_HEX_BUFLEN];
 
 	knode_check(kn);
 
-	bin_to_hex_buf(kn->id, KUID_RAW_SIZE, kuid_buf, sizeof kuid_buf);
-	host_addr_port_to_string_buf(kn->addr, kn->port, host_buf, sizeof host_buf);
-	vendor_code_to_string_buf(kn->vcode.u32, vc_buf, sizeof vc_buf);
-	gm_snprintf(buf, len,
-		"%s%s%s (%s v%u.%u) [%s] \"%s\", ref=%d%s%s%s",
+	bin_to_hex_buf(kn->id, KUID_RAW_SIZE, ARYLEN(kuid_buf));
+	host_addr_port_to_string_buf(kn->addr, kn->port, ARYLEN(host_buf));
+	vendor_code_to_string_buf(kn->vcode.u32, ARYLEN(vc_buf));
+	str_bprintf(buf, len,
+		"%s%s%s (%s v%u.%u) [%s] \"%s\", ref=%d%s%s%s%s [%s]",
 		host_buf,
 		(kn->flags & KNODE_F_PCONTACT) ? "*" : "",
 		(kn->flags & KNODE_F_FOREIGN_IP) ? "?" : "",
@@ -298,7 +352,9 @@ knode_to_string_buf(const knode_t *kn, char buf[], size_t len)
 		(kn->status != KNODE_UNKNOWN && !(kn->flags & KNODE_F_ALIVE)) ?
 			" zombie" : "",
 		(kn->flags & KNODE_F_CACHED) ? " cached" : "",
-		(kn->flags & KNODE_F_FIREWALLED) ? " fw" : "");
+		(kn->flags & KNODE_F_RPC) ? " RPC" : "",
+		(kn->flags & KNODE_F_FIREWALLED) ? " fw" : "",
+		compact_time(delta_time(tm_time(), kn->first_seen)));
 
 	return buf;
 }
@@ -307,24 +363,24 @@ knode_to_string_buf(const knode_t *kn, char buf[], size_t len)
  * Pretty-printing of node information for logs.
  * @return pointer to static data
  */
-const gchar *
+const char *
 knode_to_string(const knode_t *kn)
 {
 	static char buf[120];
 
-	return knode_to_string_buf(kn, buf, sizeof buf);
+	return knode_to_string_buf(kn, ARYLEN(buf));
 }
 
 /**
  * Second version of knode_to_string() when two different nodes need to be
  * pretty-printed in the same statement.
  */
-const gchar *
+const char *
 knode_to_string2(const knode_t *kn)
 {
 	static char buf[120];
 
-	return knode_to_string_buf(kn, buf, sizeof buf);
+	return knode_to_string_buf(kn, ARYLEN(buf));
 }
 
 /**
@@ -352,7 +408,7 @@ knode_dispose(knode_t *kn)
 
 	kuid_atom_free_null(&kn->id);
 	kn->magic = 0;
-	wfree(kn, sizeof *kn);
+	WFREE(kn);
 }
 
 /**
@@ -375,14 +431,16 @@ knode_free(knode_t *kn)
  * PATRICIA iterator callback to free Kademlia nodes
  */
 void
-knode_patricia_free(gpointer key, size_t u_kbits, gpointer value, gpointer u_d)
+knode_patricia_free(void *key, size_t u_kbits, void *value, void *u_d)
 {
 	knode_t *kn = value;
 
 	(void) u_kbits;
 	(void) u_d;
 
+	knode_check(kn);
 	g_assert(key == kn->id);
+
 	knode_free(kn);
 }
 
@@ -390,7 +448,7 @@ knode_patricia_free(gpointer key, size_t u_kbits, gpointer value, gpointer u_d)
  * Map iterator callback to free Kademlia nodes
  */
 void
-knode_map_free(gpointer key, gpointer value, gpointer unused_u)
+knode_map_free(void *key, void *value, void *unused_u)
 {
 	knode_t *kn = value;
 
@@ -400,4 +458,47 @@ knode_map_free(gpointer key, gpointer value, gpointer unused_u)
 	knode_free(kn);
 }
 
+#define KNODE_ALIVE_DECIMATION	0.85
+
+/**
+ * Convenience routine to compute theoretical probability of presence for
+ * a node, adjusted down when RPC timeouts occurred recently.
+ */
+double
+knode_still_alive_probability(const knode_t *kn)
+{
+	double p;
+	static bool inited;
+	static double decimation[KNODE_MAX_TIMEOUTS];
+
+	knode_check(kn);
+
+	if (G_UNLIKELY(!inited)) {
+		size_t i;
+
+		for (i = 0; i < N_ITEMS(decimation); i++) {
+			decimation[i] = pow(KNODE_ALIVE_DECIMATION, (double) (i + 1));
+		}
+
+		inited = TRUE;
+	}
+
+	p = stable_still_alive_probability(kn->first_seen, kn->last_seen);
+
+	/*
+	 * If RPC timeouts occurred, the theoretical probability is further
+	 * adjusted down.  The decimation is arbitrary of course, but the
+	 * rationale is that an RPC timeout somehow is an information that the
+	 * node may not be alive.  Of course, it could be an UDP drop, an IP
+	 * drop somewhere, but this is why we don't use 0.0 as the decimation!
+	 */
+
+	if (0 == kn->rpc_timeouts)
+		return p;
+	else {
+		size_t i = MIN(kn->rpc_timeouts, N_ITEMS(decimation)) - 1;
+		return p * decimation[i];
+	}
+}
+
 /* vi: set ts=4 sw=4 cindent: */
