@@ -1,6 +1,4 @@
 /*
- * $Id: lookup.h 17157 2009-11-01 22:41:55Z rmanfredi $
- *
  * Copyright (c) 2008, Raphael Manfredi
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -42,14 +40,14 @@
 
 #include "knode.h"
 #include "values.h"
-#include "token.h"
+#include "lib/sectoken.h"
 
 /**
  * A security token, as gathered during node lookups.
  */
 typedef struct lookup_token {
 	time_t retrieved;			/**< Retrieval time */
-	sec_token_t *token;			/**< The security token */
+	sectoken_remote_t *token;	/**< Their security token */
 } lookup_token_t;
 
 struct nlookup;
@@ -86,7 +84,7 @@ struct lookup_stats {
  * @param arg		additional callback opaque argument
  */
 typedef void (*lookup_cb_stats_t)(
-	const kuid_t *kuid, const struct lookup_stats *ls, gpointer arg);
+	const kuid_t *kuid, const struct lookup_stats *ls, void *arg);
 
 /**
  * Node lookup result record.
@@ -94,7 +92,7 @@ typedef void (*lookup_cb_stats_t)(
 typedef struct lookup_rc {
 	knode_t *kn;				/**< A Kademlia node */
 	void *token;				/**< The security token (NULL if none) */
-	guint8 token_len;			/**< Length of security token */
+	uint8 token_len;			/**< Length of security token */
 } lookup_rc_t;
 
 typedef enum {
@@ -128,21 +126,21 @@ lookup_result_check(const lookup_rs_t *rs)
  */
 
 void lookup_init(void);
-void lookup_close(gboolean exiting);
+void lookup_close(bool exiting);
 
-nlookup_t *lookup_bucket_refresh(const kuid_t *kuid,
-	lookup_cb_err_t done, gpointer arg);
+nlookup_t *lookup_bucket_refresh(const kuid_t *kuid, size_t bits,
+	lookup_cb_err_t done, void *arg);
 nlookup_t *lookup_find_value(const kuid_t *kuid, dht_value_type_t type,
-	lookup_cbv_ok_t ok, lookup_cb_err_t error, gpointer arg);
+	lookup_cbv_ok_t ok, lookup_cb_err_t error, void *arg);
 nlookup_t *lookup_find_node(const kuid_t *kuid,
-	lookup_cb_ok_t ok, lookup_cb_err_t error, gpointer arg);
+	lookup_cb_ok_t ok, lookup_cb_err_t error, void *arg);
 nlookup_t *lookup_store_nodes(const kuid_t *kuid,
-	lookup_cb_ok_t ok, lookup_cb_err_t error, gpointer arg);
+	lookup_cb_ok_t ok, lookup_cb_err_t error, void *arg);
 nlookup_t *lookup_token(const knode_t *kn,
-	lookup_cb_ok_t ok, lookup_cb_err_t error, gpointer arg);
+	lookup_cb_ok_t ok, lookup_cb_err_t error, void *arg);
 
 void lookup_ctrl_stats(nlookup_t *nl, lookup_cb_stats_t stats);
-void lookup_cancel(nlookup_t *nl, gboolean callback);
+void lookup_cancel(nlookup_t *nl, bool callback);
 
 #endif	/* _dht_lookup_h_ */
 
