@@ -1,6 +1,4 @@
 /*
- * $Id: ui2c.c 17077 2009-10-16 20:51:38Z rmanfredi $
- *
  * Copyright (c) 2004, Emile Roberts
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -45,14 +43,15 @@
 
 /* includes ui needs to access core */
 #include "lib/adns.h"
+#include "lib/glib-missing.h"
 #include "lib/iso3166.h"
-#include "if/core/bitzi.h"
+#include "lib/pslist.h"
+
 #include "if/core/downloads.h"
 #include "if/core/fileinfo.h"
 #include "if/core/net_stats.h"
 #include "if/core/hcache.h"
 #include "if/core/hsep.h"
-#include "if/core/http.h"
 #include "if/core/parq.h"
 #include "if/core/search.h"
 #include "if/core/share.h"
@@ -69,33 +68,13 @@
  */
 
 /*	adns interface functions (UI -> Core)*/
-gboolean
+bool
 guc_adns_resolve(const char *hostname,
-	adns_callback_t user_callback, gpointer user_data)
+	adns_callback_t user_callback, void *user_data)
 {
 	return adns_resolve(hostname, settings_dns_net(), user_callback, user_data);
 }
 
-/*	bitzi interface functions (UI -> Core)*/
-
-void
-guc_query_bitzi_by_sha1(const struct sha1 *sha1, filesize_t filesize, gboolean refresh)
-{
-    bitzi_query_by_sha1(sha1, filesize, refresh);
-}
-
-const char *
-guc_bitzi_ticket_by_sha1(const struct sha1 *sha1, filesize_t filesize)
-{
-    return bitzi_ticket_by_sha1(sha1, filesize);
-}
-
-gboolean
-guc_bitzi_has_cached_ticket(const struct sha1 *sha1)
-{
-	return bitzi_has_cached_ticket(sha1);
-}
-
 /*	download and src interface functions (UI -> Core)*/
 char *
 guc_download_build_url(const struct download *d)
@@ -121,6 +100,12 @@ guc_file_info_status_to_string(const gnet_fi_status_t *status)
 	return file_info_status_to_string(status);
 }
 
+void
+guc_file_info_clear_completed(void)
+{
+	file_info_clear_completed();
+}
+
 int
 guc_download_get_http_req_percent(const struct download *d)
 {
@@ -128,20 +113,20 @@ guc_download_get_http_req_percent(const struct download *d)
 }
 
 void
-guc_download_fallback_to_push(struct download *d, gboolean on_timeout,
-	gboolean user_request)
+guc_download_fallback_to_push(struct download *d, bool on_timeout,
+	bool user_request)
 {
 	download_fallback_to_push(d, on_timeout, user_request);
 }
 
 int
 guc_download_remove_all_from_peer(const struct guid *guid,
-	const host_addr_t addr, guint16 port, gboolean unavailable)
+	const host_addr_t addr, uint16 port, bool unavailable)
 {
 	return download_remove_all_from_peer(guid, addr, port, unavailable);
 }
 
-gboolean
+bool
 guc_download_file_exists(const struct download *d)
 {
 	return download_file_exists(d);
@@ -165,12 +150,6 @@ guc_download_pause(struct download *d)
 	download_request_pause(d);
 }
 
-void
-guc_download_abort(struct download *d)
-{
-	download_request_abort(d);
-}
-
 void
 guc_download_resume(struct download *d)
 {
@@ -189,31 +168,31 @@ guc_download_thaw_queue(void)
 	download_thaw_queue();
 }
 
-gboolean
+bool
 guc_download_queue_is_frozen(void)
 {
 	return download_queue_is_frozen();
 }
 
 void
-guc_download_clear_stopped(gboolean complete,
-	gboolean failed, gboolean unavailable, gboolean finished, gboolean now)
+guc_download_clear_stopped(bool complete,
+	bool failed, bool unavailable, bool finished, bool now)
 {
 	download_clear_stopped(complete, failed, unavailable, finished, now);
 }
 
-guint
+uint
 guc_download_handle_magnet(const char *url)
 {
 	return download_handle_magnet(url);
 }
 
-gboolean
+bool
 guc_download_new(const char *filename,
 	const char *uri,
 	filesize_t size,
 	const host_addr_t addr,
-	guint16 port,
+	uint16 port,
 	const struct guid *guid,
 	const char *hostname,
 	const struct sha1 *sha1,
@@ -221,7 +200,7 @@ guc_download_new(const char *filename,
 	time_t stamp,
 	fileinfo_t *fi,
 	gnet_host_vec_t *proxies,
-	guint32 flags,
+	uint32 flags,
 	const char *parq_id)
 {
 	return download_new(filename,
@@ -244,7 +223,7 @@ void
 guc_download_auto_new(const char *filename,
 	filesize_t size,
 	const host_addr_t addr,
-	guint16 port,
+	uint16 port,
 	const struct guid *guid,
 	const char *hostname,
 	const struct sha1 *sha1,
@@ -252,7 +231,7 @@ guc_download_auto_new(const char *filename,
 	time_t stamp,
 	fileinfo_t *fi,
 	gnet_host_vec_t *proxies,
-	guint32 flags)
+	uint32 flags)
 {
 	download_auto_new(filename,
 		size,
@@ -298,14 +277,14 @@ guc_download_total_progress(const struct download *d)
 	return download_total_progress(d);
 }
 
-gboolean
+bool
 guc_download_something_to_clear(void)
 {
 	return download_something_to_clear();
 }
 
 void
-guc_download_index_changed(const host_addr_t addr, guint16 port,
+guc_download_index_changed(const host_addr_t addr, uint16 port,
 	const struct guid *guid, filesize_t from, filesize_t to)
 {
 	download_index_changed(addr, port, guid, from, to);
@@ -319,7 +298,7 @@ guc_src_get_download(gnet_src_t src_handle)
 
 void
 guc_src_add_listener(src_listener_t cb, gnet_src_ev_t ev,
-    frequency_t t, guint32 interval)
+    frequency_t t, uint32 interval)
 {
 	src_add_listener(cb, ev, t, interval);
 }
@@ -380,7 +359,7 @@ guc_fi_resume(gnet_fi_t fih)
 	fi_resume(fih);
 }
 
-gboolean
+bool
 guc_fi_rename(gnet_fi_t fih, const char *filename)
 {
 	return fi_rename(fih, filename);
@@ -388,7 +367,7 @@ guc_fi_rename(gnet_fi_t fih, const char *filename)
 
 void
 guc_fi_add_listener(fi_listener_t cb, gnet_fi_ev_t ev,
-    frequency_t t, guint32 interval)
+    frequency_t t, uint32 interval)
 {
 	fi_add_listener(cb, ev, t, interval);
 }
@@ -454,6 +433,18 @@ guc_gnet_stats_drop_reason_to_string(msg_drop_reason_t reason)
 	return gnet_stats_drop_reason_to_string(reason);
 }
 
+const char *
+guc_gnet_stats_general_description(gnr_stats_t gs)
+{
+	return gnet_stats_general_description(gs);
+}
+
+const char *
+guc_gnet_msg_type_description(msg_type_t mt)
+{
+	return gnet_msg_type_description(mt);
+}
+
 /*	hcache interface functions (UI -> Core)*/
 void
 guc_hcache_clear_host_type(host_type_t type)
@@ -494,33 +485,19 @@ guc_hsep_get_non_hsep_triple(hsep_triple *tripledest)
 
 
 void
-guc_hsep_add_global_table_listener(GCallback cb,
-	frequency_t t, guint32 interval)
+guc_hsep_add_global_table_listener(callback_fn_t cb,
+	frequency_t t, uint32 interval)
 {
 	hsep_add_global_table_listener(cb, t, interval);
 }
 
 void
-guc_hsep_remove_global_table_listener(GCallback cb)
+guc_hsep_remove_global_table_listener(callback_fn_t cb)
 {
 	hsep_remove_global_table_listener(cb);
 }
 
 
-/*	HTTP interface functions (UI -> Core)*/
-const char *
-guc_http_range_to_string(const GSList *list)
-{
-	return http_range_to_string(list);
-}
-
-GSList *
-guc_http_range_merge(GSList *old_list, GSList *new_list)
-{
-	return http_range_merge(old_list, new_list);
-}
-
-
 /*	node interface functions (UI -> Core)*/
 void
 guc_node_add_node_added_listener(node_added_listener_t l)
@@ -571,13 +548,13 @@ guc_node_remove_node_flags_changed_listener(node_flags_changed_listener_t l)
 }
 
 void
-guc_node_add(const host_addr_t addr, guint16 port, guint32 flags)
+guc_node_add(const host_addr_t addr, uint16 port, uint32 flags)
 {
 	node_add(addr, port, flags);
 }
 
 void
-guc_node_remove_by_id(const node_id_t node_id)
+guc_node_remove_by_id(const struct nid *node_id)
 {
 	node_remove_by_id(node_id);
 }
@@ -585,17 +562,20 @@ guc_node_remove_by_id(const node_id_t node_id)
 void
 guc_node_remove_nodes_by_id(const GSList *node_list)
 {
-	node_remove_nodes_by_id(node_list);
+	pslist_t *sl = gm_slist_to_pslist(node_list);
+
+	node_remove_nodes_by_id(sl);
+	pslist_free(sl);
 }
 
-gboolean
-guc_node_get_status(const node_id_t node_id, gnet_node_status_t *s)
+bool
+guc_node_get_status(const struct nid *node_id, gnet_node_status_t *s)
 {
 	return node_get_status(node_id, s);
 }
 
 gnet_node_info_t *
-guc_node_get_info(const node_id_t node_id)
+guc_node_get_info(const struct nid *node_id)
 {
 	return node_get_info(node_id);
 }
@@ -612,14 +592,14 @@ guc_node_free_info(gnet_node_info_t *info)
 	node_free_info(info);
 }
 
-gboolean
-guc_node_fill_flags(const node_id_t node_id, gnet_node_flags_t *flags)
+bool
+guc_node_fill_flags(const struct nid *node_id, gnet_node_flags_t *flags)
 {
 	return node_fill_flags(node_id, flags);
 }
 
-gboolean
-guc_node_fill_info(const node_id_t node_id, gnet_node_info_t *info)
+bool
+guc_node_fill_info(const struct nid *node_id, gnet_node_info_t *info)
 {
 	return node_fill_info(node_id, info);
 }
@@ -662,31 +642,37 @@ guc_get_parq_dl_retry_delay(const struct download *d)
 }
 
 /*	search interface functions (UI -> Core)*/
-guint
+uint
 guc_search_handle_magnet(const char *url)
 {
 	return search_handle_magnet(url);
 }
 
 void
-guc_search_update_items(gnet_search_t sh, guint32 items)
+guc_search_update_items(gnet_search_t sh, uint32 items)
 {
 	search_update_items(sh, items);
 }
 
-guint32
+uint32
 guc_search_get_reissue_timeout(gnet_search_t sh)
 {
 	return search_get_reissue_timeout(sh);
 }
 
+unsigned
+guc_search_get_media_type(gnet_search_t sh)
+{
+	return search_get_media_type(sh);
+}
+
 void
-guc_search_set_reissue_timeout(gnet_search_t sh, guint32 timeout)
+guc_search_set_reissue_timeout(gnet_search_t sh, uint32 timeout)
 {
 	search_set_reissue_timeout(sh, timeout);
 }
 
-guint
+uint
 guc_search_get_lifetime(gnet_search_t sh)
 {
 	return search_get_lifetime(sh);
@@ -710,71 +696,95 @@ guc_search_query(gnet_search_t sh)
 	return search_query(sh);
 }
 
-gboolean
+bool
 guc_search_is_active(gnet_search_t sh)
 {
 	return search_is_active(sh);
 }
 
-gboolean
+bool
 guc_search_is_browse(gnet_search_t sh)
 {
 	return search_is_browse(sh);
 }
 
-gboolean
+bool
 guc_search_is_local(gnet_search_t sh)
 {
 	return search_is_local(sh);
 }
 
-gboolean
+bool
 guc_search_is_passive(gnet_search_t sh)
 {
 	return search_is_passive(sh);
 }
 
-gboolean
+bool
 guc_search_is_expired(gnet_search_t sh)
 {
 	return search_is_expired(sh);
 }
 
-gboolean
+bool
 guc_search_is_frozen(gnet_search_t sh)
 {
 	return search_is_frozen(sh);
 }
 
-enum search_new_result
-guc_search_new(gnet_search_t *ptr, const char *query,
-	time_t create_time, guint lifetime, guint32 reissue_timeout, guint32 flags)
+bool
+guc_search_is_whats_new(gnet_search_t sh)
 {
-	return search_new(ptr, query,
+	return search_is_whats_new(sh);
+}
+
+const char *
+guc_search_media_mask_to_string(unsigned mask)
+{
+	return search_media_mask_to_string(mask);
+}
+
+void
+guc_search_associate_sha1(gnet_search_t sh, const struct sha1 *sha1)
+{
+	search_associate_sha1(sh, sha1);
+}
+
+pslist_t *
+guc_search_associated_sha1(gnet_search_t sh)
+{
+	return search_associated_sha1(sh);
+}
+
+unsigned
+guc_search_associated_sha1_count(gnet_search_t sh)
+{
+	return search_associated_sha1_count(sh);
+}
+
+enum search_new_result
+guc_search_new(gnet_search_t *ptr, const char *query, unsigned mtype,
+	time_t create_time, uint lifetime, uint32 reissue_timeout, uint32 flags)
+{
+	return search_new(ptr, query, mtype,
 				create_time, lifetime, reissue_timeout, flags);
 }
 
-gboolean
+bool
 guc_search_browse(gnet_search_t sh,
-	const char *hostname, host_addr_t addr, guint16 port,
-	const struct guid *guid, const gnet_host_vec_t *proxies, guint32 flags)
+	const char *hostname, host_addr_t addr, uint16 port,
+	const struct guid *guid, const gnet_host_vec_t *proxies, uint32 flags)
 {
 	return search_browse(sh, hostname, addr, port, guid, proxies, flags);
 }
 
-gboolean
+bool
 guc_search_locally(gnet_search_t sh, const char *query)
 {
 	return search_locally(sh, query);
 }
 
 
-void
-guc_search_reissue(gnet_search_t sh)
-{
-	search_reissue(sh);
-}
-
 void
 guc_search_close(gnet_search_t sh)
 {
@@ -795,10 +805,10 @@ guc_search_stop(gnet_search_t sh)
 
 
 /*	settings interface functions (UI -> Core)*/
-guint16
+uint16
 guc_listen_port(void)
 {
-	guint32 port;
+	uint32 port;
 	gnet_prop_get_guint32_val(PROP_LISTEN_PORT, &port);
 	return port;
 }
@@ -835,13 +845,13 @@ guc_share_scan(void)
 	share_scan();
 }
 
-guint64
+uint64
 guc_shared_files_scanned(void)
 {
 	return shared_files_scanned();
 }
 
-guint64
+uint64
 guc_shared_kbytes_scanned(void)
 {
 	return shared_kbytes_scanned();
@@ -884,9 +894,33 @@ guc_search_request_listener_remove(search_request_listener_t l)
 }
 
 void
-guc_search_add_kept(gnet_search_t sh, guint32 kept)
+guc_search_add_kept(gnet_search_t sh, const struct guid *muid, uint32 kept)
 {
-	search_add_kept(sh, kept);
+	search_add_kept(sh, muid, kept);
+}
+
+void
+guc_guess_event_listener_add(guess_event_listener_t l)
+{
+	guess_event_listener_add(l);
+}
+
+void
+guc_guess_event_listener_remove(guess_event_listener_t l)
+{
+	guess_event_listener_remove(l);
+}
+
+void
+guc_guess_stats_listener_add(guess_stats_listener_t l)
+{
+	guess_stats_listener_add(l);
+}
+
+void
+guc_guess_stats_listener_remove(guess_stats_listener_t l)
+{
+	guess_stats_listener_remove(l);
 }
 
 /*	upload interface functions (UI -> Core)*/
@@ -964,7 +998,7 @@ guc_upload_stats_clear_all(void)
 	upload_stats_clear_all();
 }
 
-gboolean
+bool
 guc_dht_enabled(void)
 {
 	return dht_enabled();
