@@ -1,6 +1,4 @@
 /*
- * $Id: http.h 16111 2009-03-10 15:32:51Z rmanfredi $
- *
  * Copyright (c) 2003, Raphael Manfredi
  *
  * Interface definition file.  One of the files that defines structures,
@@ -22,7 +20,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -44,27 +42,6 @@ typedef enum http_state {
 	HTTP_AS_REMOVED				/**< Removed, pending free */
 } http_state_t;
 
-/**
- * HTTP range description.
- */
-
-typedef struct http_range {
-	filesize_t start;
-	filesize_t end;				/**< HTTP_OFFSET_MAX if unbounded */
-} http_range_t;
-
-#define HTTP_OFFSET_MAX	((filesize_t) -1)
-
-/*
- * Public interface, visible from the bridge.
- */
-
-#ifdef CORE_SOURCES
-
-const char *http_range_to_string(const GSList *list);
-GSList *http_range_merge(GSList *list1, GSList *list2);
-
-#endif /* CORE_SOURCES */
-
 #endif /* _if_core_http_h_ */
+
 /* vi: set ts=4 sw=4 cindent: */
