@@ -1,6 +1,4 @@
 /*
- * $Id: share.h 14132 2007-07-21 01:30:20Z cbiere $
- *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -34,10 +32,10 @@
 
 #ifdef CORE_SOURCES
 
-void shared_dir_add(const gchar *);
+void shared_dir_add(const char *);
 void share_scan(void);
-guint64 shared_files_scanned(void);
-guint64 shared_kbytes_scanned(void);
+uint64 shared_files_scanned(void);
+uint64 shared_kbytes_scanned(void);
 
 #endif /* CORE_SOURCES */
 #endif /* _if_core_share_h */
