@@ -1,6 +1,4 @@
 /*
- * $Id: sockets.h 16995 2009-09-26 22:50:43Z rmanfredi $
- *
  * Copyright (c) 2003, Raphael Manfredi
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -59,10 +57,9 @@ enum {
 	SOCK_TRACE_BOTH = 0x1 | 0x2
 };
 
-/*
+/**
  * Operating flags
  */
-
 enum {
 	SOCK_F_ESTABLISHED	= (1UL << 0),  /**< Connection was established */
 	SOCK_F_EOF			= (1UL << 1),  /**< Got an EOF condition */
@@ -74,11 +71,39 @@ enum {
 	SOCK_F_SHUTDOWN		= (1UL << 7),  /**< Set if shutdown() was called */
 	SOCK_F_OMIT_TOKEN	= (1UL << 8),  /**< If set X-Token header is omitted */
 	SOCK_F_PREPARED		= (1UL << 9),  /**< Prepared (known address) */
+	SOCK_F_SINGLE		= (1UL << 10), /**< Read one single datagram */
+	SOCK_F_CONNRESET	= (1UL << 11), /**< Got a connection reset event */
+	SOCK_F_OLD			= (1UL << 12), /**< Processing an "old" UDP datagram */
+	SOCK_F_G2			= (1UL << 13), /**< Targeting a G2 node */
 	SOCK_F_LOCAL		= (1UL << 28), /**< Is a local socket */
 	SOCK_F_UDP			= (1UL << 29), /**< Is a UDP socket */
 	SOCK_F_TCP			= (1UL << 30)  /**< Is a TCP socket */
 };
 
+/**
+ * Connection types.
+ */
+enum socket_type {
+	SOCK_TYPE_UNKNOWN = 0,
+	SOCK_TYPE_CONTROL,
+	SOCK_TYPE_DOWNLOAD,
+	SOCK_TYPE_UPLOAD,
+	SOCK_TYPE_HTTP,
+    SOCK_TYPE_SHELL,
+    SOCK_TYPE_CONNBACK,
+    SOCK_TYPE_PPROXY,
+    SOCK_TYPE_DESTROYING,
+	SOCK_TYPE_UDP
+};
+
+/**
+ * Socket buffer type.
+ */
+enum socket_buftype {
+	SOCK_BUF_RX,
+	SOCK_BUF_TX
+};
+
 #endif /* _if_core_sockets_h_ */
 
 /* vi: set ts=4 sw=4 cindent: */
