@@ -1,6 +1,4 @@
 /*
- * $Id: kuid.h 17044 2009-10-09 20:55:36Z rmanfredi $
- *
  * Copyright (c) 2008, Raphael Manfredi
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -34,7 +32,7 @@
 #define KUID_HEX_BUFLEN		(KUID_HEX_SIZE + 1)
 
 typedef struct kuid {
-	guchar v[KUID_RAW_SIZE];
+	uchar v[KUID_RAW_SIZE];
 } kuid_t;
 
 /**
@@ -50,7 +48,8 @@ kuid_copy(kuid_t *dest, const kuid_t *other)
  * Public interface.
  */
 
-gboolean kuid_eq(const kuid_t *k1, const kuid_t *k2);
+unsigned kuid_hash(const void *key);
+bool kuid_eq(const void *k1, const void *k2);
 
 const char *kuid_to_string(const kuid_t *kuid);
 const char *kuid_to_hex_string(const kuid_t *kuid);
