@@ -1,6 +1,4 @@
 /*
- * $Id: statusbar.h 12761 2007-01-16 16:07:30Z cbiere $
- *
  * Copyright (c) 2001-2003, Richard Eckart
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -45,10 +43,10 @@ typedef struct {
 #ifdef GUI_SOURCES
 
 statusbar_msgid_t statusbar_gui_message(guint timeout,
-		const gchar *fmt, ...) G_GNUC_PRINTF(2, 3);
+		const gchar *fmt, ...) G_PRINTF(2, 3);
 
 statusbar_msgid_t statusbar_gui_warning(guint timeout,
-		const gchar *fmt, ...) G_GNUC_PRINTF(2, 3);
+		const gchar *fmt, ...) G_PRINTF(2, 3);
 
 #endif /* GUI_SOURCES */
 #endif /* _if_ui_gtk_statusbar_h_ */
