@@ -1,6 +1,4 @@
 /*
- * $Id: getphysmemsize.c 14954 2007-09-28 19:23:13Z bpringlemeir $
- *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -36,22 +34,43 @@
 
 #include "common.h"
 #include "vmm.h"
-#include "lib/getphysmemsize.h"
+#include "getphysmemsize.h"
 
+#ifndef _SC_PHYS_PAGES
+/*
+ * Only include <sys/sysctl.h> if we're going to actually need it.
+ * Indeed, that header file is now deprecated on linux, but since linux
+ * has sysconf(), we don't really need to include that file there!
+ * 		--RAM, 2020-11-10.
+ */
 #if defined(I_SYS_SYSCTL) && defined(HAS_SYSCTL)
 #include <sys/sysctl.h>
 #endif
+#endif	/* !_SC_PHYS_PAGES */
 
 #ifdef I_INVENT
 #include <invent.h>
 #endif
 
+static uint64 memsize;
+
 /**
  * @return the amount of physical RAM in bytes, or zero in case of failure.
  */
-guint64
-getphysmemsize(void)
-#if defined (_SC_PHYS_PAGES)
+static uint64
+getphysmemsize_internal(void)
+#ifdef MINGW32
+{
+	uint64 mem;
+
+	if ((uint64) -1 == (mem = mingw_getphysmemsize())) {
+		g_warning("%s(): GlobalMemoryStatusEx() failed: %m", G_STRFUNC);
+		return 0;
+	}
+
+	return mem;
+}
+#elif defined(_SC_PHYS_PAGES)
 {
 	size_t pagesize = compat_pagesize();
 	long pages;
@@ -59,51 +78,46 @@ getphysmemsize(void)
 	errno = 0;
 	pages = sysconf(_SC_PHYS_PAGES);
 	if (-1L == pages && 0 != errno) {
-		g_warning("sysconf(_SC_PHYS_PAGES) failed: %s", g_strerror(errno));
+		g_warning("%s: sysconf(_SC_PHYS_PAGES) failed: %m", G_STRFUNC);
 		return 0;
 	}
-	return pagesize * (guint64) (unsigned long) pages;
+	return pagesize * (uint64) (unsigned long) pages;
 }
-#elif defined (HAS_SYSCTL) && defined (CTL_HW) && defined (HW_USERMEM64)
+#elif defined(HAS_SYSCTL) && defined(CTL_HW) && defined(HW_USERMEM64)
 {
 	/* There's also HW_PHYSMEM but HW_USERMEM is better for our needs. */
 	int mib[2] = { CTL_HW, HW_USERMEM64 };
-	guint64 amount = 0;
+	uint64 amount = 0;
 	size_t len = sizeof amount;
 
 	if (-1 == sysctl(mib, 2, &amount, &len, NULL, 0)) {
-		g_warning(
-			"settings_getphysmemsize: sysctl() for HW_USERMEM64 failed: %s",
-			g_strerror(errno));
+		g_warning("%s: sysctl() for HW_USERMEM64 failed: %m", G_STRFUNC);
 		return 0;
 	}
 
 	return amount;
 }
-#elif defined (HAS_SYSCTL) && defined (CTL_HW) && defined (HW_USERMEM)
+#elif defined(HAS_SYSCTL) && defined(CTL_HW) && defined(HW_USERMEM)
 {
 	/* There's also HW_PHYSMEM but HW_USERMEM is better for our needs. */
 	int mib[2] = { CTL_HW, HW_USERMEM };
-	guint32 amount = 0;
+	uint32 amount = 0;
 	size_t len = sizeof amount;
 
 	if (-1 == sysctl(mib, 2, &amount, &len, NULL, 0)) {
-		g_warning(
-			"settings_getphysmemsize: sysctl() for HW_USERMEM failed: %s",
-			g_strerror(errno));
+		g_warning("%s: sysctl() for HW_USERMEM failed: %m", G_STRFUNC);
 		return 0;
 	}
 
 	return amount;
 }
-#elif defined (HAS_GETINVENT)
+#elif defined(HAS_GETINVENT)
 {
 	inventory_t *inv;
 	long physmem = 0;
 
 	if (-1 == setinvent()) {
-		g_warning("settings_getphysmemsize: setinvent() failed: %s",
-			g_strerror(errno));
+		g_warning("%s: setinvent() failed: %m", G_STRFUNC);
 		return 0;
 	}
 
@@ -117,18 +131,45 @@ getphysmemsize(void)
 	endinvent();
 
 	if (-1L == physmem && 0 != errno) {
-		g_warning("settings_getphysmemsize: "
-			"getinvent() for INV_MEMORY faild: %s", g_strerror(errno));
+		g_warning("%s: getinvent() for INV_MEMORY faild: %m", G_STRFUNC);
 		return 0;
 	}
 
-	return (guint64) (unsigned long) physmem * 1024 * 1024;
+	return (uint64) (unsigned long) physmem * 1024 * 1024;
 }
 #else /* ! _SC_PHYS_PAGES && ! HAS_SYSCTL && ! HAS_GETINVENT */
 {
-	g_warning("Unable to determine amount of physical RAM");
+	g_warning("unable to determine amount of physical RAM");
 	return 0;
 }
 #endif /* _SC_PHYS_PAGES */
 
+/**
+ * Get the amount of physical RAM available.
+ *
+ * @return the amount of physical RAM in bytes, or zero in case of failure.
+ */
+uint64
+getphysmemsize(void)
+{
+	if G_UNLIKELY(0 == memsize)
+		memsize = getphysmemsize_internal();
+
+	return memsize;
+}
+
+/**
+ * Get the amount of physical RAM available, if known.
+ *
+ * The purpose is to avoid any memory allocation, which sysconf() or sysctl()
+ * may otherwise cause.
+ *
+ * @return the known amount of physical RAM in bytes, or 0 if unknown yet.
+ */
+uint64
+getphysmemsize_known(void)
+{
+	return memsize;
+}
+
 /* vi: set ts=4 sw=4 cindent: */
