@@ -1,6 +1,4 @@
 /*
- * $Id: header.h 17272 2010-02-22 17:50:23Z rmanfredi $
- *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -72,11 +70,13 @@ int header_num_lines(const header_t *h);
  */
 
 header_t *header_make(void);
+header_t *header_refcnt_inc(header_t *o);
 void header_free(header_t *o);
+void header_free_null(header_t **o_ptr);
 void header_reset(header_t *o);
 int header_append(header_t *o, const char *text, int len);
 void header_dump(FILE *out, const header_t *o, const char *trailer);
-const char *header_strerror(guint errnum);
+const char *header_strerror(uint errnum);
 char *header_get(const header_t *o, const char *field);
 char *header_get_extended(const header_t *o, const char *field, size_t *lptr);
 
@@ -86,9 +86,9 @@ header_fmt_t *header_fmt_make(const char *field, const char *separator,
 	size_t len_hint, size_t max_size);
 void header_fmt_free(header_fmt_t **hf);
 void header_fmt_set_line_length(header_fmt_t *hf, size_t maxlen);
-gboolean header_fmt_value_fits(const header_fmt_t *hf, size_t len);
-gboolean header_fmt_append(header_fmt_t *hf, const char *str, const char *sep);
-gboolean header_fmt_append_value(header_fmt_t *hf, const char *str);
+bool header_fmt_value_fits(const header_fmt_t *hf, size_t len);
+bool header_fmt_append(header_fmt_t *hf, const char *str, const char *sep);
+bool header_fmt_append_value(header_fmt_t *hf, const char *str);
 size_t header_fmt_length(const header_fmt_t *hf);
 void header_fmt_end(header_fmt_t *hf);
 const char *header_fmt_string(const header_fmt_t *hf);
