@@ -1,6 +1,4 @@
 /*
- * $Id: magnet.c 17265 2010-02-20 19:50:51Z rmanfredi $
- *
  * Copyright (c) 2006, Christian Biere
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -37,22 +35,24 @@
 
 #include "common.h"
 
-RCSID("$Id: magnet.c 17265 2010-02-20 19:50:51Z rmanfredi $")
+#include "magnet.h"
 
 #include "ascii.h"
 #include "atoms.h"
 #include "concat.h"
-#include "glib-missing.h"
 #include "gnet_host.h"
 #include "halloc.h"
-#include "magnet.h"
+#include "hstrfn.h"
+#include "once.h"
 #include "parse.h"
 #include "sequence.h"
+#include "str.h"
 #include "stringify.h"
 #include "tm.h"
+#include "tokenizer.h"
+#include "unsigned.h"
 #include "url.h"
 #include "urn.h"
-#include "unsigned.h"
 #include "utf8.h"
 #include "walloc.h"
 
@@ -69,7 +69,7 @@ RCSID("$Id: magnet.c 17265 2010-02-20 19:50:51Z rmanfredi $")
  */
 
 enum magnet_key {
-	MAGNET_KEY_NONE,
+	MAGNET_KEY_NONE = 0,
 	MAGNET_KEY_DISPLAY_NAME,		/* Display Name */
 	MAGNET_KEY_KEYWORD_TOPIC,		/* Keyword Topic */
 	MAGNET_KEY_EXACT_LENGTH,		/* eXact file Length */
@@ -79,19 +79,20 @@ enum magnet_key {
 	MAGNET_KEY_PARQ_ID,				/* PARQ ID */
 	MAGNET_KEY_GUID,				/* Servent GUID */
 	MAGNET_KEY_VENDOR,				/* Servent vendor */
+	MAGNET_KEY_DHT,					/* Servent known to publish in the DHT */
+	MAGNET_KEY_G2,					/* Servent is a G2 node */
 
 	NUM_MAGNET_KEYS
 };
 
-static const struct {
-	const char * const key;
-	const enum magnet_key id;
-} magnet_keys[] = {
+static tokenizer_t magnet_keys[] = {
 	/* Must be sorted alphabetically */
 	{ "",			MAGNET_KEY_NONE },
 	{ "as",			MAGNET_KEY_ALTERNATE_SOURCE },
 	{ "dn",			MAGNET_KEY_DISPLAY_NAME },
 	{ "kt",			MAGNET_KEY_KEYWORD_TOPIC },
+	{ "x.dht",		MAGNET_KEY_DHT },
+	{ "x.g2",		MAGNET_KEY_G2 },
 	{ "x.guid",		MAGNET_KEY_GUID },
 	{ "x.parq-id",	MAGNET_KEY_PARQ_ID },
 	{ "x.vndr",		MAGNET_KEY_VENDOR },
@@ -117,38 +118,36 @@ clear_error_str(const char ***error_str)
 }
 
 static void
-free_proxies_list(GSList *sl)
+free_proxies_list(pslist_t *sl)
 {
-	g_slist_foreach(sl, gnet_host_free_item, NULL);
-	g_slist_free(sl);
+	pslist_foreach(sl, gnet_host_free_item, NULL);
+	pslist_free(sl);
+}
+
+static once_flag_t magnet_keys_checked;
+
+static void G_COLD
+magnet_key_check(void)
+{
+	TOKENIZE_CHECK_SORTED_WITH(magnet_keys, ascii_strcasecmp);
 }
 
 static enum magnet_key
 magnet_key_get(const char *s)
 {
-	STATIC_ASSERT(G_N_ELEMENTS(magnet_keys) == NUM_MAGNET_KEYS);
-	g_assert(s);
+	STATIC_ASSERT(N_ITEMS(magnet_keys) == NUM_MAGNET_KEYS);
+	g_assert(s != NULL);
 
-#define GET_KEY(i) (magnet_keys[(i)].key)
-#define FOUND(i) G_STMT_START { \
-	return magnet_keys[(i)].id;	\
-	/* NOTREACHED */ \
-} G_STMT_END
+	ONCE_FLAG_RUN(magnet_keys_checked, magnet_key_check);
 
-	BINARY_SEARCH(const char *, s, G_N_ELEMENTS(magnet_keys), ascii_strcasecmp,
-		GET_KEY, FOUND);
-	
-#undef FOUND
-#undef GET_KEY
-
-	return MAGNET_KEY_NONE;
+	return TOKENIZE_WITH(s, ascii_strcasecmp, magnet_keys);
 }
 
 static void
 plus_to_space(char *s)
 {
 	while (s) {
-		s = strchr(s, '+');
+		s = vstrchr(s, '+');
 		if (s) {
 			*s++ = ' ';
 		}
@@ -178,7 +177,7 @@ magnet_parse_path(const char *path, const char **error_str)
 	endptr = is_strprefix(p, "/uri-res/N2R?");
 	if (endptr) {
 		struct sha1 sha1;
-		
+
 		p = endptr;
 		if (!urn_get_sha1(p, &sha1)) {
 			*error_str = "Bad SHA1 in MAGNET URI";
@@ -189,12 +188,12 @@ magnet_parse_path(const char *path, const char **error_str)
 		ms.path = atom_str_get(p);
 	}
 
-	return wcopy(&ms, sizeof ms);
+	return WCOPY(&ms);
 }
 
 static const char *
 magnet_parse_host_port(const char *hostport,
-	host_addr_t *addr, guint16 *port, const char **host, const char **host_end,
+	host_addr_t *addr, uint16 *port, const char **host, const char **host_end,
 	const char **error_str)
 {
 	const char *p;
@@ -222,7 +221,7 @@ magnet_parse_host_port(const char *hostport,
 	if (':' == *p) {
 		const char *ep2;
 		int error;
-		guint16 u;
+		uint16 u;
 
 		p++;
 		u = parse_uint16(p, &ep2, 10, &error);
@@ -247,7 +246,7 @@ magnet_parse_location(const char *uri, const char **error_str)
 	struct magnet_source *ms;
 	const char *p, *host, *host_end;
 	host_addr_t addr;
-	guint16 port;
+	uint16 port;
 
 	clear_error_str(&error_str);
 	g_return_val_if_fail(uri, NULL);
@@ -263,9 +262,9 @@ magnet_parse_location(const char *uri, const char **error_str)
 		return NULL;
 
 	if (host) {
-		char *h = g_strndup(host, host_end - host);
+		char *h = h_strndup(host, host_end - host);
 		ms->hostname = atom_str_get(h);
-		G_FREE_NULL(h);
+		HFREE_NULL(h);
 	}
 
 	ms->addr = addr;
@@ -274,59 +273,11 @@ magnet_parse_location(const char *uri, const char **error_str)
 	return ms;
 }
 
-static gboolean
-magnet_parse_legacy_addr_list(const char *proxies,
-	const char **endptr, GSList **list, const char **error_str)
-{
-	GSList *sl = NULL;
-	const char *p;
-
-	clear_error_str(&error_str);
-	g_return_val_if_fail(proxies, FALSE);
-
-	p = proxies;
-
-	if (*p != '{') {
-		*endptr = p;
-		*error_str = "Expected push-proxies list";
-		return FALSE;
-	}
-
-	p++;
-
-	while (*p != '}') {
-		host_addr_t addr;
-		guint16 port;
-		gnet_host_t *host;
-
-		p = magnet_parse_host_port(p, &addr, &port, NULL, NULL, error_str);
-		if (NULL == p)
-			goto cleanup;
-
-		host = gnet_host_new(addr, port);
-		sl = g_slist_prepend(sl, host);		/* Will reverse list below */
-
-		if (*p == ',')
-			p++;
-	}
-
-	sl = g_slist_reverse(sl);		/* Keep order of listed push-proxies */
-	p++;
-	*endptr = p;
-
-	*list = sl;
-	return TRUE;
-
-cleanup:
-	free_proxies_list(sl);
-	return FALSE;
-}
-
-static gboolean
+static bool
 magnet_parse_addr_list(const char *proxies,
-	const char **endptr, GSList **list, const char **error_str)
+	const char **endptr, pslist_t **list, const char **error_str)
 {
-	GSList *sl = NULL;
+	pslist_t *sl = NULL;
 	const char *p;
 
 	clear_error_str(&error_str);
@@ -336,7 +287,7 @@ magnet_parse_addr_list(const char *proxies,
 
 	while (*p == ':') {
 		host_addr_t addr;
-		guint16 port;
+		uint16 port;
 		gnet_host_t *host;
 
 		p = magnet_parse_host_port(p+1, &addr, &port, NULL, NULL, error_str);
@@ -344,10 +295,10 @@ magnet_parse_addr_list(const char *proxies,
 			goto cleanup;
 
 		host = gnet_host_new(addr, port);
-		sl = g_slist_prepend(sl, host);		/* Will reverse list below */
+		sl = pslist_prepend(sl, host);		/* Will reverse list below */
 	}
 
-	sl = g_slist_reverse(sl);		/* Keep order of listed push-proxies */
+	sl = pslist_reverse(sl);		/* Keep order of listed push-proxies */
 	*endptr = p;
 
 	*list = sl;
@@ -363,7 +314,7 @@ magnet_parse_proxy_location(const char *uri, const char **error_str)
 {
 	struct magnet_source *ms;
 	const char *p, *endptr = NULL;
-	GSList *sl = NULL;
+	pslist_t *sl = NULL;
 
 	clear_error_str(&error_str);
 	g_return_val_if_fail(uri, NULL);
@@ -379,20 +330,6 @@ magnet_parse_proxy_location(const char *uri, const char **error_str)
 		goto path;
 	}
 
-	/*
-	 * Legacy push:// support: push-proxy address given in a set.
-	 * We have a list of push-proxies: {addr1:port,addr2:port}.
-	 * The list may be empty if we have no known push proxies.
-	 *
-	 * FIXME: remove this in 0.96.7.	--RAM, 2009-03-22
-	 */
-
-	if (p[0] == ':' && p[1] == '{') {
-		if (!magnet_parse_legacy_addr_list(uri+1, &endptr, &sl, error_str))
-			return NULL;
-		goto path;
-	}
-
 	if (*p != ':') {
 		*error_str = "Expected / or : following the GUID";
 		return NULL;
@@ -442,9 +379,9 @@ magnet_parse_push_source(const char *uri, const char **error_str)
 	p = is_strprefix(uri, "push://");
 	g_return_val_if_fail(p, NULL);
 
-	endptr = strchr(p, ':');		/* First push-proxy host */
+	endptr = vstrchr(p, ':');		/* First push-proxy host */
 	if (NULL == endptr || GUID_HEX_SIZE != (endptr - p))
-		endptr = strchr(p, '/');	/* No push-proxy host */
+		endptr = vstrchr(p, '/');	/* No push-proxy host */
 
 	if (
 		NULL == endptr ||
@@ -492,11 +429,11 @@ magnet_handle_key(struct magnet_resource *res,
 	g_return_if_fail(res);
 	g_return_if_fail(name);
 	g_return_if_fail(value);
-	
+
 	if (!utf8_is_valid_string(value)) {
 		const char *encoding;
 		char *result;
-			
+
 		g_message("MAGNET URI key \"%s\" is not UTF-8 encoded", name);
 
 		if (MAGNET_KEY_DISPLAY_NAME != magnet_key_get(name))
@@ -530,7 +467,7 @@ magnet_handle_key(struct magnet_resource *res,
 					res->sha1 = atom_sha1_get(ms->sha1);
 				}
 				if (!ms->sha1 || sha1_eq(res->sha1, ms->sha1)) {
-					res->sources = g_slist_prepend(res->sources, ms);
+					res->sources = pslist_prepend(res->sources, ms);
 				} else {
 					magnet_source_free(&ms);
 				}
@@ -555,7 +492,7 @@ magnet_handle_key(struct magnet_resource *res,
 	case MAGNET_KEY_EXACT_LENGTH:
 		{
 			int error;
-			guint64 u;
+			uint64 u;
 
 			u = parse_uint64(value, NULL, 10, &error);
 			if (!error) {
@@ -576,6 +513,30 @@ magnet_handle_key(struct magnet_resource *res,
 		magnet_set_guid(res, value);
 		break;
 
+	case MAGNET_KEY_DHT:
+		{
+			int error;
+			uint8 u;
+
+			u = parse_uint8(value, NULL, 10, &error);
+			if (!error) {
+				magnet_set_dht(res, u);
+			}
+		}
+		break;
+
+	case MAGNET_KEY_G2:
+		{
+			int error;
+			uint8 u;
+
+			u = parse_uint8(value, NULL, 10, &error);
+			if (!error) {
+				magnet_set_g2(res, u);
+			}
+		}
+		break;
+
 	case MAGNET_KEY_NONE:
 		g_message("unhandled parameter in MAGNET URI: \"%s\"", name);
 		break;
@@ -587,7 +548,7 @@ magnet_handle_key(struct magnet_resource *res,
 	G_FREE_NULL(to_free);
 }
 
-struct magnet_resource * 
+struct magnet_resource *
 magnet_parse(const char *url, const char **error_str)
 {
 	static const struct magnet_resource zero_resource;
@@ -615,7 +576,7 @@ magnet_parse(const char *url, const char **error_str)
 		char name[16]; /* Large enough to hold longest key we know */
 
 		name[0] = '\0';
-		endptr = strchr(p, '=');
+		endptr = vstrchr(p, '=');
 		if (endptr && p != endptr) {
 			size_t name_len;
 
@@ -628,9 +589,9 @@ magnet_parse(const char *url, const char **error_str)
 			p = &endptr[1]; /* Point behind the '=' */
 		}
 
-		endptr = strchr(p, '&');
+		endptr = vstrchr(p, '&');
 		if (!endptr) {
-			endptr = strchr(p, '\0');
+			endptr = vstrchr(p, '\0');
 		}
 
 		key = magnet_key_get(name);
@@ -641,7 +602,7 @@ magnet_parse(const char *url, const char **error_str)
 			size_t value_len;
 
 			value_len = endptr - p;
-			value = g_strndup(p, value_len);
+			value = h_strndup(p, value_len);
 
 			plus_to_space(value);
 			if (url_unescape(value, TRUE)) {
@@ -649,7 +610,7 @@ magnet_parse(const char *url, const char **error_str)
 			} else {
 				g_message("badly encoded value in MAGNET URI: \"%s\"", value);
 			}
-			G_FREE_NULL(value);
+			HFREE_NULL(value);
 		}
 
 		while ('&' == endptr[0]) {
@@ -658,17 +619,17 @@ magnet_parse(const char *url, const char **error_str)
 		next = endptr;
 	}
 
-	res.sources = g_slist_reverse(res.sources);
-	res.searches = g_slist_reverse(res.searches);
+	res.sources = pslist_reverse(res.sources);
+	res.searches = pslist_reverse(res.searches);
 
-	return wcopy(&res, sizeof res);
+	return WCOPY(&res);
 }
 
 void
 magnet_source_free(struct magnet_source **ms_ptr)
 {
 	struct magnet_source *ms = *ms_ptr;
-	
+
 	if (ms) {
 		atom_str_free_null(&ms->hostname);
 		atom_str_free_null(&ms->path);
@@ -691,7 +652,7 @@ magnet_resource_free(struct magnet_resource **res_ptr)
 	struct magnet_resource *res = *res_ptr;
 
 	if (res) {
-		GSList *sl;
+		pslist_t *sl;
 
 		atom_str_free_null(&res->display_name);
 		atom_sha1_free_null(&res->sha1);
@@ -700,20 +661,17 @@ magnet_resource_free(struct magnet_resource **res_ptr)
 		atom_str_free_null(&res->guid);
 		atom_str_free_null(&res->vendor);
 
-		for (sl = res->sources; sl != NULL; sl = g_slist_next(sl)) {
+		PSLIST_FOREACH(res->sources, sl) {
 			struct magnet_source *ms = sl->data;
 			magnet_source_free(&ms);
 		}
-		g_slist_free(res->sources);
-		res->sources = NULL;
+		pslist_free_null(&res->sources);
 
-		for (sl = res->searches; sl != NULL; sl = g_slist_next(sl)) {
+		PSLIST_FOREACH(res->searches, sl) {
 			const char *s = sl->data;
 			atom_str_free_null(&s);
 		}
-		g_slist_free(res->searches);
-		res->searches = NULL;
-		
+		pslist_free_null(&res->searches);
 		wfree(res, sizeof *res);
 		*res_ptr = NULL;
 	}
@@ -723,14 +681,14 @@ struct magnet_resource *
 magnet_resource_new(void)
 {
 	static const struct magnet_resource zero_resource;
-	return wcopy(&zero_resource, sizeof zero_resource);
+	return WCOPY(&zero_resource);
 }
 
 struct magnet_source *
 magnet_source_new(void)
 {
 	static const struct magnet_source zero_source;
-	return wcopy(&zero_source, sizeof zero_source);
+	return WCOPY(&zero_source);
 }
 
 static void
@@ -738,8 +696,8 @@ magnet_add_source(struct magnet_resource *res, struct magnet_source *s)
 {
 	g_return_if_fail(res);
 	g_return_if_fail(s);
-	
-	res->sources = g_slist_prepend(res->sources, s);
+
+	res->sources = pslist_prepend(res->sources, s);
 }
 
 void
@@ -757,14 +715,15 @@ magnet_add_source_by_url(struct magnet_resource *res, const char *url)
 
 void
 magnet_add_sha1_source(struct magnet_resource *res, const struct sha1 *sha1,
-	const host_addr_t addr, const guint16 port, const struct guid *guid)
+	const host_addr_t addr, const uint16 port, const struct guid *guid,
+	const gnet_host_vec_t *proxies)
 {
 	struct magnet_source *s;
 
 	g_return_if_fail(res);
 	g_return_if_fail(sha1);
 	g_return_if_fail(!res->sha1 || sha1_eq(res->sha1, sha1));
-	g_return_if_fail(port > 0);
+	g_return_if_fail(guid != NULL || port_is_valid(port));
 
 	if (!res->sha1) {
 		magnet_set_sha1(res, sha1);
@@ -775,6 +734,22 @@ magnet_add_sha1_source(struct magnet_resource *res, const struct sha1 *sha1,
 	s->port = port;
 	s->sha1 = atom_sha1_get(sha1);
 	s->guid = guid ? atom_guid_get(guid) : NULL;
+
+	if (proxies != NULL) {
+		int i, n;
+		pslist_t *sl = NULL;
+
+		n = gnet_host_vec_count(proxies);
+		for (i = 0; i < n; i++) {
+			gnet_host_t host;
+
+			host = gnet_host_vec_get(proxies, i);
+			sl = pslist_prepend(sl, gnet_host_dup(&host));
+		}
+
+		s->proxies = sl;
+	}
+
 	magnet_add_source(res, s);
 }
 
@@ -784,7 +759,7 @@ magnet_add_search(struct magnet_resource *res, const char *search)
 	g_return_if_fail(res);
 	g_return_if_fail(search);
 
-	res->searches = g_slist_prepend(res->searches,
+	res->searches = pslist_prepend(res->searches,
 						deconstify_gchar(atom_str_get(search)));
 }
 
@@ -816,13 +791,13 @@ magnet_set_tth(struct magnet_resource *res, const struct tth *tth)
 }
 
 
-gboolean
+bool
 magnet_set_exact_topic(struct magnet_resource *res, const char *topic)
 {
 	struct sha1 sha1;
 	struct tth tth;
 
-	if (urn_get_bitprint(topic, strlen(topic), &sha1, &tth)) {
+	if (urn_get_bitprint(topic, vstrlen(topic), &sha1, &tth)) {
 		if (!res->sha1) {
 			magnet_set_sha1(res, &sha1);
 		}
@@ -835,7 +810,7 @@ magnet_set_exact_topic(struct magnet_resource *res, const char *topic)
 			magnet_set_sha1(res, &sha1);
 		}
 		return TRUE;
-	} else if (urn_get_tth(topic, strlen(topic), &tth)) {
+	} else if (urn_get_tth(topic, vstrlen(topic), &tth)) {
 		if (!res->tth) {
 			magnet_set_tth(res, &tth);
 		}
@@ -865,47 +840,43 @@ magnet_set_filesize(struct magnet_resource *res, filesize_t size)
 }
 
 static inline void
-magnet_append_item(GString **gs_ptr, gboolean escape_value,
+magnet_append_item(str_t *s, bool escape_value,
 	const char *key, const char *value)
 {
-	GString *gs;
-
-	g_return_if_fail(gs_ptr);
-	g_return_if_fail(*gs_ptr);
+	g_return_if_fail(s);
 	g_return_if_fail(key);
 	g_return_if_fail(value);
 
-	gs = *gs_ptr;
-
-	if (0 == gs->len) {
-		gs = g_string_append(gs, "magnet:?");
+	if (0 == str_len(s)) {
+		STR_CAT(s, "magnet:?");
 	} else {
-		gs = g_string_append_c(gs, '&');
+		str_putc(s, '&');
 	}
-	gs = g_string_append(gs, key);
-	gs = g_string_append_c(gs, '=');
+	str_cat(s, key);
+	str_putc(s, '=');
 
 	if (escape_value) {
 		char *escaped;
 
 		escaped = url_escape_query(value);
-		gs = g_string_append(gs, escaped);
+		str_cat(s, escaped);
 		if (escaped != value) {
 			HFREE_NULL(escaped);
 		}
 	} else {
-		gs = g_string_append(gs, value);
+		str_cat(s, value);
 	}
-
-	*gs_ptr = gs;
 }
 
+/**
+ * @return A halloc()ed string.
+ */
 static char *
 proxy_sequence_to_string(const sequence_t *s)
 {
-	GString *gs;
+	str_t *str;
 
-	gs = g_string_new(NULL);
+	str = str_new(0);
 
 	if (!sequence_is_empty(s)) {
 		sequence_iter_t *iter;
@@ -913,21 +884,24 @@ proxy_sequence_to_string(const sequence_t *s)
 		iter = sequence_forward_iterator(s);
 		while (sequence_iter_has_next(iter)) {
 			gnet_host_t *host = sequence_iter_next(iter);
-			gs = g_string_append_c(gs, ':');
-			g_string_append(gs, gnet_host_to_string(host));
+			str_putc(str, ':');
+			str_cat(str, gnet_host_to_string(host));
 		}
 		sequence_iterator_release(&iter);
 	}
 
-	return gm_string_finalize(gs);
+	return str_s2c_null(&str);
 }
 
+/**
+ * @return A halloc()ed string.
+ */
 static char *
-proxies_to_string(GSList *proxies)
+proxies_to_string(pslist_t *proxies)
 {
 	sequence_t seq;
 
-	return proxy_sequence_to_string(sequence_fill_from_gslist(&seq, proxies));
+	return proxy_sequence_to_string(sequence_fill_from_pslist(&seq, proxies));
 }
 
 /**
@@ -936,14 +910,14 @@ proxies_to_string(GSList *proxies)
  *
  * @return An empty string (""), if the list is empty or the address NULL;
  * otherwise a colon-separated list of IP:port, beginning with a colon. The
- * string is newly allocated.
+ * string is newly allocated via halloc().
  */
 char *
 magnet_proxies_to_string(const sequence_t *proxies)
 {
 	if (NULL == proxies)
-		return g_strdup("");
-	
+		return h_strdup("");
+
 	return proxy_sequence_to_string(proxies);
 }
 
@@ -969,38 +943,37 @@ magnet_source_to_string(const struct magnet_source *s)
 
 		if (s->guid) {
 			char guid_buf[GUID_HEX_SIZE + 1];
-			
-			guid_to_string_buf(s->guid, guid_buf, sizeof guid_buf);
-			concat_strings(prefix_buf, sizeof prefix_buf,
-				"push://", guid_buf, (void *) 0);
+
+			guid_to_string_buf(s->guid, ARYLEN(guid_buf));
+			concat_strings(ARYLEN(prefix_buf), "push://", guid_buf, NULL_PTR);
 			prefix = prefix_buf;
 		} else {
 			prefix = "http://";
 		}
-		
+
 		port_buf[0] = '\0';
 		if (s->hostname) {
 			host = s->hostname;
 			if (80 != s->port) {
-				gm_snprintf(port_buf, sizeof port_buf, ":%u",
-					(unsigned) s->port);
+				str_bprintf(ARYLEN(port_buf), ":%u", (unsigned) s->port);
 			}
 		} else if (s->guid) {
-			host = proxies = proxies_to_string(s->proxies);
+			proxies = proxies_to_string(s->proxies);
+			host = proxies;
 		} else {
 			host = host_addr_port_to_string(s->addr, s->port);
 		}
 		if (s->path) {
-			url = g_strconcat(prefix, host, port_buf, s->path, (void *) 0);
+			url = g_strconcat(prefix, host, port_buf, s->path, NULL_PTR);
 		} else if (s->sha1) {
 			url = g_strconcat(prefix, host, port_buf,
 					"/uri-res/N2R?", bitprint_to_urn_string(s->sha1, s->tth),
-					(void *) 0);
+					NULL_PTR);
 		} else {
-			url = g_strconcat(prefix, host, port_buf, "/", (void *) 0);
+			url = g_strconcat(prefix, host, port_buf, "/", NULL_PTR);
 		}
 
-		G_FREE_NULL(proxies);
+		HFREE_NULL(proxies);
 	}
 
 	return url;
@@ -1009,53 +982,60 @@ magnet_source_to_string(const struct magnet_source *s)
 /**
  * Create a string representation of the magnet resource.
  *
- * @return A newly allocated string.
+ * @return A newly allocated string via halloc().
  */
 char *
 magnet_to_string(const struct magnet_resource *res)
 {
-	GString *gs;
-	GSList *sl;
+	pslist_t *sl;
+	str_t *s;
 
 	g_return_val_if_fail(res, NULL);
-	
-	gs = g_string_new(NULL);
+
+	s = str_new(0);
+
 	if (res->display_name) {
-		magnet_append_item(&gs, TRUE, "dn", res->display_name);
+		magnet_append_item(s, TRUE, "dn", res->display_name);
 	}
 	if (0 != res->size) {
 		char buf[UINT64_DEC_BUFLEN];
 
-		uint64_to_string_buf(res->size, buf, sizeof buf);
-		magnet_append_item(&gs, FALSE, "xl", buf);
+		uint64_to_string_buf(res->size, ARYLEN(buf));
+		magnet_append_item(s, FALSE, "xl", buf);
 	}
 	if (res->sha1) {
-		magnet_append_item(&gs, FALSE, "xt",
+		magnet_append_item(s, FALSE, "xt",
 			bitprint_to_urn_string(res->sha1, res->tth));
 	}
 	if (res->parq_id) {
-		magnet_append_item(&gs, TRUE, "x.parq-id", res->parq_id);
+		magnet_append_item(s, TRUE, "x.parq-id", res->parq_id);
 	}
 	if (res->vendor) {
-		magnet_append_item(&gs, TRUE, "x.vndr", res->vendor);
+		magnet_append_item(s, TRUE, "x.vndr", res->vendor);
 	}
 	if (res->guid) {
-		magnet_append_item(&gs, TRUE, "x.guid", res->guid);
+		magnet_append_item(s, TRUE, "x.guid", res->guid);
+	}
+	if (res->dht) {
+		magnet_append_item(s, TRUE, "x.dht", "1");
+	}
+	if (res->g2) {
+		magnet_append_item(s, TRUE, "x.g2", "1");
 	}
 
-	for (sl = res->sources; NULL != sl; sl = g_slist_next(sl)) {
+	PSLIST_FOREACH(res->sources, sl) {
 		char *url;
 
 		url = magnet_source_to_string(sl->data);
-		magnet_append_item(&gs, TRUE, "xs", url);
+		magnet_append_item(s, TRUE, "xs", url);
 		G_FREE_NULL(url);
 	}
 
-	for (sl = res->searches; NULL != sl; sl = g_slist_next(sl)) {
-		magnet_append_item(&gs, TRUE, "kt", sl->data);
+	PSLIST_FOREACH(res->searches, sl) {
+		magnet_append_item(s, TRUE, "kt", sl->data);
 	}
 
-	return gm_string_finalize(gs);
+	return str_s2c_null(&s);
 }
 
 /*
@@ -1127,4 +1107,30 @@ magnet_set_guid(struct magnet_resource *res, const char *guid)
 	magnet_resource_set_string(&res->guid, guid);
 }
 
+/**
+ * This is a bit of a hack (an extension anyway) and should only be used
+ * for magnets with a single logical source because DHT support is only
+ * valid for a certain source.
+ */
+void
+magnet_set_dht(struct magnet_resource *res, bool dht_support)
+{
+	g_return_if_fail(res);
+
+	res->dht = booleanize(dht_support);
+}
+
+/**
+ * This is a bit of a hack (an extension anyway) and should only be used
+ * for magnets with a single logical source because G2 support is only
+ * valid for a certain source.
+ */
+void
+magnet_set_g2(struct magnet_resource *res, bool g2)
+{
+	g_return_if_fail(res);
+
+	res->g2 = booleanize(g2);
+}
+
 /* vi: set ts=4 sw=4 cindent: */
