@@ -1,6 +1,4 @@
 /*
- * $Id: map.c 16666 2009-05-05 18:27:04Z rmanfredi $
- *
  * Copyright (c) 2008, Raphael Manfredi
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -35,15 +33,18 @@
 
 #include "common.h"
 
-RCSID("$Id: map.c 16666 2009-05-05 18:27:04Z rmanfredi $")
-
 #include "map.h"
-#include "debug.h"
-#include "misc.h"
-#include "patricia.h"
-#include "walloc.h"
-#include "tm.h"					/* For tests */
 #include "atoms.h"				/* For tests */
+#include "debug.h"
+#include "htable.h"
+#include "ohash_table.h"
+#include "patricia.h"
+#include "random.h"
+#include "stringify.h"			/* For plural() */
+#include "tm.h"					/* For tests */
+#include "walloc.h"
+#include "xmalloc.h"
+
 #include "override.h"			/* Must be the last header included */
 
 /**
@@ -52,6 +53,7 @@ RCSID("$Id: map.c 16666 2009-05-05 18:27:04Z rmanfredi $")
 enum map_type {
 	MAP_HASH = 0x1,			/* Hash table from glib */
 	MAP_PATRICIA,			/* PATRICIA tree */
+	MAP_ORDERED_HASH,		/* Ordered hash table */
 
 	MAP_MAXTYPE
 };
@@ -66,8 +68,9 @@ struct map {
 	enum map_magic magic;
 	enum map_type type;
 	union {
-		GHashTable *ht;
+		htable_t *ht;
 		patricia_t *pt;
+		ohash_table_t *ot;
 	} u;
 };
 
@@ -87,14 +90,35 @@ map_check(const map_t *m)
  * @return the new map
  */
 map_t *
-map_create_hash(GHashFunc hash_func, GEqualFunc key_eq_func)
+map_create_hash(hash_fn_t hash_func, eq_fn_t key_eq_func)
 {
 	map_t *m;
 
-	m = walloc(sizeof *m);
+	WALLOC(m);
 	m->magic = MAP_MAGIC;
 	m->type = MAP_HASH;
-	m->u.ht = g_hash_table_new(hash_func, key_eq_func);
+	m->u.ht = htable_create_any(hash_func, NULL, key_eq_func);
+
+	return m;
+}
+
+/**
+ * Create a map implemented as an ordered hash table.
+ *
+ * @param hash_func		the hash function for keys
+ * @param key_eq_func	the key comparison function
+ *
+ * @return the new map
+ */
+map_t *
+map_create_ordered_hash(hash_fn_t hash_func, eq_fn_t key_eq_func)
+{
+	map_t *m;
+
+	WALLOC(m);
+	m->magic = MAP_MAGIC;
+	m->type = MAP_ORDERED_HASH;
+	m->u.ot = ohash_table_new(hash_func, key_eq_func);
 
 	return m;
 }
@@ -109,7 +133,7 @@ map_create_patricia(size_t keybits)
 {
 	map_t *m;
 
-	m = walloc(sizeof *m);
+	WALLOC(m);
 	m->magic = MAP_MAGIC;
 	m->type = MAP_PATRICIA;
 	m->u.pt = patricia_create(keybits);
@@ -122,13 +146,13 @@ map_create_patricia(size_t keybits)
  * Use map_release() to discard the map encapsulation.
  */
 map_t *
-map_create_from_hash(GHashTable *ht)
+map_create_from_hash(htable_t *ht)
 {
 	map_t *m;
 
 	g_assert(ht);
 
-	m = walloc(sizeof *m);
+	WALLOC(m);
 	m->magic = MAP_MAGIC;
 	m->type = MAP_HASH;
 	m->u.ht = ht;
@@ -136,6 +160,25 @@ map_create_from_hash(GHashTable *ht)
 	return m;
 }
 
+/**
+ * Create a map out of an existing ordered hash table.
+ * Use map_release() to discard the map encapsulation.
+ */
+map_t *
+map_create_from_ordered_hash(ohash_table_t *ot)
+{
+	map_t *m;
+
+	g_assert(ot);
+
+	WALLOC(m);
+	m->magic = MAP_MAGIC;
+	m->type = MAP_ORDERED_HASH;
+	m->u.ot = ot;
+
+	return m;
+}
+
 /**
  * Create a map out of an existing PATRICIA tree.
  * Use map_release() to discard the map encapsulation.
@@ -147,7 +190,7 @@ map_create_from_patricia(patricia_t *pt)
 
 	g_assert(pt);
 
-	m = walloc(sizeof *m);
+	WALLOC(m);
 	m->magic = MAP_MAGIC;
 	m->type = MAP_PATRICIA;
 	m->u.pt = pt;
@@ -159,10 +202,10 @@ map_create_from_patricia(patricia_t *pt)
  * Switch the implementation of an existing map to a hash table.
  * Returns the previous implementation.
  */
-gpointer
-map_switch_to_hash(map_t *m, GHashTable *ht)
+void *
+map_switch_to_hash(map_t *m, htable_t *ht)
 {
-	gpointer implementation;
+	void *implementation;
 
 	map_check(m);
 	g_assert(ht);
@@ -174,14 +217,33 @@ map_switch_to_hash(map_t *m, GHashTable *ht)
 	return implementation;
 }
 
+/**
+ * Switch the implementation of an existing map to an ordered hash table.
+ * Returns the previous implementation.
+ */
+void *
+map_switch_to_ordered_hash(map_t *m, ohash_table_t *ot)
+{
+	void *implementation;
+
+	map_check(m);
+	g_assert(ot);
+
+	implementation = map_implementation(m);
+	m->type = MAP_ORDERED_HASH;
+	m->u.ot = ot;
+
+	return implementation;
+}
+
 /**
  * Switch the implementation of an existing map to a PATRICIA tree.
  * Returns the previous implementation.
  */
-gpointer
+void *
 map_switch_to_patricia(map_t *m, patricia_t *pt)
 {
-	gpointer implementation;
+	void *implementation;
 
 	map_check(m);
 	g_assert(pt);
@@ -197,13 +259,16 @@ map_switch_to_patricia(map_t *m, patricia_t *pt)
  * Insert a key/value pair in the map.
  */
 void
-map_insert(const map_t *m, gconstpointer key, gconstpointer value)
+map_insert(const map_t *m, const void *key, const void *value)
 {
 	map_check(m);
 
 	switch (m->type) {
 	case MAP_HASH:
-		gm_hash_table_insert_const(m->u.ht, key, value);
+		htable_insert_const(m->u.ht, key, value);
+		break;
+	case MAP_ORDERED_HASH:
+		ohash_table_insert(m->u.ot, key, value);
 		break;
 	case MAP_PATRICIA:
 		patricia_insert(m->u.pt, key, value);
@@ -217,13 +282,16 @@ map_insert(const map_t *m, gconstpointer key, gconstpointer value)
  * Replace a key/value pair in the map.
  */
 void
-map_replace(const map_t *m, gconstpointer key, gconstpointer value)
+map_replace(const map_t *m, const void *key, const void *value)
 {
 	map_check(m);
 
 	switch (m->type) {
 	case MAP_HASH:
-		gm_hash_table_replace_const(m->u.ht, key, value);
+		htable_insert_const(m->u.ht, key, value);
+		break;
+	case MAP_ORDERED_HASH:
+		ohash_table_replace(m->u.ot, key, value);
 		break;
 	case MAP_PATRICIA:
 		patricia_insert(m->u.pt, key, value);		/* Does replace */
@@ -238,14 +306,17 @@ map_replace(const map_t *m, gconstpointer key, gconstpointer value)
  *
  * @return TRUE if the key was found and removed from the map.
  */
-gboolean
-map_remove(const map_t *m, gconstpointer key)
+bool
+map_remove(const map_t *m, const void *key)
 {
 	map_check(m);
 
 	switch (m->type) {
 	case MAP_HASH:
-		return gm_hash_table_remove(m->u.ht, key);
+		return htable_remove(m->u.ht, key);
+		break;
+	case MAP_ORDERED_HASH:
+		return ohash_table_remove(m->u.ot, key);
 		break;
 	case MAP_PATRICIA:
 		return patricia_remove(m->u.pt, key);
@@ -261,14 +332,16 @@ map_remove(const map_t *m, gconstpointer key)
 /**
  * Check whether map contains the key.
  */
-gboolean
-map_contains(const map_t *m, gconstpointer key)
+bool
+map_contains(const map_t *m, const void *key)
 {
 	map_check(m);
 
 	switch (m->type) {
 	case MAP_HASH:
-		return g_hash_table_lookup_extended(m->u.ht, key, NULL, NULL);
+		return htable_contains(m->u.ht, key);
+	case MAP_ORDERED_HASH:
+		return ohash_table_contains(m->u.ot, key);
 	case MAP_PATRICIA:
 		return patricia_contains(m->u.pt, key);
 	case MAP_MAXTYPE:
@@ -280,14 +353,16 @@ map_contains(const map_t *m, gconstpointer key)
 /**
  * Lookup a key in the map.
  */
-gpointer
-map_lookup(const map_t *m, gconstpointer key)
+void *
+map_lookup(const map_t *m, const void *key)
 {
 	map_check(m);
 
 	switch (m->type) {
 	case MAP_HASH:
-		return g_hash_table_lookup(m->u.ht, key);
+		return htable_lookup(m->u.ht, key);
+	case MAP_ORDERED_HASH:
+		return ohash_table_lookup(m->u.ot, key);
 	case MAP_PATRICIA:
 		return patricia_lookup(m->u.pt, key);
 	case MAP_MAXTYPE:
@@ -306,7 +381,9 @@ map_count(const map_t *m)
 
 	switch (m->type) {
 	case MAP_HASH:
-		return g_hash_table_size(m->u.ht);
+		return htable_count(m->u.ht);
+	case MAP_ORDERED_HASH:
+		return ohash_table_count(m->u.ot);
 	case MAP_PATRICIA:
 		return patricia_count(m->u.pt);
 	case MAP_MAXTYPE:
@@ -318,15 +395,16 @@ map_count(const map_t *m)
 /**
  * Extended lookup of a key in the map, returning both key/value pointers.
  */
-gboolean
-map_lookup_extended(const map_t *m, gconstpointer key,
-	gpointer *okey, gpointer *oval)
+bool
+map_lookup_extended(const map_t *m, const void *key, void **okey, void **oval)
 {
 	map_check(m);
 
 	switch (m->type) {
 	case MAP_HASH:
-		return g_hash_table_lookup_extended(m->u.ht, key, okey, oval);
+		return htable_lookup_extended(m->u.ht, key, (const void **) okey, oval);
+	case MAP_ORDERED_HASH:
+		return ohash_table_lookup_extended(m->u.ot, key, okey, oval);
 	case MAP_PATRICIA:
 		return patricia_lookup_extended(m->u.pt, key, okey, oval);
 	case MAP_MAXTYPE:
@@ -335,19 +413,49 @@ map_lookup_extended(const map_t *m, gconstpointer key,
 	return FALSE;
 }
 
+/**
+ * Mark map as thread-safe.
+ *
+ * If the underlying implementation does not implement thread-safety, this
+ * causes a fatal error.
+ */
+void
+map_thread_safe(const map_t *m)
+{
+	const char *type = NULL;
+
+	map_check(m);
+
+	switch (m->type) {
+	case MAP_HASH:
+		htable_thread_safe(m->u.ht);
+		return;
+	case MAP_ORDERED_HASH:
+		type = "ordered hash";
+		break;
+	case MAP_PATRICIA:
+		type = "PATRICIA";
+		break;
+	case MAP_MAXTYPE:
+		g_assert_not_reached();
+	}
+
+	s_error("%s(): %s implementation is not thread-safe yet", G_STRFUNC, type);
+}
+
 /**
  * Structure used to handle foreach() trampoline for PATRICIA.
  */
 struct pat_foreach {
-	map_cb_t cb;		/* Registered user callback */
-	gpointer u;			/* User callback additional arg */
+	keyval_fn_t cb;		/* Registered user callback */
+	void *u;			/* User callback additional arg */
 };
 
 /**
  * foreach() trampoline for PATRICIA.
  */
 static void
-pat_foreach_wrapper(gpointer key, size_t u_keybits, gpointer value, gpointer u)
+pat_foreach_wrapper(void *key, size_t u_keybits, void *value, void *u)
 {
 	struct pat_foreach *ctx = u;
 
@@ -360,14 +468,17 @@ pat_foreach_wrapper(gpointer key, size_t u_keybits, gpointer value, gpointer u)
  * Iterate on each item of the map, applying callback.
  */
 void
-map_foreach(const map_t *m, map_cb_t cb, gpointer u)
+map_foreach(const map_t *m, keyval_fn_t cb, void *u)
 {
 	map_check(m);
 	g_assert(cb);
 
 	switch (m->type) {
 	case MAP_HASH:
-		g_hash_table_foreach(m->u.ht, cb, u);
+		htable_foreach(m->u.ht, (ckeyval_fn_t) cb, u);
+		break;
+	case MAP_ORDERED_HASH:
+		ohash_table_foreach(m->u.ot, cb, u);
 		break;
 	case MAP_PATRICIA:
 		{
@@ -388,16 +499,15 @@ map_foreach(const map_t *m, map_cb_t cb, gpointer u)
  * Structure used to handle foreach_remove() trampoline for PATRICIA.
  */
 struct pat_foreach_remove {
-	map_cbr_t cb;		/* Registered user callback */
-	gpointer u;			/* User callback additional arg */
+	keyval_rm_fn_t cb;	/* Registered user callback */
+	void *u;			/* User callback additional arg */
 };
 
 /**
  * foreach() trampoline for PATRICIA.
  */
-static gboolean
-pat_foreach_remove_wrapper(
-	gpointer key, size_t u_keybits, gpointer value, gpointer u)
+static bool
+pat_foreach_remove_wrapper(void *key, size_t u_keybits, void *value, void *u)
 {
 	struct pat_foreach_remove *ctx = u;
 
@@ -411,14 +521,17 @@ pat_foreach_remove_wrapper(
  *
  * @return the amount of items deleted.
  */
-size_t map_foreach_remove(const map_t *m, map_cbr_t cb, gpointer u)
+size_t
+map_foreach_remove(const map_t *m, keyval_rm_fn_t cb, void *u)
 {
 	map_check(m);
 	g_assert(cb);
 
 	switch (m->type) {
 	case MAP_HASH:
-		return g_hash_table_foreach_remove(m->u.ht, cb, u);
+		return htable_foreach_remove(m->u.ht, (ckeyval_rm_fn_t) cb, u);
+	case MAP_ORDERED_HASH:
+		return ohash_table_foreach_remove(m->u.ot, cb, u);
 	case MAP_PATRICIA:
 		{
 			struct pat_foreach_remove ctx;
@@ -438,7 +551,7 @@ size_t map_foreach_remove(const map_t *m, map_cbr_t cb, gpointer u)
 /**
  * Returns the underlying map implementation.
  */
-gpointer
+void *
 map_implementation(const map_t *m)
 {
 	map_check(m);
@@ -446,6 +559,8 @@ map_implementation(const map_t *m)
 	switch (m->type) {
 	case MAP_HASH:
 		return m->u.ht;
+	case MAP_ORDERED_HASH:
+		return m->u.ot;
 	case MAP_PATRICIA:
 		return m->u.pt;
 	case MAP_MAXTYPE:
@@ -455,15 +570,14 @@ map_implementation(const map_t *m)
 	return NULL;
 }
 
-
 /**
  * Release the map encapsulation, returning the underlying implementation
  * object (will need to be cast back to the proper type for perusal).
  */
-gpointer
+void *
 map_release(map_t *m)
 {
-	gpointer implementation;
+	void *implementation;
 
 	map_check(m);
 
@@ -471,7 +585,7 @@ map_release(map_t *m)
 
 	m->type = MAP_MAXTYPE;
 	m->magic = 0;
-	wfree(m, sizeof *m);
+	WFREE(m);
 
 	return implementation;
 }
@@ -486,7 +600,10 @@ map_destroy(map_t *m)
 
 	switch (m->type) {
 	case MAP_HASH:
-		g_hash_table_destroy(m->u.ht);
+		htable_free_null(&m->u.ht);
+		break;
+	case MAP_ORDERED_HASH:
+		ohash_table_destroy_null(&m->u.ot);
 		break;
 	case MAP_PATRICIA:
 		patricia_destroy(m->u.pt);
@@ -497,7 +614,21 @@ map_destroy(map_t *m)
 
 	m->type = MAP_MAXTYPE;
 	m->magic = 0;
-	wfree(m, sizeof *m);
+	WFREE(m);
+}
+
+/**
+ * Destroy a map, nullifying its pointer.
+ */
+void
+map_destroy_null(map_t **m_ptr)
+{
+	map_t *m = *m_ptr;
+
+	if (m != NULL) {
+		map_destroy(m);
+		*m_ptr = NULL;
+	}
 }
 
 /***
@@ -541,11 +672,11 @@ test_map_remove(void *o, sha1_t *keys, size_t count)
 		map_remove(m, &keys[i]);
 }
 
-static double
+static double G_COLD
 timeit(
 	void (*f)(void *, sha1_t *, size_t),
 	void *o, sha1_t *keys, size_t count, size_t iter, const char *what,
-	gboolean verbose)
+	bool verbose)
 {
 	size_t i;
 	tm_t start, end;
@@ -558,14 +689,13 @@ timeit(
 	elapsed = tm_elapsed_f(&end, &start);
 
 	if (verbose)
-		g_message("%s (%lu items, %lu loop%s): %f s (average: %f s)", what,
-			(unsigned long) count, (gulong) iter, iter == 1 ? "" : "s",
-			elapsed, elapsed / iter);
+		g_debug("%s (%zu items, %zu loop%s): %F s (average: %F s)", what,
+			count, PLURAL(iter), elapsed, elapsed / iter);
 
 	return elapsed;
 }
 
-void
+void G_COLD
 map_test(void)
 {
 	sha1_t *keys;
@@ -576,12 +706,12 @@ map_test(void)
 	struct {
 		unsigned insertion, contains, removal;
 	} faster = { 0, 0, 0};
-	gboolean verbose = common_stats > 1;
+	bool verbose = common_stats > 1;
 
 	if (common_stats <= 0)
 		return;
 
-	keys = g_malloc(ITEM_COUNT * sizeof *keys);
+	XMALLOC_ARRAY(keys, ITEM_COUNT);
 
 	for (i = 0; i < ITEM_COUNT; i++)
 		random_bytes(keys[i].data, SHA1_RAW_SIZE);
@@ -614,8 +744,8 @@ map_test(void)
 			1, "map PATRICIA reloading", verbose);
 
 		if (verbose)
-			g_message("PATRICIA insertion %s than hash with %lu items",
-				ptime < htime ? "faster" : "slower", (unsigned long) count);
+			g_info("PATRICIA insertion %s than hash with %zu items",
+				ptime < htime ? "faster" : "slower", count);
 
 		if (ptime < htime)
 			faster.insertion++;
@@ -627,8 +757,8 @@ map_test(void)
 			LOOPS, "map PATRICIA contains", verbose);
 
 		if (verbose)
-			g_message("PATRICIA contains %s than hash with %lu items",
-				ptime < htime ? "faster" : "slower", (unsigned long) count);
+			g_info("PATRICIA contains %s than hash with %zu items",
+				ptime < htime ? "faster" : "slower", count);
 
 		if (ptime < htime)
 			faster.contains++;
@@ -640,8 +770,8 @@ map_test(void)
 			1, "map PATRICIA remove", verbose);
 
 		if (verbose)
-			g_message("PATRICIA remove %s than hash with %lu items",
-				ptime < htime ? "faster" : "slower", (unsigned long) count);
+			g_info("PATRICIA remove %s than hash with %zu items",
+				ptime < htime ? "faster" : "slower", count);
 
 		if (ptime < htime)
 			faster.removal++;
@@ -651,17 +781,17 @@ map_test(void)
 	}
 
 	if (faster.insertion)
-		g_message("PATRICIA insert was faster than hash in %d out of %d tests",
+		g_info("PATRICIA insert was faster than hash in %d out of %d tests",
 			faster.insertion, tests);
 	if (faster.contains)
-		g_message(
+		g_info(
 			"PATRICIA contains was faster than hash in %d out of %d tests",
 			faster.contains, tests);
 	if (faster.removal)
-		g_message("PATRICIA remove was faster than hash in %d out of %d tests",
+		g_info("PATRICIA remove was faster than hash in %d out of %d tests",
 			faster.removal, tests);
 
-	G_FREE_NULL(keys);
+	XFREE_NULL(keys);
 }
 
 /* vi: set ts=4 sw=4 cindent: */
