@@ -1,6 +1,4 @@
 /*
- * $Id: pmsg.h 16596 2009-04-30 16:50:53Z rmanfredi $
- *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -62,29 +60,56 @@
  * kind.
  */
 
-typedef void (*pdata_free_t)(gpointer p, gpointer arg);
+typedef void (*pdata_free_t)(void *p, void *arg);
+
+enum pdata_magic { PDATA_MAGIC = 0x0ad505b9 };
 
 typedef struct pdata {
+	enum pdata_magic magic;
 	pdata_free_t d_free;		/**< Free routine */
-	gpointer d_arg;				/**< Argument to free routine */
+	void *d_arg;				/**< Argument to free routine */
 	int d_refcnt;				/**< Reference count */
 	char *d_arena;				/**< First byte in buffer */
 	char *d_end;				/**< First byte after buffer */
 	char d_embedded[1];			/**< Start of embedded arena */
 } pdata_t;
 
-#define pdata_start(x)		((x)->d_arena)
-#define pdata_len(x)		((size_t) ((x)->d_end - (x)->d_arena))
-#define pdata_addref(x)		do { (x)->d_refcnt++; } while (0)
+static inline void
+pdata_check(const pdata_t * const pd)
+{
+	g_assert(pd != NULL);
+	g_assert(PDATA_MAGIC == pd->magic);
+}
+
+static inline char *
+pdata_start(const pdata_t *pd)
+{
+	pdata_check(pd);
+	return pd->d_arena;
+}
+
+static inline size_t
+pdata_len(const pdata_t *pd)
+{
+	pdata_check(pd);
+	return ptr_diff(pd->d_end, pd->d_arena);
+}
+
+static inline void
+pdata_addref(pdata_t *pd)
+{
+	pdata_check(pd);
+	pd->d_refcnt++;
+}
 
 /*
  * A message block
  */
 
-struct mqueue;
-
 typedef struct pmsg pmsg_t;
-typedef gboolean (*pmsg_check_t)(pmsg_t *mb, const struct mqueue *q);
+
+typedef bool (*pmsg_check_t)(const pmsg_t *mb, const void *arg);
+typedef bool (*pmsg_hook_t)(const pmsg_t *mb);
 
 enum pmsg_magic {
 	PMSG_MAGIC		= 0x2fa50be3U,
@@ -96,29 +121,19 @@ struct pmsg {
 	const char *m_rptr;			/**< First unread byte in buffer */
 	char *m_wptr;				/**< First unwritten byte in buffer */
 	pdata_t *m_data;			/**< Data buffer */
-	guint m_prio;				/**< Message priority (0 = normal) */
-	pmsg_check_t m_check;		/**< Optional check before sending */
+	uint8 m_flags;				/**< Message flags */
+	uint8 m_prio;				/**< Message priority (0 = normal) */
+	uint16 m_refcnt;			/**< Refs to this message block */
+	union {
+		pmsg_check_t m_check;	/**< Optional check before sending */
+		pmsg_hook_t m_hook;		/**< Optional check before transmitting */
+	} m_u;
 };
 
-typedef void (*pmsg_free_t)(pmsg_t *mb, gpointer arg);
-
-#define PMSG_PRIO_MASK		0x00ffffff	/**< Only lower bits are relevant */
-
-#define pmsg_start(x)		((x)->m_data->d_arena)
-#define pmsg_phys_len(x)	pdata_len((x)->m_data)
-#define pmsg_is_writable(x)	((x)->m_data->d_refcnt == 1)
-#define pmsg_prio(x)		((x)->m_prio & PMSG_PRIO_MASK)
-
-#define pmsg_is_unread(x)	((x)->m_rptr == (x)->m_data->d_arena)
-#define pmsg_read_base(x)	((x)->m_rptr)
-
-#define pmsg_check(x,y)		((x)->m_check ? (x)->m_check((x), (y)) : TRUE)
-
-/* Available room for pmsg_write() calls */
-#define pmsg_available(x)	((x)->m_data->d_end - (x)->m_wptr)
+typedef void (*pmsg_free_t)(pmsg_t *mb, void *arg);
 
 /*
- * Message priorities.
+ * Message priorities (16-bit).
  */
 
 #define PMSG_P_DATA		0			/**< Regular data, lowest priority */
@@ -126,72 +141,247 @@ typedef void (*pmsg_free_t)(pmsg_t *mb, gpointer arg);
 #define PMSG_P_URGENT	2			/**< Urgent message */
 #define PMSG_P_HIGHEST	3			/**< Highest priority */
 
+#define PMSG_P_COUNT	4			/**< Amount of priorities defined */
+
 /*
- * Flags defined in highest bits of `m_prio'.
+ * Message flags.
  */
 
-#define PMSG_PF_EXT		0x80000000	/**< Message block uses extended form */
-#define PMSG_PF_SENT	0x40000000	/**< Message was successfully sent */
+#define PMSG_PF_EXT		(1U << 7)	/**< Message block uses extended form */
+#define PMSG_PF_SENT	(1U << 6)	/**< Message was successfully sent */
+#define PMSG_PF_ACKME	(1U << 5)	/**< Request remote acknowledgment */
+#define PMSG_PF_COMP	(1U << 4)	/**< Compression already attempted / done */
+#define PMSG_PF_HOOK	(1U << 3)	/**< Use ``m_check'' as standalone hook */
 
-#define pmsg_is_extended(mb) ((mb)->m_prio & PMSG_PF_EXT)
-#define pmsg_was_sent(mb) ((mb)->m_prio & PMSG_PF_SENT)
-#define pmsg_mark_sent(mb) \
-G_STMT_START { \
-	(mb)->m_prio |= PMSG_PF_SENT; \
-} G_STMT_END
+static inline void
+pmsg_check(const pmsg_t * const mb)
+{
+	g_assert(mb != NULL);
+	g_assert((PMSG_MAGIC == mb->magic) ^ (PMSG_EXT_MAGIC == mb->magic));
+	g_assert((PMSG_MAGIC == mb->magic) ^ (0 != (PMSG_PF_EXT & mb->m_flags)));
+}
+
+static inline char *
+pmsg_phys_base(const pmsg_t *mb)
+{
+	pmsg_check(mb);
+	pdata_check(mb->m_data);
+
+	return mb->m_data->d_arena;
+}
+
+static inline size_t
+pmsg_phys_len(const pmsg_t *mb)
+{
+	pmsg_check(mb);
+	return pdata_len(mb->m_data);
+}
+
+static inline bool
+pmsg_is_writable(const pmsg_t *mb)
+{
+	pmsg_check(mb);
+	pdata_check(mb->m_data);
+	return 1 == mb->m_data->d_refcnt;
+}
+
+static inline unsigned
+pmsg_prio(const pmsg_t *mb)
+{
+	pmsg_check(mb);
+	return mb->m_prio;
+}
+
+static inline unsigned
+pmsg_refcnt(const pmsg_t *mb)
+{
+	pmsg_check(mb);
+	return mb->m_refcnt;
+}
+
+static inline bool
+pmsg_is_unread(const pmsg_t *mb)
+{
+	pmsg_check(mb);
+	pdata_check(mb->m_data);
+	return mb->m_rptr == mb->m_data->d_arena;
+}
+
+static inline const char *
+pmsg_start(const pmsg_t *mb)
+{
+	pmsg_check(mb);
+	return mb->m_rptr;
+}
+
+/**
+ * Pre-send checks.
+ */
+static inline bool
+pmsg_can_send(const pmsg_t *mb, const void *arg)
+{
+	pmsg_check(mb);
+	return (NULL != mb->m_u.m_check && !(mb->m_flags & PMSG_PF_HOOK)) ?
+		mb->m_u.m_check(mb, arg) : TRUE;
+}
+
+/**
+ * Pre-transmit hook.
+ */
+static inline bool
+pmsg_can_transmit(const pmsg_t *mb)
+{
+	pmsg_check(mb);
+	return (NULL != mb->m_u.m_hook && (mb->m_flags & PMSG_PF_HOOK)) ?
+		mb->m_u.m_hook(mb) : TRUE;
+}
+
+/**
+ * Available room for pmsg_write() calls
+ */
+static inline size_t
+pmsg_available(const pmsg_t *mb)
+{
+	pmsg_check(mb);
+	pdata_check(mb->m_data);
+	return ptr_diff(mb->m_data->d_end, mb->m_wptr);
+}
+
+static inline bool
+pmsg_is_extended(const pmsg_t *mb)
+{
+	pmsg_check(mb);
+	return 0 != (mb->m_flags & PMSG_PF_EXT);
+}
+
+static inline bool
+pmsg_was_sent(const pmsg_t *mb)
+{
+	pmsg_check(mb);
+	return 0 != (mb->m_flags & PMSG_PF_SENT);
+}
+
+
+static inline bool
+pmsg_is_reliable(const pmsg_t *mb)
+{
+	pmsg_check(mb);
+	return 0 != (mb->m_flags & PMSG_PF_ACKME);
+}
+
+static inline bool
+pmsg_is_compressed(const pmsg_t *mb)
+{
+	pmsg_check(mb);
+	return 0 != (mb->m_flags & PMSG_PF_COMP);
+}
+
+/**
+ * TX layer marks message as being sent when it was sent over the network.
+ */
+static inline void
+pmsg_mark_sent(pmsg_t *mb)
+{
+	mb->m_flags |= PMSG_PF_SENT;
+}
+
+/**
+ * Clear the "sent" marker on message.
+ */
+static inline void
+pmsg_clear_sent(pmsg_t *mb)
+{
+	mb->m_flags &= ~PMSG_PF_SENT;
+}
+
+/**
+ * On unreliable medium (e.g. UDP), flag message as requiring a reliable
+ * transmission, if possible.  The message will be marked as "sent" only
+ * if it was acknowledged.
+ */
+static inline void
+pmsg_mark_reliable(pmsg_t *mb)
+{
+	mb->m_flags |= PMSG_PF_ACKME;
+}
+
+/**
+ * Clear the reliable marker.
+ */
+static inline void
+pmsg_clear_reliable(pmsg_t *mb)
+{
+	mb->m_flags &= ~PMSG_PF_ACKME;
+}
+
+/**
+ * Mark message as "compressed", whether or not it actually is.
+ *
+ * This signals the TX layers that the data has either already been compressed
+ * or that compression was already attempted and did not yield any significant
+ * gain.  Hence it would be a waste of time to attempt a compression again.
+ */
+static inline void
+pmsg_mark_compressed(pmsg_t *mb)
+{
+	mb->m_flags |= PMSG_PF_COMP;
+}
 
 /*
  * Public interface
  */
 
-static inline void
-pmsg_check_consistency(const pmsg_t * const mb)
-{
-	g_assert(mb);
-	g_assert((PMSG_MAGIC == mb->magic) ^ (PMSG_EXT_MAGIC == mb->magic));
-	g_assert((PMSG_MAGIC == mb->magic) ^ (0 != (PMSG_PF_EXT & mb->m_prio)));
-}
-
 void pmsg_init(void);
 void pmsg_close(void);
 
-pmsg_t *pmsg_new(int prio, gconstpointer buf, int len);
+pmsg_t *pmsg_new(int prio, const void *buf, int len);
 pmsg_t * pmsg_new_extend(
-	int prio, gconstpointer buf, int len,
-	pmsg_free_t free_cb, gpointer arg);
+	int prio, const void *buf, int len,
+	pmsg_free_t free_cb, void *arg);
 pmsg_t *pmsg_alloc(int prio, pdata_t *db, int roff, int woff);
-pmsg_t *pmsg_clone(pmsg_t *mb);
-pmsg_t *pmsg_clone_extend(pmsg_t *mb, pmsg_free_t free_cb, gpointer arg);
+pmsg_t *pmsg_ref(pmsg_t *mb);
+pmsg_t *pmsg_clone(const pmsg_t *mb);
+pmsg_t *pmsg_clone_plain(const pmsg_t *mb);
+pmsg_t *pmsg_clone_extend(const pmsg_t *mb, pmsg_free_t free_cb, void *arg);
 pmsg_free_t pmsg_replace_ext(
-	pmsg_t *mb, pmsg_free_t nfree, gpointer narg, gpointer *oarg);
-gpointer pmsg_get_metadata(pmsg_t *mb);
-pmsg_check_t pmsg_set_check(pmsg_t *mb, pmsg_check_t check);
+	pmsg_t *mb, pmsg_free_t nfree, void *narg, void **oarg);
+void *pmsg_get_metadata(const pmsg_t *mb);
+void pmsg_set_send_callback(pmsg_t *mb, pmsg_check_t check);
+void pmsg_set_transmit_hook(pmsg_t *mb, pmsg_hook_t hook);
 void pmsg_free(pmsg_t *mb);
 void pmsg_free_null(pmsg_t **mb_ptr);
-int pmsg_write(pmsg_t *mb, gconstpointer data, int len);
+int pmsg_write(pmsg_t *mb, const void *data, int len);
 int pmsg_writable_length(const pmsg_t *mb);
-int pmsg_read(pmsg_t *mb, gpointer data, int len);
+int pmsg_read(pmsg_t *mb, void *data, int len);
 int pmsg_discard(pmsg_t *mb, int len);
 int pmsg_discard_trailing(pmsg_t *mb, int len);
+int pmsg_copy(pmsg_t *dest, pmsg_t *src, int len);
+pmsg_t *pmsg_split(pmsg_t *mb, int offset);
+void pmsg_compact(pmsg_t *mb);
+void pmsg_fractional_compact(pmsg_t *mb, int n);
 void pmsg_reset(pmsg_t *mb);
 
 pdata_t *pdata_new(int len);
 pdata_t *pdata_allocb(void *buf, int len,
-	pdata_free_t freecb, gpointer freearg);
+	pdata_free_t freecb, void *freearg);
 pdata_t *pdata_allocb_ext(void *buf, int len,
-	pdata_free_t freecb, gpointer freearg);
-void pdata_free_nop(gpointer p, gpointer arg);
+	pdata_free_t freecb, void *freearg);
+void pdata_free_nop(void *p, void *arg);
 void pdata_unref(pdata_t *db);
 
-struct iovec *pmsg_slist_to_iovec(slist_t *slist,
+iovec_t *pmsg_slist_to_iovec(slist_t *slist,
 				int *iovcnt_ptr, size_t *size_ptr);
 void pmsg_slist_discard(slist_t *slist, size_t n_bytes);
 void pmsg_slist_append(slist_t *slist, const void *data, size_t n_bytes);
+size_t pmsg_slist_size(const slist_t *slist);
+size_t pmsg_slist_read(slist_t *slist, void *buf, size_t len);
+void pmsg_slist_discard_all(slist_t *slist);
+void pmsg_slist_free_all(slist_t **slist_ptr);
 
 static inline void
 pmsg_slist_free(slist_t **slist_ptr)
 {
-	slist_free_all(slist_ptr, (slist_destroy_cb) pmsg_free);
+	slist_free_all(slist_ptr, (free_fn_t) pmsg_free);
 }
 
 /**
@@ -277,19 +467,19 @@ pmsg_seek(pmsg_t *mb, pmsg_offset_t offset)
  * Write a single byte.
  */
 static inline void
-pmsg_write_u8(pmsg_t *mb, guint8 val)
+pmsg_write_u8(pmsg_t *mb, uint8 val)
 {
 	g_assert(pmsg_is_writable(mb));	/* Not shared, or would corrupt data */
 	g_assert(pmsg_available(mb) >= 1);
 
-	*(guint8 *) mb->m_wptr++ = val;
+	*(uint8 *) mb->m_wptr++ = val;
 }
 
 /**
  * Write a 16-bit value in big-endian format.
  */
 static inline void
-pmsg_write_be16(pmsg_t *mb, guint16 val)
+pmsg_write_be16(pmsg_t *mb, uint16 val)
 {
 	g_assert(pmsg_is_writable(mb));	/* Not shared, or would corrupt data */
 	g_assert(pmsg_available(mb) >= 2);
@@ -301,7 +491,7 @@ pmsg_write_be16(pmsg_t *mb, guint16 val)
  * Write a 16-bit value in little-endian format.
  */
 static inline void
-pmsg_write_le16(pmsg_t *mb, guint16 val)
+pmsg_write_le16(pmsg_t *mb, uint16 val)
 {
 	g_assert(pmsg_is_writable(mb));	/* Not shared, or would corrupt data */
 	g_assert(pmsg_available(mb) >= 2);
@@ -313,7 +503,7 @@ pmsg_write_le16(pmsg_t *mb, guint16 val)
  * Write a 32-bit value in big-endian format.
  */
 static inline void
-pmsg_write_be32(pmsg_t *mb, guint32 val)
+pmsg_write_be32(pmsg_t *mb, uint32 val)
 {
 	g_assert(pmsg_is_writable(mb));	/* Not shared, or would corrupt data */
 	g_assert(pmsg_available(mb) >= 4);
@@ -325,7 +515,7 @@ pmsg_write_be32(pmsg_t *mb, guint32 val)
  * Write a 32-bit value in little-endian format.
  */
 static inline void
-pmsg_write_le32(pmsg_t *mb, guint32 val)
+pmsg_write_le32(pmsg_t *mb, uint32 val)
 {
 	g_assert(pmsg_is_writable(mb));	/* Not shared, or would corrupt data */
 	g_assert(pmsg_available(mb) >= 4);
@@ -337,7 +527,7 @@ pmsg_write_le32(pmsg_t *mb, guint32 val)
  * Write a 64-bit value in big-endian format.
  */
 static inline void
-pmsg_write_be64(pmsg_t *mb, guint64 val)
+pmsg_write_be64(pmsg_t *mb, uint64 val)
 {
 	g_assert(pmsg_is_writable(mb));	/* Not shared, or would corrupt data */
 	g_assert(pmsg_available(mb) >= 8);
@@ -349,7 +539,7 @@ pmsg_write_be64(pmsg_t *mb, guint64 val)
  * Write a 64-bit value in little-endian format.
  */
 static inline void
-pmsg_write_le64(pmsg_t *mb, guint64 val)
+pmsg_write_le64(pmsg_t *mb, uint64 val)
 {
 	g_assert(pmsg_is_writable(mb));	/* Not shared, or would corrupt data */
 	g_assert(pmsg_available(mb) >= 8);
@@ -366,19 +556,19 @@ pmsg_write_time(pmsg_t *mb, time_t val)
 	g_assert(pmsg_is_writable(mb));	/* Not shared, or would corrupt data */
 	g_assert(pmsg_available(mb) >= 4);
 
-	mb->m_wptr = poke_be32(mb->m_wptr, (guint32) val);
+	mb->m_wptr = poke_be32(mb->m_wptr, (uint32) val);
 }
 
 /**
- * Write gboolean.
+ * Write boolean.
  */
 static inline void
-pmsg_write_boolean(pmsg_t *mb, gboolean val)
+pmsg_write_boolean(pmsg_t *mb, bool val)
 {
 	g_assert(pmsg_is_writable(mb));	/* Not shared, or would corrupt data */
 	g_assert(pmsg_available(mb) >= 1);
 
-	*(guint8 *) mb->m_wptr++ = val ? 1 : 0;
+	*(uint8 *) mb->m_wptr++ = booleanize(val);
 }
 
 /**
@@ -393,7 +583,10 @@ pmsg_write_float_be(pmsg_t *mb, float val)
 	mb->m_wptr = poke_float_be32(mb->m_wptr, val);
 }
 
+void pmsg_write_ule64(pmsg_t *mb, uint64 val);
+void pmsg_write_fixed_string(pmsg_t *mb, const char *str, size_t n);
 void pmsg_write_ipv4_or_ipv6_addr(pmsg_t *mb, host_addr_t addr);
+void pmsg_write_string(pmsg_t *mb, const char *str, size_t length);
 
 #endif	/* _pmsg_h_ */
 
