@@ -1,6 +1,4 @@
 /*
- * $Id: sequence.c 17291 2010-02-25 20:20:39Z rmanfredi $
- *
  * Copyright (c) 2009, Raphael Manfredi
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -35,9 +33,8 @@
 
 #include "common.h"
 
-RCSID("$Id: sequence.c 17291 2010-02-25 20:20:39Z rmanfredi $")
-
 #include "sequence.h"
+#include "glib-missing.h"
 #include "walloc.h"
 #include "override.h"			/* Must be the last header included */
 
@@ -60,6 +57,8 @@ struct sequence_iterator {
 	union {
 		GSList *gsl;
 		GList *gl;
+		plist_t *pl;
+		pslist_t *psl;
 		list_iter_t *li;
 		slist_iter_t *sli;
 		hash_list_iter_t *hli;
@@ -91,7 +90,7 @@ sequence_create_from_gslist(GSList *gsl)
 {
 	sequence_t *s;
 
-	s = walloc(sizeof *s);
+	WALLOC(s);
 	return sequence_fill_from_gslist(s, gsl);
 }
 
@@ -104,10 +103,36 @@ sequence_create_from_glist(GList *gl)
 {
 	sequence_t *s;
 
-	s = walloc(sizeof *s);
+	WALLOC(s);
 	return sequence_fill_from_glist(s, gl);
 }
 
+/**
+ * Create a sequence out of an existing pslist_t.
+ * Use sequence_release() to discard the sequence encapsulation.
+ */
+sequence_t *
+sequence_create_from_pslist(pslist_t *pl)
+{
+	sequence_t *s;
+
+	WALLOC(s);
+	return sequence_fill_from_pslist(s, pl);
+}
+
+/**
+ * Create a sequence out of an existing plist_t.
+ * Use sequence_release() to discard the sequence encapsulation.
+ */
+sequence_t *
+sequence_create_from_plist(plist_t *pl)
+{
+	sequence_t *s;
+
+	WALLOC(s);
+	return sequence_fill_from_plist(s, pl);
+}
+
 /**
  * Create a sequence out of an existing list_t.
  * Use sequence_release() to discard the sequence encapsulation.
@@ -119,7 +144,7 @@ sequence_create_from_list(list_t *l)
 
 	g_assert(l != NULL);
 
-	s = walloc(sizeof *s);
+	WALLOC(s);
 	return sequence_fill_from_list(s, l);
 }
 
@@ -134,7 +159,7 @@ sequence_create_from_slist(slist_t *sl)
 
 	g_assert(sl != NULL);
 
-	s = walloc(sizeof *s);
+	WALLOC(s);
 	return sequence_fill_from_slist(s, sl);
 }
 
@@ -149,7 +174,7 @@ sequence_create_from_hash_list(hash_list_t *hl)
 
 	g_assert(hl != NULL);
 
-	s = walloc(sizeof *s);
+	WALLOC(s);
 	return sequence_fill_from_hash_list(s, hl);
 }
 
@@ -164,7 +189,7 @@ sequence_create_from_vector(vector_t *vec)
 
 	g_assert(vec != NULL);
 
-	s = walloc(sizeof *s);
+	WALLOC(s);
 	return sequence_fill_from_vector(s, vec);
 }
 
@@ -192,6 +217,30 @@ sequence_fill_from_glist(sequence_t *s, GList *gl)
 	return s;
 }
 
+/**
+ * Fill sequence object with plist_t.
+ */
+sequence_t *
+sequence_fill_from_plist(sequence_t *s, plist_t *pl)
+{
+	s->magic = SEQUENCE_MAGIC;
+	s->type = SEQUENCE_PLIST;
+	s->u.pl = pl;
+	return s;
+}
+
+/**
+ * Fill sequence object with pslist_t.
+ */
+sequence_t *
+sequence_fill_from_pslist(sequence_t *s, pslist_t *pl)
+{
+	s->magic = SEQUENCE_MAGIC;
+	s->type = SEQUENCE_PSLIST;
+	s->u.psl = pl;
+	return s;
+}
+
 /**
  * Fill sequence object with list_.
  */
@@ -260,6 +309,8 @@ sequence_type_to_string(const sequence_t *s)
 	case SEQUENCE_GSLIST:		return "GSList";
 	case SEQUENCE_GLIST:		return "GList";
 	case SEQUENCE_LIST:			return "list_t";
+	case SEQUENCE_PSLIST:		return "pslist_t";
+	case SEQUENCE_PLIST:		return "plist_t";
 	case SEQUENCE_SLIST:		return "slist_t";
 	case SEQUENCE_HLIST:		return "hash_list_t";
 	case SEQUENCE_VECTOR:		return "vector_t";
@@ -288,6 +339,12 @@ sequence_foreach(const sequence_t *s, GFunc func, void *data)
 	case SEQUENCE_LIST:
 		list_foreach(s->u.l, func, data);
 		break;
+	case SEQUENCE_PSLIST:
+		pslist_foreach(s->u.psl, func, data);
+		break;
+	case SEQUENCE_PLIST:
+		plist_foreach(s->u.pl, func, data);
+		break;
 	case SEQUENCE_SLIST:
 		slist_foreach(s->u.sl, func, data);
 		break;
@@ -305,7 +362,7 @@ sequence_foreach(const sequence_t *s, GFunc func, void *data)
 /**
  * Is the sequence empty?
  */
-gboolean
+bool
 sequence_is_empty(const sequence_t *s)
 {
 	sequence_check(s);
@@ -317,6 +374,10 @@ sequence_is_empty(const sequence_t *s)
 		return NULL == s->u.gl;
 	case SEQUENCE_LIST:
 		return 0 == list_length(s->u.l);
+	case SEQUENCE_PSLIST:
+		return NULL == s->u.psl;
+	case SEQUENCE_PLIST:
+		return NULL == s->u.pl;
 	case SEQUENCE_SLIST:
 		return 0 == slist_length(s->u.sl);
 	case SEQUENCE_HLIST:
@@ -330,10 +391,42 @@ sequence_is_empty(const sequence_t *s)
 	return FALSE;
 }
 
+/**
+ * @return how many items are held in the sequence.
+ */
+size_t
+sequence_count(const sequence_t *s)
+{
+	sequence_check(s);
+
+	switch (s->type) {
+	case SEQUENCE_GSLIST:
+		return g_slist_length(s->u.gsl);
+	case SEQUENCE_GLIST:
+		return g_list_length(s->u.gl);
+	case SEQUENCE_LIST:
+		return list_length(s->u.l);
+	case SEQUENCE_PSLIST:
+		return pslist_length(s->u.psl);
+	case SEQUENCE_PLIST:
+		return plist_length(s->u.pl);
+	case SEQUENCE_SLIST:
+		return slist_length(s->u.sl);
+	case SEQUENCE_HLIST:
+		return hash_list_length(s->u.hl);
+	case SEQUENCE_VECTOR:
+		return vector_length(s->u.vec);
+	case SEQUENCE_MAXTYPE:
+		g_assert_not_reached();
+	}
+
+	return 0;
+}
+
 /**
  * Returns the underlying sequence implementation.
  */
-gpointer
+void *
 sequence_implementation(const sequence_t *s)
 {
 	sequence_check(s);
@@ -345,6 +438,10 @@ sequence_implementation(const sequence_t *s)
 		return s->u.gl;
 	case SEQUENCE_LIST:
 		return s->u.l;
+	case SEQUENCE_PSLIST:
+		return s->u.psl;
+	case SEQUENCE_PLIST:
+		return s->u.pl;
 	case SEQUENCE_SLIST:
 		return s->u.sl;
 	case SEQUENCE_HLIST:
@@ -364,13 +461,13 @@ sequence_implementation(const sequence_t *s)
  *
  * The supplied pointer to the sequence is nullified upon return.
  */
-gpointer
+void *
 sequence_release(sequence_t **s_ptr)
 {
 	sequence_t *s = *s_ptr;
 
 	if (s != NULL) {
-		gpointer implementation;
+		void *implementation;
 
 		sequence_check(s);
 
@@ -378,7 +475,7 @@ sequence_release(sequence_t **s_ptr)
 
 		s->type = SEQUENCE_MAXTYPE;
 		s->magic = 0;
-		wfree(s, sizeof *s);
+		WFREE(s);
 
 		*s_ptr = NULL;
 		return implementation;
@@ -397,14 +494,20 @@ sequence_destroy(sequence_t *s)
 
 	switch (s->type) {
 	case SEQUENCE_GSLIST:
-		g_slist_free(s->u.gsl);
+		gm_slist_free_null(&s->u.gsl);
 		break;
 	case SEQUENCE_GLIST:
-		g_list_free(s->u.gl);
+		gm_list_free_null(&s->u.gl);
 		break;
 	case SEQUENCE_LIST:
 		list_free(&s->u.l);
 		break;
+	case SEQUENCE_PSLIST:
+		pslist_free_null(&s->u.psl);
+		break;
+	case SEQUENCE_PLIST:
+		plist_free_null(&s->u.pl);
+		break;
 	case SEQUENCE_SLIST:
 		slist_free(&s->u.sl);
 		break;
@@ -420,7 +523,7 @@ sequence_destroy(sequence_t *s)
 
 	s->type = SEQUENCE_MAXTYPE;
 	s->magic = 0;
-	wfree(s, sizeof *s);
+	WFREE(s);
 }
 
 /**
@@ -433,7 +536,7 @@ sequence_forward_iterator(const sequence_t *s)
 
 	g_assert(s != NULL);
 
-	si = walloc(sizeof *si);
+	WALLOC(si);
 	si->magic = SEQUENCE_ITER_MAGIC;
 	si->direction = SEQ_ITER_FORWARD;
 	si->type = s->type;
@@ -448,6 +551,12 @@ sequence_forward_iterator(const sequence_t *s)
 	case SEQUENCE_LIST:
 		si->u.li = list_iter_before_head(s->u.l);
 		break;
+	case SEQUENCE_PSLIST:
+		si->u.psl = s->u.psl;
+		break;
+	case SEQUENCE_PLIST:
+		si->u.pl = s->u.pl;
+		break;
 	case SEQUENCE_SLIST:
 		si->u.sli = slist_iter_before_head(s->u.sl);
 		break;
@@ -467,7 +576,7 @@ sequence_forward_iterator(const sequence_t *s)
 /**
  * Check whether we have a next item to be iterated over.
  */
-gboolean
+bool
 sequence_iter_has_next(const sequence_iter_t *si)
 {
 	if (!si)
@@ -483,6 +592,10 @@ sequence_iter_has_next(const sequence_iter_t *si)
 		return si->u.gl != NULL;
 	case SEQUENCE_LIST:
 		return list_iter_has_next(si->u.li);
+	case SEQUENCE_PSLIST:
+		return si->u.psl != NULL;
+	case SEQUENCE_PLIST:
+		return si->u.pl != NULL;
 	case SEQUENCE_SLIST:
 		return slist_iter_has_next(si->u.sli);
 	case SEQUENCE_HLIST:
@@ -499,10 +612,10 @@ sequence_iter_has_next(const sequence_iter_t *si)
 /**
  * Get next item, moving iterator cursor by one position.
  */
-gpointer
+void *
 sequence_iter_next(sequence_iter_t *si)
 {
-	gpointer next = NULL;
+	void *next = NULL;
 
 	sequence_iter_check(si);
 	g_assert(SEQ_ITER_FORWARD == si->direction);
@@ -518,6 +631,14 @@ sequence_iter_next(sequence_iter_t *si)
 		break;
 	case SEQUENCE_LIST:
 		return list_iter_next(si->u.li);
+	case SEQUENCE_PSLIST:
+		next = si->u.psl ? si->u.psl->data : NULL;
+		si->u.psl = pslist_next(si->u.psl);
+		break;
+	case SEQUENCE_PLIST:
+		next = si->u.pl ? si->u.pl->data : NULL;
+		si->u.pl = plist_next(si->u.pl);
+		break;
 	case SEQUENCE_SLIST:
 		return slist_iter_next(si->u.sli);
 	case SEQUENCE_HLIST:
@@ -537,13 +658,13 @@ sequence_iter_next(sequence_iter_t *si)
  * instead unless ``check'' is TRUE in which case we panic.
  */
 sequence_iter_t *
-sequence_backward_iterator(const sequence_t *s, gboolean check)
+sequence_backward_iterator(const sequence_t *s, bool check)
 {
 	sequence_iter_t *si;
 
 	g_assert(s != NULL);
 
-	si = walloc(sizeof *si);
+	WALLOC(si);
 	si->magic = SEQUENCE_ITER_MAGIC;
 	si->direction = SEQ_ITER_BACKWARD;
 	si->type = s->type;
@@ -560,6 +681,14 @@ sequence_backward_iterator(const sequence_t *s, gboolean check)
 	case SEQUENCE_LIST:
 		si->u.li = list_iter_after_tail(s->u.l);
 		break;
+	case SEQUENCE_PSLIST:
+		if (check)
+			goto panic;
+		si->u.psl = s->u.psl;	/* Forward iteration from head */
+		break;
+	case SEQUENCE_PLIST:
+		si->u.pl = plist_last(s->u.pl);
+		break;
 	case SEQUENCE_SLIST:
 		if (check)
 			goto panic;
@@ -586,10 +715,10 @@ panic:
 /**
  * Get previous item, moving iterator cursor by one position backwards.
  */
-gpointer
+void *
 sequence_iter_previous(sequence_iter_t *si)
 {
-	gpointer prev = NULL;
+	void *prev = NULL;
 
 	sequence_iter_check(si);
 	g_assert(SEQ_ITER_BACKWARD == si->direction);
@@ -606,6 +735,15 @@ sequence_iter_previous(sequence_iter_t *si)
 		break;
 	case SEQUENCE_LIST:
 		return list_iter_previous(si->u.li);
+	case SEQUENCE_PSLIST:
+		/* Forward iteration only */
+		prev = si->u.psl ? si->u.psl->data : NULL;
+		si->u.psl = pslist_next(si->u.psl);
+		break;
+	case SEQUENCE_PLIST:
+		prev = si->u.pl ? si->u.pl->data : NULL;
+		si->u.pl = plist_prev(si->u.pl);
+		break;
 	case SEQUENCE_SLIST:
 		/* Forward iteration only */
 		return slist_iter_next(si->u.sli);
@@ -623,7 +761,7 @@ sequence_iter_previous(sequence_iter_t *si)
 /**
  * Check whether we have a previous item to be iterated over.
  */
-gboolean
+bool
 sequence_iter_has_previous(const sequence_iter_t *si)
 {
 	if (!si)
@@ -640,6 +778,10 @@ sequence_iter_has_previous(const sequence_iter_t *si)
 		return si->u.gl != NULL;
 	case SEQUENCE_LIST:
 		return list_iter_has_previous(si->u.li);
+	case SEQUENCE_PSLIST:
+		return si->u.psl != NULL;
+	case SEQUENCE_PLIST:
+		return si->u.pl != NULL;
 	case SEQUENCE_SLIST:
 		/* Forward iteration only */
 		return slist_iter_has_next(si->u.sli);
@@ -668,6 +810,8 @@ sequence_iterator_release(sequence_iter_t **iter_ptr)
 		switch (si->type) {
 		case SEQUENCE_GSLIST:
 		case SEQUENCE_GLIST:
+		case SEQUENCE_PSLIST:
+		case SEQUENCE_PLIST:
 			break;
 		case SEQUENCE_LIST:
 			list_iter_free(&si->u.li);
@@ -686,7 +830,7 @@ sequence_iterator_release(sequence_iter_t **iter_ptr)
 		}
 
 		si->magic = 0;
-		wfree(si, sizeof *si);
+		WFREE(si);
 		*iter_ptr = NULL;
 	}
 }
