@@ -1,6 +1,4 @@
 /*
- * $Id: tiger.h 16103 2009-03-10 14:53:39Z rmanfredi $
- *
  * Copyright (c) 2003, Jeroen Asselman
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -39,7 +37,7 @@
 #include "common.h"
 
 void tiger_check(void);
-void tiger(gconstpointer data, guint64 length, char hash[24]);
+void tiger(const void *data, uint64 length, char hash[24]);
 
 #endif /* _tiger_h_ */
 /* vi: set ts=4 sw=4 cindent: */
