@@ -1,6 +1,4 @@
 /*
- * $Id: online.c 17171 2009-11-03 20:08:11Z rmanfredi $
- *
  * Copyright (c) 2002-2003, Richard Eckart
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -35,12 +33,9 @@
 
 #include "common.h"
 
-RCSID("$Id: online.c 17171 2009-11-03 20:08:11Z rmanfredi $")
-
 #include "cmd.h"
 
 #include "if/gnet_property.h"
-#include "if/gnet_property_priv.h"
 
 #include "lib/override.h"		/* Must be the last header included */
 
