@@ -1,6 +1,4 @@
 /*
- * $Id: columns.h 14659 2007-08-23 15:48:20Z cbiere $
- *
  * Copyright (c) 2001-2004, Raphael Manfredi & Richard Eckart
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -84,15 +82,16 @@ enum {
  * Upload stats columns.
  */
 
-enum {
+enum c_us {
     c_us_filename,
     c_us_size,
     c_us_attempts,
     c_us_complete,
     c_us_norm,
-#define UPLOAD_STATS_GUI_VISIBLE_COLUMNS ((guint) c_us_norm + 1)
-	c_us_stat,
+	c_us_rtime,
+	c_us_dtime,
 
+#define UPLOAD_STATS_GUI_VISIBLE_COLUMNS ((guint) c_us_num)
 	c_us_num
 };
 
@@ -121,6 +120,7 @@ enum c_fi {
 	c_fi_uploaded,
 	c_fi_sources,
 	c_fi_created,
+	c_fi_modified,
 	c_fi_status,
 
 #define FILEINFO_VISIBLE_COLUMNS ((guint) c_fi_num)
@@ -136,11 +136,12 @@ enum c_sr_columns {
 	c_sr_ext,
     c_sr_charset,
     c_sr_size,
+	c_sr_mime,
 	c_sr_count,
     c_sr_loc,
-    c_sr_meta,	/**< meta-data info column */
     c_sr_vendor,
     c_sr_info,
+    c_sr_address,
     c_sr_route,
     c_sr_protocol,
     c_sr_hops,
