@@ -1,6 +1,4 @@
 /*
- * $Id: gtk-missing.c 16397 2009-04-04 17:42:58Z cbiere $
- *
  * Copyright (c) 2002-2003, Richard Eckart
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -37,16 +35,22 @@
 
 #include "gui.h"
 
-RCSID("$Id: gtk-missing.c 16397 2009-04-04 17:42:58Z cbiere $")
-
 #include "columns.h"
 
+#ifdef USE_GTK1
+/* For gtk_window_iconify() */
+#include <gdk/gdkx.h>
+#include <X11/Xlib.h>
+#endif
+
 #include "if/gui_property.h"
 #include "if/gui_property_priv.h"
 
 #include "lib/glib-missing.h"
 #include "lib/misc.h"
+#include "lib/str.h"
 #include "lib/walloc.h"
+
 #include "lib/override.h"		/* Must be the last header included */
 
 /**
@@ -83,7 +87,7 @@ gtk_clist_set_column_name(GtkCList *clist, int col, const char *title)
 			0 != strcmp(clist->column[col].title, title)
 		) {
 			G_FREE_NULL(clist->column[col].title);
-			clist->column[col].title = g_strdup(title);
+			clist->column[col].title = NOT_LEAKING(g_strdup(title));
 		}
 	}
 }
@@ -96,7 +100,7 @@ clist_sync_rows(GtkCList *clist, void (*func)(int, void *))
 
 	g_return_if_fail(clist);
 	g_return_if_fail(func);
-	
+
 	i = 0;
 	for (iter = clist->row_list; NULL != iter; iter = g_list_next(iter), i++) {
 		GtkCListRow *row;
@@ -112,7 +116,7 @@ clist_copy_text(GtkCList *clist, int row, int column)
 	char *text;
 
 	g_return_val_if_fail(clist, NULL);
-	
+
 	if (
 		row < 0 ||
 		column < 0 ||
@@ -159,7 +163,7 @@ clist_get_cursor_row(GtkCList *clist)
 	int row;
 
 	g_return_val_if_fail(clist, -1);
-	
+
 	row = clist_get_focus_row(clist);
 	return row >= 0 && clist_row_is_selected(clist, row) ? row : -1;
 }
@@ -186,8 +190,11 @@ gtk_main_flush(void)
 			break;
 	}
 
-	if (i > GTK_ITERATION_MAX && !val)
-		g_warning("gtk_main_flush: too much work");
+	if (i > GTK_ITERATION_MAX && !val) {
+		if (GUI_PROPERTY(gui_debug)) {
+			g_warning("%s(): too much work", G_STRFUNC);
+		}
+	}
 
     return val;
 }
@@ -212,7 +219,7 @@ option_menu_select_item_by_data(GtkOptionMenu *option_menu, gconstpointer data)
         }
     }
 
-	g_warning("option_menu_select_item_by_data: no item with data %p", data);
+	g_warning("%s(): no item with data %p", G_STRFUNC, data);
 }
 
 
@@ -290,7 +297,7 @@ gtk_entry_printf(GtkEntry *entry, const gchar *format, ...)
     va_start(args, format);
 
     if (format != NULL)
-        gm_vsnprintf(buf, sizeof(buf), format, args);
+        str_vbprintf(ARYLEN(buf), format, args);
     else
         buf[0] = '\0';
 
@@ -312,7 +319,7 @@ void gtk_label_printf(GtkLabel *label, const gchar *format, ...)
     va_start(args, format);
 
     if (format != NULL)
-        gm_vsnprintf(buf, sizeof(buf), format, args);
+        str_vbprintf(ARYLEN(buf), format, args);
     else
         buf[0] = '\0';
 
@@ -425,7 +432,7 @@ w_tree_iter_new(void)
 {
 	GtkTreeIter *iter;
 
-	iter = walloc(sizeof *iter);
+	WALLOC(iter);
 	return iter;
 }
 
@@ -438,8 +445,8 @@ w_tree_iter_copy(GtkTreeIter *iter)
 {
 	GtkTreeIter *copy;
 
-	copy = walloc(sizeof *copy);
-	*copy = *iter;
+	WALLOC(copy);
+	*copy = *iter;		/* Struct copy */
 	return copy;
 }
 
@@ -449,7 +456,7 @@ w_tree_iter_copy(GtkTreeIter *iter)
 void
 w_tree_iter_free(GtkTreeIter *iter)
 {
-	wfree(iter, sizeof *iter);
+	WFREE(iter);
 }
 
 /**
@@ -459,7 +466,7 @@ void
 ht_w_tree_iter_free(gpointer p)
 {
 	GtkTreeIter *iter = p;
-	wfree(iter, sizeof *iter);
+	WFREE(iter);
 }
 
 
@@ -467,7 +474,7 @@ typedef struct collect_data_struct {
     GSList *results;
     GSList *to_unselect;
 	GCompareFunc cfn;
-	tree_selection_get_data_func gdf;	
+	tree_selection_get_data_func gdf;
 	GtkTreeView *tv;
 	const gchar *name; /* name of the treeview widget (for debugging) */
 	guint column;
@@ -576,7 +583,7 @@ tree_selection_collect_data(GtkTreeSelection *selection,
     /*
      * Cleanup before exit.
      */
-    g_slist_free(cdata.to_unselect);
+    gm_slist_free_null(&cdata.to_unselect);
 
     return cdata.results;
 }
@@ -603,10 +610,10 @@ on_tree_view_motion_notify(GtkWidget *widget,
 	g_assert(tvm != NULL);
 	g_assert(tvm->cb != NULL);
 
-#if 0 
+#if 0
 	{
 		gchar type[32];
-#define EVENT_TYPE(x) case x: gm_snprintf(type, sizeof type, "%s", #x); break;
+#define EVENT_TYPE(x) case x: str_bprintf(type, sizeof type, "%s", #x); break;
 		switch (event->type) {
 		EVENT_TYPE(GDK_NOTHING)
 		EVENT_TYPE(GDK_DESTROY)
@@ -643,14 +650,17 @@ on_tree_view_motion_notify(GtkWidget *widget,
 		EVENT_TYPE(GDK_WINDOW_STATE)
 		EVENT_TYPE(GDK_SETTING)
 		default:
-			gm_snprintf(type, sizeof type, "%ld", (gulong) event->type);
+			str_bprintf(type, sizeof type, "%ld", (ulong) event->type);
 		}
 #undef EVENT_TYPE
 
-		g_message("on_tree_view_motion_notify(): type=%s, x=%d, y=%d, axes=%p, x_root=%d, y_root=%d",
-				type,
-				(gint) event->x, (gint) event->y, event->axes,
-				(gint) event->x_root, (gint) event->y_root);
+		if (GUI_PROPERTY(gui_debug) {
+			g_debug("on_tree_view_motion_notify(): "
+				"type=%s, x=%d, y=%d, axes=%p, x_root=%d, y_root=%d",
+					type,
+					(gint) event->x, (gint) event->y, event->axes,
+					(gint) event->x_root, (gint) event->y_root);
+		}
 	}
 #endif /* 0 */
 
@@ -700,7 +710,8 @@ tree_view_motion_set_callback(GtkTreeView *tv,
 	g_return_val_if_fail(tv, NULL);
 	g_return_val_if_fail(cb, NULL);
 
-	tvm = g_malloc(sizeof *tvm);
+	WALLOC(tvm);
+	tvm->ready = FALSE;
 	tvm->tv = GTK_TREE_VIEW(g_object_ref(tv));
 	tvm->cb = cb;
 	tvm->timeout_id = g_timeout_add(interval, tree_view_motion_timeout, tvm);
@@ -719,7 +730,7 @@ tree_view_motion_clear_callback(tree_view_motion_t **ptr)
 		g_source_remove(tvm->timeout_id);
 		g_object_unref(tvm->tv);
 		tvm->tv = NULL;
-		G_FREE_NULL(tvm);
+		WFREE_TYPE_NULL(tvm);
 		*ptr = NULL;
 	}
 }
@@ -728,9 +739,9 @@ void
 tree_view_set_fixed_height_mode(GtkTreeView *tv, gboolean fixed)
 {
 	g_return_if_fail(tv);
-	
+
 #if GTK_CHECK_VERSION(2, 4, 0)
-    g_object_set(GTK_TREE_VIEW(tv), "fixed_height_mode", fixed, (void *) 0);
+    g_object_set(GTK_TREE_VIEW(tv), "fixed_height_mode", fixed, NULL_PTR);
 #endif /* GTK+ >= 2.4.0 */
 }
 
@@ -826,7 +837,7 @@ widget_init_choices(GtkWidget *widget, GtkSignalFunc func,
 		GtkCombo *combo = GTK_COMBO(widget);
 		const gchar *title;
     	guint i;
-		
+
 		for (i = 0; (title = def->data.guint32.choices[i].title) != NULL; i++) {
 			GtkWidget *list_item;
 			GList *l;
@@ -1187,6 +1198,20 @@ gtk_progress_bar_set_text(GtkProgressBar *pb, const gchar *text)
 
 	gtk_progress_set_format_string(GTK_PROGRESS(pb), buf);
 }
+
+void
+gtk_window_iconify(GtkWindow *window)
+{
+	GtkWidget *widget;
+
+	g_return_if_fail(GTK_IS_WINDOW(window));
+
+	widget = GTK_WIDGET(window);
+	if (widget != NULL && widget->window != NULL) {
+		XIconifyWindow(GDK_DISPLAY(), GDK_WINDOW_XWINDOW(widget->window),
+			((_XPrivDisplay) GDK_DISPLAY())->default_screen);
+	}
+}
 #endif
 
 
