@@ -1,6 +1,4 @@
 /*
- * $Id: search_cb.c 15804 2008-09-04 22:41:07Z cbiere $
- *
  * Copyright (c) 2001-2005, Raphael Manfredi, Richard Eckart
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -39,11 +37,8 @@
 
 #include "gtk/gui.h"
 
-RCSID("$Id: search_cb.c 15804 2008-09-04 22:41:07Z cbiere $")
-
 #include "search_cb.h"
 
-#include "gtk/bitzi.h"
 #include "gtk/columns.h"
 #include "gtk/filter_core.h"
 #include "gtk/misc.h"
@@ -60,13 +55,16 @@ RCSID("$Id: search_cb.c 15804 2008-09-04 22:41:07Z cbiere $")
 #include "lib/atoms.h"
 #include "lib/cq.h"
 #include "lib/glib-missing.h"
+#include "lib/halloc.h"
+#include "lib/misc.h"			/* Fox xml_indent() */
 #include "lib/utf8.h"
+#include "lib/vendors.h"		/* For T_GTKG */
 
 #include "lib/override.h"		/* Must be the last header included */
 
-static record_t *selected_record; 
+static record_t *selected_record;
 
-char * 
+char *
 search_gui_details_get_text(GtkWidget *widget)
 {
 	char *text = NULL;
@@ -112,18 +110,10 @@ search_set_xml_metadata(const record_t *rc)
 {
 	char *indented;
 
-	indented = (rc && rc->xml) ? search_xml_indent(rc->xml) : NULL;
+	indented = (rc && rc->xml) ? xml_indent(rc->xml, NULL) : NULL;
 	set_text_buffer(gui_main_window_lookup("text_result_info_xml"),
 		EMPTY_STRING(indented));
-	G_FREE_NULL(indented);
-}
-
-void
-search_gui_set_bitzi_metadata_text(const char *text)
-{
-	g_return_if_fail(text);
-
-	set_text_buffer(gui_main_window_lookup("text_result_info_bitzi"), text);
+	HFREE_NULL(indented);
 }
 
 static GtkCList *clist_search_details;
@@ -147,7 +137,7 @@ search_gui_append_detail(const gchar *name, const gchar *value)
 	titles[1] = EMPTY_STRING(value);
     gtk_clist_append(clist_search_details, (gchar **) titles);
 }
-	
+
 /**
  *	Activates/deactivates buttons and popups based on what is selected
  */
@@ -168,7 +158,6 @@ search_gui_refresh_details(const record_t *rc)
 	search_gui_set_details(rc);
     gtk_clist_thaw(clist_search_details);
 	search_set_xml_metadata(rc);
-	search_gui_set_bitzi_metadata(rc);
 }
 
 record_t *
@@ -231,14 +220,13 @@ static cevent_t *row_selected_ev;
 #define ROW_SELECT_TIMEOUT	100 /* milliseconds */
 
 static void
-row_selected_expire(cqueue_t *unused_cq, gpointer unused_udata)
+row_selected_expire(cqueue_t *cq, gpointer unused_udata)
 {
 	search_t *search;
 
-	(void) unused_cq;
 	(void) unused_udata;
 
-	row_selected_ev = NULL;
+	cq_zero(cq, &row_selected_ev);
 
     search = search_gui_get_current_search();
 	if (search) {
@@ -253,7 +241,7 @@ static void
 selected_row_changed(GtkCTree *ctree)
 {
 	int row;
-	
+
 	if (selected_record) {
 		search_gui_unref_record(selected_record);
 		selected_record = NULL;
@@ -271,9 +259,9 @@ selected_row_changed(GtkCTree *ctree)
 	}
 
 	if (row_selected_ev) {
-		cq_resched(callout_queue, row_selected_ev, ROW_SELECT_TIMEOUT);
+		cq_resched(row_selected_ev, ROW_SELECT_TIMEOUT);
 	} else {
-		row_selected_ev = cq_insert(callout_queue, ROW_SELECT_TIMEOUT,
+		row_selected_ev = cq_main_insert(ROW_SELECT_TIMEOUT,
 							row_selected_expire, NULL);
 	}
 }
@@ -339,6 +327,7 @@ search_gui_browse_selected(void)
 		rs = rc->results_set;
 		flags |= 0 != (rs->status & ST_FIREWALL) ? SOCK_F_PUSH : 0;
 		flags |= 0 != (rs->status & ST_TLS) ? SOCK_F_TLS : 0;
+		flags |= ((rs->status & ST_G2) && T_GTKG != rs->vendor) ? SOCK_F_G2 : 0;
 
 		(void) search_gui_new_browse_host(
 				rs->hostname, rs->addr, rs->port,
@@ -373,54 +362,6 @@ search_cb_collect_ctree_data(GtkCTree *ctree,
 	return g_slist_reverse(data_list);
 }
 
-/**
- * Queue a bitzi queries from the search context menu
- */
-void
-on_popup_search_metadata_activate(GtkMenuItem *unused_menuitem,
-	gpointer unused_udata)
-{
-    GList *node_list;
-	GSList *data_list;
-    search_t *search;
-	guint32 bitzi_debug;
-
-	(void) unused_menuitem;
-	(void) unused_udata;
-
-    search = search_gui_get_current_search();
-    g_assert(search != NULL);
-
-    gtk_clist_freeze(GTK_CLIST(search->tree));
-
-	node_list = g_list_copy(GTK_CLIST(search->tree)->selection);
-	data_list = search_cb_collect_ctree_data(GTK_CTREE(search->tree),
-					node_list, gui_record_sha1_eq);
-
-	/* Make sure the column is actually visible. */
-	{
-		static const gint min_width = 80;
-		GtkCList *clist = GTK_CLIST(search->tree);
-
-    	gtk_clist_set_column_visibility(clist, c_sr_meta, TRUE);
-		if (clist->column[c_sr_meta].width < min_width)
-    		gtk_clist_set_column_width(clist, c_sr_meta, min_width);
-	}
-	
-	/* Queue up our requests */
-    gnet_prop_get_guint32_val(PROP_BITZI_DEBUG, &bitzi_debug);
-	if (bitzi_debug)
-		g_message("on_popup_search_metadata_activate: %d items, %p",
-			  g_slist_position(data_list, g_slist_last(data_list)) + 1,
-			  cast_to_gconstpointer(data_list));
-
-	G_SLIST_FOREACH(data_list, search_gui_queue_bitzi_by_sha1);
-
-	gtk_clist_thaw(GTK_CLIST(search->tree));
-	g_slist_free(data_list);
-	g_list_free(node_list);
-}
-
 void
 on_popup_search_copy_magnet_activate(GtkMenuItem *unused_item,
 	gpointer unused_udata)
@@ -436,7 +377,7 @@ on_popup_search_copy_magnet_activate(GtkMenuItem *unused_item,
 	if (selected_record) {
 		char *magnet = search_gui_get_magnet(search, selected_record);
 		clipboard_set_text(gui_main_window(), magnet);
-		G_FREE_NULL(magnet);
+		HFREE_NULL(magnet);
 	}
 }
 
@@ -446,7 +387,7 @@ search_gui_callbacks_shutdown(void)
 	/*
  	 *	Remove delayed callbacks
  	 */
-	cq_cancel(callout_queue, &row_selected_ev);
+	cq_cancel(&row_selected_ev);
 	search_gui_clear_details();
 	if (selected_record) {
 		search_gui_unref_record(selected_record);
