@@ -1,6 +1,4 @@
 /*
- * $Id: uploads.c 15242 2008-04-22 23:10:56Z cbiere $
- *
  * Copyright (c) 2001-2003, Richard Eckart
  *
  *----------------------------------------------------------------------
@@ -19,14 +17,12 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
 #include "common.h"
 
-RCSID("$Id: uploads.c 15242 2008-04-22 23:10:56Z cbiere $")
-
 #include "gtk/gui.h"
 
 #include "gtk/columns.h"
@@ -38,8 +34,8 @@ RCSID("$Id: uploads.c 15242 2008-04-22 23:10:56Z cbiere $")
 #include "if/bridge/ui2c.h"
 
 #include "lib/host_addr.h"
-#include "lib/glib-missing.h"
 #include "lib/iso3166.h"
+#include "lib/str.h"
 #include "lib/tm.h"
 #include "lib/utf8.h"
 #include "lib/walloc.h"
@@ -131,12 +127,8 @@ find_row(gnet_upload_t u, upload_row_data_t **data)
 {
     GtkCList *clist;
     GList *iter;
-    upload_row_data_t fake;
     gint row = 0;
 
-    fake.handle = u;
-    fake.valid  = TRUE;
-
     clist = GTK_CLIST(gui_main_window_lookup("clist_uploads"));
 
     for (iter = clist->row_list; iter != NULL; iter = g_list_next(iter)) {
@@ -146,7 +138,7 @@ find_row(gnet_upload_t u, upload_row_data_t **data)
 		g_assert(r);
 		rd = r->data;
 		g_assert(rd);
-		
+
         if (rd->valid && (rd->handle == u)) {
             /* found */
 
@@ -158,8 +150,7 @@ find_row(gnet_upload_t u, upload_row_data_t **data)
         row++;
     }
 
-    g_warning("%s: upload not found [handle=%u]",
-        G_GNUC_PRETTY_FUNCTION, u);
+    g_warning("%s(): upload not found [handle=%u]", G_STRFUNC, u);
 
     return -1;
 }
@@ -189,8 +180,8 @@ uploads_gui_update_upload_info(gnet_upload_info_t *u)
     clist_uploads = GTK_CLIST(gui_main_window_lookup("clist_uploads"));
     row =  find_row(u->upload_handle, &rd);
 	if (row == -1) {
-        g_warning("%s: no matching row found [handle=%u]",
-            G_GNUC_PRETTY_FUNCTION, u->upload_handle);
+        g_warning("%s(): no matching row found [handle=%u]",
+            G_STRFUNC, u->upload_handle);
 		return;
 	}
 
@@ -206,13 +197,13 @@ uploads_gui_update_upload_info(gnet_upload_info_t *u)
 		g_strlcpy(size_tmp, short_size(u->file_size, show_metric_units()),
 			sizeof size_tmp);
 
-        range_len = gm_snprintf(range_tmp, sizeof range_tmp, "%s%s",
+        range_len = str_bprintf(range_tmp, sizeof range_tmp, "%s%s",
             short_size(u->range_end - u->range_start + 1,
 				show_metric_units()),
 			u->partial ? _(" (partial)") : "");
 
 		if (u->range_start)
-			range_len += gm_snprintf(
+			range_len += str_bprintf(
 				&range_tmp[range_len], sizeof(range_tmp)-range_len,
 					" @ %s", short_size(u->range_start, show_metric_units()));
 
@@ -239,7 +230,7 @@ uploads_gui_update_upload_info(gnet_upload_info_t *u)
 		gtk_clist_set_foreground(clist_uploads, row, color);
 	}
 
-	gm_snprintf(range_tmp, sizeof range_tmp, "%5.02f%%",
+	str_bprintf(range_tmp, sizeof range_tmp, "%5.02f%%",
 		100.0 * uploads_gui_progress(&status, rd));
 	gtk_clist_set_text(clist_uploads, row, c_ul_progress, range_tmp);
 
@@ -273,7 +264,7 @@ uploads_gui_add_upload(gnet_upload_info_t *u)
 
     clist_uploads = gui_main_window_lookup("clist_uploads");
 
-	memset(titles, 0, sizeof(titles));
+	ZERO(&titles);
 
     if ((u->range_start == 0) && (u->range_end == 0)) {
         titles[c_ul_size] = titles[c_ul_range] =  "...";
@@ -281,13 +272,13 @@ uploads_gui_add_upload(gnet_upload_info_t *u)
         g_strlcpy(size_tmp, short_size(u->file_size, show_metric_units()),
 			sizeof size_tmp);
 
-        range_len = gm_snprintf(range_tmp, sizeof range_tmp, "%s%s",
+        range_len = str_bprintf(range_tmp, sizeof range_tmp, "%s%s",
             short_size(u->range_end - u->range_start + 1,
 				show_metric_units()),
 			u->partial ? _(" (partial)") : "");
 
         if (u->range_start)
-            range_len += gm_snprintf(
+            range_len += str_bprintf(
                 &range_tmp[range_len], sizeof(range_tmp)-range_len,
                 " @ %s", short_size(u->range_start, show_metric_units()));
 
@@ -307,7 +298,7 @@ uploads_gui_add_upload(gnet_upload_info_t *u)
 	titles[c_ul_progress]   = "...";
 	titles[c_ul_status]   = "...";
 
-    data = walloc0(sizeof *data);
+    WALLOC0(data);
     data->handle      = u->upload_handle;
     data->range_start = u->range_start;
     data->range_end   = u->range_end;
@@ -354,7 +345,7 @@ uploads_gui_update_display(time_t now)
             data->last_update = now;
             guc_upload_get_status(data->handle, &status);
 
-			gm_snprintf(tmp, sizeof tmp, "%5.02f%%",
+			str_bprintf(tmp, sizeof tmp, "%5.02f%%",
 				100.0 * uploads_gui_progress(&status, data));
 			gtk_clist_set_text(clist, row, c_ul_progress, tmp);
 
@@ -495,7 +486,7 @@ uploads_gui_init(void)
 /**
  * Unregister callbacks in the backend and clean up.
  */
-void
+void G_COLD
 uploads_gui_shutdown(void)
 {
 	GtkCList *clist;
