@@ -1,6 +1,4 @@
 /*
- * $Id: gnet_stats.c 16990 2009-09-25 15:23:43Z rmanfredi $
- *
  * Copyright (c) 2001-2003, Richard Eckart
  *
  *----------------------------------------------------------------------
@@ -19,14 +17,12 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
 #include "gtk/gui.h"
 
-RCSID("$Id: gnet_stats.c 16990 2009-09-25 15:23:43Z rmanfredi $")
-
 #include "gtk/gnet_stats.h"
 #include "gtk/notebooks.h"
 #include "gtk/misc.h"
@@ -37,9 +33,12 @@ RCSID("$Id: gnet_stats.c 16990 2009-09-25 15:23:43Z rmanfredi $")
 #include "if/core/gnutella.h"
 #include "if/bridge/ui2c.h"
 
+#include "lib/cstr.h"
+#include "lib/entropy.h"
+#include "lib/str.h"
 #include "lib/stringify.h"
 #include "lib/tm.h"
-#include "lib/glib-missing.h"
+
 #include "lib/override.h"		/* Must be the last header included */
 
 static GtkTreeView *treeview_gnet_stats_messages;
@@ -48,7 +47,6 @@ static GtkTreeView *treeview_gnet_stats_flowc;
 static GtkTreeView *treeview_gnet_stats_recv;
 static GtkTreeView *treeview_gnet_stats_general;
 static GtkTreeView *treeview_gnet_stats_horizon;
-static GtkNotebook *notebook_gnet_stats;
 
 static const gchar * const msg_stats_label[] = {
 	N_("Type"),
@@ -67,11 +65,30 @@ enum gnet_stats_nb_page {
 	GNET_STATS_NB_PAGE_FLOWC,
 	GNET_STATS_NB_PAGE_RECV,
 	GNET_STATS_NB_PAGE_HORIZON,
+	GNET_STATS_NB_PAGE_FILE_TRANSFERS,
 
 	NUM_GNET_STATS_NB_PAGES
 };
 
 static void gnet_stats_update_drop_reasons(const gnet_stats_t *);
+static enum gnet_stats_nb_page gnet_stats_notebook_current_page;
+
+static void
+on_gnet_stats_notebook_switch_page(GtkNotebook *unused_notebook,
+	GtkNotebookPage *unused_page, int page_num, void *unused_udata)
+{
+	(void) unused_notebook;
+	(void) unused_page;
+	(void) unused_udata;
+
+	g_return_if_fail(UNSIGNED(page_num) < NUM_GNET_STATS_NB_PAGES);
+
+	entropy_harvest_single(VARLEN(page_num));
+
+	gnet_stats_notebook_current_page = page_num;
+	gui_prop_set_guint32_val(PROP_GNET_STATS_NOTEBOOK_TAB, page_num);
+	gnet_stats_gui_update_display(tm_time());
+}
 
 /***
  *** Private functions
@@ -106,12 +123,12 @@ pkt_stat_str(gchar *dst, size_t size, const guint64 *val_tbl,
 	gint type, gboolean perc)
 {
 	if (0 == val_tbl[type])
-		g_strlcpy(dst, "-", size);
+		cstr_bcpy(dst, size, "-");
 	else {
 		if (!perc)
 			uint64_to_string_buf(val_tbl[type], dst, size);
 		else
-			gm_snprintf(dst, size, "%.2f%%",
+			str_bprintf(dst, size, "%.2f%%",
 			    (gfloat) val_tbl[type] / val_tbl[MSG_TOTAL] * 100.0);
 	}
 
@@ -124,50 +141,31 @@ byte_stat_str(gchar *dst, gulong n, const guint64 *val_tbl,
 	gint type, gboolean perc)
 {
 	if (0 == val_tbl[type])
-		g_strlcpy(dst, "-", n);
+		cstr_bcpy(dst, n, "-");
 	else if (!perc)
-		g_strlcpy(dst, compact_size(val_tbl[type], show_metric_units()), n);
+		cstr_bcpy(dst, n, compact_size(val_tbl[type], show_metric_units()));
 	else
-		gm_snprintf(dst, n, "%.2f%%",
+		str_bprintf(dst, n, "%.2f%%",
 		    (gfloat) val_tbl[type] / val_tbl[MSG_TOTAL] * 100.0);
 
 	return dst;
 }
 
-static const gchar *
+static void
 drop_stat_str(gchar *dst, size_t size, const gnet_stats_t *stats, gint reason,
-	gint selected_type, gboolean percent)
+	gint selected_type)
 {
-	guint32 total = stats->pkg.dropped[MSG_TOTAL];
-
 	if (stats->drop_reason[reason][selected_type] == 0)
-		g_strlcpy(dst, "-", size);
-	else if (percent)
-		gm_snprintf(dst, size, "%.2f%%",
-		    (gfloat) stats->drop_reason[reason][selected_type] / total * 100);
+		cstr_bcpy(dst, size, "-");
 	else
 		uint64_to_string_buf(stats->drop_reason[reason][selected_type],
 			dst, size);
-
-	return dst;
 }
 
-static const gchar *
+static inline void
 general_stat_str(gchar *dst, size_t size, const gnet_stats_t *stats, gint type)
 {
-	if (stats->general[type] == 0)
-		g_strlcpy(dst, "-", size);
-	else if (
-		type == GNR_QUERY_COMPACT_SIZE ||
-		type == GNR_IGNORED_DATA ||
-		type == GNR_SUNK_DATA
-	)
-		g_strlcpy(dst,
-			compact_size(stats->general[type], show_metric_units()), size);
-	else
-		uint64_to_string_buf(stats->general[type], dst, size);
-
-	return dst;
+	gnet_stats_gui_general_to_string_buf(dst, size, stats, type);
 }
 
 static void
@@ -183,7 +181,7 @@ add_column(GtkTreeView *treeview, gint column_id, gint width, gfloat xalign,
 	g_object_set(renderer,
 		"xalign", xalign,
 		"ypad", GUI_CELL_RENDERER_YPAD,
-		(void *) 0);
+		NULL_PTR);
 	column = gtk_tree_view_column_new_with_attributes(label, renderer,
 		"text", column_id,
 		NULL);
@@ -193,13 +191,15 @@ add_column(GtkTreeView *treeview, gint column_id, gint width, gfloat xalign,
 		"reorderable", TRUE,
 		"resizable", TRUE,
 		"sizing", GTK_TREE_VIEW_COLUMN_FIXED,
-		(void *) 0);
+		NULL_PTR);
 	gtk_tree_view_append_column(treeview, column);
+	gui_column_map(column, treeview);	/* Capture resize events */
 }
 
 static void
 gnet_stats_update_general(const gnet_stats_t *stats)
 {
+	static uint64 general[GNR_TYPE_COUNT];
 	GtkListStore *store;
 	GtkTreeIter iter;
 	gint n;
@@ -210,10 +210,14 @@ gnet_stats_update_general(const gnet_stats_t *stats)
 		return;
 
 	for (n = 0; n < GNR_TYPE_COUNT; n++) {
-		gchar buf[32];
+		gchar buf[UINT64_DEC_BUFLEN];
+
+		if (stats->general[n] != general[n]) {
+			general[n] = stats->general[n];
+			general_stat_str(ARYLEN(buf), stats, n);
+			gtk_list_store_set(store, &iter, 1, buf, (-1));
+		}
 
-		general_stat_str(buf, sizeof buf, stats, n);
-		gtk_list_store_set(store, &iter, 1, buf, (-1));
 		if (!gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter))
 			break;
 	}
@@ -222,9 +226,11 @@ gnet_stats_update_general(const gnet_stats_t *stats)
 static void
 gnet_stats_update_drop_reasons(const gnet_stats_t *stats)
 {
+	static uint64 drop_reason[MSG_DROP_REASON_COUNT][MSG_TYPE_COUNT];
 	GtkListStore *store;
 	GtkTreeIter iter;
 	gint n;
+	guint i = GUI_PROPERTY(gnet_stats_drop_reasons_type);
 
 	store = GTK_LIST_STORE(gtk_tree_view_get_model(
 				treeview_gnet_stats_drop_reasons));
@@ -232,12 +238,14 @@ gnet_stats_update_drop_reasons(const gnet_stats_t *stats)
 		return;
 
 	for (n = 0; n < MSG_DROP_REASON_COUNT; n++) {
-		gchar buf[32];
+		gchar buf[UINT64_DEC_BUFLEN];
+
+		if (stats->drop_reason[n][i] != drop_reason[n][i]) {
+			drop_reason[n][i] = stats->drop_reason[n][i];
+			drop_stat_str(ARYLEN(buf), stats, n, i);
+			gtk_list_store_set(store, &iter, 1, buf, (-1));
+		}
 
-		drop_stat_str(buf, sizeof buf, stats, n,
-			GUI_PROPERTY(gnet_stats_drop_reasons_type),
-			GUI_PROPERTY(gnet_stats_bytes));
-		gtk_list_store_set(store, &iter, 1, buf, (-1));
 		if (!gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter))
 			break;
 	}
@@ -246,7 +254,7 @@ gnet_stats_update_drop_reasons(const gnet_stats_t *stats)
 static void
 gnet_stats_update_messages(const gnet_stats_t *stats)
 {
-	static char str[num_c_gs][32];
+	static char str[num_c_gs][UINT64_DEC_BUFLEN];
 	static const size_t len = sizeof(str[0]);
 	GtkTreeView *treeview = treeview_gnet_stats_messages;
 	GtkListStore *store;
@@ -255,7 +263,7 @@ gnet_stats_update_messages(const gnet_stats_t *stats)
 	gboolean bytes = FALSE;
 	gint n;
 
-	STATIC_ASSERT(num_c_gs == G_N_ELEMENTS(msg_stats_label));
+	STATIC_ASSERT(num_c_gs == N_ITEMS(msg_stats_label));
 
 	gui_prop_get_boolean_val(PROP_GNET_STATS_PERC, &perc);
 	gui_prop_get_boolean_val(PROP_GNET_STATS_BYTES, &bytes);
@@ -303,7 +311,6 @@ gnet_stats_update_messages(const gnet_stats_t *stats)
 		if (!gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter))
 			break;
 	}
-
 }
 
 static void
@@ -382,14 +389,17 @@ gnet_stats_gui_horizon_init(void)
 	guint32 width[num_c_horizon];
 	c_horizon_t n;
 
-	STATIC_ASSERT(num_c_horizon == G_N_ELEMENTS(types));
-	model = GTK_TREE_MODEL(gtk_list_store_newv(G_N_ELEMENTS(types), types));
+	STATIC_ASSERT(num_c_horizon == N_ITEMS(types));
+	model = GTK_TREE_MODEL(gtk_list_store_newv(N_ITEMS(types), types));
 	treeview = treeview_gnet_stats_horizon = GTK_TREE_VIEW(
 	    gui_main_window_lookup("treeview_gnet_stats_horizon"));
 
 	gui_prop_get_guint32(PROP_GNET_STATS_HORIZON_COL_WIDTHS,
-		width, 0, G_N_ELEMENTS(width));
-	for (n = 0; n < G_N_ELEMENTS(width); n++) {
+		width, 0, N_ITEMS(width));
+
+	gui_parent_widths_saveto(treeview, PROP_GNET_STATS_HORIZON_COL_WIDTHS);
+
+	for (n = 0; n < N_ITEMS(width); n++) {
 		GtkTreeIter iter;
 		gint i;
 
@@ -429,8 +439,8 @@ gnet_stats_gui_flowc_init(void)
 	guint32 width[STATS_FLOWC_COLUMNS];
 	guint n;
 
-	STATIC_ASSERT(STATS_FLOWC_COLUMNS == G_N_ELEMENTS(types));
-	model = GTK_TREE_MODEL(gtk_list_store_newv(G_N_ELEMENTS(types), types));
+	STATIC_ASSERT(STATS_FLOWC_COLUMNS == N_ITEMS(types));
+	model = GTK_TREE_MODEL(gtk_list_store_newv(N_ITEMS(types), types));
 
 	treeview = treeview_gnet_stats_flowc = GTK_TREE_VIEW(
 	    gui_main_window_lookup("treeview_gnet_stats_flowc"));
@@ -448,10 +458,13 @@ gnet_stats_gui_flowc_init(void)
 
 	gui_prop_get_guint32(PROP_GNET_STATS_FC_COL_WIDTHS,
 		width, 0, STATS_FLOWC_COLUMNS);
-	for (n = 0; n < G_N_ELEMENTS(width); n++) {
+
+	gui_parent_widths_saveto(treeview, PROP_GNET_STATS_FC_COL_WIDTHS);
+
+	for (n = 0; n < N_ITEMS(width); n++) {
 		gchar buf[16];
 
-		gm_snprintf(buf, sizeof(buf), "%d%c", n - 1,
+		str_bprintf(ARYLEN(buf), "%d%c", n - 1,
 				n + 1 < STATS_FLOWC_COLUMNS ? '\0' : '+');
 		add_column(treeview, n, width[n], (gfloat) (n != 0),
 			n == 0 ? _("Type") : buf);
@@ -470,18 +483,21 @@ gnet_stats_gui_drop_reasons_init(void)
 		G_TYPE_STRING,
 		G_TYPE_STRING
 	};
-	guint32 width[G_N_ELEMENTS(types)];
+	guint32 width[N_ITEMS(types)];
 	guint n;
 
-	STATIC_ASSERT(2 == G_N_ELEMENTS(types));
-	model = GTK_TREE_MODEL(gtk_list_store_newv(G_N_ELEMENTS(types), types));
+	STATIC_ASSERT(2 == N_ITEMS(types));
+	model = GTK_TREE_MODEL(gtk_list_store_newv(N_ITEMS(types), types));
 
 	treeview = treeview_gnet_stats_drop_reasons = GTK_TREE_VIEW(
 	    gui_main_window_lookup("treeview_gnet_stats_drop_reasons"));
-	gui_prop_get_guint32(PROP_GNET_STATS_DROP_REASONS_COL_WIDTHS,
-		width, 0, G_N_ELEMENTS(width));
 
-	for (n = 0; n < G_N_ELEMENTS(types); n++) {
+	gui_prop_get_guint32(PROP_GNET_STATS_DROP_REASONS_COL_WIDTHS,
+		width, 0, N_ITEMS(width));
+
+	gui_parent_widths_saveto(treeview, PROP_GNET_STATS_DROP_REASONS_COL_WIDTHS);
+
+	for (n = 0; n < N_ITEMS(types); n++) {
 		GtkTreeIter iter;
 		gint i;
 
@@ -508,18 +524,21 @@ gnet_stats_gui_general_init(void)
 		G_TYPE_STRING,
 		G_TYPE_STRING
 	};
-	guint32 width[G_N_ELEMENTS(types)];
+	guint32 width[N_ITEMS(types)];
 	guint n;
 
-	STATIC_ASSERT(2 == G_N_ELEMENTS(types));
-	model = GTK_TREE_MODEL(gtk_list_store_newv(G_N_ELEMENTS(types), types));
+	STATIC_ASSERT(2 == N_ITEMS(types));
+	model = GTK_TREE_MODEL(gtk_list_store_newv(N_ITEMS(types), types));
 
 	treeview = treeview_gnet_stats_general = GTK_TREE_VIEW(
 	    gui_main_window_lookup("treeview_gnet_stats_general"));
-	gui_prop_get_guint32(PROP_GNET_STATS_GENERAL_COL_WIDTHS,
-		width, 0, G_N_ELEMENTS(width));
 
-	for (n = 0; n < G_N_ELEMENTS(types); n++) {
+	gui_prop_get_guint32(PROP_GNET_STATS_GENERAL_COL_WIDTHS,
+		width, 0, N_ITEMS(width));
+
+	gui_parent_widths_saveto(treeview, PROP_GNET_STATS_GENERAL_COL_WIDTHS);
+
+	for (n = 0; n < N_ITEMS(types); n++) {
 		GtkTreeIter iter;
 		gint i;
 
@@ -551,12 +570,12 @@ gnet_stats_gui_messages_init(void)
 		G_TYPE_STRING,	/* c_gs_gen_queued */
 		G_TYPE_STRING	/* c_gs_generated */
 	};
-	guint32 width[G_N_ELEMENTS(types)];
+	guint32 width[N_ITEMS(types)];
 	guint n;
 
-	STATIC_ASSERT(num_c_gs == G_N_ELEMENTS(msg_stats_label));
-	STATIC_ASSERT(num_c_gs == G_N_ELEMENTS(types));
-	model = GTK_TREE_MODEL(gtk_list_store_newv(G_N_ELEMENTS(types), types));
+	STATIC_ASSERT(num_c_gs == N_ITEMS(msg_stats_label));
+	STATIC_ASSERT(num_c_gs == N_ITEMS(types));
+	model = GTK_TREE_MODEL(gtk_list_store_newv(N_ITEMS(types), types));
 
 	treeview = treeview_gnet_stats_messages = GTK_TREE_VIEW(
 	    gui_main_window_lookup("treeview_gnet_stats_messages"));
@@ -566,7 +585,7 @@ gnet_stats_gui_messages_init(void)
 		gint i;
 
 		gtk_list_store_append(GTK_LIST_STORE(model), &iter);
-		for (i = 0; (guint) i < G_N_ELEMENTS(msg_stats_label); i++) {
+		for (i = 0; (guint) i < N_ITEMS(msg_stats_label); i++) {
 			gtk_list_store_set(GTK_LIST_STORE(model), &iter,
 				i, i == 0 ? msg_type_str(n) : "-",
 				(-1));
@@ -574,9 +593,11 @@ gnet_stats_gui_messages_init(void)
 	}
 
 	gui_prop_get_guint32(PROP_GNET_STATS_MSG_COL_WIDTHS,
-		width, 0, G_N_ELEMENTS(width));
+		width, 0, N_ITEMS(width));
 
-	for (n = 0; (guint) n < G_N_ELEMENTS(msg_stats_label); n++) {
+	gui_parent_widths_saveto(treeview, PROP_GNET_STATS_MSG_COL_WIDTHS);
+
+	for (n = 0; (guint) n < N_ITEMS(msg_stats_label); n++) {
 		add_column(treeview, n, width[n], (gfloat) (n != 0),
 			_(msg_stats_label[n]));
 	}
@@ -603,22 +624,25 @@ gnet_stats_gui_recv_init(void)
 		G_TYPE_STRING,
 		G_TYPE_STRING
 	};
-	guint32 width[G_N_ELEMENTS(types)];
+	guint32 width[N_ITEMS(types)];
 	guint n;
 
-	STATIC_ASSERT(STATS_RECV_COLUMNS == G_N_ELEMENTS(types));
-	STATIC_ASSERT(STATS_RECV_COLUMNS == G_N_ELEMENTS(width));
-	model = GTK_TREE_MODEL(gtk_list_store_newv(G_N_ELEMENTS(types), types));
+	STATIC_ASSERT(STATS_RECV_COLUMNS == N_ITEMS(types));
+	STATIC_ASSERT(STATS_RECV_COLUMNS == N_ITEMS(width));
+	model = GTK_TREE_MODEL(gtk_list_store_newv(N_ITEMS(types), types));
 
 	treeview = treeview_gnet_stats_recv = GTK_TREE_VIEW(
 	    gui_main_window_lookup("treeview_gnet_stats_recv"));
-	gui_prop_get_guint32(PROP_GNET_STATS_RECV_COL_WIDTHS,
-		width, 0, G_N_ELEMENTS(width));
 
-	for (n = 0; n < G_N_ELEMENTS(width); n++) {
+	gui_prop_get_guint32(PROP_GNET_STATS_RECV_COL_WIDTHS,
+		width, 0, N_ITEMS(width));
+
+	gui_parent_widths_saveto(treeview, PROP_GNET_STATS_RECV_COL_WIDTHS);
+
+	for (n = 0; n < N_ITEMS(width); n++) {
 		gchar buf[16];
 
-		gm_snprintf(buf, sizeof(buf), "%d%c", n - 1,
+		str_bprintf(ARYLEN(buf), "%d%c", n - 1,
 				n + 1 < STATS_RECV_COLUMNS ? '\0' : '+');
 		add_column(treeview, n, width[n], (gfloat) (n != 0),
 			n == 0 ? _("Type") : buf);
@@ -647,8 +671,8 @@ gnet_stats_gui_recv_init(void)
 void
 gnet_stats_gui_init(void)
 {
-	notebook_gnet_stats = GTK_NOTEBOOK(
-							gui_main_window_lookup("gnet_stats_notebook"));
+	GtkNotebook *notebook_gnet_stats =
+		GTK_NOTEBOOK(gui_main_window_lookup("gnet_stats_notebook"));
 
 	/*
 	 * Initialize stats tables.
@@ -662,51 +686,45 @@ gnet_stats_gui_init(void)
 	gnet_stats_gui_recv_init();
 
 	guc_hsep_add_global_table_listener(
-		(GCallback) gnet_stats_gui_horizon_update, FREQ_UPDATES, 0);
+		(callback_fn_t) gnet_stats_gui_horizon_update, FREQ_UPDATES, 0);
 
 	main_gui_add_timer(gnet_stats_gui_timer);
+
+	/*
+	 * Restore previous page.
+	 */
+
+	{
+		uint32 page;
+
+		gui_prop_get_guint32_val(PROP_GNET_STATS_NOTEBOOK_TAB, &page);
+
+		if (page >= NUM_GNET_STATS_NB_PAGES)
+			page = GNET_STATS_NB_PAGE_STATS;
+
+		gnet_stats_notebook_current_page = page;
+		gtk_notebook_set_current_page(notebook_gnet_stats, page);
+	}
+
+	gui_signal_connect(notebook_gnet_stats,
+		"switch-page", on_gnet_stats_notebook_switch_page, NULL);
 }
 
 void
 gnet_stats_gui_shutdown(void)
 {
-	static const struct {
-		property_t prop;
-		GtkTreeView **tv;
-	} widths[] = {
-		{ 	PROP_GNET_STATS_GENERAL_COL_WIDTHS,
-			&treeview_gnet_stats_general },
-		{ 	PROP_GNET_STATS_DROP_REASONS_COL_WIDTHS,
-			&treeview_gnet_stats_drop_reasons },
-		{ 	PROP_GNET_STATS_MSG_COL_WIDTHS,
-			&treeview_gnet_stats_messages },
-		{ 	PROP_GNET_STATS_FC_COL_WIDTHS,
-			&treeview_gnet_stats_flowc },
-		{ 	PROP_GNET_STATS_RECV_COL_WIDTHS,
-			&treeview_gnet_stats_recv },
-		{ 	PROP_GNET_STATS_HORIZON_COL_WIDTHS,
-			&treeview_gnet_stats_horizon },
-	};
-	size_t i;
-
 	guc_hsep_remove_global_table_listener(
-	    (GCallback) gnet_stats_gui_horizon_update);
-
-	for (i = 0; i < G_N_ELEMENTS(widths); i++) {
-		tree_view_save_widths(*(widths->tv), widths->prop);
-	}
+	    (callback_fn_t) gnet_stats_gui_horizon_update);
 }
 
 void
 gnet_stats_gui_update_display(time_t now)
 {
 	static gnet_stats_t stats;
-	gint current_page;
 
 	guc_gnet_stats_get(&stats);
 
-	current_page = gtk_notebook_get_current_page(notebook_gnet_stats);
-	switch ((enum gnet_stats_nb_page) current_page) {
+	switch (gnet_stats_notebook_current_page) {
 	case GNET_STATS_NB_PAGE_STATS:
 		gnet_stats_update_general(&stats);
 		gnet_stats_update_drop_reasons(&stats);
@@ -735,6 +753,7 @@ gnet_stats_gui_update_display(time_t now)
 	case GNET_STATS_NB_PAGE_RECV:
 		gnet_stats_update_recv(&stats);
 		break;
+	case GNET_STATS_NB_PAGE_FILE_TRANSFERS:
 	case NUM_GNET_STATS_NB_PAGES:
 		break;
 	}
