@@ -1,6 +1,4 @@
 /*
- * $Id: nodes_cb.c 13951 2007-06-24 00:48:45Z cbiere $
- *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -36,8 +34,6 @@
 
 #include "gtk/gui.h"
 
-RCSID("$Id: nodes_cb.c 13951 2007-06-24 00:48:45Z cbiere $")
-
 #include "nodes_cb.h"
 #include "gtk/gtkcolumnchooser.h"
 #include "gtk/nodes_common.h"
@@ -115,23 +111,6 @@ on_entry_host_changed(GtkEditable *editable, gpointer unused_udata)
 	G_FREE_NULL(e);
 }
 
-gboolean
-on_treeview_nodes_button_press_event(GtkWidget *unused_widget,
-		GdkEventButton *event, gpointer unused_udata)
-{
-	(void) unused_widget;
-	(void) unused_udata;
-
-    if (3 == event->button) {
-        /* right click section (popup menu) */
-
-        gtk_menu_popup(GTK_MENU(gui_popup_nodes()), NULL, NULL, NULL, NULL,
-			event->button, event->time);
-        return TRUE;
-	}
-	return FALSE;
-}
-
 /**
  * Creates and pops up the column chooser for ``treeview_nodes''
  */
@@ -144,7 +123,8 @@ on_popup_nodes_config_cols_activate(GtkItem *unused_menuitem,
 	(void) unused_menuitem;
 	(void) unused_udata;
     cc = gtk_column_chooser_new(gui_main_window_lookup("treeview_nodes"));
-    gtk_menu_popup(GTK_MENU(cc), NULL, NULL, NULL, NULL, 1, 0);
+    gtk_menu_popup(GTK_MENU(cc), NULL, NULL, NULL, NULL, 1,
+		gtk_get_current_event_time());
 
     /* GtkColumnChooser takes care of cleaning up itself */
 	return TRUE;
