@@ -1,6 +1,4 @@
 /*
- * $Id: html_view.c 16990 2009-09-25 15:23:43Z rmanfredi $
- *
  * Copyright (c) 2007 Christian Biere
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -35,15 +33,19 @@
 
 #include "gui.h"
 
-RCSID("$Id: html_view.c 16990 2009-09-25 15:23:43Z rmanfredi $")
-
 #include "lib/ascii.h"
 #include "lib/concat.h"
 #include "lib/glib-missing.h"
+#include "lib/halloc.h"
+#include "lib/hstrfn.h"
 #include "lib/html.h"
+#include "lib/mempcpy.h"
+#include "lib/str.h"
 #include "lib/utf8.h"
 #include "lib/walloc.h"
 
+#include "lib/override.h"
+
 #define STYLE_TAG(x) static const gchar STYLE_TAG_ ## x [] = #x ;
 STYLE_TAG(ANCHOR)
 STYLE_TAG(ANCHOR_EXTERN)
@@ -72,7 +74,7 @@ struct html_view {
 struct html_context {
 	struct html_view *html_view;
 	struct html_output *output;
-	GString *title;
+	str_t *title;
 	const gchar *lang, *href;
 
 #if GTK_CHECK_VERSION(2,0,0)
@@ -83,11 +85,9 @@ struct html_context {
 static struct html_context *
 html_context_alloc(void)
 {
-	static const struct html_context zero_html_context;
 	struct html_context *ctx;
 
-	ctx = walloc(sizeof *ctx);
-	*ctx = zero_html_context;
+	WALLOC0(ctx);
 	return ctx;
 }
 
@@ -96,23 +96,19 @@ html_context_free(struct html_context **ctx_ptr)
 {
 	struct html_context *ctx = *ctx_ptr;
 	if (ctx) {
-		if (ctx->title) {
-			g_string_free(ctx->title, TRUE);
-			ctx->title = NULL;
-		}
+		str_destroy_null(&ctx->title);
 		html_output_free(&ctx->output);
+		WFREE(ctx);
 		*ctx_ptr = NULL;
 	}
 }
-	
+
 static struct html_view *
 html_view_alloc(void)
 {
-	static const struct html_view zero_html_view;
 	struct html_view *ctx;
 
-	ctx = walloc(sizeof *ctx);
-	*ctx = zero_html_view;
+	WALLOC0(ctx);
 	return ctx;
 }
 
@@ -120,13 +116,13 @@ static void
 html_output_print(struct html_output *output, const struct array *text)
 {
 	struct html_context *ctx;
-   
+
 	if (text->size <= 0)
 		return;
 
 	ctx = html_output_get_udata(output);
 	if (ctx->title) {
-		g_string_append_len(ctx->title, text->data, text->size);
+		str_cat_len(ctx->title, text->data, text->size);
 		return;
 	}
 #if GTK_CHECK_VERSION(2,0,0)
@@ -147,12 +143,12 @@ html_output_print(struct html_output *output, const struct array *text)
 			str = array_init(text->data, text->size);
 			to_free = NULL;
 		} else {
-			to_free = g_strndup(text->data, text->size);
+			to_free = h_strndup(text->data, text->size);
 			str = array_from_string(lazy_utf8_to_ui_string(to_free));
 		}
 		gtk_text_insert(ctx->html_view->widget,
 			NULL, NULL, NULL, str.data, str.size);
-		G_FREE_NULL(to_free);
+		HFREE_NULL(to_free);
 	}
 #endif
 }
@@ -176,6 +172,7 @@ utf8_char(guint32 codepoint)
 	unsigned len;
 
 	STATIC_ASSERT(sizeof buf.str > 4);
+
 	len = utf8_encode(codepoint, buf.str);
 	buf.str[len] = '\0';
 	return buf;
@@ -196,6 +193,9 @@ html_output_tag(struct html_output *output, const struct array *tag)
 	gboolean closing;
 	GtkTextBuffer *buffer;
 
+	/* Since we don't ZERO the whole short_string_t in utf8_char() */
+	G_IGNORE_PUSH(-Wmaybe-uninitialized);
+
 	if (!special.initialized) {
 
 		special.initialized = TRUE;
@@ -204,11 +204,12 @@ html_output_tag(struct html_output *output, const struct array *tag)
 		special.soft_hyphen = utf8_char(0x00AD);
 		special.nbsp = utf8_char(0x00A0);
 		special.em_dash = utf8_char(0x2014);
-		concat_strings(special.list_item_prefix.str,
-			sizeof special.list_item_prefix.str,
-			" ", special.bullet.str, " ", (void *) 0);
+		concat_strings(ARYLEN(special.list_item_prefix.str),
+			" ", special.bullet.str, " ", NULL_PTR);
 	}
-	
+
+	G_IGNORE_POP;
+
 	style = NULL;
 	text = NULL;
 	attr = NULL;
@@ -227,7 +228,7 @@ html_output_tag(struct html_output *output, const struct array *tag)
 				GtkTextIter start, end;
 				GtkTextTag *anchor;
 
-				anchor = gtk_text_buffer_create_tag(buffer, NULL, (void *) 0);
+				anchor = gtk_text_buffer_create_tag(buffer, NULL, NULL_PTR);
 				g_object_set_data(G_OBJECT(anchor), "href",
 					deconstify_gchar(ctx->href));
 				gtk_text_buffer_get_iter_at_mark(buffer,
@@ -258,13 +259,14 @@ html_output_tag(struct html_output *output, const struct array *tag)
 				GtkTextTagTable *table;
 				gchar name[256];
 				size_t n;
+				char *p;
 
 				n = sizeof name - 2;
 				n = MIN(value.size, n);
 				name[0] = '#';
-				memcpy(&name[1], value.data, n);
-				name[n + 1] = '\0';
-				
+				p = mempcpy(&name[1], value.data, n);
+				*p = '\0';
+
 				table = gtk_text_buffer_get_tag_table(buffer);
 				if (NULL == gtk_text_tag_table_lookup(table, name)) {
 					GtkTextIter iter;
@@ -295,10 +297,10 @@ html_output_tag(struct html_output *output, const struct array *tag)
 		if (!closing) {
 			struct array value;
 			static gchar alt[1024];
-			
+
 			value = html_get_attribute(tag, HTML_ATTR_ALT);
 			if (value.data) {
-				gm_snprintf(alt, sizeof alt, "\n[image alt=\"%.*s\"]\n",
+				str_bprintf(ARYLEN(alt), "\n[image alt=\"%.*s\"]\n",
 					(int)value.size, value.data);
 				text = alt;
 			}
@@ -308,18 +310,18 @@ html_output_tag(struct html_output *output, const struct array *tag)
 				gchar *filename;
 				GtkTextIter iter;
 
-				filename = g_strndup(value.data, value.size);
+				filename = h_strndup(value.data, value.size);
 				pixbuf = gdk_pixbuf_new_from_file(filename, NULL);
 				if (pixbuf) {
 					gtk_text_buffer_get_end_iter(buffer, &iter);
 					gtk_text_buffer_insert_pixbuf(buffer, &iter, pixbuf);
 				} else {
 					static gchar msg[1024];
-					gm_snprintf(msg, sizeof msg,
+					str_bprintf(ARYLEN(msg),
 						"\n[Image not found (\"%s\")]\n", filename);
 					text = msg;
 				}
-				G_FREE_NULL(filename);
+				HFREE_NULL(filename);
 			}
 			if (!text) {
 				text = "\n[image]\n";
@@ -360,21 +362,21 @@ html_output_tag(struct html_output *output, const struct array *tag)
 			margin = gtk_text_buffer_create_tag(buffer, NULL,
 						"left-margin",		width * 2,
 						"left-margin-set",	TRUE,
-						(void *) 0);
+						NULL_PTR);
 
 			gtk_text_buffer_get_iter_at_mark(buffer, &start, ctx->start[id]);
 			gtk_text_buffer_get_end_iter(buffer, &end);
 			gtk_text_buffer_apply_tag(buffer, margin, &start, &end);
 		} else {
 			GtkTextIter iter;
-		
+
 			gtk_text_buffer_get_end_iter(buffer, &iter);
 			gtk_text_buffer_insert(buffer, &iter, "\n", (-1));
 
 			gtk_text_buffer_get_end_iter(buffer, &iter);
 			gtk_text_buffer_insert_with_tags_by_name(buffer, &iter,
 					special.list_item_prefix.str, (-1),
-					STYLE_TAG_BOLD, (void *) 0);
+					STYLE_TAG_BOLD, NULL_PTR);
 			gtk_text_buffer_get_end_iter(buffer, &iter);
 			ctx->start[id] = gtk_text_buffer_create_mark(buffer, NULL,
 					&iter, TRUE);
@@ -411,12 +413,11 @@ html_output_tag(struct html_output *output, const struct array *tag)
 
 				window = gtk_widget_get_toplevel(
 							GTK_WIDGET(ctx->html_view->widget));
-				gtk_window_set_title(GTK_WINDOW(window), ctx->title->str);
-				g_string_free(ctx->title, TRUE);
-				ctx->title = NULL;
+				gtk_window_set_title(GTK_WINDOW(window), str_2c(ctx->title));
+				str_destroy_null(&ctx->title);
 			}
 		} else {
-			ctx->title = g_string_new("");
+			ctx->title = str_new(0);
 		}
 		break;
 	case HTML_TAG_HR:
@@ -426,12 +427,12 @@ html_output_tag(struct html_output *output, const struct array *tag)
 			gtk_text_buffer_get_end_iter(buffer, &iter);
 			gtk_text_buffer_insert_with_tags_by_name(buffer, &iter,
 					"\n    \n", (-1),
-					STYLE_TAG_CENTER, STYLE_TAG_UNDERLINE, (void *) 0);
+					STYLE_TAG_CENTER, STYLE_TAG_UNDERLINE, NULL_PTR);
 		}
 		text = "\n";
 		break;
 	case HTML_TAG_COMMENT:
-#if 0 
+#if 0
 		{
 			GtkTextIter iter;
 
@@ -440,7 +441,7 @@ html_output_tag(struct html_output *output, const struct array *tag)
 					&iter, TRUE);
 			gtk_text_buffer_get_end_iter(buffer, &iter);
 			gtk_text_buffer_insert_with_tags_by_name(buffer, &iter,
-					tag->data, tag->size, STYLE_TAG_ITALIC, (void *) 0);
+					tag->data, tag->size, STYLE_TAG_ITALIC, NULL_PTR);
 		}
 		closing = TRUE;
 		text = "\n";
@@ -455,7 +456,7 @@ html_output_tag(struct html_output *output, const struct array *tag)
 				lang = gtk_text_buffer_create_tag(buffer, NULL,
 						"language",		ctx->lang,
 						"language-set",	TRUE,
-						(void *) 0);
+						NULL_PTR);
 				gtk_text_buffer_get_iter_at_mark(buffer,
 					&start, ctx->start[id]);
 				gtk_text_buffer_get_end_iter(buffer, &end);
@@ -494,7 +495,7 @@ html_output_tag(struct html_output *output, const struct array *tag)
 		if (closing) {
 			if (ctx->start[id]) {
 				GtkTextIter start, end;
-		
+
 				gtk_text_buffer_get_iter_at_mark(buffer,
 						&start, ctx->start[id]);
 				gtk_text_buffer_get_end_iter(buffer, &end);
@@ -513,7 +514,7 @@ html_output_tag(struct html_output *output, const struct array *tag)
 
 		gtk_text_buffer_get_end_iter(buffer, &iter);
 		gtk_text_buffer_insert_with_tags_by_name(buffer, &iter,
-			text, (-1), attr, (void *) 0);
+			text, (-1), attr, NULL_PTR);
 	}
 }
 #else	/* Gtk+ < 2.0 */
@@ -572,12 +573,11 @@ html_output_tag(struct html_output *output, const struct array *tag)
 
 				window = gtk_widget_get_toplevel(
 							GTK_WIDGET(ctx->html_view->widget));
-				gtk_window_set_title(GTK_WINDOW(window), ctx->title->str);
-				g_string_free(ctx->title, TRUE);
-				ctx->title = NULL;
+				gtk_window_set_title(GTK_WINDOW(window), str_2c(ctx->title));
+				str_destroy_null(&ctx->title);
 			}
 		} else {
-			ctx->title = g_string_new("");
+			ctx->title = str_new(0);
 		}
 		break;
 	case HTML_TAG_A:
@@ -606,7 +606,7 @@ html_output_tag(struct html_output *output, const struct array *tag)
 	case NUM_HTML_TAG:
 		g_assert_not_reached();
 	}
-		
+
 	if (text) {
 		gtk_text_insert(ctx->html_view->widget, NULL, NULL, NULL, text, (-1));
 	}
@@ -620,11 +620,9 @@ static void
 set_cursor_if_appropriate(GtkTextView *text_view, gint x, gint y)
 {
 	GSList *tags, *sl;
-	GtkTextBuffer *buffer;
 	GtkTextIter iter;
 	gboolean hovering = FALSE;
 
-	buffer = gtk_text_view_get_buffer(text_view);
 	gtk_text_view_get_iter_at_location(text_view, &iter, x, y);
 	tags = gtk_text_iter_get_tags(&iter);
 
@@ -648,7 +646,7 @@ visibility_notify_event(GtkWidget *text_view, GdkEventVisibility *event)
 
 	(void) event;
 	gdk_window_get_pointer(text_view->window, &wx, &wy, NULL);
-	gtk_text_view_window_to_buffer_coords(GTK_TEXT_VIEW(text_view), 
+	gtk_text_view_window_to_buffer_coords(GTK_TEXT_VIEW(text_view),
 		GTK_TEXT_WINDOW_WIDGET, wx, wy, &bx, &by);
 	set_cursor_if_appropriate(GTK_TEXT_VIEW(text_view), bx, by);
 	return FALSE;
@@ -660,14 +658,14 @@ motion_notify_event(GtkWidget *text_view, GdkEventMotion *event)
 {
 	gint x, y;
 
-	gtk_text_view_window_to_buffer_coords(GTK_TEXT_VIEW(text_view), 
+	gtk_text_view_window_to_buffer_coords(GTK_TEXT_VIEW(text_view),
 		GTK_TEXT_WINDOW_WIDGET, event->x, event->y, &x, &y);
 	set_cursor_if_appropriate(GTK_TEXT_VIEW(text_view), x, y);
 	gdk_window_get_pointer(text_view->window, NULL, NULL, NULL);
 	return FALSE;
 }
 
-/* Looks at all tags covering the position of iter in the text view, 
+/* Looks at all tags covering the position of iter in the text view,
  * and if one of them is a link, follow it by showing the page identified
  * by the data attached to it.
  */
@@ -706,7 +704,7 @@ key_press_event(GtkWidget *text_view, GdkEventKey *event)
 	GtkTextBuffer *buffer;
 
 	switch (event->keyval) {
-	case GDK_Return: 
+	case GDK_Return:
 	case GDK_KP_Enter:
 		buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(text_view));
 		gtk_text_buffer_get_iter_at_mark(buffer, &iter,
@@ -739,7 +737,7 @@ event_after(GtkWidget *text_view, GdkEvent *ev)
 	/* we shouldn't follow a link if the user has selected something */
 	gtk_text_buffer_get_selection_bounds(buffer, &start, &end);
 	if (gtk_text_iter_get_offset(&start) == gtk_text_iter_get_offset(&end)) {
-		gtk_text_view_window_to_buffer_coords(GTK_TEXT_VIEW(text_view), 
+		gtk_text_view_window_to_buffer_coords(GTK_TEXT_VIEW(text_view),
 			GTK_TEXT_WINDOW_WIDGET, event->x, event->y, &x, &y);
 		gtk_text_view_get_iter_at_location(GTK_TEXT_VIEW(text_view),
 			&iter, x, y);
@@ -779,73 +777,74 @@ html_view_load(struct html_view *html_view)
 		"visibility-notify-event", visibility_notify_event, NULL);
 	gui_signal_connect(widget, "key-press-event", key_press_event, NULL);
 	gui_signal_connect(widget, "event-after", event_after, NULL);
-		  
+
  	gtk_text_view_set_buffer(GTK_TEXT_VIEW(widget), NULL);
 	{
 		GtkTextBuffer *buffer;
-		
+
 		buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(widget));
 
 		gtk_text_buffer_create_tag(buffer, STYLE_TAG_WORD_WRAP,
 				"wrap_mode",		GTK_WRAP_WORD,
-				(void *) 0);
+				NULL_PTR);
 		gtk_text_buffer_create_tag(buffer, STYLE_TAG_MONOSPACE,
 				"family",			"monospace",
-				(void *) 0);
+				NULL_PTR);
 		gtk_text_buffer_create_tag(buffer, STYLE_TAG_STRETCH,
 				"stretch",			PANGO_STRETCH_ULTRA_EXPANDED,
 				"stretch-set",		TRUE,
-				(void *) 0);
+				NULL_PTR);
 		gtk_text_buffer_create_tag(buffer,	STYLE_TAG_ANCHOR,
-				"foreground",		"blue", 
-				"underline",		PANGO_UNDERLINE_SINGLE, 
-				(void *) 0);
+				"foreground",		"blue",
+				"underline",		PANGO_UNDERLINE_SINGLE,
+				NULL_PTR);
 		gtk_text_buffer_create_tag(buffer,	STYLE_TAG_ANCHOR_EXTERN,
-				"foreground",		"red", 
-				"underline",		PANGO_UNDERLINE_SINGLE, 
-				(void *) 0);
+				"foreground",		"red",
+				"underline",		PANGO_UNDERLINE_SINGLE,
+				NULL_PTR);
 		gtk_text_buffer_create_tag(buffer, STYLE_TAG_BOLD,
 				"weight",			PANGO_WEIGHT_BOLD,
-				(void *) 0);
+				NULL_PTR);
 		gtk_text_buffer_create_tag(buffer, STYLE_TAG_ITALIC,
 				"style",			PANGO_STYLE_ITALIC,
-				(void *) 0);
+				NULL_PTR);
 		gtk_text_buffer_create_tag(buffer, STYLE_TAG_CENTER,
 				"justification",	GTK_JUSTIFY_CENTER,
-				(void *) 0);
+				NULL_PTR);
 		gtk_text_buffer_create_tag(buffer, STYLE_TAG_UNDERLINE,
 				"underline",		PANGO_UNDERLINE_SINGLE,
-				(void *) 0);
+				NULL_PTR);
 		gtk_text_buffer_create_tag(buffer, STYLE_TAG_HEADING_1,
 				"weight",			PANGO_WEIGHT_BOLD,
 				"size",				15 * PANGO_SCALE,
-				(void *) 0);
+				NULL_PTR);
 		gtk_text_buffer_create_tag(buffer, STYLE_TAG_HEADING_2,
 				"weight",			PANGO_WEIGHT_BOLD,
 				"size",				14 * PANGO_SCALE,
-				(void *) 0);
+				NULL_PTR);
 		gtk_text_buffer_create_tag(buffer, STYLE_TAG_HEADING_3,
 				"weight",			PANGO_WEIGHT_BOLD,
 				"size",				13 * PANGO_SCALE,
-				(void *) 0);
+				NULL_PTR);
 		gtk_text_buffer_create_tag(buffer, STYLE_TAG_HEADING_4,
 				"weight",			PANGO_WEIGHT_BOLD,
 				"size",				12 * PANGO_SCALE,
-				(void *) 0);
+				NULL_PTR);
 	}
 #else	/* Gtk+ < 2.0 */
 
+	(void) widget;
 	ctx->html_view = html_view;
 	gtk_text_freeze(html_view->widget);
 	gtk_text_set_word_wrap(html_view->widget, TRUE);
-	
+
 #endif	/* Gtk+ >= 2.0 */
 
 	ctx->output = html_output_alloc();
 	html_output_set_udata(ctx->output, ctx);
 	html_output_set_print(ctx->output, html_output_print);
 	html_output_set_tag(ctx->output, html_output_tag);
-	
+
 #if GTK_CHECK_VERSION(2,0,0)
 #else
 	gtk_text_thaw(html_view->widget);
@@ -879,7 +878,7 @@ html_view_load_memory(GtkWidget *widget, const struct array memory)
 {
 	struct html_view *html_view;
 	struct html_context *ctx;
-	
+
 	g_return_val_if_fail(widget, NULL);
 	g_return_val_if_fail(memory.data, NULL);
 
@@ -904,7 +903,7 @@ html_view_clear(struct html_view *html_view)
 	{
 		GtkTextBuffer *buffer;
 		GtkTextIter start, end;
-		
+
 		buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(html_view->widget));
 		gtk_text_buffer_get_start_iter(buffer, &start);
 		gtk_text_buffer_get_end_iter(buffer, &end);
@@ -937,9 +936,9 @@ html_view_free(struct html_view **html_view_ptr)
 			for (iter = html_view->to_free; iter; iter = g_slist_next(iter)) {
 				G_FREE_NULL(iter->data);
 			}
-			g_slist_free(html_view->to_free);
+			gm_slist_free_null(&html_view->to_free);
 		}
-		wfree(html_view, sizeof *html_view);
+		WFREE(html_view);
 		*html_view_ptr = NULL;
 	}
 }
