@@ -1,6 +1,4 @@
 /*
- * $Id: main.h 17033 2009-10-05 20:51:00Z rmanfredi $
- *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -30,6 +28,15 @@
 
 #include <gtk/gtk.h>
 
+/*
+ * Declare all the windows created by glade (through create_xxx()) here
+ * so that we can refer to them via gui_xxx() in the code.
+ *
+ * The actual association (mapping between the output of create_xxx() from
+ * glade and the value returned by the gui_xxx() routine) is done at GUI
+ * initialization time in main_gui_early_init() through gui_xxx_set().
+ */
+
 #define WIDGET(name) \
 	GtkWidget *gui_ ## name (void); \
 	GtkWidget *gui_ ## name ## _lookup(const gchar *id);
@@ -37,6 +44,7 @@
 WIDGET(dlg_about)
 WIDGET(dlg_ancient)
 WIDGET(dlg_faq)
+WIDGET(dlg_glossary)
 WIDGET(dlg_prefs)
 WIDGET(dlg_quit)
 WIDGET(main_window)
@@ -51,18 +59,23 @@ WIDGET(shutdown_window)
 #undef WIDGET
 
 void main_gui_early_init(gint argc, gchar **argv, gboolean disable_xshm);
+void main_gui_disable_ancient(bool v);
 void main_gui_init(void);
 void main_gui_exit(int n);
-void main_gui_run(const gchar *geometry_spec);
+void main_gui_run(const gchar *geometry_spec, const gboolean minimized);
 void main_gui_shutdown(void);
 void main_gui_timer(time_t now);
 void main_gui_shutdown_tick(guint);
 void main_gui_show_prefences(void);
 
 gboolean main_gui_window_visible(void);
+bool main_gui_ancient_is_disabled(void);
 
 typedef void (*main_gui_visibility_cb)(gboolean visible);
 
+void main_gui_add_visibility_listener(main_gui_visibility_cb);
+void main_gui_remove_visibility_listener(main_gui_visibility_cb);
+
 void main_gui_add_page_visibility_listener(main_gui_visibility_cb, int page);
 void main_gui_remove_page_visibility_listener(main_gui_visibility_cb, int page);
 
