@@ -1,6 +1,4 @@
 /*
- * $Id: settings_cb.c 17077 2009-10-16 20:51:38Z rmanfredi $
- *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -36,7 +34,9 @@
 
 #include "gui.h"
 
-RCSID("$Id: settings_cb.c 17077 2009-10-16 20:51:38Z rmanfredi $")
+#ifdef USE_GTK2
+#include "gtk2/column_sort.h"
+#endif
 
 #include "misc.h"
 #include "settings_cb.h"
@@ -47,7 +47,10 @@ RCSID("$Id: settings_cb.c 17077 2009-10-16 20:51:38Z rmanfredi $")
 #include "if/gnet_property.h"
 #include "if/bridge/ui2c.h"
 
-#include "lib/glib-missing.h"	/* For g_strlcpy() */
+#include "lib/cstr.h"
+#include "lib/halloc.h"
+#include "lib/pslist.h"
+#include "lib/str.h"
 #include "lib/override.h"		/* Must be the last header included */
 
 /*
@@ -156,7 +159,8 @@ on_button_config_remove_dir_clicked(GtkButton *unused_button,
 	GtkTreeModel *model;
 	GtkTreeIter iter;
 	GtkTreeSelection *selection;
-	GString *gs;
+	pslist_t *sl, *pl_dirs = NULL;
+	char *dirs;
 
 	(void) unused_button;
 	(void) unused_udata;
@@ -169,24 +173,26 @@ on_button_config_remove_dir_clicked(GtkButton *unused_button,
 
 	/* Regenerate the string property holding a list of paths */
 	selection = gtk_tree_view_get_selection(tv);
-	gs = g_string_new("");
 
 	do {
-		gchar *dir = NULL;
+		char *pathname = NULL;
 
 		/* Skip items selected for removal */
 		if (gtk_tree_selection_iter_is_selected(selection, &iter))
 			continue;
 
-		gtk_tree_model_get(model, &iter, 0, &dir, (-1));
-		if ('\0' != gs->str[0])
-			gs = g_string_append_c(gs, ':');
-		gs = g_string_append(gs, dir);
-		G_FREE_NULL(dir);
+		gtk_tree_model_get(model, &iter, 0, &pathname, (-1));
+		pl_dirs = pslist_prepend(pl_dirs, pathname);
 	} while (gtk_tree_model_iter_next(model, &iter));
 
-	gnet_prop_set_string(PROP_SHARED_DIRS_PATHS, gs->str);
-	g_string_free(gs, TRUE);
+	dirs = dirlist_to_string(pl_dirs);
+	gnet_prop_set_string(PROP_SHARED_DIRS_PATHS, dirs);
+	HFREE_NULL(dirs);
+
+	PSLIST_FOREACH(pl_dirs, sl) {
+		G_FREE_NULL(sl->data);
+	}
+	pslist_free_null(&pl_dirs);
 }
 #endif /* USE_GTK2 */
 
@@ -249,7 +255,7 @@ on_entry_config_force_ipv6_activate(GtkEditable *unused_editable,
 	if (
 		string_to_host_addr(text, &endptr, &addr)
 			&& '\0' == endptr[0]
-			&& NET_TYPE_IPV6 == host_addr_net(addr)
+			&& host_addr_is_ipv6(addr)
 	) {
 		gnet_prop_set_ip_val(PROP_FORCED_LOCAL_IP6, addr);
 	} else if (0 == strcmp(text, "") || 0 == strcmp(text, "<none>")) {
@@ -271,7 +277,7 @@ on_entry_config_force_ipv6_changed(GtkEditable *editable, gpointer unused_udata)
 	g_strstrip(text);
 	sensitive = string_to_host_addr(text, &endptr, &addr)
 			&& '\0' == endptr[0]
-			&& NET_TYPE_IPV6 == host_addr_net(addr);
+			&& host_addr_is_ipv6(addr);
 	gtk_widget_set_sensitive(
         gui_dlg_prefs_lookup("checkbutton_config_force_ipv6"),
 		sensitive);
@@ -298,7 +304,7 @@ on_entry_config_ipv6_trt_prefix_activate(GtkEditable *unused_editable,
 	if (
 		string_to_host_addr(text, &endptr, &addr)
 			&& '\0' == endptr[0]
-			&& NET_TYPE_IPV6 == host_addr_net(addr)
+			&& host_addr_is_ipv6(addr)
 	) {
 		gnet_prop_set_ip_val(PROP_IPV6_TRT_PREFIX, addr);
 	} else if (0 == strcmp(text, "") || 0 == strcmp(text, "<none>")) {
@@ -321,7 +327,7 @@ on_entry_config_ipv6_trt_prefix_changed(GtkEditable *editable,
 	g_strstrip(text);
 	sensitive = string_to_host_addr(text, &endptr, &addr)
 			&& '\0' == endptr[0]
-			&& NET_TYPE_IPV6 == host_addr_net(addr);
+			&& host_addr_is_ipv6(addr);
 	gtk_widget_set_sensitive(
         gui_dlg_prefs_lookup("checkbutton_config_ipv6_trt_enable"),
 		sensitive);
@@ -351,29 +357,31 @@ on_entry_server_hostname_changed(GtkEditable *editable, gpointer unused_udata)
     gchar *text = STRTRACK(gtk_editable_get_chars(editable, 0, -1));
 
 	(void) unused_udata;
-	
+
 	g_strstrip(text);
 	gtk_widget_set_sensitive(
         gui_dlg_prefs_lookup("checkbutton_give_server_hostname"),
-        strlen(text) > 3);		/* Minimum: "x.cx" */
+        vstrlen(text) > 3);		/* Minimum: "x.cx" */
 	G_FREE_NULL(text);
 }
 
 enum dbg_cols {
 	dbg_col_saved = 0,
+	dbg_col_internal,
 	dbg_col_type,
 	dbg_col_name,
 	dbg_col_value,
-	
-#ifdef USE_GTK2	
+
+#ifdef USE_GTK2
 	dbg_col_property,
 #endif /* USE_GTK2 */
-	
+
 	num_dbg_cols
 };
 
 #ifdef USE_GTK2
 static tree_view_motion_t *tvm_dbg_property;
+static struct sorting_context dbg_column_sort;
 
 static void
 update_tooltip(GtkTreeView *tv, GtkTreePath *path)
@@ -428,13 +436,11 @@ on_enter_notify(GtkWidget *widget, GdkEventCrossing *unused_event,
 
 static gboolean
 on_leave_notify(GtkWidget *widget, GdkEventCrossing *unused_event,
-	gpointer data)
+	gpointer unused_data)
 {
-	GtkTreeView *tv;
-
 	(void) unused_event;
+	(void) unused_data;
 
-	tv = GTK_TREE_VIEW(data);
 	update_tooltip(GTK_TREE_VIEW(widget), NULL);
 	tree_view_motion_clear_callback(&tvm_dbg_property);
 	return FALSE;
@@ -465,7 +471,8 @@ on_cell_edited(GtkCellRendererText *unused_renderer, const gchar *path_str,
 	u = 0;
 	gtk_tree_model_get(model, &iter, dbg_col_property, &u, (-1));
 	prop = (property_t) u;
-	gnet_prop_set_from_string(prop,	text);
+	if (!gnet_prop_is_internal(prop))
+		gnet_prop_set_from_string(prop,	text);
 	gtk_list_store_set(GTK_LIST_STORE(model), &iter,
 		dbg_col_value, gnet_prop_to_string(prop),
 		(-1));
@@ -503,7 +510,15 @@ on_button_dbg_property_refresh_clicked(GtkButton *unused_button,
 	(void) unused_udata;
 
 	tv = GTK_TREE_VIEW(gui_dlg_prefs_lookup("treeview_dbg_property"));
+	g_assert(GTK_IS_TREE_VIEW(tv));
+
 	model = gtk_tree_view_get_model(tv);
+	if (!model) {
+		on_entry_dbg_property_pattern_activate(NULL, NULL);
+		return;
+	}
+	g_assert(GTK_IS_TREE_MODEL(model));
+
 	gtk_tree_model_foreach(GTK_TREE_MODEL(model), refresh_property, NULL);
 }
 
@@ -527,7 +542,7 @@ dbg_property_update_selection(void)
 	} else {
 		text = _("<no property selected>");
 	}
-	
+
     widget = gui_dlg_prefs_lookup("label_dbg_property_default");
 	gtk_label_set_text(GTK_LABEL(widget), text);
 }
@@ -540,6 +555,36 @@ on_cursor_changed(GtkTreeView *unused_tv, gpointer unused_udata)
 	dbg_property_update_selection();
 }
 
+static void
+on_dbg_column_clicked(GtkTreeViewColumn *column, void *udata)
+{
+	(void) udata;
+
+	column_sort_tristate(column, &dbg_column_sort);
+}
+
+static inline const char *
+dbg_get_data(GtkTreeModel *model, GtkTreeIter *iter, gint column)
+{
+	GValue value;
+
+	ZERO(&value);
+	gtk_tree_model_get_value(model, iter, column, &value);
+	return g_value_get_string(&value);
+}
+
+static gint
+dbg_column_cmp(
+	GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b, gpointer col)
+{
+	const char *sa, *sb;
+	gint column = pointer_to_int(col);
+
+	sa = dbg_get_data(model, a, column);
+	sb = dbg_get_data(model, b, column);
+	return strcmp(sa, sb);
+}
+
 static void
 dbg_tree_init(void)
 {
@@ -550,6 +595,7 @@ dbg_tree_init(void)
 		const enum dbg_cols id;
 	} columns[] = {
 		{ N_("Saved"),	 	  0, FALSE, dbg_col_saved },
+		{ N_("Internal"), 	  0, FALSE, dbg_col_internal },
 		{ N_("Type"),		  0, FALSE, dbg_col_type },
 		{ N_("Property"),	  0, FALSE, dbg_col_name },
 		{ N_("Value"),		200, TRUE,  dbg_col_value  },
@@ -559,20 +605,21 @@ dbg_tree_init(void)
 	GtkTreeView *tv;
 	guint i;
 
-	STATIC_ASSERT(G_N_ELEMENTS(columns) == (guint) num_dbg_cols);
-	
+	STATIC_ASSERT(N_ITEMS(columns) == (guint) num_dbg_cols);
+
 	tv = GTK_TREE_VIEW(gui_dlg_prefs_lookup("treeview_dbg_property"));
-	store = GTK_LIST_STORE(gtk_list_store_new(G_N_ELEMENTS(columns),
+	store = GTK_LIST_STORE(gtk_list_store_new(N_ITEMS(columns),
+				G_TYPE_STRING,		/* Saved? */
+				G_TYPE_STRING,		/* Internal? */
 				G_TYPE_STRING,		/* Type */
 				G_TYPE_STRING,		/* Name */
 				G_TYPE_STRING,		/* Value */
-				G_TYPE_STRING,		/* Persistent */
 				G_TYPE_UINT));		/* property_t */
 
 	gtk_tree_view_set_model(tv, GTK_TREE_MODEL(store));
 	g_object_unref(store);
 
-	for (i = 0; i < G_N_ELEMENTS(columns); i++) {
+	for (i = 0; i < N_ITEMS(columns); i++) {
 		GtkTreeViewColumn *column;
 		GtkCellRenderer *renderer;
 
@@ -586,39 +633,45 @@ dbg_tree_init(void)
 			gui_signal_connect(renderer, "edited", on_cell_edited, NULL);
 			g_object_set(renderer,
 					"editable", TRUE,
-					(void *) 0);
+					NULL_PTR);
 		}
 
 		column = gtk_tree_view_column_new_with_attributes(
 				_(columns[i].title), renderer,
 				"text", i,
-				(void *) 0);
+				NULL_PTR);
 
 		g_object_set(renderer,
 				"xalign", 0.0,
 				"xpad", GUI_CELL_RENDERER_XPAD,
 				"ypad", GUI_CELL_RENDERER_YPAD,
-				(void *) 0);
+				NULL_PTR);
 		g_object_set(column,
 				"min-width", 1,
 				"resizable", TRUE,
 				"reorderable", FALSE,
-				(void *) 0);
-		
+				NULL_PTR);
+
 		if (columns[i].width) {
 			g_object_set(column,
 				"fixed-width", columns[i].width,
 				"sizing", GTK_TREE_VIEW_COLUMN_FIXED,
-				(void *) 0);
+				NULL_PTR);
 		} else {
 			g_object_set(column,
 				"sizing", GTK_TREE_VIEW_COLUMN_AUTOSIZE,
-				(void *) 0);
+				NULL_PTR);
 		}
-		
+
 
 		gtk_tree_view_column_set_sort_column_id(column, i);
 		gtk_tree_view_append_column(tv, column);
+
+		gtk_tree_sortable_set_sort_func(
+			GTK_TREE_SORTABLE(gtk_tree_view_get_model(tv)),
+			i, dbg_column_cmp, uint_to_pointer(i), NULL);
+
+		column_sort_tristate_register(column, on_dbg_column_clicked, NULL);
 	}
 
 	gui_signal_connect(tv, "enter-notify-event", on_enter_notify, tv);
@@ -626,24 +679,24 @@ dbg_tree_init(void)
 	gui_signal_connect(tv, "cursor-changed", on_cursor_changed, tv);
 }
 
-void
-dbg_property_show_list(const GSList *props)
+static void
+dbg_property_show_list(const pslist_t *props)
 {
-	const GSList *sl;
+	const pslist_t *sl;
 	GtkTreeView *tv;
 	GtkListStore *store;
-	
+
 	tv = GTK_TREE_VIEW(gui_dlg_prefs_lookup("treeview_dbg_property"));
 	if (!gtk_tree_view_get_model(tv))
 		dbg_tree_init();
-		
+
 	store = GTK_LIST_STORE(gtk_tree_view_get_model(tv));
 	gtk_list_store_clear(store);
 
 	if (!props) {
 		const gchar *text = _("<no property selected>");
 		GtkWidget *widget;
-		
+
 		widget = gui_dlg_prefs_lookup("label_dbg_property_limits");
 		gtk_label_set_text(GTK_LABEL(widget), text);
 		/* Gtk+ 2.x has editable column cells */
@@ -661,7 +714,7 @@ dbg_property_show_list(const GSList *props)
 #endif /* USE_GTK2 */
 	}
 
-	for (sl = props; NULL != sl; sl = g_slist_next(sl)) {
+	PSLIST_FOREACH(props, sl) {
 		GtkTreeIter iter;
 		property_t prop;
 
@@ -669,11 +722,12 @@ dbg_property_show_list(const GSList *props)
 
 		gtk_list_store_append(store, &iter);
 		gtk_list_store_set(store, &iter,
-			dbg_col_saved,		gnet_prop_is_saved(prop) ? _("Yes") : _("No"),
-			dbg_col_type,		gnet_prop_type_to_string(prop),
-			dbg_col_name,		gnet_prop_name(prop),
-			dbg_col_value,		gnet_prop_to_string(prop),
-			dbg_col_property,	(guint) prop,
+			dbg_col_saved,     gnet_prop_is_saved(prop) ? _("Yes") : _("No"),
+			dbg_col_internal,  gnet_prop_is_internal(prop) ? _("Yes") : _("No"),
+			dbg_col_type,      gnet_prop_type_to_string(prop),
+			dbg_col_name,      gnet_prop_name(prop),
+			dbg_col_value,     gnet_prop_to_string(prop),
+			dbg_col_property,  (guint) prop,
 			(-1));
 	}
 }
@@ -734,6 +788,9 @@ dbg_property_set_row(GtkCList *clist, gint row, property_t prop)
 		case dbg_col_saved:
 			text = gnet_prop_is_saved(prop) ? _("Yes") : _("No");
 			break;
+		case dbg_col_internal:
+			text = gnet_prop_is_internal(prop) ? _("Yes") : _("No");
+			break;
 		case dbg_col_type:
 			text = gnet_prop_type_to_string(prop);
 			break;
@@ -753,22 +810,23 @@ dbg_property_set_row(GtkCList *clist, gint row, property_t prop)
 }
 
 static void
-dbg_property_show_list(const GSList *props)
+dbg_property_show_list(const pslist_t *props)
 {
 	GtkCList *clist;
-	const GSList *sl;
+	const pslist_t *sl;
 
    	clist = GTK_CLIST(gui_dlg_prefs_lookup("clist_dbg_property"));
 	gtk_clist_freeze(clist);
 	gtk_clist_clear(clist);
 
-	for (sl = props; NULL != sl; sl = g_slist_next(sl)) {
-		static const gchar * const titles[num_dbg_cols] = { "", "", "", "", };
+	PSLIST_FOREACH(props, sl) {
+		static const char * const titles[num_dbg_cols] =
+			{ "", "", "", "", "", };
 		property_t prop = GPOINTER_TO_UINT(sl->data);
 		gint row;
-		
+
     	row = gtk_clist_append(clist, deconstify_gpointer(titles));
-		dbg_property_set_row(clist, row, prop);	
+		dbg_property_set_row(clist, row, prop);
 	}
 	gtk_clist_sort(clist);
 	gtk_clist_columns_autosize(clist);
@@ -790,12 +848,15 @@ on_entry_dbg_property_value_activate(GtkEditable *editable,
 		property_t prop;
 		gint row;
 
-		text = STRTRACK(gtk_editable_get_chars(editable, 0, -1));
 		row = GPOINTER_TO_INT(clist->selection->data);
 		prop = GPOINTER_TO_UINT(gtk_clist_get_row_data(clist, row));
-		gnet_prop_set_from_string(prop,	text);
-		dbg_property_set_row(clist, row, prop);
-		G_FREE_NULL(text);
+
+		if (!gnet_prop_is_internal(prop)) {
+			char *text = STRTRACK(gtk_editable_get_chars(editable, 0, -1));
+			gnet_prop_set_from_string(prop,	text);
+			dbg_property_set_row(clist, row, prop);
+			G_FREE_NULL(text);
+		}
 	}
 }
 
@@ -814,7 +875,7 @@ on_clist_dbg_property_select_row(GtkCList *unused_clist, gint unused_row,
 }
 
 static gboolean dbg_property_cmp_name_inverted = TRUE;
-static gint 
+static gint
 dbg_property_cmp_name(GtkCList *unused_clist,
 	gconstpointer ptr1, gconstpointer ptr2)
 {
@@ -828,7 +889,7 @@ dbg_property_cmp_name(GtkCList *unused_clist,
 }
 
 static gboolean dbg_property_cmp_type_inverted = TRUE;
-static gint 
+static gint
 dbg_property_cmp_type(GtkCList *unused_clist,
 	gconstpointer ptr1, gconstpointer ptr2)
 {
@@ -842,7 +903,7 @@ dbg_property_cmp_type(GtkCList *unused_clist,
 }
 
 static gboolean dbg_property_cmp_saved_inverted = TRUE;
-static gint 
+static gint
 dbg_property_cmp_saved(GtkCList *unused_clist,
 	gconstpointer ptr1, gconstpointer ptr2)
 {
@@ -861,7 +922,7 @@ on_clist_dbg_property_click_column(GtkCList *clist, gint column,
 	gpointer unused_udata)
 {
 	gboolean do_sort = FALSE;
-	
+
 	(void) unused_udata;
 
 	g_assert(column >= 0 && column < num_dbg_cols);
@@ -888,8 +949,9 @@ on_clist_dbg_property_click_column(GtkCList *clist, gint column,
 	case num_dbg_cols:
 		g_assert_not_reached();
 	}
-			
-	gtk_clist_sort(clist);
+
+	if (do_sort)
+		gtk_clist_sort(clist);
 }
 #endif /* USE_GTK1 */
 
@@ -897,7 +959,7 @@ void
 on_entry_dbg_property_pattern_activate(GtkEditable *unused_editable,
 	gpointer unused_udata)
 {
-	static gchar old_pattern[1024];
+	static gchar old_pattern[1024] = { '\0' };
    	gchar *text;
 
 	(void) unused_editable;
@@ -908,19 +970,18 @@ on_entry_dbg_property_pattern_activate(GtkEditable *unused_editable,
         0, -1));
 	g_strstrip(text);
 
-	if (0 != strcmp(text, old_pattern)) {
-		GSList *props;
+	if (0 != strcmp(text, old_pattern) || '\0' == text[0]) {
+		pslist_t *props;
 
-		g_strlcpy(old_pattern, text, sizeof old_pattern);
+		cstr_bcpy(ARYLEN(old_pattern), text);
 		props = gnet_prop_get_by_regex(text, NULL);
 		if (!props)
 			statusbar_gui_warning(10,
 				_("No property name matches the pattern \"%s\"."), text);
-		
+
 		dbg_property_show_list(props);
 		dbg_property_update_selection();
-		g_slist_free(props);
-		props = NULL;
+		pslist_free_null(&props);
 	}
 	G_FREE_NULL(text);
 }
@@ -946,7 +1007,7 @@ on_menu_sidebar_visible_activate(GtkMenuItem *menuitem, gpointer unused_udata)
 	 * visibility status changes.
 	 */
 #ifdef USE_GTK1
-	{	
+	{
 		GtkPaned *paned;
 		gboolean sidebar;
 
