@@ -1,6 +1,4 @@
 /*
- * $Id: upload_stats.h 6425 2004-08-24 23:18:29Z rmanfredi $
- *
  * Copyright (c) 2001-2003, Richard Eckart
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -29,7 +27,13 @@
 #include "if/core/uploads.h"
 #include "if/ui/gtk/upload_stats.h"
 
-void upload_stats_gui_init(void);
 void upload_stats_gui_shutdown(void);
+void upload_stats_common_gui_init(void);
+void upload_stats_common_gui_shutdown(void);
+void upload_stats_gui_clear_model();
+
+void upload_stats_gui_update_model(struct ul_stats *us);
 
 #endif /* _gtk_uploads_stats_h_ */
+
+/* vi: set ts=4 sw=4 cindent: */
