@@ -1,6 +1,4 @@
 /*
- * $Id: uploads_common.c 16990 2009-09-25 15:23:43Z rmanfredi $
- *
  * Copyright (c) 2001-2003, Richard Eckart
  *
  *----------------------------------------------------------------------
@@ -19,14 +17,12 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
 #include "gui.h"
 
-RCSID("$Id: uploads_common.c 16990 2009-09-25 15:23:43Z rmanfredi $")
-
 #include "uploads.h"			/* For upload_row_data_t */
 #include "uploads_common.h"
 #include "search.h"
@@ -40,9 +36,10 @@ RCSID("$Id: uploads_common.c 16990 2009-09-25 15:23:43Z rmanfredi $")
 
 #include "lib/concat.h"
 #include "lib/host_addr.h"
-#include "lib/glib-missing.h"	/* For gm_snprintf() */
-#include "lib/tm.h"
+#include "lib/str.h"
 #include "lib/stringify.h"
+#include "lib/tm.h"
+
 #include "lib/override.h"		/* Must be the last header included */
 
 #define IO_STALLED	60	/**< If nothing exchanged after that many secs */
@@ -110,19 +107,21 @@ uploads_gui_status_str(const gnet_upload_status_t *u,
     case GTA_UL_COMPLETE:
 		{
 			time_delta_t d = delta_time(data->last_update, data->start_date);
+			time_delta_t x = delta_time(data->last_update,
+								MAX(data->send_date, data->start_date));
 	        filesize_t requested = data->range_end - data->range_start + 1;
 			size_t rw;
 
-			rw = gm_snprintf(tmpstr, sizeof(tmpstr),
-				"%s (%s) %s %s#%u", _("Completed"),
-				d > 0 ? short_rate(requested / d, show_metric_units())
-						: _("< 1s"),
-				d > 0 ? short_time(d) : "",
+			rw = str_bprintf(ARYLEN(tmpstr),
+				"%s (%s in %s) %s %s#%u", _("Completed"),
+				short_rate(requested / MAX(x, 1), show_metric_units()),
+				short_time(x),
+				short_time2(d),
 				u->parq_quick ? _("(quick) ") : "",
 				u->reqnum);
 
 			if (u->error_count)
-				rw += gm_snprintf(&tmpstr[rw], sizeof(tmpstr)-rw,
+				rw += str_bprintf(ARYPOSLEN(tmpstr, rw),
 					_(" E=%u"), u->error_count);
 		}
         break;
@@ -131,23 +130,36 @@ uploads_gui_status_str(const gnet_upload_status_t *u,
 		{
 			/* Time Remaining at the current rate, in seconds  */
 			filesize_t tr = (data->range_end + 1 - u->pos) / MAX(1, u->avg_bps);
-			gdouble p = uploads_gui_progress(u, data);
+			double p = uploads_gui_progress(u, data);
 			time_t now = tm_time();
-			gboolean stalled = delta_time(now, data->last_update) > IO_STALLED;
-			gchar pbuf[32];
+			bool stalled = delta_time(now, data->last_update) > IO_STALLED;
+			char pbuf[32];
+			char dbuf[32];
+			char cbuf[32];
 			size_t rw;
 
-			gm_snprintf(pbuf, sizeof pbuf, "%5.02f%% ", p * 100.0);
-			rw = gm_snprintf(tmpstr, sizeof tmpstr, _("%s(%s) TR: %s %s#%u"),
+			if (u->bw_penalty != 0)
+				str_bprintf(ARYLEN(dbuf), " [/%u]", 1U << u->bw_penalty);
+
+			if (u->bw_cap != 0) {
+				str_bprintf(ARYLEN(cbuf), " [<%s]",
+					short_rate(u->bw_cap, show_metric_units()));
+			}
+
+			str_bprintf(ARYLEN(pbuf), "%5.02f%% ", p * 100.0);
+			rw = str_bprintf(ARYLEN(tmpstr),
+				_("%s(%s)%s%s TR: %s %s#%u"),
 				p > 1.0 ? pbuf : "",
 				stalled ? _("stalled")
 					: short_rate(u->bps, show_metric_units()),
+				u->bw_penalty != 0 ? dbuf : "",
+				u->bw_cap != 0 ? cbuf : "",
 				short_time(tr),
 				u->parq_quick ? _("(quick) ") : "",
 				u->reqnum);
 
 			if (u->error_count)
-				rw += gm_snprintf(&tmpstr[rw], sizeof(tmpstr)-rw,
+				rw += str_bprintf(ARYPOSLEN(tmpstr, rw),
 					_(" E=%u"), u->error_count);
 		}
 		break;
@@ -157,11 +169,11 @@ uploads_gui_status_str(const gnet_upload_status_t *u,
 
     case GTA_UL_EXPECTING:
 		if (u->error_count)
-			gm_snprintf(tmpstr, sizeof(tmpstr),
+			str_bprintf(ARYLEN(tmpstr),
 				_("%s %s#%u E=%u"), _("Waiting for further request..."),
 				u->parq_quick ? _("(quick) ") : "", u->reqnum, u->error_count);
 		else
-			gm_snprintf(tmpstr, sizeof(tmpstr),
+			str_bprintf(ARYLEN(tmpstr),
 				"%s %s#%u", _("Waiting for further request..."),
 				u->parq_quick ? _("(quick) ") : "", u->reqnum);
 		break;
@@ -200,13 +212,12 @@ uploads_gui_status_str(const gnet_upload_status_t *u,
 			queued = u->parq_position > available;
 
 			if (u->parq_retry > 0) {
-				gm_snprintf(tbuf, sizeof tbuf,
-							" %s,", short_time(u->parq_retry));
+				str_bprintf(ARYLEN(tbuf), " %s,", short_time(u->parq_retry));
 			} else {
 				tbuf[0] = '\0';
 			}
 
-			rw = gm_snprintf(tmpstr, sizeof tmpstr,
+			rw = str_bprintf(ARYLEN(tmpstr),
 						_("%s [%d] (slot %d/%d)%s %s %s"),
 						u->parq_frozen ? _("Frozen") :
 						queued ? _("Queued") : _("Waiting"),
@@ -218,7 +229,7 @@ uploads_gui_status_str(const gnet_upload_status_t *u,
 						short_time(u->parq_lifetime));
 
 			if (u->error_count)
-				rw += gm_snprintf(&tmpstr[rw], sizeof(tmpstr)-rw,
+				rw += str_bprintf(ARYPOSLEN(tmpstr, rw),
 					_(" E=%u"), u->error_count);
 		}
 		break;
@@ -294,7 +305,7 @@ upload_should_remove(time_t now, const upload_row_data_t *ul)
 /**
  * @return A pointer to a static buffer holding the host address as string.
  */
-const gchar * 
+const gchar *
 uploads_gui_host_string(const gnet_upload_info_t *u)
 {
 	static gchar buf[1024];
@@ -306,13 +317,14 @@ uploads_gui_host_string(const gnet_upload_info_t *u)
 		peer = NULL;
 	}
 
-	concat_strings(buf, sizeof buf,
+	concat_strings(ARYLEN(buf),
 		host_addr_to_string(u->addr),
-		u->encrypted ? " (E) " : " ",
+		u->encrypted ? (u->tls_upgraded ? " (e) " : " (E) ") : "",
+		u->g2 ? " [G2] " : "",
 		peer ? " <" : "",
 		peer ? peer : "",
 		peer ? ">"  : "",
-		(void *) 0);
+		NULL_PTR);
 
 	return buf;
 }
